@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -57,7 +70,6 @@ import javax.xml.parsers.ParserConfigurationException;
 import org.xml.sax.SAXException;
 import org.xml.sax.EntityResolver;
 import org.xml.sax.InputSource;
-import com.sun.org.apache.xerces.internal.parsers.DOMParser;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Node;
@@ -67,16 +79,17 @@ import org.w3c.dom.NamedNodeMap;
  *
  * @author  pa100654
  */
-public class CLIDescriptorsReader 
+public final class CLIDescriptorsReader 
 {
-    
-    public static int DONT_SERIALIZE = 0;
-    public static int SERIALIZE_COMMANDS_TO_FILES = 1;
-    public static int SERIALIZE_COMMANDS_TO_FILE = 2;
-    public static String SERIALIZED_DESCRIPTOR_FILE = ".CLIDescriptors";
-    public static String SERIALIZED_COMMANDS_DEFAULT_DIR = "cli";
-    public static String ENVIRONMENT_PREFIX = "environment-prefix";
-    public static String ENVIRONMENT_FILENAME = "environment-filename";
+    public static final int DONT_SERIALIZE = 0;
+    public static final int SERIALIZE_COMMANDS_TO_FILES = 1;
+    public static final int SERIALIZE_COMMANDS_TO_FILE = 2;
+    public static final String SERIALIZED_DESCRIPTOR_FILE = ".CLIDescriptors";
+    public static final String SERIALIZED_COMMANDS_DEFAULT_DIR = "cli";
+    public static final String ENVIRONMENT_PREFIX = "environment-prefix";
+    public static final String ENVIRONMENT_FILENAME = "environment-filename";
+    private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
+    private static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
     //static instance of itself
     private static CLIDescriptorsReader cliDescriptorsReader;
     
@@ -91,16 +104,16 @@ public class CLIDescriptorsReader
     private ValidCommandsList commandsList = null;
     
     // List of valid options from all the components
-    private HashMap validOptions = null;
+    private HashMap<String, ValidOption> validOptions = null;
 
     // List of options to be replaced by all the commands in the descriptor files
-    private HashMap replaceOptions = null;
+    private HashMap<String, ValidOption> replaceOptions = null;
 
     // List of descriptors (URLs)
-    private Vector descriptors = null;
+    private Vector<URL> descriptors = null;
     
     // Vector of Properties from all the descriptor files
-    private Vector properties = null;
+    private Vector<Properties> properties = null;
     
     // Default command to use
     private String defaultCommand = null;
@@ -110,11 +123,11 @@ public class CLIDescriptorsReader
     
 
 
-    private static String DESCRIPTOR_FILE_NAME = "CLIDescriptor.xml";
+    private static final String DESCRIPTOR_FILE_NAME = "CLIDescriptor.xml";
     /** 
         Construct CLIDescriptorsReader object 
     */
-    protected CLIDescriptorsReader()
+    private CLIDescriptorsReader()
     {
         initialize();
     }
@@ -124,7 +137,7 @@ public class CLIDescriptorsReader
         Construct CLIDescriptorsReader object with the given arguments.
         @param descriptor, the URL for the CLI Descriptor file
     */
-    protected CLIDescriptorsReader(URL descriptor)
+    private CLIDescriptorsReader( final URL descriptor)
     {
         setDescriptor(descriptor);
         initialize();
@@ -135,7 +148,7 @@ public class CLIDescriptorsReader
         Construct CLIDescriptorsReader object with the given arguments.
 	@param descriptors, the URL's for the CLI Descriptor file
     */
-    protected CLIDescriptorsReader(Vector descriptors) 
+    private CLIDescriptorsReader( final Vector<URL> descriptors) 
     {
         this.descriptors = descriptors;
         initialize();
@@ -148,16 +161,16 @@ public class CLIDescriptorsReader
     private void initialize()
     {
         commandsList = new ValidCommandsList();
-        validOptions = new HashMap();
-        replaceOptions = new HashMap();
-        properties = new Vector();
+        validOptions = new HashMap<String, ValidOption>();
+        replaceOptions = new HashMap<String, ValidOption>();
+        properties = new Vector<Properties>();
     }
 
 
     /**
      * returns the instance of the CLIDescriptorsReader
      */
-    public static CLIDescriptorsReader getInstance()
+    public static synchronized CLIDescriptorsReader getInstance()
     {
         if (cliDescriptorsReader == null)
         {
@@ -228,7 +241,7 @@ public class CLIDescriptorsReader
       Returns the properties list of all the commands module's
       @return Iterator the list of Properties
      */
-    public Iterator getProperties() throws CommandValidationException
+    public Iterator<Properties> getProperties() throws CommandValidationException
     {
         return properties.iterator();
     }
@@ -260,6 +273,7 @@ public class CLIDescriptorsReader
             //if command could not be found
         int saveSerializedDescriptor = getSerializeDescriptorsProperty();
         setSerializeDescriptorsProperty(DONT_SERIALIZE);
+            //setSerializeDescriptorsProperty(SERIALIZE_COMMANDS_TO_FILE);
         loadAllCommandsFromFilesOrDescriptors();
             //put back setting for serizliedDescriptor
         setSerializeDescriptorsProperty(saveSerializedDescriptor);
@@ -358,6 +372,7 @@ public class CLIDescriptorsReader
         }
         else if (serializeDescriptors == SERIALIZE_COMMANDS_TO_FILES)
         {
+                // currently not supported
             loadCommandsFromMultipleFiles();
         }
         else if (serializeDescriptors == SERIALIZE_COMMANDS_TO_FILE)
@@ -370,56 +385,12 @@ public class CLIDescriptorsReader
     /**
      *
      */
-    private void loadCommandsFromMultipleFiles() 
-	throws CommandValidationException
+    private void loadCommandsFromMultipleFiles() throws CommandValidationException
     {
-        File commandsDir = new File(serializeDir);
-
-        if ((commandsDir != null) && (commandsDir.exists()) && 
-              (getSerializedCommandsFileList(commandsDir).length > 0))
-        {
-            getDescriptors();
-            if (descriptors == null)
-            {
-		LocalStringsManager lsm = 
-		    LocalStringsManagerFactory.getFrameworkLocalStringsManager();
-		throw new CommandValidationException(lsm.getString("NoDescriptorsDefined"));
-            }
-            boolean isSerializedCommandsLatest = true;
-            for (int i = 0; (i < descriptors.size()) && 
-                            isSerializedCommandsLatest; i++)
-            {
-                URL descriptorURL = (URL) descriptors.get(i);
-                File descriptorFile = getDescriptorFile(descriptorURL);
-                if ((descriptorFile != null) && (descriptorFile.exists()))
-                {
-                    if (descriptorFile.lastModified() > commandsDir.lastModified())
-                    {
-                        isSerializedCommandsLatest = false;
-                        break;
-                    }
-                }
-                else
-                {
-                  // should we throw a CommandValidationException?   
-                }
-            }
-            if (isSerializedCommandsLatest)
-            {
-                if (!loadSerializedCommands())
-                {
-                    readDescriptors();
-                }
-            }
-            else
-            {
-                readDescriptors();
-            }
-        }
-        else
-        {
-            readDescriptors();
-        }
+            //currently not supported and may not be a feasible feature
+            //since reading all the serialized commands is timing consuming.
+            //so for now, read the descriptor file only.
+        readDescriptors();
     }
    
     
@@ -429,49 +400,14 @@ public class CLIDescriptorsReader
     private void loadCommandsFromSingleFile()
                     throws CommandValidationException
     {
-        File serializedDescriptorFile = getSerializedDescriptorFile();
-        if (serializedDescriptorFile.exists())
-    	    CLILogger.getInstance().printDebugMessage("Yes the file exists");
-        if (!((serializedDescriptorFile != null) && 
-              (serializedDescriptorFile.exists())))
-        {
-            readDescriptors();
-            return;
-        }
+        final Vector<URL> serializedDescriptorFile = getSerializedDescriptorFile();
 
-        getDescriptors();
-        if (descriptors == null)
-        {
-	    LocalStringsManager lsm = 
-		    LocalStringsManagerFactory.getFrameworkLocalStringsManager();
-	    throw new CommandValidationException(lsm.getString("NoDescriptorsDefined"));
+            //if there are no serialized descriptor file, then directly read 
+            //and parse the CLIDescriptor.xml file
+        if (serializedDescriptorFile.size() > 0) {
+            loadSerializedDescriptorFile(serializedDescriptorFile);
         }
-        boolean isSerializedDescriptorLatest = true;
-        for (int i = 0; (i < descriptors.size()) && 
-                        isSerializedDescriptorLatest; i++)
-        {
-            URL descriptorURL = (URL) descriptors.get(i);
-            File descriptorFile = getDescriptorFile(descriptorURL);
-            if ((descriptorFile != null) && (descriptorFile.exists()))
-            {
-                if (descriptorFile.lastModified() > 
-                        serializedDescriptorFile.lastModified())
-                {
-                    isSerializedDescriptorLatest = false;
-                    break;
-                }
-            }
-            else
-            {
-              // should we throw a CommandValidationException?   
-            }
-        }
-        if (isSerializedDescriptorLatest)
-        {
-            loadSerializedDescriptorFile();
-        }
-        else
-        {
+        else {
             readDescriptors();
         }
     }
@@ -481,26 +417,21 @@ public class CLIDescriptorsReader
         Parse and read the XML Descriptors file and populate it into 
         commandsList
      */
-    private void readDescriptors() throws CommandValidationException
+    public void readDescriptors() throws CommandValidationException
     {
         commandsList.removeAllCommands();
-        //DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
-        //factory.setValidating(true);   
-        //factory.setNamespaceAware(true);
         try 
         {
+            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
             // setup schema processing
-            DOMParser parser = new DOMParser();
-            parser.setFeature( "http://apache.org/xml/features/dom/defer-node-expansion", true);
-            parser.setFeature( "http://xml.org/sax/features/validation", true );
-            parser.setFeature( "http://xml.org/sax/features/namespaces", true );
-            parser.setFeature( "http://apache.org/xml/features/validation/schema", true );
-            parser.setFeature ("http://apache.org/xml/features/dom/include-ignorable-whitespace", false);
-            parser.setEntityResolver(new CLIEntityResolver());
+            factory.setValidating(true);   
+            factory.setNamespaceAware(true);
+            factory.setIgnoringElementContentWhitespace(true);
+            // setting the schema validation
+            factory.setAttribute(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
 
-            //factory.setValidating(true);
-	        // CLILogger.getInstance().printDebugMessage("Validation against DTD is " + factory.isValidating());
-            //DocumentBuilder builder = factory.newDocumentBuilder();
+            DocumentBuilder builder = factory.newDocumentBuilder();
+            builder.setEntityResolver(new CLIEntityResolver());
             if (descriptors == null)
             {
                 getDescriptors();
@@ -513,17 +444,18 @@ public class CLIDescriptorsReader
             }
             for (int i = 0; i < descriptors.size(); i++)
             {
+                CLILogger.getInstance().printDebugMessage(i + " descriptor = " + descriptors.get(i));
+                
                 URL descriptorURL = (URL) descriptors.get(i);
-                //InputSource is = new InputSource(descriptorURL.toString());
-                //Document document = builder.parse(is);
-                parser.parse(new InputSource(descriptorURL.toString()));
-                Document document = parser.getDocument();
+                InputSource is = new InputSource(descriptorURL.toString());
+                Document document = builder.parse(is);
                 generateOptionsAndCommands(document);
             }
                 //call this last after all descriptors have been read since we want to
                 //replace all the options with the options defined in ReplaceOptions
                 //element in the last XML in the list.
             replaceOptionsInCommandsList(replaceOptions);
+            validateOptionsInCommand();
             
             if (serializeDescriptors == SERIALIZE_COMMANDS_TO_FILES)
             {
@@ -542,6 +474,10 @@ public class CLIDescriptorsReader
             if (sxe.getException() != null)
                x = sxe.getException();
 	    throw new CommandValidationException(x);
+        } 
+        catch (ParserConfigurationException pxe)
+        {
+            throw new CommandValidationException(pxe);
         }
         catch (IOException ioe) 
         {
@@ -565,44 +501,38 @@ public class CLIDescriptorsReader
     }
     
     /**
-     * Returns the File object of the single serialized file, containing all
-     * the commands
-     * @return File file object of the serialized file containing all commands
+     * Returns the Vector contains all the serialized descriptor files. 
+     * @return Vector.  an empty vector is returned if there are no
+     * serialized descriptor file.
      */
-    private File getSerializedDescriptorFile()
+    private Vector<URL> getSerializedDescriptorFile() throws CommandValidationException
     {
-        File file = null;
+        Vector<URL> descriptorFiles = new Vector<URL>();
         try
         {
-            file = new File(serializeDir + "/" + SERIALIZED_DESCRIPTOR_FILE);
-        }
-        catch (NullPointerException npe)
-        {
-        }
-        
-        return file;
-    }
-    
-    
-    /**
-     * Returns the file object of the component
-     * @param  descriptor the URL object of the component
-     * @return File file object of the descriptor
-     */
-    private File getDescriptorFile(URL descriptor)
-    {
-        File file = null;
-        try
-        {
-            file = new File(descriptor.getFile());
-            if (!file.exists())
-                file = null;
-        }
-        catch (NullPointerException e)
-        {
+            final String sDescriptorFile = serializeDir + "/" +
+                                           SERIALIZED_DESCRIPTOR_FILE;
             
+            Enumeration<URL> urls = CLIDescriptorsReader.class.getClassLoader().getResources(sDescriptorFile);
+            if ((urls == null) || (!urls.hasMoreElements()))
+            {
+                //return an empty vector
+                return descriptorFiles;
+            }
+
+            while (urls.hasMoreElements())
+            {
+                URL url = (URL) urls.nextElement();
+                descriptorFiles.add(url);
+            }
         }
-        return file;
+        catch(IOException ioe)
+        {
+   	       LocalStringsManager lsm = 
+		   LocalStringsManagerFactory.getFrameworkLocalStringsManager();
+	       CLILogger.getInstance().printMessage(lsm.getString("CouldNotLoadDescriptor"));
+        }
+        return descriptorFiles;
     }
     
     
@@ -615,14 +545,13 @@ public class CLIDescriptorsReader
                     throws CommandValidationException
     {
         File commandsDir = new File(serializeDir);
+        String[] commandFiles = getSerializedCommandsFileList(commandsDir);
 
         if (!((commandsDir != null) && (commandsDir.exists()) && 
-              (getSerializedCommandsFileList(commandsDir).length > 0)))
+              (commandFiles.length > 0)))
         {
             return false;
         }
-
-        String[] commandFiles = getSerializedCommandsFileList(commandsDir);
         
         boolean loaded = true;
         for (int i = 0; (i < commandFiles.length) && loaded; i++)
@@ -654,37 +583,47 @@ public class CLIDescriptorsReader
     
     
     /**
-     * Loads the serialized commands from a single file which is 
-        SERIALIZED_COMMANDS_FILE
+     * Loads commands from serialized descriptor file
      */
-    private void loadSerializedDescriptorFile()
+    private void loadSerializedDescriptorFile(final Vector<URL> descriptorFile) throws CommandValidationException
     {
         try
         {
-            File fileName = getSerializedDescriptorFile();
-            // expecting the additional checking for validity of SERIALIZED_COMMANDS_FILE
-            // is already done prior to calling this method.
-            ObjectInputStream in = new ObjectInputStream(new FileInputStream(fileName));
-            
-            /** read CommandProperties to serialized file **/
-            defaultCommand = (String) in.readObject();
-            helpClass = (String) in.readObject();
-            properties = (Vector) in.readObject();
-
-            commandsList = (ValidCommandsList) in.readObject();
-            CLILogger.getInstance().printDebugMessage("++++++++++++++++++++++++++++ Commands loaded from file");
+            for (int ii=0; ii<descriptorFile.size(); ii++) {
+                final URL descriptorURL = (URL) descriptorFile.get(ii);
+                ObjectInputStream ois = new ObjectInputStream(descriptorURL.openStream());
+                
+                defaultCommand = (String) ois.readObject();
+                helpClass = (String) ois.readObject();
+                properties = (Vector) ois.readObject();
+                
+                final ValidCommandsList validCommandsList = (ValidCommandsList)ois.readObject();
+                    //if commandsList is not empty then need to load each
+                    //command in commandsList.
+                if (commandsList != null  && commandsList.size() > 0) {
+                    final Iterator<ValidCommand> commands = validCommandsList.getCommands();
+                    while (commands.hasNext()) {
+                        commandsList.addCommand((ValidCommand)commands.next());
+                    }
+                }
+                else {
+                    commandsList = validCommandsList;
+                }
+                
+                CLILogger.getInstance().printDebugMessage("++++++++++++++++++++++++++++ Commands loaded from file");
+            }
         }
         catch(FileNotFoundException fnfe)
         {
-            
+            throw new CommandValidationException(fnfe);
         }
         catch(IOException ioe)
         {
-            
+            throw new CommandValidationException(ioe);
         }
         catch(ClassNotFoundException cnfe)
         {
-            
+            throw new CommandValidationException(cnfe);
         }
     }
     
@@ -735,7 +674,7 @@ public class CLIDescriptorsReader
         {
             if (descriptors == null)
             {
-                descriptors = new Vector();
+                descriptors = new Vector<URL>();
             }
             descriptors.add(descriptor);
             //reverse the order in the Vector so that the last CLIDescriptor.xml will be parsed first.
@@ -748,7 +687,7 @@ public class CLIDescriptorsReader
         Sets the descriptor files (components)
 	@param descriptors the descriptor files containing the command specifications
      */
-    public void setDescriptors(Vector descriptors) 
+    public void setDescriptors(Vector<URL> descriptors) 
     {
         this.descriptors = descriptors;
     }
@@ -759,6 +698,7 @@ public class CLIDescriptorsReader
         from the descriptor file
      */
     private void generateOptionsAndCommands(Document document)
+        throws CommandValidationException
     {
         if (document != null) 
         {
@@ -804,7 +744,17 @@ public class CLIDescriptorsReader
                         if (grandKidNodeName.equalsIgnoreCase("Option")) 
                         {
                             ValidOption option = generateOption(document, grandKid);
-                            validOptions.put(option.getName(), option);
+                            final String optionName =  option.getName();
+                                //do not allow duplication option name defined
+                            if (!validOptions.containsKey(optionName)) {
+                                validOptions.put(optionName, option);
+                            }
+                            else {
+                                LocalStringsManager lsm = 
+                                                    LocalStringsManagerFactory.getFrameworkLocalStringsManager();
+                                throw new CommandValidationException(lsm.getString("DuplicateOptionDeclaration", new Object[]{optionName}) );
+                            }
+                            
                         }
                         else if (grandKidNodeName.equalsIgnoreCase("ReplaceOption")) 
                         {
@@ -882,6 +832,7 @@ public class CLIDescriptorsReader
      @return returns a ValidCommand
      */
     private ValidCommand generateCommand(Document document, Node grandKid)
+        throws CommandValidationException
     {
         ValidCommand command = new ValidCommand();
 
@@ -908,31 +859,33 @@ public class CLIDescriptorsReader
                 nextGrandKid = nextGrandKid.getNextSibling()) 
         {
             String grandKidName = nextGrandKid.getNodeName();
+            LocalStringsManager lsm = LocalStringsManagerFactory.getFrameworkLocalStringsManager();
             if (grandKidName.equalsIgnoreCase("ValidOption"))
             {
                 final NamedNodeMap validOptionNodeMap = 
                                 nextGrandKid.getAttributes();
                 final String validOption = 
                         validOptionNodeMap.getNamedItem("name").getNodeValue();
+                if (command.hasValidOption(validOption)) {
+                    throw new CommandValidationException(lsm.getString("OptionAlreadyDefined",
+                                                                       new Object[] {validOption,
+                                                                       command.getName()} ));
+                }
+                
                 String deprecatedOption = null;
                 if (validOptionNodeMap.getNamedItem("deprecatedoption") != null)
                 {
                     deprecatedOption = validOptionNodeMap.getNamedItem("deprecatedoption").getNodeValue();
                 }
                 String defaultValue = null;
-                if (validOptionNodeMap.getNamedItem("defaultvalue") != null)
-                {
+                if (validOptionNodeMap.getNamedItem("defaultvalue") != null) {
                     defaultValue = validOptionNodeMap.getNamedItem("defaultvalue").getNodeValue();
                 }
 
                 final ValidOption option = findOption(validOption);
-                if (option == null)
-                {
-                    CLILogger.getInstance().printDebugMessage("Valid option (" + 
-							      validOption + 
-							      ")is not found in xml : " + 
-							      command.getName());
-                    return command;
+                if (option == null) {
+                    throw new CommandValidationException(lsm.getString("ValidOptionNotDefined",
+                                                                       new Object[] {validOption} ));
                 }
                 ValidOption newOption = new ValidOption(option);
                 if (deprecatedOption != null)
@@ -948,6 +901,12 @@ public class CLIDescriptorsReader
                                 nextGrandKid.getAttributes();
                 final String reqdOption = 
                         reqdOptionNodeMap.getNamedItem("name").getNodeValue();
+                if (command.hasRequiredOption(reqdOption)) {
+                    throw new CommandValidationException(lsm.getString("OptionAlreadyDefined",
+                                                                       new Object[] {reqdOption,
+                                                                       command.getName()} ));
+                }
+                
                 String deprecatedOption = null;
                 if (reqdOptionNodeMap.getNamedItem("deprecatedoption") != null)
                 {
@@ -957,11 +916,8 @@ public class CLIDescriptorsReader
                 final ValidOption option = findOption(reqdOption);
                 if (option == null)
                 {
-                    CLILogger.getInstance().printDebugMessage("Required option (" + 
-							      reqdOption + 
-							      ")is not found in xml : " + 
-							      command.getName());
-                    return command;
+                    throw new CommandValidationException(lsm.getString("RequiredOptionNotDefined",
+                                                                       new Object[] {reqdOption} ));
                 }
                 if (deprecatedOption != null)
                     option.setDeprecatedOption(deprecatedOption);
@@ -970,14 +926,17 @@ public class CLIDescriptorsReader
             else if (grandKidName.equalsIgnoreCase("DeprecatedOption"))
             {
                 final String deprecatedOption = nextGrandKid.getFirstChild().getNodeValue();
+                if (command.hasDeprecatedOption(deprecatedOption)) {
+                    throw new CommandValidationException(lsm.getString("OptionAlreadyDefined",
+                                                                       new Object[] {deprecatedOption,
+                                                                       command.getName()} ));
+                }
+                
                 final ValidOption option = findOption(deprecatedOption);
                 if (option == null)
                 {
-                    CLILogger.getInstance().printDebugMessage("Deprecated option (" + 
-							      deprecatedOption + 
-							      ")is not found in xml : " + 
-							      command.getName());
-                    return command;
+                    throw new CommandValidationException(lsm.getString("DeprecatedOptionNotDefined",
+                                                                       new Object[] {deprecatedOption} ));
                 }
                 command.addDeprecatedOption(option);
             }
@@ -994,7 +953,7 @@ public class CLIDescriptorsReader
                         String propertyNameAttr = 
                                     nodeMap.getNamedItem("name").getNodeValue();
 
-                        Vector values = new Vector(); 
+                        Vector<String> values = new Vector<String>(); 
                         for (Node nextGreatGreatGrandKid = nextGreatGrandKid.getFirstChild();
                                 nextGreatGreatGrandKid != null;
                                 nextGreatGreatGrandKid = nextGreatGreatGrandKid.getNextSibling()) 
@@ -1192,6 +1151,60 @@ public class CLIDescriptorsReader
             }
         }
     }
+
+
+        /**
+         *  This method checks for clashing option names and short options in a command.
+         **/  
+    private void validateOptionsInCommand() throws CommandValidationException
+    {
+        final Iterator commands = commandsList.getCommands();
+        while (commands.hasNext())
+        {
+            final ValidCommand command = (ValidCommand) commands.next();
+            Vector<ValidOption> vvo = command.getOptions();
+            final LocalStringsManager lsm = LocalStringsManagerFactory.getFrameworkLocalStringsManager();
+            
+            
+            int ii = 1;
+            final int size = vvo.size();
+            for (ValidOption vo : vvo)
+            {
+                final String optionName = vo.getName();
+                    //get rest of the options and compare
+                if (ii<size) {
+                    java.util.List<ValidOption> sub_vvo = vvo.subList(ii,size);
+                    for (ValidOption sub_vo : sub_vvo)
+                    {
+//System.out.println("***** Compare: optionName = " + optionName + " and " + sub_vo.getName() + " for " + command.getName());
+                        if (optionName.equals(sub_vo.getName()))
+                        {
+                            throw new CommandValidationException(lsm.getString("OptionAlreadyDefined",
+                                                                 new Object[] {optionName, command.getName()} ));
+                        }
+                            //check short option clash
+                        if (vo.hasShortName()) 
+                        {
+                            final Vector<String> shortNames = vo.getShortNames();
+                            for (String shortName : shortNames)
+                            {
+                                final Vector<String> sub_shortNames = sub_vo.getShortNames();
+                                if (sub_shortNames.contains(shortName))
+                                {
+                                    throw new CommandValidationException(lsm.getString("ShortOptionAlreadyDefined",
+                                                                         new Object[] {shortName,
+                                                                         vo.getName(), sub_vo.getName(),
+                                                                         command.getName()} ));
+                                }
+                            }
+                        }
+                    }
+                }
+                ii++;
+            }
+        }
+    }
+    
     
 
     public String getEnvironmentPrefix(){
@@ -1233,12 +1246,23 @@ public class CLIDescriptorsReader
         {
             //read CLI descriptor
             final CLIDescriptorsReader cliDescriptorsReader = CLIDescriptorsReader.getInstance();
+            
+            final String SERIALIZE_TYPE = System.getProperty("SERIALIZE_TYPE");
 
-            //cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.SERIALIZE_COMMANDS_TO_FILE);
-            cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.SERIALIZE_COMMANDS_TO_FILES);
-            //cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.DONT_SERIALIZE);
+                //default is to serialize the commands to files
+            if (SERIALIZE_TYPE == null)
+               cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.SERIALIZE_COMMANDS_TO_FILES);
+            else {
+                if (SERIALIZE_TYPE.equals("SINGLE_FILE"))
+                    cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.SERIALIZE_COMMANDS_TO_FILE);
+                if (SERIALIZE_TYPE.equals("MULTIPLE_FILES"))
+                    cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.SERIALIZE_COMMANDS_TO_FILES);
+                if (SERIALIZE_TYPE.equals("DONT_SERIALIZE"))            
+                    cliDescriptorsReader.setSerializeDescriptorsProperty(CLIDescriptorsReader.DONT_SERIALIZE);
+            }
+            
 
-            Iterator commands = cliDescriptorsReader.getCommandsList().getCommands();
+            cliDescriptorsReader.readDescriptors();
 
                 /*
             while(commands.hasNext())
@@ -1257,6 +1281,7 @@ public class CLIDescriptorsReader
         catch(CommandValidationException cve)
         {
             System.out.println(cve.getLocalizedMessage());
+            System.exit(1);
         }
     }
 }
