@@ -235,7 +235,7 @@ public class Config extends ConfigBean implements Serializable
 						null, null);
 		this.createAttribute(SECURITY_SERVICE, "anonymous-role", "AnonymousRole", 
 						AttrProp.CDATA,
-						null, "ANYONE");
+						null, "AttributeDeprecated");
 		this.createAttribute(SECURITY_SERVICE, "audit-enabled", "AuditEnabled", 
 						AttrProp.CDATA,
 						null, "false");
@@ -623,11 +623,6 @@ public class Config extends ConfigBean implements Serializable
 		return (ManagementRules)this.getValue(MANAGEMENT_RULES);
 	}
 
-	// This attribute is an array, possibly empty
-	public void setSystemProperty(int index, SystemProperty value) {
-		this.setValue(SYSTEM_PROPERTY, index, value);
-	}
-
 	// Get Method
 	public SystemProperty getSystemProperty(int index) {
 		return (SystemProperty)this.getValue(SYSTEM_PROPERTY, index);
@@ -696,11 +691,6 @@ public class Config extends ConfigBean implements Serializable
 		return null;
 		
 	}
-	// This attribute is an array, possibly empty
-	public void setElementProperty(int index, ElementProperty value) {
-		this.setValue(ELEMENT_PROPERTY, index, value);
-	}
-
 	// Get Method
 	public ElementProperty getElementProperty(int index) {
 		return (ElementProperty)this.getValue(ELEMENT_PROPERTY, index);
@@ -783,6 +773,8 @@ public class Config extends ConfigBean implements Serializable
 	*/
 	public void setName(String v, boolean overwrite) throws StaleWriteConfigException {
 		setAttributeValue(ServerTags.NAME, v, overwrite);
+                /*hack, do it every time, manually, see bugs.sun.com - 6595919 */
+		setAttributeValue("Name", v, overwrite);
 	}
 	/**
 	* Modify  the Name of the Element config
@@ -790,6 +782,8 @@ public class Config extends ConfigBean implements Serializable
 	*/
 	public void setName(String v) {
 		setAttributeValue(ServerTags.NAME, v);
+                /*hack, do it every time, manually, see bugs.sun.com - 6595919 */
+		setAttributeValue("Name", v);
 	}
 	/**
 	* Getter for DynamicReconfigurationEnabled of the Element config
