@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 package com.sun.enterprise.deployment;
 
@@ -304,13 +317,16 @@ public class Application extends RootDeploymentDescriptor
 
                 //for archive deployment, we check the sub archives by its
                 //file extension; for directory deployment, we check the sub
-                //directories by its name, i.e. "_war" "_rar" etc.
+                //directories by its name. We are now supporting directory
+                //names with both "_suffix" and ".suffix".
 
                 //Section EE.8.4.2.1.a
                 String name = subModule.getName();
                 String uri = deriveArchiveUri(appRoot, subModule, directory);
                 if ( (!directory && name.endsWith(".war")) 
-                        || (directory && name.endsWith("_war")) ) {
+                        || (directory && 
+                            (name.endsWith("_war") || 
+                             name.endsWith(".war"))) ) {
                     String contextRoot = 
                     uri.substring(uri.lastIndexOf('/')+1, uri.lastIndexOf('.'));
                     ModuleDescriptor md = new ModuleDescriptor();
@@ -321,14 +337,18 @@ public class Application extends RootDeploymentDescriptor
                 }
                 //Section EE.8.4.2.1.b
                 else if ( (!directory && name.endsWith(".rar"))
-                            || (directory && name.endsWith("_rar")) ) {
+                            || (directory && 
+                                (name.endsWith("_rar") || 
+                                 name.endsWith(".rar"))) ) {
                     ModuleDescriptor md = new ModuleDescriptor();
                     md.setArchiveUri(uri);
                     md.setModuleType(ModuleType.RAR);
                     app.addModule(md);
                 } 
                 else if ( (!directory && name.endsWith(".jar")) 
-                            || (directory && name.endsWith("_jar")) ) {
+                            || (directory && 
+                                (name.endsWith("_jar") || 
+                                 name.endsWith(".jar"))) ) {
                     try {
                         //Section EE.8.4.2.1.d.i
                         AppClientArchivist acArchivist = new AppClientArchivist();
@@ -1928,9 +1948,14 @@ public class Application extends RootDeploymentDescriptor
                 return false;
             }
 
+            // now we are supporting directory names with 
+            // ".suffix" and "_suffix"
             if (name.endsWith("_war") 
+                    || name.endsWith(".war")
                     || name.endsWith("_rar") 
-                    || name.endsWith("_jar")) {
+                    || name.endsWith(".rar")
+                    || name.endsWith("_jar") 
+                    || name.endsWith(".jar")) {
                 return true;
             }
 
