@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
  package com.sun.enterprise.deployment;
 
@@ -64,7 +77,7 @@ import com.sun.enterprise.util.LocalStringManagerImpl;
 
     /** 
     * I am an object that represents all the deployment information about
-    * a web application.
+    * a web app [{0}]lication.
     * @author Danny Coward
     */
 
@@ -126,7 +139,7 @@ public class WebBundleDescriptor extends BundleDescriptor
    
    private SunWebApp sunWebApp=null;
    
-    /** Constrct an empty web app.
+    /** Constrct an empty web app [{0}].
     */
     public WebBundleDescriptor() {
         sessionTimeout = SESSION_TIMEOUT_DEFAULT;
@@ -184,7 +197,8 @@ public class WebBundleDescriptor extends BundleDescriptor
 	// (this problem occured for WebComponents as well)
 	//this.getWebComponentDescriptorsSet().addAll(webBundleDescriptor.getWebComponentDescriptorsSet());
 	for (Iterator itr = webBundleDescriptor.getWebComponentDescriptorsSet().iterator(); itr.hasNext();) {
-	    WebComponentDescriptor webComponentDescriptor = (WebComponentDescriptor)itr.next();
+	    WebComponentDescriptor webComponentDescriptor = 
+                new WebComponentDescriptor((WebComponentDescriptor)itr.next());
 	    webComponentDescriptor.setWebBundleDescriptor(this);
 	    this.getWebComponentDescriptorsSet().add(webComponentDescriptor);
 	}
@@ -315,7 +329,10 @@ public class WebBundleDescriptor extends BundleDescriptor
 		return srd;
 	    }
 	}
-	throw new IllegalArgumentException("This web application has no service refernce by the name " + name);
+        throw new IllegalArgumentException(localStrings.getLocalString(
+            "enterprise.deployment.exceptionwebapphasnoservicerefbyname",
+            "This web app [{0}] has no service reference by the name of [{1}]",
+            new Object[] {getName(), name}));
     }
 
 /**
@@ -349,8 +366,8 @@ public class WebBundleDescriptor extends BundleDescriptor
 	    }
 	}
 	throw new IllegalArgumentException(localStrings.getLocalString(
-								       "enterprise.deployment.exceptionwebcomphasnojmsdestrefbyname",
-								       "This web component has no JMS destination reference by the name of {0}", new Object[] {name}));
+								       "enterprise.deployment.exceptionwebapphasnojmsdestrefbyname",
+								       "This web app [{0}] has no resource environment reference by the name of [{1}]", new Object[] {getName(), name}));
     }
 
 	/**
@@ -375,7 +392,7 @@ public class WebBundleDescriptor extends BundleDescriptor
     }
     
 	/**
-	* Sets the Set of Mime Mappings for this web application.
+	* Sets the Set of Mime Mappings for this web app [{0}]lication.
 	*/
     public void setMimeMappings(Set mimeMappings) {
 	this.mimeMappings = mimeMappings;
@@ -552,7 +569,7 @@ public class WebBundleDescriptor extends BundleDescriptor
     }
     
     /**
-    * Return true if this web app can be distributed across different processes.
+    * Return true if this web app [{0}] can be distributed across different processes.
     */
     
     public boolean isDistributable() {
@@ -560,7 +577,7 @@ public class WebBundleDescriptor extends BundleDescriptor
     }
     
      /**
-    * Sets whether this web app can be distributed across different processes.
+    * Sets whether this web app [{0}] can be distributed across different processes.
     */
     public void setDistributable(boolean isDistributable) {
 	this.isDistributable = isDistributable;
@@ -602,7 +619,7 @@ public class WebBundleDescriptor extends BundleDescriptor
 	}
 	throw new IllegalArgumentException(localStrings.getLocalString(
 							      "enterprise.deployment.exceptionwebapphasnoejbrefbyname",
-							      "This web app has no ejb reference by the name of {0} ", new Object[] {name}));
+							      "This web app [{0}] has no ejb reference by the name of [{1}] ", new Object[] {getName(), name}));
     }
     
      /**
@@ -618,7 +635,7 @@ public class WebBundleDescriptor extends BundleDescriptor
 	}
 	throw new IllegalArgumentException(localStrings.getLocalString(
 							      "enterprise.deployment.exceptionwebapphasnoresourcerefbyname",
-							      "This web app has no resource reference by the name of {0}", new Object[] {name}));
+							      "This web app [{0}] has no resource reference by the name of [{1}]", new Object[] {getName(), name}));
     }
     
      /**
@@ -634,13 +651,7 @@ public class WebBundleDescriptor extends BundleDescriptor
     
     public Set<EntityManagerFactoryReferenceDescriptor> 
         getEntityManagerFactoryReferenceDescriptors() {
-        if( this.entityManagerFactoryReferences == null ) {
-            this.entityManagerFactoryReferences = 
-                new HashSet<EntityManagerFactoryReferenceDescriptor>();
-        }
-        return this.entityManagerFactoryReferences = 
-            new HashSet<EntityManagerFactoryReferenceDescriptor>
-            (entityManagerFactoryReferences);
+        return entityManagerFactoryReferences;
     }
 
     /**
@@ -656,8 +667,10 @@ public class WebBundleDescriptor extends BundleDescriptor
 		return next;
 	    }
 	}
-	throw new IllegalArgumentException("This web component has no entity "
-                 + "manager factory reference by the name " + name);
+        throw new IllegalArgumentException(localStrings.getLocalString(
+            "exceptionwebapphasnoentitymgrfactoryrefbyname",
+            "This web app [{0}] has no entity manager factory reference by the name of [{1}]",
+            new Object[] {getName(), name}));
     }
 
     public void addEntityManagerFactoryReferenceDescriptor
@@ -669,13 +682,7 @@ public class WebBundleDescriptor extends BundleDescriptor
 
     public Set<EntityManagerReferenceDescriptor> 
         getEntityManagerReferenceDescriptors() {
-        if( this.entityManagerReferences == null ) {
-            this.entityManagerReferences = 
-                new HashSet<EntityManagerReferenceDescriptor>();
-        }
-        return this.entityManagerReferences = 
-            new HashSet<EntityManagerReferenceDescriptor>
-            (this.entityManagerReferences);
+        return entityManagerReferences;
     }
 
     /**
@@ -691,8 +698,10 @@ public class WebBundleDescriptor extends BundleDescriptor
 		return next;
 	    }
 	}
-	throw new IllegalArgumentException("This web component has no entity "
-                 + "manager refernce by the name " + name);
+        throw new IllegalArgumentException(localStrings.getLocalString(
+            "exceptionwebapphasnoentitymgrrefbyname",
+            "This web app [{0}] has no entity manager reference by the name of [{1}]",
+            new Object[] {getName(), name}));
     }
 
     public void addEntityManagerReferenceDescriptor
@@ -702,7 +711,17 @@ public class WebBundleDescriptor extends BundleDescriptor
         this.changed();
     }
 
-
+    /**
+     * {@inheritDoc}
+     */
+    @Override public Collection<? extends PersistenceUnitDescriptor>
+            findReferencedPUs() {
+        Collection<PersistenceUnitDescriptor> pus =
+                new HashSet<PersistenceUnitDescriptor>(
+                        findReferencedPUsViaPURefs(this));
+        pus.addAll(findReferencedPUsViaPCRefs(this));
+        return pus;
+    }
 
     /** Return my set of environment properties.
     */
@@ -793,15 +812,14 @@ public class WebBundleDescriptor extends BundleDescriptor
 		return mdr;
 	    }
 	}
-	throw new IllegalArgumentException("This ejb has no message destination refernce by the name " + name);
+        throw new IllegalArgumentException(localStrings.getLocalString(
+                "exceptionwebapphasnomsgdestrefbyname",
+                "This web app [{0}] has no message destination reference by the name of [{1}]",
+                new Object[] {getName(), name}));
     }
 
     public Set<LifecycleCallbackDescriptor>
         getPostConstructDescriptors() {
-        if (postConstructDescs == null) {
-            postConstructDescs =
-                new HashSet<LifecycleCallbackDescriptor>();
-        }
         return postConstructDescs;
     }
 
@@ -828,10 +846,6 @@ public class WebBundleDescriptor extends BundleDescriptor
 
     public Set<LifecycleCallbackDescriptor>
         getPreDestroyDescriptors() {
-        if (preDestroyDescs == null) {
-            preDestroyDescs =
-                new HashSet<LifecycleCallbackDescriptor>();
-        }
         return preDestroyDescs;
     }
 
@@ -899,22 +913,6 @@ public class WebBundleDescriptor extends BundleDescriptor
         return(getInjectableResourcesByClass(className, this));
     }
 
-    public InjectionInfo getInjectionInfoByClass(String className,
-                              JndiNameEnvironment jndiNameEnv) {
-        LifecycleCallbackDescriptor postConstructDesc =
-            getPostConstructDescriptorByClass(className, jndiNameEnv);
-        String postConstructMethodName = (postConstructDesc != null) ?
-            postConstructDesc.getLifecycleCallbackMethod() : null;
-        LifecycleCallbackDescriptor preDestroyDesc =
-            getPreDestroyDescriptorByClass(className, jndiNameEnv);
-        String preDestroyMethodName = (preDestroyDesc != null) ?
-            preDestroyDesc.getLifecycleCallbackMethod() : null;
-        return new InjectionInfo(className, 
-                                 postConstructMethodName, preDestroyMethodName,
-                                 getInjectableResourcesByClass(className,
-                                                               jndiNameEnv));
-    }
-
     public InjectionInfo getInjectionInfoByClass(String className) {
         return(getInjectionInfoByClass(className, this));
     }
@@ -1088,9 +1086,9 @@ public class WebBundleDescriptor extends BundleDescriptor
 	    }
 	}
 	throw new IllegalArgumentException(localStrings.getLocalString(
-		"enterprise.deployment.exceptionbeanhasnoenvpropertybyname",
-		"This bean has no environment property by the name of {0}", 
-                new Object[] {name}));
+		"enterprise.deployment.exceptionwebapphasnoenvpropertybyname",
+		"This web app [{0}] has no environment property by the name of [{1}]", 
+                new Object[] {getName(), name}));
     }
 
     /** Removes this given environment property from my list.*/
