@@ -1,27 +1,42 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 package com.sun.enterprise.deployment.node;
 
+import java.util.Map;
+import java.util.HashMap;
 
 import com.sun.enterprise.deployment.xml.ApplicationTagNames;
 import org.xml.sax.SAXParseException;
@@ -65,8 +80,6 @@ import com.sun.enterprise.util.LocalStringManagerImpl;
  * @version 
  */
 public class SaxParserHandler extends DefaultHandler {
-
-
     public static final String JAXP_SCHEMA_LANGUAGE =
         "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
     public static final String JAXP_SCHEMA_SOURCE = 
@@ -74,9 +87,24 @@ public class SaxParserHandler extends DefaultHandler {
     public static final String W3C_XML_SCHEMA =
         "http://www.w3.org/2001/XMLSchema";
     
-    protected static Hashtable mapping = null; 
-    private static Hashtable rootNodes = null;
-    private List nodes = new ArrayList();
+    // allows use of 'final' in static initializer for _mappingStuff
+    // @Immutable
+    private static final class MappingStuff {
+        public final Hashtable<String,String> mMapping;
+        private final Map<String,Class> mRootNodesMutable; // use only when initializing
+        public final Map<String,Class> mRootNodes;
+        
+        MappingStuff() {
+            mMapping    = new Hashtable<String,String>();
+            mRootNodesMutable  = new HashMap<String,Class>();
+            mRootNodes         = Collections.unmodifiableMap( mRootNodesMutable );
+        }
+    };
+    
+    private static volatile boolean _MappingStuffInited = false;
+    private static final MappingStuff _mappingStuff = new MappingStuff(); 
+    
+    private final List<XMLNode> nodes = new ArrayList<XMLNode>();
     public XMLNode topNode = null;
     protected String publicID=null;
     private StringBuffer elementData=null;
@@ -85,12 +113,16 @@ public class SaxParserHandler extends DefaultHandler {
     private boolean stopOnXMLErrors = false;
 
     private boolean pushedNamespaceContext=false;
-    private NamespaceSupport namespaces;
+    private final NamespaceSupport namespaces;
 
     // for i18N
-    private static LocalStringManagerImpl localStrings=
+    private static final LocalStringManagerImpl localStrings=
 	    new LocalStringManagerImpl(SaxParserHandler.class);    
     
+    protected static Hashtable<String,String> getMapping() {
+        return _mappingStuff.mMapping;
+    }
+    
     public SaxParserHandler() {
         Init();
 
@@ -98,11 +130,17 @@ public class SaxParserHandler extends DefaultHandler {
         namespaces = new NamespaceSupport();
     }
 
+    /**
+        @return the Map of root nodes.
+     */
     private static void Init() {
-
-        if (mapping==null) {
-            mapping = new Hashtable();
-            rootNodes= new Hashtable();
+        if ( _MappingStuffInited ) {    // '_MappingStuffInited' MUST be 'volatile'
+            return;
+        }
+        
+        synchronized( SaxParserHandler.class ) {
+            final Map<String,Class> rootNodes= _mappingStuff.mRootNodesMutable;
+            final Hashtable<String,String> mapping = _mappingStuff.mMapping;
             
             String rootNode  = ApplicationNode.registerBundle(mapping);
             rootNodes.put(rootNode, ApplicationNode.class);
@@ -110,13 +148,13 @@ public class SaxParserHandler extends DefaultHandler {
             rootNode  = EjbBundleNode.registerBundle(mapping);
             rootNodes.put(rootNode, EjbBundleNode.class);
 
-	    rootNode = ConnectorNode.registerBundle(mapping);
+            rootNode = ConnectorNode.registerBundle(mapping);
             rootNodes.put(rootNode, ConnectorNode.class);
             
             rootNode = WebBundleNode.registerBundle(mapping);
             rootNodes.put(rootNode, WebBundleNode.class);
 
-	    rootNode = AppClientNode.registerBundle(mapping);
+            rootNode = AppClientNode.registerBundle(mapping);
             rootNodes.put(rootNode, AppClientNode.class);
             
             rootNode = WebServicesDescriptorNode.ROOT_ELEMENT.getQName();
@@ -128,28 +166,30 @@ public class SaxParserHandler extends DefaultHandler {
             rootNode = PersistenceNode.ROOT_ELEMENT.getQName();
             rootNodes.put(rootNode, PersistenceNode.class);
 
-	    rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_APPLICATION_RUNTIME_TAG, ApplicationRuntimeNode.class);
-            ApplicationRuntimeNode.registerBundle(mapping);
-	    rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_WEB_RUNTIME_TAG, WebBundleRuntimeNode.class);
-            WebBundleRuntimeNode.registerBundle(mapping);
-	    rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_EJB_RUNTIME_TAG, EjbBundleRuntimeNode.class);
-            EjbBundleRuntimeNode.registerBundle(mapping);              
-	    rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_APPCLIENT_RUNTIME_TAG, AppClientRuntimeNode.class);
-            AppClientRuntimeNode.registerBundle(mapping);     
-	    rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_CONNECTOR_RUNTIME_TAG, com.sun.enterprise.deployment.node.runtime.connector.ConnectorNode.class);
-            com.sun.enterprise.deployment.node.runtime.connector.ConnectorNode.registerBundle(mapping);              
+            rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_APPLICATION_RUNTIME_TAG, ApplicationRuntimeNode.class);
+                ApplicationRuntimeNode.registerBundle(mapping);
+            rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_WEB_RUNTIME_TAG, WebBundleRuntimeNode.class);
+                WebBundleRuntimeNode.registerBundle(mapping);
+            rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_EJB_RUNTIME_TAG, EjbBundleRuntimeNode.class);
+                EjbBundleRuntimeNode.registerBundle(mapping);              
+            rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_APPCLIENT_RUNTIME_TAG, AppClientRuntimeNode.class);
+                AppClientRuntimeNode.registerBundle(mapping);     
+            rootNodes.put(com.sun.enterprise.deployment.xml.RuntimeTagNames.S1AS_CONNECTOR_RUNTIME_TAG, com.sun.enterprise.deployment.node.runtime.connector.ConnectorNode.class);
+                com.sun.enterprise.deployment.node.runtime.connector.ConnectorNode.registerBundle(mapping);              
             
             // post treatment, let's remove the URL from the DTD so we use local copies...
             for (java.util.Enumeration publicIDs=mapping.keys();publicIDs.hasMoreElements();) {
-                String publicID = (String) publicIDs.nextElement();
-                String dtd = (String) mapping.get(publicID);
+                final String publicID = (String) publicIDs.nextElement();
+                final String dtd = (String) mapping.get(publicID);
                 mapping.put(publicID, dtd.substring(dtd.lastIndexOf('/')+1));
             }
-        }    
+
+            _MappingStuffInited = true;
+        }
     }
     
     public static void registerMapping(String publicID, String systemID) {
-        mapping.put(publicID, systemID);
+        getMapping().put(publicID, systemID);
     }
     
                         
@@ -175,9 +215,9 @@ public class SaxParserHandler extends DefaultHandler {
                     }
                     return new InputSource(fileName);
             }
-            if (mapping.containsKey(publicID)) {                    
+            if ( getMapping().containsKey(publicID)) {                    
                 this.publicID = publicID;
-                return new InputSource(new BufferedInputStream(getDTDUrlFor((String) mapping.get(publicID))));                
+                return new InputSource(new BufferedInputStream(getDTDUrlFor((String) getMapping().get(publicID))));                
             } 
         } catch(Exception ioe) {
 	    ioe.printStackTrace();
@@ -315,7 +355,7 @@ public class SaxParserHandler extends DefaultHandler {
         
         if (nodes.isEmpty()) {
             // this must be a root element...
-            Class rootNodeClass = (Class) rootNodes.get(localName);
+            Class rootNodeClass = (Class) _mappingStuff.mRootNodes.get(localName);
             if (rootNodeClass==null) {
                 DOLUtils.getDefaultLogger().log(Level.SEVERE, "enterprise.deployment.backend.invalidDescriptorMappingFailure",
                         new Object[] {localName , " not supprted !"});                
