@@ -1,32 +1,56 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 package com.sun.enterprise.admin.server.core;
 
 //JDK imports
+import com.sun.enterprise.admin.event.ClusterEvent;
 import com.sun.enterprise.admin.event.MBeanElementChangeEvent;
 import com.sun.enterprise.admin.event.MBeanElementChangeEventListener;
+import com.sun.enterprise.connectors.util.ResourcesUtil;
+import com.sun.enterprise.server.pluggable.PluggableFeatureFactory;
+import com.sun.enterprise.util.SystemPropertyConstants;
 import java.io.File;
+import java.lang.reflect.Constructor;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.util.Properties;
+import java.io.FilenameFilter;
+import java.net.URL;
+import java.net.URLClassLoader;
+
 
 //JMX imports
 import javax.management.MBeanServer;
@@ -70,15 +94,15 @@ import com.sun.logging.LogDomains;
 
 //Utility methods
 import com.sun.enterprise.util.io.FileUtils;
+import com.sun.enterprise.util.JvmInfoUtil;
 import com.sun.enterprise.util.i18n.StringManager;
+import com.sun.enterprise.util.RegistrationUtil;
 
 //autodeploy
-import com.sun.enterprise.deployment.autodeploy.AutoDeployControllerImpl;
 import com.sun.enterprise.deployment.autodeploy.AutoDeployController;
-import com.sun.enterprise.deployment.autodeploy.AutoDeployConstants;
-import com.sun.enterprise.deployment.autodeploy.AutoDeploymentException;
 import com.sun.enterprise.deployment.autodeploy.AutoDeployControllerFactroy;
 import com.sun.enterprise.deployment.autodeploy.AutoDeployControllerFactroyImpl;
+import com.sun.enterprise.deployment.autodeploy.AutoDeploymentException;
 
 //web service
 import com.sun.enterprise.webservice.WsUtil;
@@ -97,9 +121,16 @@ import com.sun.enterprise.Switch;
 import com.sun.enterprise.admin.monitor.callflow.Agent;
 import com.sun.enterprise.admin.monitor.callflow.AgentAdapter;
 
+import com.sun.enterprise.server.logging.AMXLoggingHook;
+
+import com.sun.enterprise.util.FeatureAvailability;
+
+import com.sun.appserv.management.util.misc.RunnableBase;
+import com.sun.enterprise.interceptor.DynamicInterceptor;
+
 /**
  * Admin service is a singleton in every instance and acts as gateway to server
- * administration tasks. 
+ * administration tasks.
  */
 public class AdminService {
 
@@ -136,35 +167,34 @@ public class AdminService {
     public static final Logger sLogger 
             = LogDomains.getLogger(AdminConstants.kLoggerName);
 
-    private static StringManager localStrings =
+    private static final StringManager localStrings =
         StringManager.getManager(AdminService.class);
 
     private static AdminService adminService = null;
 
+    private static final String COM_SUN_APPSERV = "com.sun.appserv";
+    
     /** prefix for administrative domain name */
-    private static final String ADMIN_DOMAIN_NAME_PREFIX = "com.sun.appserv.";
+    private static final String ADMIN_DOMAIN_NAME_PREFIX = COM_SUN_APPSERV + ".";
 
     /** default administrative domain name */
-    private static final String DEF_ADMIN_DOMAIN_NAME="com.sun.appserv.server";
+    private static final String DEF_ADMIN_DOMAIN_NAME=ADMIN_DOMAIN_NAME_PREFIX + "server";
 
-    private static String  kTempDirNamePrefix   = "s1astemp";
-    private static String  kGUITempDirName      = "gui";
-    private static String  kTempDirNameSuffix   = 
+    private static final String  kTempDirNamePrefix   = "s1astemp";
+    private static final String  kGUITempDirName      = "gui";
+    private static final String  kTempDirNameSuffix   = 
             "" + ((System.getProperty(Constants.IAS_ROOT) != null) ?
             System.getProperty(Constants.IAS_ROOT).hashCode() : 0);
     /* The temporary directory name should be a function of domain name, 
      * admin-server id and the install root, otherwise it won't
      * work for the multiple installations on the same machine. */
-    private static int          sMBeanServerID;
-    public static final int     kDefaultImpl = 0;
     private static final String SS_MBEAN_CLASS = "com.sun.enterprise.admin.mbeans.SystemServicesMBean";
-    private String adminServiceType;
-    private ServerContext context;
-    private AdminContext adminContext;
-    private ServerLifecycle adminChannel;
-    private String mTempDirPath;
-    private String mGUITempDirPath;
-    private AutoDeployController autoDeployController;
+    private volatile String adminServiceType;
+    private volatile ServerContext context;
+    private volatile AdminContext adminContext;
+    private volatile ServerLifecycle adminChannel;
+    private volatile String mTempDirPath;
+    private volatile String mGUITempDirPath;
     
     /** A field to denote the default name for all the config and runtime mbeans that are 
      * generated from the meta-information (descriptor, like admin-mbean-descriptor.xml) */
@@ -172,15 +202,16 @@ public class AdminService {
     public static final String DAS_DIAGNOSTIC_MBEAN_CLASS_NAME =
                 "com.sun.enterprise.admin.mbeans.DomainDiagnostics";
 
+    private final AutoDeployStarter autoDeployStarter = new AutoDeployStarter();
+    
 
     /**
      * private constructor. AdminService instance should be initialized
      * through package method createAdminService(). 
      */
     private AdminService() {
-        sMBeanServerID = kDefaultImpl; //the only implementation for now
     }
-
+    
     /**
      * Create admin service. Admin Service is initialized by an internal
      * lifecycle module (AdminServiceLifeCycle).
@@ -261,48 +292,56 @@ public class AdminService {
      * @throws LifeCycleException in case the initialzation fails.
      */
     void init() throws ServerLifecycleException {
-
         if (isDas()) {
             // remove restart required state file when starting up DAS
             RRStateFactory.removeStateFile();
         }
         adminChannel = new AdminChannelLifecycle();
         adminChannel.onInitialization(context);
+        
+        // Publish PID for this VM in the config directory
+        // This publishing has to happen after the AdminService init
+        publishPID();        
 
-        MBeanServer mbs = null;
+        final DynamicInterceptor dyn = (DynamicInterceptor)getMBeanServer();
+        final MBeanServer delegateMBeanServer = dyn.getDelegateMBeanServer();
+        adminContext.setMBeanServer( dyn );
+        
+        final SunoneInterceptor sunone =
+            SunoneInterceptor.createInstance(adminContext, dyn, delegateMBeanServer);
+        dyn.addHook( COM_SUN_APPSERV, sunone );
+        FeatureAvailability.getInstance().registerFeature(
+            FeatureAvailability.SUN_ONE_INTERCEPTOR_FEATURE, "true");
+        AMXLoggingHook.enableLoggingHook();
+        sunone.registerConfigMBeans();
+            
+        sLogger.log(Level.INFO, "core.sunone_interceptor_enabled");
+        initCallFlow();
 
-        try {
-            if (sMBeanServerID == kDefaultImpl) {
-                SunoneInterceptor.setAdminContext(adminContext);
-                mbs = SunoneInterceptor.getMBeanServerInstance();
-                MBeanServerFactory.initialize(this, mbs);
-            } else {
-                throw new RuntimeException(localStrings.getString(
-                        "admin.server.core.mbs_init_error"));
-            }
-            if (adminContext != null) {
-                adminContext.setMBeanServer(mbs);
-            }
-            sLogger.log(Level.INFO, "core.mbs_init_ok");
-	    initCallFlow();
-        }
-        catch (Throwable t) {
-            sLogger.log(Level.FINEST, t.getMessage(), t);
-            if (isDas()) {
-                sLogger.log(Level.SEVERE, "core.mbs_init_exception", t);
-            }
-            else {
-                sLogger.log(Level.WARNING, "core.mbs_init_exception", t);
-            }
-            throw new ServerLifecycleException(t.getMessage());
-        }
+        //initialize JKS properties.
+        setupJKS();
     }
 
+    private void setupJKS() {
+        String configDir = System.getProperty("com.sun.aas.instanceRoot") + 
+            File.separator + "config";
+        java.io.File nssFile = new File(configDir, "key3.db");
+        if (!nssFile.exists()) {
+            if (System.getProperty("javax.net.ssl.keyStore") == null) {
+                System.setProperty("javax.net.ssl.keyStore", configDir + 
+                    File.separator + "keystore.jks");
+                System.setProperty("javax.net.ssl.trustStore", configDir +
+                    File.separator + "cacerts.jks");
+            }
+        }
+    } 
+
     private void initCallFlow() {
         try {
-            Class cl = Class.forName(
+            final Class cl = Class.forName(
                     "com.sun.enterprise.admin.monitor.callflow.AgentImpl");
-            Agent agent = (Agent) cl.newInstance();
+            final Method method = cl.getMethod("getInstance", null);
+            final Agent agent = (Agent) method.invoke(null, null);
             Switch.getSwitch().setCallFlowAgent(agent);
         } catch (Throwable t) {
             sLogger.log(Level.SEVERE, "core.callflow_agent_init_failed", t);
@@ -359,10 +398,6 @@ public class AdminService {
      * Admin service is ready. All other services have started up successfully.
      */
     void ready() throws ServerLifecycleException {
-        if (isDas()) {
-           //check and start autodeploy service  -FIX for 4933846
-           startAutoDeployService();
-        }
         if (adminChannel != null) {
             adminChannel.onReady(context);
         }
@@ -372,12 +407,80 @@ public class AdminService {
         // would be cleaner to issue JMX Notifications for a variety
         // of events, but that would mean adding an MBean for AdminService.
         notifyAMXThatAdminServiceIsReady();
+        
+        // notify all node agents in this domain to know THIS DAS. This is 
+        // useful when DAS has moved. Also, only the running Node Agents will
+        // be notified.
+        notifyNodeagents();
+        
+        if (isDas()) {
+           // autoDeployStarter *must* run in a separate thread, or the
+           // this call won't return because it will wait for the server
+           // to start using the current thread, which must return in order
+           // for the server to be considered to have started.
+           autoDeployStarter.submit(RunnableBase.HowToRun.RUN_IN_SEPARATE_THREAD);
+        }
+        
+        NoopClusterEventListener noop = new NoopClusterEventListener();
+        if (!isDas()) {
+            AdminEventListenerRegistry.addEventListener(ClusterEvent.eventType, noop);
+        }
+
+        //Everything has started successfully. Try to start registration thread
+        if(isDas())
+            startRegistrationDaemon();
+
+    }
+
+    /**
+     * Starts a background process that would check for registration
+     */
+    private void startRegistrationDaemon() {
+        //Provide a back door to skip starting registration daemon.
+        Boolean enableRegistrationDaemon = Boolean.valueOf(
+                System.getProperty("com.sun.enterprise.registration.ENABLE_REGISTRATION_DAEMON", "true"));
+        if (!enableRegistrationDaemon) {
+            sLogger.log(Level.FINER, "Registration Daemon not disabled. Will not start the Daemon.");
+            return;
+        }
+
+        File registrationHome = RegistrationUtil.getRegistrationHome();
+        
+        //Put all jars in this directory into classpath for the registrationclassloader
+        final File [] files = registrationHome.listFiles(new FilenameFilter() {
+            public boolean accept(File dir, String name) {
+                return (name.endsWith(".jar") || name.endsWith(".zip"));
+            }
+        });
+        //start registration daemon only if the dir exist and it does contain some jars.
+        if(files != null && files.length != 0 ) {
+            try {
+                URL[] registrationJars = new URL[files.length];
+                for (int i = 0; i < files.length; i++) {
+                    registrationJars[i] = files[i].toURI().toURL();
+                }
+                URLClassLoader classLoaderRegistration = new URLClassLoader(registrationJars);
+                Class registrationDaemonClass = null;
+                registrationDaemonClass = Class.forName("com.sun.enterprise.registration.RegistrationDaemon", true, classLoaderRegistration);
+                Method m = registrationDaemonClass.getMethod("start", File.class);
+                m.invoke(null,RegistrationUtil.getServiceTagRegistry());
+            } catch (Exception e) {
+                //Log a info message and let the appserver start normally.
+                sLogger.log(Level.INFO, "general.fatal_error_initializing_registration_daemon", e.getMessage() );
+                if(sLogger.isLoggable(Level.FINER)) {
+                    //More details for debugging
+                    sLogger.log(Level.FINER, "Exception while initialing registration daemon" + e);
+                }
+            }
+        }
+
     }
 
     /**
      * Stop admin service. This is called when shutdown process starts.
      */
     void stop() throws ServerLifecycleException {
+        
         if (adminChannel != null) {
             adminChannel.onShutdown();
         }
@@ -393,7 +496,8 @@ public class AdminService {
         }
         if (canRunUserApps()) {
             //autodeploy service stoped
-            stopAutoDeployService();            
+            autoDeployStarter.waitDone();
+            autoDeployStarter.stopAutoDeployService();            
             stopNormalInstance();
         }
         // as we are killing the VM, there is no real need of shutting down MBS.
@@ -454,7 +558,8 @@ public class AdminService {
      * Set admin context.
      */
     protected void setAdminContext(AdminContext ctx) {
-         adminContext = ctx;
+        adminContext = ctx;
+        adminContext.setMBeanServer( getMBeanServer() );
     }
 
     /**
@@ -639,8 +744,7 @@ public class AdminService {
             if (!gd.exists()) {
                 final boolean s = gd.mkdirs();
                 if (!s) {
-                   final String msg = localStrings.getString("admin.server.core.gui_dir_failure", mGUITempDirPath);
-                   throw new RuntimeException(msg);
+                   sLogger.log(Level.WARNING, "core.gui_tmp_folder_creation_failed", mGUITempDirPath);
                 }
             }
         }
@@ -736,35 +840,40 @@ public class AdminService {
     private static final String ADMIN_JAVAROOT = "com.sun.aas.javaRoot";
     private static final String PLUGIN_JAVAROOT = "JAVA_HOME";
 
-    /**
-     *to start autodeploy service
-     */
-    private void startAutoDeployService(){
-        try {
-            AutoDeployControllerFactroy controllerFactory = new AutoDeployControllerFactroyImpl();
-            autoDeployController = controllerFactory.createAutoDeployController(context);
-            if(autoDeployController !=null ) {
-                autoDeployController.enableAutoDeploy();
+    private final class AutoDeployStarter extends RunnableBase {
+        // note that methods using this variable are 'synchronized', both
+        // for visibilility and mutual exclusion purposes.
+        private AutoDeployController mAutoDeployController;
+         
+        AutoDeployStarter() {
+        }
+        
+        protected synchronized void doRun() throws AutoDeploymentException {
+            // block until the server has started
+            FeatureAvailability.getInstance().waitForFeature(
+                FeatureAvailability.SERVER_STARTED_FEATURE,"AutoDeployStarter");
+            
+            try {
+                final AutoDeployControllerFactroy /*[sic]*/ factory =
+                    new AutoDeployControllerFactroyImpl();
+                mAutoDeployController = factory.createAutoDeployController(context);
+                if( mAutoDeployController !=null ) {
+                    mAutoDeployController.enableAutoDeploy();
+                }
+            }
+            catch( Throwable t ) {
+                sLogger.log(Level.WARNING, "general.unexpected_exception", t);
             }
-        } catch(Throwable tw) {
-            sLogger.log(Level.FINE, "general.unexpected_exception", tw);
         }
-    }
-    
-    /**
-     *to stop autodeploy service
-     */
-    private void stopAutoDeployService(){
-        try {             
-            if(autoDeployController !=null) {
-             autoDeployController.disableAutoDeploy();
-             autoDeployController=null;
-            } 
-        } catch(Throwable tw) {            
-            sLogger.log(Level.FINE, "general.unexpected_exception", tw);
+        
+        private synchronized void stopAutoDeployService(){
+            if(mAutoDeployController !=null) {
+                mAutoDeployController.disableAutoDeploy();
+                mAutoDeployController = null;
+            }
         }
-    }
-    
+    }    
+            
     /**
      * MBeanRegistry is used to generate all dotted names
      * available in the system. This is achived by iterating
@@ -818,7 +927,7 @@ public class AdminService {
     }
 
     private void registerLogManagerMBean() {
-        MBeanServer mbs = adminContext.getMBeanServer();
+        final MBeanServer mbs = FeatureAvailability.getInstance().getMBeanServer();
         try {
             Object mbean =
                     com.sun.enterprise.server.logging.LogMBean.getInstance();
@@ -866,15 +975,9 @@ public class AdminService {
             final Object impl = Class.forName(SS_MBEAN_CLASS).newInstance();
             mbs.registerMBean(impl, on);
             sLogger.finer("Admin Message: System Services MBean Registered with on: " + on.toString());
-            if (isDas()) {
-                //register SystemService MBean under the old 8.0 name as well
-                //to backward compatibility with 8.0 RI client - hardcoded
-                mbs.registerMBean(impl, new ObjectName("ias:type=system-services"));
-                sLogger.finer("Admin Message: System Services MBean Registered with sibling name: " + on.toString());
-            }
         }
         catch (final Exception e) {
-            throw new ServerLifecycleException(e);
+            sLogger.log(Level.WARNING, "core.system.service.mbean.not.registered", on.toString());
         }
     }
     
@@ -889,7 +992,7 @@ public class AdminService {
     public static final String AMX_LOADER_DEFAULT_OBJECTNAME    =
         "amx-support:name=mbean-loader";
     
-    private ObjectName mAMXLoaderObjectName   = null;
+    private volatile ObjectName mAMXLoaderObjectName   = null;
         
     /**
         Initializes AMX MBeans
@@ -912,21 +1015,15 @@ public class AdminService {
             sLogger.log(Level.INFO, "mbean.init_amx_success");
         }
         catch(Exception e) {
-            // why is this being ignored????
+            // this is really a fatal error; not even the AMX loader works
             sLogger.log(Level.SEVERE, "mbean.init_amx_failed", e);
-            //ignore
-            throw new Error( e );
+            throw new RuntimeException( e );
         }
     }
     
-        private MBeanServer
+        private static MBeanServer
     getMBeanServer() {
-        try {
-            return AppServerMBeanServerFactory.getMBeanServerInstance();
-        }
-        catch( Exception e ) {
-            throw new RuntimeException( e );
-        }
+        return FeatureAvailability.getInstance().getMBeanServer();
     }
     
 
@@ -946,29 +1043,14 @@ public class AdminService {
             
     private void initiateCustomMBeanLoading() throws ServerLifecycleException {
         try {
-        final MBeanServer mbs = adminContext.getMBeanServer();
-        final ConfigContext cc = context.getConfigContext();
-        new CustomMBeanRegistrationHelper(mbs, cc).registerMBeans();
+            final MBeanServer mbs = getMBeanServer();
+            final ConfigContext cc = context.getConfigContext();
+            new CustomMBeanRegistrationHelper(mbs, cc).registerMBeans();
         } catch (final Exception e) {
-            e.printStackTrace();
-            throw new ServerLifecycleException(e); // we must not abort the startup, we must not affect other operations that may follow
+            sLogger.log(Level.WARNING, "core.custom.mbean.registration.failure");
         }
         initializeMBeanEventListeners();
     }
-    private void registerJVMMonitoringMBeans() {
-        try {
-            //register the thread-dump related MBeans
-            final ObjectName dummy1 = new ObjectName("foo:bar=bar1"), dummy2 = new ObjectName("foo:bar=bar2");
-            if (this.isDas()) {
-                final Object jvmic = Class.forName("com.sun.enterprise.admin.mbeans.jvm.JVMInformationCollector").newInstance();
-                com.sun.enterprise.admin.common.MBeanServerFactory.getMBeanServer().registerMBean(jvmic, dummy1);
-            }
-            final Object jvmi = Class.forName("com.sun.enterprise.admin.mbeans.jvm.JVMInformation").newInstance();
-            com.sun.enterprise.admin.common.MBeanServerFactory.getMBeanServer().registerMBean(jvmi, dummy2);
-        } catch (final Exception e) {
-            e.printStackTrace();
-        }
-    }
 
     // createDASDiagnosticMBean
     private void createDASDiagnosticMBean() {
@@ -999,4 +1081,65 @@ public class AdminService {
             e.printStackTrace(); //ok to squelch
         }
     }
+    
+    private void registerJVMMonitoringMBeans()  throws ServerLifecycleException {
+        try {
+            //  Use a dummy ObjectName; the MBeans modify the supplied one
+            final ObjectName dummy = new ObjectName("dummy:name=dummy");
+            
+            if (this.isDas()) {
+                final Object jvmInfoCollector =
+                    Class.forName("com.sun.enterprise.admin.mbeans.jvm.JVMInformationCollector").newInstance();
+                
+                // com.sun.appserv:type=JVMInformationCollector,category=monitor,server=server
+                final ObjectName actualObjectName =
+                    getMBeanServer().registerMBean( jvmInfoCollector, dummy).getObjectName();
+            }
+            
+            // com.sun.appserv:type=JVMInformation,category=monitor,server=server
+            final Object jvmInfo = Class.forName("com.sun.enterprise.admin.mbeans.jvm.JVMInformation").newInstance();
+            final ObjectName actualObjectName = getMBeanServer().registerMBean( jvmInfo, dummy ).getObjectName();
+        }
+        catch ( final Exception e) {
+            sLogger.log(Level.WARNING, "core.jvm.mbeans.not.registered");
+        }
+    }
+
+    private void notifyNodeagents() {
+        try { 
+            if (isDAS()) {
+                Class naNotifierClass = Class.forName(
+                    "com.sun.enterprise.ee.nodeagent.NodeAgentNotifier");
+                Class[] params = new Class[1];
+                params[0] = ServerContext.class;
+                Constructor c = naNotifierClass.getConstructor(params);
+                Object naNotifier = c.newInstance(context);
+                Thread notifyNodeAgents = new Thread((Runnable)naNotifier);
+                notifyNodeAgents.setDaemon(true);
+                notifyNodeAgents.start();
+            }
+        } catch (ConfigException ex) {
+            ex.printStackTrace();
+            sLogger.log(Level.WARNING, "core.could_not_determine_whether_instance_is_das");
+        } catch (Exception ex) {
+            ex.printStackTrace();
+            sLogger.log(Level.WARNING, 
+                "core.notification_for_nodeagents_could_not_be_invoked");
+        }
+    }
+    
+    private boolean isDAS() throws ConfigException {
+      return ServerHelper.isDAS( 
+       AdminService.getAdminService().getAdminContext().getAdminConfigContext(), 
+       context.getInstanceName());
+    }
+
+    private void publishPID() {
+        JvmInfoUtil jvminfo = new JvmInfoUtil();
+	System.out.println(System.getProperty("com.sun.aas.instanceRoot") +
+                       "/config/" + SystemPropertyConstants.PID_FILE);
+        jvminfo.logPID(System.getProperty("com.sun.aas.instanceRoot") + 
+                       File.separator + "config" + File.separator + 
+		       SystemPropertyConstants.PID_FILE);
+    }    
 }
