@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.appclient.jws;
@@ -33,9 +46,11 @@ import java.util.Date;
 import java.util.Properties;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletRequest;
 import javax.servlet.ServletResponse;
 import javax.servlet.ServletException;
+import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletRequestWrapper;
 import javax.servlet.http.HttpServletResponse;
@@ -64,7 +79,7 @@ import javax.servlet.http.HttpServletResponse;
  *
  * @author tjquinn
  */
-public class JWSSystemServlet extends org.apache.catalina.servlets.DefaultServlet {
+public class JWSSystemServlet extends HttpServlet {
     
     private static final String ARG_QUERY_PARAM_NAME = "arg";
     private static final String PROP_QUERY_PARAM_NAME = "prop";
@@ -81,6 +96,7 @@ public class JWSSystemServlet extends org.apache.catalina.servlets.DefaultServle
 
     private Logger _logger=LogDomains.getLogger(NamingConventions.JWS_LOGGER);
 
+    RequestDispatcher defaultDispatcher = null;
 
     /** Creates a new instance of JWSSystemServlet */
     public JWSSystemServlet() {
@@ -91,6 +107,10 @@ public class JWSSystemServlet extends org.apache.catalina.servlets.DefaultServle
      */
     public void init() throws ServletException {
         super.init();
+        String dispatcherName = getInitParameter("defaultDispatcherName");
+        if (dispatcherName==null)
+        	dispatcherName = "default";
+         defaultDispatcher = getServletConfig().getServletContext().getNamedDispatcher(dispatcherName);
         try {
             /*
              *Locate - or create - the data structure object.
@@ -172,9 +192,10 @@ public class JWSSystemServlet extends org.apache.catalina.servlets.DefaultServle
             }
             
             /*
-             *Delegate.
+             * Send to the dispatcher for the default servlet.
              */
-            super.service(wrappedRequest, resp);
+            defaultDispatcher.forward(wrappedRequest, resp);
+
         } catch (Throwable thr) {
             throw new ServletException("Error processing static content for path " + content.getPath(), thr);
         }
@@ -267,8 +288,8 @@ public class JWSSystemServlet extends org.apache.catalina.servlets.DefaultServle
         Properties answer = new Properties();
         
         answer.setProperty("request.scheme", request.getScheme());
-        answer.setProperty("request.host", request.getLocalName());
-        answer.setProperty("request.port", Integer.toString(request.getLocalPort()));
+        answer.setProperty("request.host", request.getServerName());
+        answer.setProperty("request.port", Integer.toString(request.getServerPort()));
         
         /*
          *Treat query parameters with the name "arg" as command line arguments to the
