@@ -1,28 +1,43 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.connectors;
 
+import com.sun.enterprise.connectors.util.ResourcesUtil;
+import com.sun.enterprise.connectors.util.JmsRaUtil;
 import com.sun.enterprise.deployment.*;
 import com.sun.enterprise.admin.monitor.registry.MonitoringLevel;
 import com.sun.enterprise.connectors.system.ActiveJmsResourceAdapter;
@@ -73,7 +88,7 @@ public class ActiveInboundResourceAdapter extends
     protected ResourceAdapter resourceadapter_; //runtime instance
 
     //beanID -> endpoint factory and its activation spec
-    private Hashtable factories_; 
+    private Hashtable<String, MessageEndpointFactoryInfo> factories_; 
 
     protected String moduleName_;
     static Logger _logger = LogDomains.getLogger(LogDomains.RSR_LOGGER);
@@ -97,9 +112,9 @@ public class ActiveInboundResourceAdapter extends
     public ActiveInboundResourceAdapter(
             ResourceAdapter ra, ConnectorDescriptor desc, String moduleName, 
             ClassLoader jcl) throws ConnectorRuntimeException {
-        super(desc,moduleName,jcl,false);
+        super(desc,moduleName,jcl);
         this.resourceadapter_ = ra;
-        this.factories_       = new Hashtable();
+        this.factories_       = new Hashtable<String, MessageEndpointFactoryInfo>();
         this.moduleName_      = moduleName;
         try {
                 loadRAConfiguration();
@@ -144,13 +159,21 @@ public class ActiveInboundResourceAdapter extends
                 cre.initCause( pex.getException() );	    
                 throw cre;
         } catch (Throwable t) {
-                _logger.log(Level.SEVERE,"rardeployment.start_failed",t);
+            _logger.log(Level.SEVERE,"rardeployment.start_failed",t);
+	    t.printStackTrace(); 
             String i18nMsg = localStrings.getString(
                 "rardeployment.start_failed", t.getMessage() );
         		ConnectorRuntimeException cre = 
         		    new ConnectorRuntimeException( i18nMsg );
-        	        cre.initCause( t );	    
-        		throw cre;
+	     if (t.getCause() != null) {	
+		/* Put the correct cause of the exception
+ 	 	 * Fixes issue : 6543333
+		 */
+       	        cre.initCause( t.getCause());	    
+  	     } else {
+		cre.initCause(t);
+	     } 
+       	     throw cre;
         }
     }
 
@@ -195,9 +218,6 @@ public class ActiveInboundResourceAdapter extends
     private void registerWorkStats() {
         if (getWorkStatsMonitoringLevel() != MonitoringLevel.OFF) {
             
-            final ConnectorWorkMgmtStatsImpl workStatsImpl 
-                            = new ConnectorWorkMgmtStatsImpl(this);
-
             ((MonitorableWorkManager)this.getBootStrapContext().getWorkManager())
                                                 .setMonitoringEnabled(true);
             
@@ -210,7 +230,7 @@ public class ActiveInboundResourceAdapter extends
                         String moduleName = ActiveInboundResourceAdapter.this.getModuleName();
                         //@todo :: after MBeans are modified
                         //Dont register system RARs as of now until MBean changes are complete.
-                        if (ResourcesUtil.getInstance().belongToSystemRar(moduleName)) {
+                        if (ResourcesUtil.createInstance().belongToSystemRar(moduleName)) {
                             if (!ConnectorAdminServiceUtils.isJMSRA(moduleName)) {
                                 return null;
                             }
@@ -283,7 +303,7 @@ public class ActiveInboundResourceAdapter extends
             return;
         }
         obtainServerXMLvalue();
-        ResourcesUtil resUtil = ResourcesUtil.getInstance();
+        ResourcesUtil resUtil = ResourcesUtil.createInstance();
 
         if(isServer() && !resUtil.belongToSystemRar(moduleName_)) {
             createAllConnectorResources();
@@ -303,10 +323,9 @@ public class ActiveInboundResourceAdapter extends
                              (connectionDefs_.length != 0)) {
             destroyAllConnectorResources();
             //deactivateEndpoints as well!
-            Iterator iter = getAllEndpointFactoryInfo().iterator();
+            Iterator<MessageEndpointFactoryInfo> iter = getAllEndpointFactories().iterator();
             while (iter.hasNext()) {
-                MessageEndpointFactoryInfo element = 
-                    (MessageEndpointFactoryInfo) iter.next();
+                MessageEndpointFactoryInfo element = iter.next(); 
                 try {
                     this.resourceadapter_.endpointDeactivation(
                         element.getEndpointFactory(),element.getActivationSpec());
@@ -363,7 +382,7 @@ public class ActiveInboundResourceAdapter extends
      * @return <code>MessageEndpointFactoryIndo</code> object.
      */
     public MessageEndpointFactoryInfo getEndpointFactoryInfo(String id) {
-        return (MessageEndpointFactoryInfo) factories_.get(id);
+        return factories_.get(id);
     }
 
     /*
@@ -372,7 +391,7 @@ public class ActiveInboundResourceAdapter extends
      *         A shallow copy only to avoid concurrency issues.
      */
     public Set getAllEndpointFactoryInfo() {
-        Hashtable infos = (Hashtable) factories_.clone();
+        Hashtable infos = (Hashtable<String, MessageEndpointFactoryInfo>) factories_.clone();
         return infos.entrySet();
     }
 
@@ -403,7 +422,7 @@ public class ActiveInboundResourceAdapter extends
      * @return a <code>Collection</code> of <code>MessageEndpointFactory</code>
      *         objects.
      */
-    public Collection getAllEndpointFactories() {
+    public Collection<MessageEndpointFactoryInfo> getAllEndpointFactories() {
         return factories_.values();
     }
 
@@ -508,27 +527,25 @@ public class ActiveInboundResourceAdapter extends
                                   new ElementProperty[]{},
                                   getDescriptor().getConfigProperties());
             }
-            
+
             //HACK !
             if (this.moduleName_.equals(ConnectorRuntime.DEFAULT_JMS_ADAPTER)) {
-		    if (ConnectorRuntime.getRuntime().isServer()) {
+                    if (ConnectorRuntime.getRuntime().isServer()) {
                         hackMergedProps(mergedProps);
-		    }
+                    }
             }
-            
             logMergedProperties(mergedProps);
             SetMethodAction setMethodAction = new SetMethodAction
                                 (this.resourceadapter_, mergedProps);
             setMethodAction.run();
-	} catch(Exception e) {
-	    String i18nMsg = localStrings.getString(
-	        "ccp_adm.wrong_params_for_create", e.getMessage() );
-	    ConnectorRuntimeException cre = 	
+        } catch(Exception e) {
+            String i18nMsg = localStrings.getString(
+                "ccp_adm.wrong_params_for_create", e.getMessage() );
+            ConnectorRuntimeException cre =
                 new ConnectorRuntimeException( i18nMsg );
-            cre.initCause( e );		
-	    throw cre;
-	}
-
+            cre.initCause( e );
+            throw cre;
+        }
     }
 
     /**
@@ -538,30 +555,40 @@ public class ActiveInboundResourceAdapter extends
      * that the passed in connection URL is one additional
      * URL, apart from the default URL derived from brokerhost:brokerport
      * and reported a PE connection url limitation.
-     *    
+     *
      */
-    private void hackMergedProps(Set mergedProps) {
-        
+   private void hackMergedProps(Set mergedProps) {
+
         String brokerType = null;
-        
+
         for (Iterator iter = mergedProps.iterator(); iter.hasNext();) {
             EnvironmentProperty element = (EnvironmentProperty) iter.next();
             if (element.getName().equals(ActiveJmsResourceAdapter.BROKERTYPE)) {
                      brokerType = element.getValue();
             }
         }
-        
-        if (brokerType.equals(ActiveJmsResourceAdapter.LOCAL) 
-                || brokerType.equals(ActiveJmsResourceAdapter.EMBEDDED)) {
-              for (Iterator iter = mergedProps.iterator(); iter.hasNext();) {
+	boolean cluster = false;
+	try {
+		cluster = JmsRaUtil.isClustered();
+	} catch (Exception e) {
+		e.printStackTrace();
+	}
+	// hack is required only for nonclustered nonremote brokers.
+	if (!cluster) {
+        if (brokerType.equals(ActiveJmsResourceAdapter.LOCAL)
+                || brokerType.equals(ActiveJmsResourceAdapter.EMBEDDED)
+		|| brokerType.equals(ActiveJmsResourceAdapter.DIRECT)) 
+        {              
+		for (Iterator iter = mergedProps.iterator(); iter.hasNext();) {
                 EnvironmentProperty element = (EnvironmentProperty) iter.next();
                 if (element.getName().equals(ActiveJmsResourceAdapter.CONNECTION_URL)) {
                     iter.remove();
                 }
-             }
-        }
+          }
+    	}
+	}
     }
-    
+
 
     private void logMergedProperties(Set mergedProps) {
 			_logger.fine("Passing in the following properties " +
