@@ -1,28 +1,42 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.connectors;
 
+import com.sun.enterprise.config.ConfigException;
 import java.util.logging.*;
 import java.util.*;
 
@@ -37,6 +51,7 @@ import com.sun.enterprise.connectors.system.*;
 import com.sun.enterprise.connectors.authentication.*;
 import java.sql.Connection;
 import java.sql.SQLException;
+import java.io.PrintWriter;
 
 /**
  * This class is the entry point to connector backend module.
@@ -50,30 +65,28 @@ import java.sql.SQLException;
 
 
 public final class ConnectorRuntime implements ConnectorConstants {
-    static Logger _logger = LogDomains.getLogger(LogDomains.RSR_LOGGER);     
+    
+    static final Logger _logger = LogDomains.getLogger(LogDomains.RSR_LOGGER);     
      
-    private static ConnectorRegistry _registry = null;
+    private volatile int environment = CLIENT;
 
-    private static ConnectorRuntime _runtime;
-
- 
-    private boolean debug=true;
-    private int environment = CLIENT;
-
-    private static ConnectorAdminObjectAdminServiceImpl 
-                   adminObjectAdmService = null;
-    private static ConnectorConfigurationParserServiceImpl 
-                   configParserAdmService =null;
-    private static ConnectorConnectionPoolAdminServiceImpl 
-                   ccPoolAdmService =null;
-    private static ConnectorResourceAdminServiceImpl 
-                   connectorResourceAdmService =null;
-    private static ConnectorSecurityAdminServiceImpl 
-                   connectorSecurityAdmService =null;
-    private static ResourceAdapterAdminServiceImpl 
-                   resourceAdapterAdmService = null;
-    private static ConnectorServiceImpl connectorService=null;
+    private final ConnectorAdminObjectAdminServiceImpl 
+                   adminObjectAdmService;
+    private final ConnectorConfigurationParserServiceImpl 
+                   configParserAdmService;
+    private final ConnectorConnectionPoolAdminServiceImpl 
+                   ccPoolAdmService;
+    private final ConnectorResourceAdminServiceImpl 
+                   connectorResourceAdmService;
+    private final ConnectorSecurityAdminServiceImpl 
+                   connectorSecurityAdmService;
+    private final ResourceAdapterAdminServiceImpl 
+                   resourceAdapterAdmService;
+    private final ConnectorServiceImpl connectorService;
 
+    private boolean isSystemResourcesLoaded = false;
+    
+    private static final ConnectorRuntime _runtime = new ConnectorRuntime();    
 
     /**
      * Returns the ConnectorRuntime instance.
@@ -84,47 +97,14 @@ public final class ConnectorRuntime implements ConnectorConstants {
      */
 
     public static ConnectorRuntime getRuntime() {
-        if (_runtime == null) {
-            _runtime = new ConnectorRuntime();
-            com.sun.enterprise.util.Utility.setDataDirectProperty();
-            createServices();
-        }
         return _runtime;
     }
-
-    /** Initializes the execution environment. If the execution environment
-     *  is appserv runtime it is set to ConnectorConstants.SERVER else 
-     *  it is set ConnectorConstants.CLIENT
-     *  @param environment set to ConnectorConstants.SERVER if execution 
-     *              environment is appserv runtime else set to 
-     *              ConnectorConstants.CLIENT
-     */
-
-    public void initialize(int environment) {
-        this.environment = environment;
-        initServices();
-    }
-
-    /**
-     * Returns the execution environment. 
-     * @return ConnectorConstants.SERVER if execution environment is 
-     *         appserv runtime 
-     *         else it returns ConnectorConstants.CLIENT
-     */
-
-    public int getEnviron() {
-        return environment;
-    }
-
+    
     /** 
      * Private constructor. It is private as it follows singleton pattern.
      */
-
     private ConnectorRuntime() {
-        _registry = ConnectorRegistry.getInstance();
-    }
-
-    private static void createServices() {
+        com.sun.enterprise.util.Utility.setEnvironment();
         adminObjectAdmService = (ConnectorAdminObjectAdminServiceImpl)
            ConnectorAdminServicesFactory.getService(ConnectorAdminService.AOR);
         ccPoolAdmService = ( ConnectorConnectionPoolAdminServiceImpl)
@@ -139,11 +119,31 @@ public final class ConnectorRuntime implements ConnectorConstants {
         connectorService = new ConnectorServiceImpl();
     }
 
-    private void initServices() {
-        connectorService.initialize(getEnviron()); 
+    /** Initializes the execution environment. If the execution environment
+     *  is appserv runtime it is set to ConnectorConstants.SERVER else 
+     *  it is set ConnectorConstants.CLIENT
+     *  @param environment set to ConnectorConstants.SERVER if execution 
+     *              environment is appserv runtime else set to 
+     *              ConnectorConstants.CLIENT
+     */
+
+    public void initialize(int environment) {
+        this.environment = environment;
+        connectorService.initialize(getEnviron());
     }
 
-    /** 
+    /**
+     * Returns the execution environment. 
+     * @return ConnectorConstants.SERVER if execution environment is 
+     *         appserv runtime 
+     *         else it returns ConnectorConstants.CLIENT
+     */
+
+    public int getEnviron() {
+        return environment;
+    }
+
+   /** 
      *  Destroys/deletes the Active resource adapter object from the 
      *  connector container. Active resource adapter abstracts the rar
      *  deployed. It checks whether any resources (pools and connector
@@ -202,11 +202,10 @@ public final class ConnectorRuntime implements ConnectorConstants {
     public void  createActiveResourceAdapter(
             ConnectorDescriptor connectorDescriptor, 
             String moduleName, 
-            String moduleDir, 
-            boolean writeSunDescriptor) throws ConnectorRuntimeException 
+            String moduleDir) throws ConnectorRuntimeException 
     {
         resourceAdapterAdmService.createActiveResourceAdapter(
-               connectorDescriptor,moduleName,moduleDir,writeSunDescriptor);
+               connectorDescriptor,moduleName,moduleDir);
 
     }
 
@@ -221,11 +220,10 @@ public final class ConnectorRuntime implements ConnectorConstants {
      */ 
 
     public void  createActiveResourceAdapter(String moduleDir , 
-                String moduleName,
-                boolean writeSunDescriptor) throws ConnectorRuntimeException 
+                String moduleName) throws ConnectorRuntimeException 
     {
         resourceAdapterAdmService.createActiveResourceAdapter(
-           moduleDir,moduleName,writeSunDescriptor); 
+           moduleDir,moduleName); 
     }
 
     public ConnectionManager obtainConnectionManager(String poolName) 
@@ -242,6 +240,21 @@ public final class ConnectorRuntime implements ConnectorConstants {
                 getAvailableConnectionManager(poolName, forceNoLazyAssoc);
         return mgr;
     } 
+
+    /** Returns the MCF instances in scenarions where a pool has to 
+     *  return multiple mcfs. Should be used only during JMS RA recovery.
+     *  @param poolName Name of the pool.MCFs pertaining to this pool is
+     *         created/returned.
+     *  @return created MCF instances
+     *  @throws ConnectorRuntimeException if creation/retrieval of MCFs fails
+     */
+
+    public ManagedConnectionFactory[]  obtainManagedConnectionFactories(
+           String poolName) throws ConnectorRuntimeException
+    {
+
+        return ccPoolAdmService.obtainManagedConnectionFactories(poolName);
+    }
     /** Returns the MCF instance. If the MCF is already created and
      *  present in connectorRegistry that instance is returned. Otherwise it
      *  is created explicitly and added to ConnectorRegistry.
@@ -854,7 +867,7 @@ public final class ConnectorRuntime implements ConnectorConstants {
      */
 
     public static boolean isServer() {
-        return connectorService.isServer(); 
+        return getRuntime().connectorService.isServer(); 
     }
 
     public void loadDeferredResourceAdapter(String rarName) 
@@ -880,7 +893,7 @@ public final class ConnectorRuntime implements ConnectorConstants {
      *                Connection Definition class
      */
     public static Set getConnectionDefinitionProperties(String connectionDefinitionClassName) {
-        return ccPoolAdmService.getConnectionDefinitionProperties(
+        return getRuntime().ccPoolAdmService.getConnectionDefinitionProperties(
             connectionDefinitionClassName);
     }
                                                                                                                                                
@@ -888,6 +901,11 @@ public final class ConnectorRuntime implements ConnectorConstants {
       * Gets the properties of the Java bean connection definition class that
       * have setter methods defined and the default values as provided by the
       * Connection Definition java bean developer.
+      * This method is used to get properties of jdbc-data-source<br>
+      * To get Connection definition properties for Connector Connection Pool,
+      * use ConnectorRuntime.getMCFConfigProperties()<br>
+      * When the connection definition class is not found, standard JDBC
+      * properties (of JDBC 3.0 Specification) will be returned.<br>
       *
       * @param connectionDefinitionClassName
       *                     The Connection Definition Java bean class for which
@@ -895,7 +913,7 @@ public final class ConnectorRuntime implements ConnectorConstants {
       * @return Map [property, defaultValue]
       */
     public static Map getConnectionDefinitionPropertiesAndDefaults(String connectionDefinitionClassName) {
-        return ccPoolAdmService.getConnectionDefinitionPropertiesAndDefaults(
+        return getRuntime().ccPoolAdmService.getConnectionDefinitionPropertiesAndDefaults(
             connectionDefinitionClassName);
     }
  
@@ -949,7 +967,7 @@ public final class ConnectorRuntime implements ConnectorConstants {
      * Calls the stop method for all J2EE Connector 1.5 spec compliant RARs
      */    
     public static void stopAllActiveResourceAdapters(){
-        resourceAdapterAdmService.stopAllActiveResourceAdapters();
+        getRuntime().resourceAdapterAdmService.stopAllActiveResourceAdapters();
     }   
     
     /**
@@ -1057,23 +1075,86 @@ public final class ConnectorRuntime implements ConnectorConstants {
     }
 
     /**
-     * Lookup using JNDI to get PM Resource
-     * @param name
-     * @return PMResource
-     * @throws NamingException
+     * Does lookup of "__pm" datasource. If found, it will be returned.<br><br>
+     *
+     * If not found and <b>force</b> is true, this api will try to get a wrapper datasource specified
+     *  by the jdbcjndi name. The motivation for having this
+     * API is to provide the CMP backend/ JPA-Java2DB a means of acquiring a connection during
+     * the codegen phase. If a user is trying to deploy an JPA-Java2DB app on a remote server,
+     * without this API, a resource reference has to be present both in the DAS
+     * and the server instance. This makes the deployment more complex for the
+     * user since a resource needs to be forcibly created in the DAS Too.
+     * This API will mitigate this need.
+     * When the resource is not enabled, datasource wrapper provided will not be of
+     * type "__pm"
+     *
+     * @param jndiName  jndi name of the resource
+     * @param force provide the resource (in DAS)  even if it is not enabled in DAS
+     * @return DataSource representing the resource.
+     * @throws NamingException when not able to get the datasource.
      */
-    public Object lookupPMResource(String name) throws NamingException{
-        return connectorResourceAdmService.lookup(name+PM_JNDI_SUFFIX);
+    public Object lookupPMResource(String jndiName, boolean force) throws NamingException{
+        Object result ;
+        try{
+            result = connectorResourceAdmService.lookup(jndiName+PM_JNDI_SUFFIX);
+        }catch(NamingException ne){
+            if(force && ResourcesUtil.isDAS()){
+                _logger.log(Level.FINE, "jdbc.unable_to_lookup_resource",new Object[] {jndiName});
+                result = lookupDataSourceInDAS(jndiName);
+            }else{
+                throw ne;
+            }
+        }
+        return result;
+    }
+    /**
+     * Does lookup of non-tx-datasource. If found, it will be returned.<br><br>
+     *
+     * If not found and <b>force</b> is true,  this api will try to get a wrapper datasource specified
+     * by the jdbcjndi name. The motivation for having this
+     * API is to provide the CMP backend/ JPA-Java2DB a means of acquiring a connection during
+     * the codegen phase. If a user is trying to deploy an JPA-Java2DB app on a remote server,
+     * without this API, a resource reference has to be present both in the DAS
+     * and the server instance. This makes the deployment more complex for the
+     * user since a resource needs to be forcibly created in the DAS Too.
+     * This API will mitigate this need.
+     *
+     * @param jndiName  jndi name of the resource
+     * @param force provide the resource (in DAS)  even if it is not enabled in DAS
+     * @return DataSource representing the resource.
+     * @throws NamingException when not able to get the datasource.
+     */
+    public Object lookupNonTxResource(String jndiName, boolean force) throws NamingException{
+        Object result ;
+        try{
+            result = connectorResourceAdmService.lookup(jndiName+NON_TX_JNDI_SUFFIX);
+        }catch(NamingException ne){
+            if(force && ResourcesUtil.isDAS()){
+                _logger.log(Level.FINE, "jdbc.unable_to_lookup_resource",new Object[] {jndiName});
+                result = lookupDataSourceInDAS(jndiName);
+            }else{
+                throw ne;
+            }
+        }
+        return result;
     }
 
+
     /**
-     * Lookup using JNDI to get NonTx Resource
-     * @param name
-     * @return  NonTx Resource
-     * @throws NamingException
+     * Get a wrapper datasource specified by the jdbcjndi name
+     * This API is intended to be used in the DAS. The motivation for having this
+     * API is to provide the CMP backend/ JPA-Java2DB a means of acquiring a connection during
+     * the codegen phase. If a user is trying to deploy an JPA-Java2DB app on a remote server,
+     * without this API, a resource reference has to be present both in the DAS
+     * and the server instance. This makes the deployment more complex for the
+     * user since a resource needs to be forcibly created in the DAS Too.
+     * This API will mitigate this need.
+     *
+     * @param jndiName  jndi name of the resource
+     * @return DataSource representing the resource.
      */
-    public Object lookupNonTxResource(String name) throws NamingException{
-        return connectorResourceAdmService.lookup(name+NON_TX_JNDI_SUFFIX);
+    private Object lookupDataSourceInDAS(String jndiName){
+        return connectorResourceAdmService.lookupDataSourceInDAS(jndiName);
     }
 
     /**
@@ -1094,6 +1175,24 @@ public final class ConnectorRuntime implements ConnectorConstants {
                 ConnectorConstants.DEFAULT_JMS_ADAPTER,
                 ConnectorConstants.JAXR_RA_NAME};
     }
-     
+    
+    /**
+     * Loads all system RA resources not used till now
+     * This method is used when user accesses jndi tree.
+     */
+    public synchronized void loadAllSystemRAResources(){
+        if(!isSystemResourcesLoaded){
+            ResourcesUtil.createInstance().loadSystemRAResources();
+            isSystemResourcesLoaded = true;
+        }
+    }
+
+    public PrintWriter getResourceAdapterLogWriter(){
+        Logger logger =  LogDomains.getLogger(LogDomains.RSR_LOGGER);
+        RAWriterAdapter writerAdapter = new RAWriterAdapter(logger);
+        return new PrintWriter(writerAdapter);
+    }
 
 }
+
+
