@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.deployment.client;
@@ -29,11 +42,13 @@ import com.sun.appserv.management.client.ConnectionSource;
 import com.sun.appserv.management.client.ProxyFactory;
 import com.sun.appserv.management.config.*;
 import com.sun.appserv.management.deploy.DeploymentSupport;
+import com.sun.appserv.management.util.jmx.JMXUtil;
 import com.sun.enterprise.deployapi.SunTarget;
 import com.sun.enterprise.deployment.backend.DeploymentStatus;
 import com.sun.enterprise.deployment.util.DeploymentProperties;
 import com.sun.enterprise.deployment.util.DOLUtils;
 import com.sun.enterprise.deployment.util.XModuleType;
+import com.sun.enterprise.admin.common.ObjectNames;
 
 import java.io.File;
 import java.io.FileOutputStream;
@@ -54,6 +69,8 @@ public class DeploymentClientUtils {
 
     private static final String APPS_CONFIG_MBEAN = 
         "com.sun.appserv:type=applications,category=config";
+    private static ObjectName applicationsMBean =
+        JMXUtil.newObjectName(APPS_CONFIG_MBEAN);
 
     public static final String mapModuleTypeToXType(ModuleType moduleType) {
         if (ModuleType.EAR.equals(moduleType)) {
@@ -73,7 +90,6 @@ public class DeploymentClientUtils {
     public static final ModuleType getModuleType(
         MBeanServerConnection mbsc, String moduleID) throws Exception {
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[]{"java.lang.String"};
             Object[] params = new Object[]{moduleID};
             Integer modType = (Integer) mbsc.invoke(applicationsMBean, "getModuleType", params, signature);
@@ -88,7 +104,6 @@ public class DeploymentClientUtils {
                                                             SunTarget target, Map options) throws Exception {
         //System.out.println("DBG_PRINT : del-app-ref for target = " + target.getName());
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[]{"java.lang.String", "java.lang.String", "java.util.Map"};
             Object[] params = new Object[]{target.getName(), moduleID, options};
             return (DeploymentStatus) (mbsc.invoke(
@@ -101,7 +116,6 @@ public class DeploymentClientUtils {
                                                             SunTarget target, Map options) throws Exception {
         //System.out.println("DBG_PRINT : app stop for target = " + target.getName());
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[]{"java.lang.String", "java.lang.String", "java.util.Map"};
             Object[] params = new Object[]{moduleID, target.getName(), options};
             return (DeploymentStatus) (mbsc.invoke(
@@ -114,7 +128,6 @@ public class DeploymentClientUtils {
                                         SunTarget target, Map options) throws Exception {
         //System.out.println("DBG_PRINT : create-app-ref for target = " + target.getName());
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             Object[] params = new Object[]{target.getName(), moduleID, options};
             String[] signature = new String[]{"java.lang.String", "java.lang.String", "java.util.Map"};
             return (DeploymentStatus) (mbsc.invoke(
@@ -127,7 +140,6 @@ public class DeploymentClientUtils {
                                                             SunTarget target, Map options) throws Exception {
         //System.out.println("DBG_PRINT : app start for target = " + target.getName());
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[]{"java.lang.String", "java.lang.String", "java.util.Map"};
             Object[] params = new Object[]{moduleID, target.getName(), options};
             return (DeploymentStatus) (mbsc.invoke(
@@ -143,7 +155,6 @@ public class DeploymentClientUtils {
         }
     
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[] {"java.lang.String"};
             Object[] params = new Object[] {moduleID};
             Object result = mbsc.invoke(applicationsMBean, "getModuleType", 
@@ -157,7 +168,6 @@ public class DeploymentClientUtils {
 
     public static boolean isLifecycleModule(MBeanServerConnection mbsc, String moduleID) throws Exception {
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[] {"java.lang.String"};
             Object[] params = new Object[] {moduleID};
             Boolean ret = (Boolean)mbsc.invoke(applicationsMBean, "isLifecycleModuleRegistered", params, signature);
@@ -171,7 +181,6 @@ public class DeploymentClientUtils {
     public static DeploymentStatus createLifecycleModuleReference(MBeanServerConnection mbsc,
         String moduleID, String target, Map options) throws Exception {
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[] {"java.lang.String", "java.lang.String", "java.util.Map"};
             Object[] params = new Object[] {moduleID, target, options};
             return((DeploymentStatus)mbsc.invoke(applicationsMBean, "createLifecycleModuleReference", params, signature));
@@ -182,7 +191,6 @@ public class DeploymentClientUtils {
     public static DeploymentStatus removeLifecycleModuleReference(MBeanServerConnection mbsc,
         String moduleID, String target) throws Exception {
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             String[] signature = new String[] {"java.lang.String", "java.lang.String"};
             Object[] params = new Object[] {moduleID, target};
             return((DeploymentStatus)mbsc.invoke(applicationsMBean, "removeLifecycleModuleReference", params, signature));
@@ -194,7 +202,6 @@ public class DeploymentClientUtils {
                                                         SunTarget target, boolean enable) throws Exception {
         //System.out.println("DBG_PRINT : Changing state in target  " + target.getName() + " to " + enable);
         if (mbsc!=null) {
-            ObjectName applicationsMBean = new ObjectName(APPS_CONFIG_MBEAN);
             Object[] params = new Object[]{moduleID, type, target.getName()};
             String[] signature = new String[]{"java.lang.String", "java.lang.String", "java.lang.String"};
 
@@ -405,8 +412,7 @@ public class DeploymentClientUtils {
             //Note that we still rely on the SystemsServiceMBean to retrieve 
             //the client jar location from the server.  This should only 
             //happen on DAS.
-            ObjectName ssMBean = 
-                new ObjectName("ias:type=system-services,server=server");
+            ObjectName ssMBean = ObjectNames.getSystemServicesObjectName();
             
             String filePath = (String) mbsc.invoke(
                 ssMBean, "getClientStubJarLocation", params, signature);
