@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -55,6 +68,8 @@ import com.sun.enterprise.deployment.util.ModuleDescriptor;
 import com.sun.enterprise.deployment.WebBundleDescriptor;
 import com.sun.enterprise.deployment.WebService;
 import com.sun.enterprise.management.deploy.DeploymentCallback;
+import com.sun.enterprise.resource.Resource;
+import com.sun.enterprise.resource.ResourceUtilities;
 import com.sun.enterprise.util.i18n.StringManager;
 import com.sun.enterprise.util.io.FileUtils;
 import com.sun.enterprise.util.RelativePathResolver;
@@ -65,6 +80,7 @@ import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import java.util.List;
 
 import javax.enterprise.deploy.shared.ModuleType;
 
@@ -126,6 +142,8 @@ public class J2EECPhase extends DeploymentPhase {
             // to J2EECPhase deployment status
             req.setCurrentDeploymentStatus(status);
 
+            req.setDescriptor(deploymentCtx.getApplication(req.getName()));
+
             deployer = DeployerFactory.getDeployer(req);
             
             //prePhaseNotify(getPrePhaseEvent(deployer.getPreEventInfo()));
@@ -146,6 +164,10 @@ public class J2EECPhase extends DeploymentPhase {
             }
             
             if(isRedeploy) {
+                // clear the deploymentCtx cache now
+                deploymentCtx.removeApplication(req.getName());               
+                req.setDescriptor(null);
+
                 target = (DeploymentTarget)req.getTarget();
                 
                 // In the case of redeploy to domain,
@@ -167,7 +189,12 @@ public class J2EECPhase extends DeploymentPhase {
             }
             
             deployer.doRequestFinish();
-            
+
+            // cache the updated application object in deployment context 
+            deploymentCtx.addApplication(req.getName(), req.getDescriptor());
+ 
+            parseAndValidateSunResourcesXMLFiles(req);
+
             // check if an abort operation has been issued
             // throw exception if true
             DeploymentServiceUtils.checkAbort(req.getName());
@@ -204,6 +231,8 @@ public class J2EECPhase extends DeploymentPhase {
         } catch(Throwable t) {
             String msg =
             localStrings.getString("enterprise.deployment.phasing.j2eec.error" );
+            if (t.getCause()!= null) 
+                msg += t.getCause().toString();
 
             // For any failure during the J2EEC phase (during fresh deploy or redeploy), cleanup the domain.xml
             // so that the config is left in a clean state without any hanging j2ee-app/module elements without
@@ -468,4 +497,14 @@ public class J2EECPhase extends DeploymentPhase {
         String wsdlDir = bundle.getWsdlDir();
         return entry.substring(wsdlDir.length()+1);
     }
+
+    // this API parses the sun-resources.xml files and check for duplicates
+    // conflicts within the archive and with domain.xml
+    private void parseAndValidateSunResourcesXMLFiles(DeploymentRequest req) 
+        throws Exception {
+        List<Resource> resourceList = DeploymentServiceUtils.getResourceList(
+            req, true, deploymentCtx);
+        ResourceUtilities.getResourceConflictsWithDomainXML(
+            resourceList, DeploymentServiceUtils.getConfigContext());
+    }
 }
