@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the License).  You may not use this file except in
- * compliance with the License.
- *
- * You can obtain a copy of the license at
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * Header Notice in each file and include the License file
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.
- * If applicable, add the following below the CDDL Header,
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information:
- * "Portions Copyrighted [year] [name of copyright owner]"
- *
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -70,7 +83,6 @@ import java.util.Enumeration;
 import java.util.Iterator;
 
 import com.sun.appserv.server.util.PreprocessorUtil;
-import com.sun.enterprise.util.Print;
 import com.sun.logging.LogDomains;
 import com.sun.enterprise.util.i18n.StringManager;
 
@@ -93,37 +105,38 @@ public class EJBClassLoader
         implements JasperAdapter, InstrumentableClassLoader {
 
     /** logger for this class */
-    static Logger _logger=LogDomains.getLogger(LogDomains.LOADER_LOGGER);
+    static final Logger _logger=LogDomains.getLogger(LogDomains.LOADER_LOGGER);
 
     /** list of url entries of this class loader */
-    private List<URLEntry> urlSet = Collections.synchronizedList(new ArrayList());
+    private final List<URLEntry> urlSet = Collections.synchronizedList(new ArrayList());
 
     /** cache of not found resources */
-    private Map notFoundResources   = new HashMap();
+    private final Map<String,String> notFoundResources   = new HashMap<String,String>();
 
     /** cache of not found classes */
-    private Map notFoundClasses     = new HashMap();
+    private final Map<String,String> notFoundClasses     = new HashMap<String,String>();
 
     /** state flag to track whether this instance has been shut off.  */
-    private boolean doneCalled = false;
+    private volatile boolean doneCalled = false;
     /** snapshot of classloader state at the time done was called */
-    private String doneSnapshot;
+    private volatile String doneSnapshot;
 
     /** streams opened by this loader */
-    private Vector<SentinelInputStream> streams = null;
+    private volatile Vector<SentinelInputStream> streams = null;
 
-    private ArrayList<ClassTransformer> transformers =
+    private final ArrayList<ClassTransformer> transformers =
             new ArrayList<ClassTransformer>(1);
 
-    private static StringManager sm = 
+    private static final StringManager sm = 
         StringManager.getManager(EJBClassLoader.class);
 
+    private static final String LINE_SEP = System.getProperty("line.separator");
+    
     /**
      * Constructor.
      */
     public EJBClassLoader() {
         super(new URL[0]);
-
         if (_logger.isLoggable(Level.FINE)) {
             _logger.log(Level.FINE,
                         "ClassLoader: " + this + " is getting created.");
@@ -158,7 +171,7 @@ public class EJBClassLoader
         // before setting doneCalled = true.
         doneSnapshot = "EJBClassLoader.done() called ON " + this.toString()
             + "\n AT " + new Date() +
-            " \n BY :" + Print.printStackTraceToString();
+            " \n BY :" + printStackTraceToString();
         doneCalled = true;
 
         // closes the jar handles and sets the url entries to null
@@ -189,9 +202,9 @@ public class EJBClassLoader
         if (this.notFoundClasses != null)   { this.notFoundClasses.clear();   }
 
         // sets all the objects to null
-        this.urlSet              = null;
-        this.notFoundResources   = null;
-        this.notFoundClasses     = null;
+        //this.urlSet              = null;
+        //this.notFoundResources   = null;
+        //this.notFoundClasses     = null;
     }
 
 
@@ -791,6 +804,22 @@ public class EJBClassLoader
         return stream;
     }
 
+    /**
+     *Formats the caller's stack trace to a string.
+     */
+    private String printStackTraceToString() {
+        StringBuilder sb = new StringBuilder();
+        StackTraceElement[] elements = Thread.currentThread().getStackTrace();
+        /*
+         *Start with element 1 and not 0 to avoid displaying the stack frame
+         *for the call to this method.
+         */
+        for (int i = 1; i < elements.length; i++) {
+            sb.append(elements[i].toString()).append(LINE_SEP);
+        }
+        return sb.toString();
+    }
+    
     /**
      *Looks up the key in the logger's resource bundle and substitutes any
      *arguments provided into the looked-up string.
@@ -1050,10 +1079,18 @@ public class EJBClassLoader
 
             if (obj instanceof URLEntry) {
                 URLEntry e = (URLEntry) obj;
+                try {
+                	//try comparing URIs
+					if (source.toURI().equals(e.source.toURI())) {
+					    tf = true;
+					}
+				} catch (URISyntaxException e1) {
+					//fall back to comparing URLs
                 if (source.equals(e.source)) {
                     tf = true;
                 }
             }
+            }
 
             return tf;
         }
@@ -1062,8 +1099,13 @@ public class EJBClassLoader
          * Since equals is overridden, we need to override hashCode as well.
          */
         public int hashCode() {
+            try {
+				return source.toURI().hashCode();
+			} catch (URISyntaxException e) {
+				//fall back to URL's hashCode
             return source.hashCode();
         }
+        }
 
     }
 
@@ -1071,7 +1113,7 @@ public class EJBClassLoader
      *Returns the vector of open streams; creates it if needed.
      *@return Vector<SentinelInputStream> holding open streams
      */
-    private Vector<SentinelInputStream> getStreams() {
+    private synchronized Vector<SentinelInputStream> getStreams() {
         if (streams == null) {
             streams = new Vector<SentinelInputStream>();
         }
@@ -1247,13 +1289,30 @@ public class EJBClassLoader
          */
         protected URLConnection openConnection(URL u) throws IOException {
             if (u != mURL) { // ref compare on purpose
-                // This should never happen
-                throw new IOException("Cannot open a foreign URL; this.url=" + mURL
-                    + "; foreign.url=" + u);
+                String path = u.getPath();
+
+                //check if it is a jar reference
+                if (u.getProtocol().equals("jar")) {
+                    if (path.indexOf("!/") != -1) {
+                        path = path.substring(path.indexOf("!/") + 2);
+                        //find resource constructs a url with our special urlhandler
+                        URL u2 = findResource(path);
+                        if (u2 != null) {
+                            return u2.openConnection();
+                        }		
+                    }
+                    // This should never happen. An invalid jar reference?
+                    throw new IOException("Cannot open a foreign URL; this.url=" + mURL
+                            + "; foreign.url=" + u);
+                } else {
+                    //not a jar
+                    return u.openConnection();
+                }
             }
             return new InternalJarURLConnection(u, mRes, mName);
         }
 
+
         /**
          * Ties the URL that this handler is associated with to the handler, so
          * that it can be asserted that somehow no other URLs are mangled in (this
