@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.naming;
@@ -38,6 +51,7 @@ import java.lang.reflect.Method;
 import java.lang.reflect.Proxy;
 import java.lang.reflect.InvocationHandler;
 
+import javax.enterprise.deploy.shared.ModuleType;
 import javax.xml.namespace.QName;
 import javax.xml.rpc.ServiceFactory;
 import javax.xml.rpc.Service;
@@ -51,13 +65,18 @@ import org.omg.CORBA.ORB;
 import com.sun.enterprise.*;
 import com.sun.enterprise.util.*;
 import com.sun.enterprise.deployment.*;
+import com.sun.enterprise.instance.BaseManager;
+import com.sun.enterprise.deployment.phasing.DeploymentServiceUtils;
+import com.sun.enterprise.deployment.backend.DeployableObjectType;
 import com.sun.enterprise.webservice.WsUtil;
 import com.sun.enterprise.webservice.ServiceInvocationHandler;
 import com.sun.enterprise.webservice.JAXWSServiceDelegate;
+import com.sun.enterprise.webservice.WSContainerResolver;
 import com.sun.enterprise.distributedtx.UserTransactionImpl;
 import com.sun.enterprise.naming.java.*;
 import com.sun.enterprise.log.Log;
 
+
 import com.sun.ejb.Container;
 import com.sun.ejb.EJBUtils;
 
@@ -104,6 +123,9 @@ public final class NamingManagerImpl implements NamingManager
     public static final String EIS_STRING = "/eis/";
 
     private static final String CORBANAME = "corbaname:";
+
+    private static final String IIOPURL = "iiop://";
+
  
     public NamingManagerImpl() throws NamingException
     {
@@ -386,11 +408,13 @@ public final class NamingManagerImpl implements NamingManager
             }
 
             if( destinationObject instanceof javax.jms.Queue ) {
-                if( !next.getRefType().equals("javax.jms.Queue") ) {
+                if( !next.getRefType().equals("javax.jms.Queue") && 
+                    !next.getRefType().equals("javax.jms.Destination")) {
                     throw new InvalidNameException(localStrings.getLocalString("naming.destinationRefTypeMismatch", "", new Object[] {next.getName(), next.getRefType()}));
                 }
             } else if( destinationObject instanceof javax.jms.Topic ) {
-                if( !next.getRefType().equals("javax.jms.Topic") ) {
+                if( !next.getRefType().equals("javax.jms.Topic") && 
+                    !next.getRefType().equals("javax.jms.Destination")) {
                     throw new InvalidNameException(localStrings.getLocalString("naming.destinationRefTypeMismatch", "", new Object[] {next.getName(), next.getRefType()}));
                 }
             }
@@ -568,6 +592,21 @@ public final class NamingManagerImpl implements NamingManager
                     J2EEEnvWrapper.MUTABLE_RESOURCE_REF);
                 }
             } 
+	    else if (next.isJMSConnectionFactory()) {
+		try {
+		    Object res = initialContext.lookup(resJndi);
+		    obj = new J2EEEnvWrapper(res, J2EEEnvWrapper.RESOURCE_ADAPTER);
+		} catch (NamingException ne) {
+                    String msg = localStrings.getLocalString
+                    ("naming.unresolved.warning", "",
+                    new Object[] {logicalJndiName, resJndi});
+                    _logger.log(Level.FINE,msg);
+		}
+                if ( obj == null ) { // perhaps the resource is not yet deployed
+                    obj = new J2EEEnvWrapper(resJndi,
+                    J2EEEnvWrapper.MUTABLE_RESOURCE_REF);
+		}
+	    }
 
 	    else if (next.isORB()) {
 		_logger.fine(" we have an ORB resource " + next);
@@ -621,6 +660,50 @@ public final class NamingManagerImpl implements NamingManager
             }
             String logicalJndiName = descriptorToLogicalJndiName(next);
 
+            // Set WSDL File URL here if it null (happens during server restart)
+            if((next.getWsdlFileUrl() == null)  && 
+                (next.getWsdlFileUri() != null)) {
+                try {
+                    if(next.getWsdlFileUri().startsWith("http")) {
+                        // HTTP URLs set as is
+                        next.setWsdlFileUrl(new URL(next.getWsdlFileUri()));
+                    } else if((new File(next.getWsdlFileUri())).isAbsolute()) {
+                        // Absolute WSDL file paths set as is
+                        next.setWsdlFileUrl((new File(next.getWsdlFileUri())).toURL());
+                    } else {
+                        // Relative WSDL file paths prefixed with module dir
+                        BaseManager mgr=null;
+                        if(next.getBundleDescriptor().getApplication().isVirtual()) {
+                            ModuleType mType = next.getBundleDescriptor().getModuleType();
+                            if(mType.equals(ModuleType.WAR)) {
+                                mgr = DeploymentServiceUtils.getInstanceManager(DeployableObjectType.WEB);
+                            } else if(mType.equals(ModuleType.EAR)) {
+                                mgr = DeploymentServiceUtils.getInstanceManager(DeployableObjectType.APP);
+                            } else if(mType.equals(ModuleType.EJB)) {
+                                mgr = DeploymentServiceUtils.getInstanceManager(DeployableObjectType.EJB);
+                            } else if(mType.equals(ModuleType.CAR)) {
+                                mgr = DeploymentServiceUtils.getInstanceManager(DeployableObjectType.CAR);
+                            }
+                            String deployedDir = 
+                                mgr.getLocation(next.getBundleDescriptor().getApplication().getRegistrationName());
+                            if(deployedDir != null) {
+                                File fileURL;
+                                if(next.getBundleDescriptor().getApplication().isVirtual()) {
+                                    fileURL = new File(deployedDir+File.separator+next.getWsdlFileUri());
+                                } else {
+                                    fileURL = new File(deployedDir+File.separator+
+                                            next.getBundleDescriptor().getModuleDescriptor().getArchiveUri().replaceAll("\\.", "_") +
+                                            File.separator +next.getWsdlFileUri());
+                                }
+                                next.setWsdlFileUrl(fileURL.toURL());
+                            }
+                        }
+                    }
+                } catch (Throwable mex) {
+                    throw new NamingException(mex.getLocalizedMessage());
+                }
+            }
+            
             // Delay creation of service ref until access time
             // to avoid classloading difficulties.  (E.g. Setting
             // up service-ref for web component requires classes
@@ -830,7 +913,14 @@ public final class NamingManagerImpl implements NamingManager
 		    ORB orb = ORBManager.getORB();		
 		    obj = (Object) orb.string_to_object(remoteJndiName);
 		} else {
-		    obj = ic.lookup(remoteJndiName);
+		    if (remoteJndiName.startsWith(IIOPURL)) {
+		        Properties env = new Properties();
+			env.put("java.naming.corba.orb",ORBManager.getORB());
+			Context ctx = new InitialContext(env);
+			obj = ctx.lookup(remoteJndiName);
+		    } else {
+		        obj = ic.lookup(remoteJndiName);
+		    }
 		}
 		
 		obj = EJBUtils.resolveEjbRefObject(remoteEjbRef, obj);
@@ -1025,6 +1115,8 @@ public final class NamingManagerImpl implements NamingManager
 
         try {
 
+            WSContainerResolver.set(desc);
+            
             ClassLoader cl = Thread.currentThread().getContextClassLoader();
             
             serviceInterfaceClass = cl.loadClass(desc.getServiceInterface());
@@ -1042,6 +1134,7 @@ public final class NamingManagerImpl implements NamingManager
                     Class serviceImplClass  = cl.loadClass(serviceImplName);
                     serviceDelegate = (Service) serviceImplClass.newInstance();
                 } else {
+                  
                     // The target is probably a post JAXRPC-1.1- based service;
                     // If Service Interface class is set, check if it is indeed a subclass of Service
                     // initiateInstance should not be called if the user has given javax.xml.ws.Service itself
@@ -1172,7 +1265,10 @@ public final class NamingManagerImpl implements NamingManager
             NamingException ne = new NamingException();
             ne.initCause(e);
             throw ne;
+        } finally {
+            WSContainerResolver.unset();
         }
+        
         return returnObj;
     }
     
