@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.resource;
@@ -26,7 +39,7 @@ package com.sun.enterprise.resource;
 import com.sun.enterprise.ManagementObjectManager;
 
 import com.sun.enterprise.server.ResourceDeployer;
-import com.sun.enterprise.server.ResourcesUtil;
+import com.sun.enterprise.connectors.util.ResourcesUtil;
 import com.sun.enterprise.config.serverbeans.Resources;
 import com.sun.enterprise.config.serverbeans.ElementProperty;
 import com.sun.enterprise.config.serverbeans.SecurityMap;
@@ -51,7 +64,7 @@ import com.sun.enterprise.util.i18n.StringManager;
 import com.sun.enterprise.PoolManager;
 import com.sun.enterprise.connectors.ConnectorConstants;
 import com.sun.enterprise.connectors.util.ConnectionPoolObjectsUtils;
-import com.sun.enterprise.connectors.util.SecurityMapUtils;
+import com.sun.enterprise.connectors.util.SecurityMapUtils;                                           
 
 /**
  *
@@ -224,10 +237,10 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
     }
 
     private ConnectorConnectionPool getConnectorConnectionPool(
-	     com.sun.enterprise.config.serverbeans.ConnectorConnectionPool 
-             domainCcp ) throws Exception {
-        ConnectorConnectionPool ccp = 
-                       new ConnectorConnectionPool(domainCcp.getName());
+            com.sun.enterprise.config.serverbeans.ConnectorConnectionPool
+                    domainCcp) throws Exception {
+        ConnectorConnectionPool ccp =
+                new ConnectorConnectionPool(domainCcp.getName());
         ccp.setSteadyPoolSize(domainCcp.getSteadyPoolSize());
         ccp.setMaxPoolSize(domainCcp.getMaxPoolSize());
         ccp.setMaxWaitTimeInMillis(domainCcp.getMaxWaitTimeInMillis());
@@ -235,69 +248,97 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
         ccp.setIdleTimeoutInSeconds(domainCcp.getIdleTimeoutInSeconds());
         ccp.setFailAllConnections(domainCcp.isFailAllConnections());
         ccp.setAuthCredentialsDefinedInPool(
-            isAuthCredentialsDefinedInPool(domainCcp));
-	//The line below will change for 9.0. We will get this from
-	//the domain.xml
-	ccp.setConnectionValidationRequired( 
-	    isIsConnectionValidationRequired() );
-	String txSupport = domainCcp.getTransactionSupport();
-	int txSupportIntVal = parseTransactionSupportString( txSupport );
+                isAuthCredentialsDefinedInPool(domainCcp));
+        //The line below will change for 9.0. We will get this from
+        //the domain.xml
+        ccp.setConnectionValidationRequired(domainCcp.isIsConnectionValidationRequired());
 
-        if (txSupportIntVal == -1 ) {
-	    //if transaction-support attribute is null load the value
-	    //from the ra.xml
-	    if (_logger.isLoggable( Level.FINE ) ) {
-	        _logger.fine("Got transaction-support attr null from domain.xml");
-	    }
-	    txSupportIntVal = ConnectionPoolObjectsUtils.getTransactionSupportFromRaXml( 
-	        domainCcp.getResourceAdapterName() );
-		
-	} else {
-	    //We got some valid transaction-support attribute value
-	    //so go figure if it is valid.
-	    //The tx support is valid if it is less-than/equal-to
-	    //the value specified in the ra.xml
-	    if (! isTxSupportConfigurationSane( txSupportIntVal, 
-	        domainCcp.getResourceAdapterName() ) ) {
-                
-	        String i18nMsg = localStrings.getString(
-	            "ccp_deployer.incorrect_tx_support");
-	        ConnectorRuntimeException cre = new 
-	            ConnectorRuntimeException(i18nMsg);
-                
-                _logger.log(Level.SEVERE, "rardeployment.incorrect_tx_support", 
-	            ccp.getName());
-	        throw cre;	
-	    }
-	}
+        String txSupport = domainCcp.getTransactionSupport();
+        int txSupportIntVal = parseTransactionSupportString(txSupport);
+
+        if (txSupportIntVal == -1) {
+            //if transaction-support attribute is null load the value
+            //from the ra.xml
+            if (_logger.isLoggable(Level.FINE)) {
+                _logger.fine("Got transaction-support attr null from domain.xml");
+            }
+            txSupportIntVal = ConnectionPoolObjectsUtils.getTransactionSupportFromRaXml(
+                    domainCcp.getResourceAdapterName());
+
+        } else {
+            //We got some valid transaction-support attribute value
+            //so go figure if it is valid.
+            //The tx support is valid if it is less-than/equal-to
+            //the value specified in the ra.xml
+            if (!isTxSupportConfigurationSane(txSupportIntVal,
+                    domainCcp.getResourceAdapterName())) {
+
+                String i18nMsg = localStrings.getString(
+                        "ccp_deployer.incorrect_tx_support");
+                ConnectorRuntimeException cre = new
+                        ConnectorRuntimeException(i18nMsg);
+
+                _logger.log(Level.SEVERE, "rardeployment.incorrect_tx_support",
+                        ccp.getName());
+                throw cre;
+            }
+        }
         if (_logger.isLoggable(Level.FINE)) {
-	    _logger.fine("setting txSupportVal to " + txSupportIntVal + 
-	        " in pool " + domainCcp.getName() );
-	}
-	ccp.setTransactionSupport( txSupportIntVal );
+            _logger.fine("setting txSupportVal to " + txSupportIntVal +
+                    " in pool " + domainCcp.getName());
+        }
+        ccp.setTransactionSupport(txSupportIntVal);
 
         //Always for ccp	
-	ccp.setLazyConnectionAssoc( false );
-	ccp.setAssociateWithThread( false );
-	ccp.setNonComponent( false );
-	ccp.setNonTransactional( false );
-	ccp.setConnectionLeakTracing( false );
+        ccp.setNonComponent(false);
+        ccp.setNonTransactional(false);
+        ccp.setConnectionLeakTracingTimeout(domainCcp.getConnectionLeakTimeoutInSeconds());
+        ccp.setConnectionReclaim(domainCcp.isConnectionLeakReclaim());
 
-	//IMPORTANT
-	//Here all properties that will be checked by the 
-	//convertElementPropertyToPoolProperty method need to be set to 
-	//their default values
-	ccp.setMatchConnections( true );
+        ccp.setMatchConnections(domainCcp.isMatchConnections());
+        ccp.setAssociateWithThread(domainCcp.isAssociateWithThread());
+        
+        boolean lazyConnectionEnlistment = domainCcp.isLazyConnectionEnlistment();
+        boolean lazyConnectionAssociation = domainCcp.isLazyConnectionAssociation();
 
-	convertElementPropertyToPoolProperty( ccp, domainCcp );
+        if (lazyConnectionAssociation) {
+            if (lazyConnectionEnlistment) {
+                ccp.setLazyConnectionAssoc(true);
+                ccp.setLazyConnectionEnlist(true);
+            } else {
+                _logger.log(Level.SEVERE,
+                        "conn_pool_obj_utils.lazy_enlist-lazy_assoc-invalid-combination",
+                        domainCcp.getName());
+                String i18nMsg = localStrings.getString(
+                        "cpou.lazy_enlist-lazy_assoc-invalid-combination",  domainCcp.getName());
+                throw new RuntimeException(i18nMsg);
+            }
+        } else {
+            ccp.setLazyConnectionAssoc(lazyConnectionAssociation);
+            ccp.setLazyConnectionEnlist(lazyConnectionEnlistment);
+        }
+
+        ccp.setMaxConnectionUsage(domainCcp.getMaxConnectionUsageCount());
+        ccp.setValidateAtmostOncePeriod(
+                domainCcp.getValidateAtmostOncePeriodInSeconds());
+
+        ccp.setConCreationRetryAttempts(
+                domainCcp.getConnectionCreationRetryAttempts());
+        ccp.setConCreationRetryInterval(
+                domainCcp.getConnectionCreationRetryIntervalInSeconds());
+
+        //IMPORTANT
+        //Here all properties that will be checked by the
+        //convertElementPropertyToPoolProperty method need to be set to
+        //their default values
+        convertElementPropertyToPoolProperty(ccp, domainCcp);
         return ccp;
     }
     
     private void populateConnectorConnectionPool( ConnectorConnectionPool ccp,
         String connectionDefinitionName, String rarName, 
 	ElementProperty[] props, SecurityMap[] securityMaps) 
-	throws ConnectorRuntimeException
-    {
+	throws ConnectorRuntimeException {
     
 	ConnectorRegistry _registry = ConnectorRegistry.getInstance();
         ConnectorDescriptor connectorDescriptor = _registry.getDescriptor(rarName);
@@ -331,7 +372,8 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
         cdi.setConnectionFactoryInterface(cdd.getConnectionFactoryIntf());
         cdi.setConnectionClass(cdd.getConnectionImpl());
         cdi.setConnectionInterface(cdd.getConnectionIntf());
-        Set mergedProps = mergeProps(props, cdd.getConfigProperties());
+        Set mergedProps = mergeProps(props, cdd.getConfigProperties(),
+                                      rarName);
         cdi.setMCFConfigProperties(mergedProps);
         cdi.setResourceAdapterConfigProperties(
                     connectorDescriptor.getConfigProperties());
@@ -340,7 +382,8 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
              
     }
 
-    private Set mergeProps(ElementProperty[] props, Set defaultMCFProps) {
+    private Set mergeProps(ElementProperty[] props, Set defaultMCFProps,
+                            String rarName) {
         HashSet mergedSet = new HashSet();
 
         Object[] defaultProps = ( defaultMCFProps == null ) ? 
@@ -348,6 +391,15 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
 	    defaultMCFProps.toArray();
 
         for (int i =0; i< defaultProps.length; i++) {
+             if (rarName.trim().equals(
+                            ConnectorRuntime.DEFAULT_JMS_ADAPTER)){
+                 EnvironmentProperty ep1 =
+                                  (EnvironmentProperty)defaultProps[i];
+                 if (ep1.getName().equals("AddressList")
+                     && ep1.getValue().equals("localhost")) {
+                    continue;
+                 }
+             }
 	     mergedSet.add(defaultProps[i]);
         }
 
@@ -379,65 +431,56 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
 	        + txSupport + "  raXmlTxSupport => " + raXmlTxSupport);
 	}
 
-        return (txSupport > raXmlTxSupport) ?  false : true ;
+        return (txSupport <= raXmlTxSupport);
        
     }
 
     
     private int parseTransactionSupportString( String txSupport ) {
-        int txSupportIntVal = ConnectorConstants.UNDEFINED_TRANSACTION_INT;
         return ConnectionPoolObjectsUtils.parseTransactionSupportString( txSupport );
     }
-   
-    public boolean isIsConnectionValidationRequired() {
-        String val = System.getProperty( "validation" );
-        return ( val != null ) ? (new Boolean(val)).booleanValue() : false;
-    }
-
-    public boolean isMatchConnections() {
-        String val= System.getProperty( "matchConnections" );
-        return ( val != null ) ? (new Boolean(val)).booleanValue() : true;
-    }
 
     /**
      * The idea is to convert the ElementProperty values coming from the admin
      * connection pool to standard pool attributes thereby making it
      * easy in case of a reconfig
      */
-    public void convertElementPropertyToPoolProperty( ConnectorConnectionPool ccp, 
-	com.sun.enterprise.config.serverbeans.ConnectorConnectionPool domainCcp) 
-    {
+    public void convertElementPropertyToPoolProperty(ConnectorConnectionPool ccp,
+                                                     com.sun.enterprise.config.serverbeans.ConnectorConnectionPool domainCcp) {
         ElementProperty[] elemProps = domainCcp.getElementProperty();
-	if ( elemProps == null ) {
-	    return;
-	}
-        for( ElementProperty ep : elemProps ) {
-	    
-	    if (ep != null &&  "MATCHCONNECTIONS".equals( ep.getName().toUpperCase()) ) {
-		//the foreach loop seems to handle change in hte underlying datastructure.
-	        domainCcp.removeElementProperty( ep );
-		if (_logger.isLoggable( Level.FINE) ) {
-                    _logger.fine("@@@@ ConnectorConnectionPoolDeployer::  Setting matchConnections");		
+        if (elemProps == null) {
+            return;
+        }
+        for (ElementProperty ep : elemProps) {
+            if (ep != null) {
+                if ("MATCHCONNECTIONS".equals(ep.getName().toUpperCase())) {
+                    //the foreach loop seems to handle change in the underlying datastructure.
+                    domainCcp.removeElementProperty(ep);
+                    if (_logger.isLoggable(Level.FINE)) {
+                        _logger.fine(" ConnectorConnectionPoolDeployer::  Setting matchConnections");
+                    }
+                    ccp.setMatchConnections(toBoolean(ep.getValue(), true));
+                } else if ("LAZYCONNECTIONASSOCIATION".equals(ep.getName().toUpperCase())) {
+                    ConnectionPoolObjectsUtils.setLazyEnlistAndLazyAssocProperties(ep.getValue(), domainCcp, ccp);
+                    boolean assoc = toBoolean(ep.getValue(), false);
+                    _logger.log(Level.FINE, "ccp_deployer.lazy_con_assoc_value",
+                            new Object[]{ccp.getName(), String.valueOf(assoc)});
+                } else if ("LAZYCONNECTIONENLISTMENT".equals(ep.getName().toUpperCase())) {
+                    boolean enlist = toBoolean(ep.getValue(), false);
+                    ccp.setLazyConnectionEnlist(enlist);
+                    _logger.log(Level.FINE, "ccp_deployer.lazy_con_enlist_value",
+                            new Object[]{ccp.getName(), String.valueOf(enlist)});
                 }
-		ccp.setMatchConnections( toBoolean( ep.getValue(), true ));
-            }else if ( "LAZYCONNECTIONASSOCIATION".equals(ep.getName().toUpperCase()) ){
-                ConnectionPoolObjectsUtils.setLazyEnlistAndLazyAssocProperties(ep.getValue(),domainCcp, ccp);
-                boolean assoc =   toBoolean( ep.getValue(), false );
-                _logger.log(Level.FINE,"ccp_deployer.lazy_con_assoc_value",new Object[]{ccp.getName(), String.valueOf(assoc)});
-            }else if ( "LAZYCONNECTIONENLISTMENT".equals(ep.getName().toUpperCase()) ){
-                boolean enlist = toBoolean( ep.getValue(), false );
-                ccp.setLazyConnectionEnlist( enlist ) ;
-                _logger.log(Level.FINE,"ccp_deployer.lazy_con_enlist_value",new Object[]{ccp.getName(), String.valueOf(enlist)});
-	    }
-	}
+            }
+        }
     }
 
-    private boolean toBoolean( Object prop, boolean defaultVal ) {
-        if ( prop == null ) {
-	    return defaultVal;
-	}
+    private boolean toBoolean(Object prop, boolean defaultVal) {
+        if (prop == null) {
+            return defaultVal;
+        }
 
-	return (new Boolean( ((String)prop).toLowerCase())).booleanValue(); 
+        return Boolean.valueOf((String) prop);
     }
 
     private boolean isAuthCredentialsDefinedInPool(
@@ -447,10 +490,10 @@ public class ConnectorConnectionPoolDeployer extends GlobalResourceDeployer
             return false;
         }
 
-        for( int i =0; i < elemProps.length; i++ ) { 
+        for( int i =0; i < elemProps.length; i++ ) {
             ElementProperty ep = elemProps[i];
-            if (ep.getName().equalsIgnoreCase("UserName") || 
-                ep.getName().equalsIgnoreCase("User") || 
+            if (ep.getName().equalsIgnoreCase("UserName") ||
+                ep.getName().equalsIgnoreCase("User") ||
                 ep.getName().equalsIgnoreCase("Password")) {
                 return true;
             }
