@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.security.auth.realm.jdbc;
@@ -120,6 +133,7 @@ public final class JDBCRealm extends IASRealm {
      */
     public synchronized void init(Properties props)
             throws BadRealmException, NoSuchRealmException{
+        super.init(props);
         String jaasCtx = props.getProperty(IASRealm.JAAS_CONTEXT_PARAM);
         String dbUser = props.getProperty(PARAM_DB_USER);
         String dbPassword = props.getProperty(PARAM_DB_PASSWORD);
@@ -284,6 +298,7 @@ public final class JDBCRealm extends IASRealm {
         String[] groups = null;
         if (isUserValid(username, password)) {
             groups = findGroups(username);
+            groups = addAssignGroups(groups);
             setGroupNames(username, groups);
         }
         return groups;
@@ -305,7 +320,7 @@ public final class JDBCRealm extends IASRealm {
             String hpwd = hashPassword(password);
             connection = getConnection();
             statement =  connection.prepareStatement(passwordQuery);
-            statement.setString(1,user);
+            statement.setString(1, user);
             rs = statement.executeQuery();
             String pwd = null;
             if (rs.next()) {
@@ -316,8 +331,14 @@ public final class JDBCRealm extends IASRealm {
                     valid = pwd.equals(hpwd);
                 }
             }
+        } catch(SQLException ex) {
+                _logger.log(Level.SEVERE, "jdbcrealm.invaliduserreason", 
+                        new String [] {user,ex.toString()});
+            if (_logger.isLoggable(Level.FINE)) {
+                _logger.log(Level.FINE, "Cannot validate user", ex);
+            }
         } catch(Exception ex) {
-            _logger.log(Level.SEVERE, "jdbcrealm.invaliduser", user);
+                _logger.log(Level.SEVERE, "jdbcrealm.invaliduser", user);
             if (_logger.isLoggable(Level.FINE)) {
                 _logger.log(Level.FINE, "Cannot validate user", ex);
             }
@@ -383,7 +404,7 @@ public final class JDBCRealm extends IASRealm {
         try{
             connection = getConnection();
             statement =  connection.prepareStatement(groupQuery);
-            statement.setString(1,user);
+            statement.setString(1, user);
             rs = statement.executeQuery();
             final List<String> groups = new ArrayList<String>();
             while (rs.next()) {
@@ -391,8 +412,15 @@ public final class JDBCRealm extends IASRealm {
             }
             final String[] groupArray = new String[groups.size()];
             return groups.toArray(groupArray);
+        } catch(SQLException ex) {
+            _logger.log(Level.SEVERE, "jdbcrealm.grouperrorreason", 
+                    new String [] {user,ex.toString()});
+            if (_logger.isLoggable(Level.FINE)) {
+                _logger.log(Level.FINE, "Cannot load group", ex);
+            }
+            return null;
         } catch(Exception ex) {
-            _logger.log(Level.SEVERE, "jdbcrealm.grouperror", user);
+            _logger.log(Level.SEVERE, "jdbcrealm.grouperror", user); 
             if (_logger.isLoggable(Level.FINE)) {
                 _logger.log(Level.FINE, "Cannot load group", ex);
             }
@@ -437,7 +465,7 @@ public final class JDBCRealm extends IASRealm {
         final String dbPassword = this.getProperty(PARAM_DB_PASSWORD);
         try{
             final DataSource dataSource = 
-                (DataSource)ConnectorRuntime.getRuntime().lookupNonTxResource(dsJndi);
+                (DataSource)ConnectorRuntime.getRuntime().lookupNonTxResource(dsJndi,false);
             
             Connection connection = null;
             if (dbUser != null && dbPassword != null) {
