@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -36,6 +49,8 @@
  */
 package com.sun.enterprise.server;
 
+import com.sun.appserv.management.util.misc.RunnableBase;
+import com.sun.appserv.server.ServerLifecycleException;
 import com.sun.enterprise.admin.event.AdminEventListenerException;
 import com.sun.enterprise.admin.event.AdminEventResult;
 import com.sun.enterprise.admin.event.BaseDeployEvent;
@@ -47,12 +62,16 @@ import com.sun.enterprise.config.serverbeans.ApplicationHelper;
 import com.sun.enterprise.config.serverbeans.ApplicationRef;
 import com.sun.enterprise.config.serverbeans.Server;
 import com.sun.enterprise.config.serverbeans.ServerBeansFactory;
+import com.sun.enterprise.connectors.util.ResourcesUtil;
 import com.sun.enterprise.deployment.Application;
 import com.sun.enterprise.deployment.backend.DeployableObjectType;
 import com.sun.enterprise.instance.BaseManager;
 import com.sun.enterprise.management.StateManageable;
+import com.sun.enterprise.server.ondemand.OnDemandServer;
 import com.sun.logging.LogDomains;
 import java.io.File;
+import java.text.MessageFormat;
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
 import java.util.logging.Level;
@@ -271,12 +290,67 @@ public abstract class AbstractManager implements MonitorListener {
         }
 
         // loads all the deployed applications
+        ArrayList<SystemAppStarter> systemAppStarters = new ArrayList<SystemAppStarter>();
+        
+        long systemAppLoadStartTime = System.currentTimeMillis();
+        boolean isSystemAppLoadLoggingOn = _logger.isLoggable(Level.FINE);
         for (int i=0; i<appList.size(); i++) {
 
             // registration name of this app
             String id = (String) appList.get(i);
-            loadOneSystemApp(id, jsr77);
 
+            /**
+             * The system app controlled by on-demand framework should not be
+             * loaded from this path. All appserver system apps are managed by
+             * On-demand system app loader. Any system app inserted by addons 
+             * will be loaded in this path.
+             */
+            if (OnDemandServer.isOnDemandOff() || 
+               !OnDemandServer.getSystemAppLoader().isOnDemandSystemApp(id)) {
+                /*
+                 *For each system app of this manager's type create a starter
+                 *object and submit it for concurrent execution.  Keep
+                 *track of all such objects to wait for their completion and
+                 *to query their success.
+                 */
+                try {
+                    if (configManager.isSystem(id)) {
+                        SystemAppStarter starter = new SystemAppStarter(id, jsr77);
+                        systemAppStarters.add(starter); //loadOneSystemApp(id, jsr77);
+                        starter.submit();
+                        if (isSystemAppLoadLoggingOn) {
+                            _logger.fine(getClass().getName() + " submitted system app load request for " + id);
+                        }
+                    }
+                } catch (ConfigException ce) {
+                    // Ignore this - it would come from the isSystem invocation.
+                }
+            } 
+        }
+        
+        /*
+         * Wait for the system apps to complete their start-up.  Errors will be
+         * thrown, and therefore logged, from the thread doing the work.  Wait for
+         * all to complete before aborting the startup if any failed.
+         */
+        boolean systemAppsStartedOK = true;
+        for (SystemAppStarter starter : systemAppStarters) {
+            if (isSystemAppLoadLoggingOn) {
+                _logger.fine("About to wait for completion of load request for " + starter.getID());
+            }
+            boolean ok = starter.waitDone() == null; 
+            systemAppsStartedOK &= ok;
+            if (isSystemAppLoadLoggingOn) {
+                _logger.fine("Completion received for " + starter.getID() + " reports " + (ok ? "success" : "failure"));
+            }
+        }
+        
+        if ( ! systemAppsStartedOK) {
+            throw new RuntimeException(
+                    _logger.getResourceBundle().getString("core.unexpected_error_occured_while_loading_app"));
+        } else if (isSystemAppLoadLoggingOn) {
+            _logger.fine(getClass().getName() + " has started its system apps successfully after " + 
+                    (System.currentTimeMillis() - systemAppLoadStartTime) + " ms");
         }
         systemAppsLoaded = true;
     }
@@ -299,7 +373,7 @@ public abstract class AbstractManager implements MonitorListener {
         try {
             // if app is enabled
             ConfigContext ctx = configManager.getConfigContext();
-            if (ResourcesUtil.getInstance().belongToSystemRar(id) ||
+            if (ResourcesUtil.createInstance().belongToSystemRar(id) ||
                     (this.configManager.isSystem(id) 
                      && isEnabled(ctx, id))) {
 
@@ -531,7 +605,6 @@ public abstract class AbstractManager implements MonitorListener {
 
             Server server = ServerBeansFactory.getServerBean(config);
             ApplicationRef appRef = server.getApplicationRefByRef(moduleName);
-
             return ((app != null && app.isEnabled()) && 
                         (appRef != null && appRef.isEnabled()));
         } catch (ConfigException e) {
@@ -550,4 +623,27 @@ public abstract class AbstractManager implements MonitorListener {
         result.addException(event.getEffectiveDestination(), 
                             new AdminEventListenerException(msg));
     }
+    
+    /**
+     *Represents the start-up of a system app, typically run concurrently with
+     *other system-app start up work.
+     */
+    private class SystemAppStarter extends RunnableBase {
+        
+        private final String id;
+        private final boolean jsr77;
+        
+        public SystemAppStarter(String id, boolean jsr77) {
+            this.id = id;
+            this.jsr77 = jsr77;
+        }
+
+        protected void doRun() throws Exception {
+            loadOneSystemApp(id, jsr77);
+        }
+        
+        public String getID() {
+            return id;
+        }
+    }
 }
