@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -37,6 +50,7 @@
 package com.sun.enterprise.server;
 
 import com.sun.enterprise.appclient.jws.AppclientJWSSupportManager;
+import com.sun.enterprise.connectors.util.ResourcesUtil;
 import java.io.File;
 import java.util.List;
 import java.util.ArrayList;
@@ -619,10 +633,8 @@ public class ApplicationManager extends AbstractManager
                 //this event's config context so that it can 
                 //find the application node while loading embedded RAR configs
                 //and connector resources.
-                ResourcesUtil.getInstance().setConfigContext(event.getConfigContext());
-                ResourcesUtil.getInstance().setReinitConfigContext(false);
-                ServerContext sc = ApplicationServer.getServerContext();
-                connecorResourcesLoader =  new ConnectorResourcesLoader(sc);
+                ResourcesUtil.setEventConfigContext(event.getConfigContext());
+                connecorResourcesLoader =  new ConnectorResourcesLoader();
 
                 // do this before loading rar
                 if (event.getLoadUnloadAction() == Constants.LOAD_RAR) {
@@ -651,7 +663,7 @@ public class ApplicationManager extends AbstractManager
         } catch (ConfigException ce) {
             throw new AdminEventListenerException(ce.getMessage()); 
         } finally {
-            ResourcesUtil.getInstance().setReinitConfigContext(true);
+            ResourcesUtil.resetEventConfigContext();
         }
     }
 
@@ -727,7 +739,6 @@ public class ApplicationManager extends AbstractManager
     public synchronized void applicationEnabled(ApplicationDeployEvent event) 
             throws AdminEventListenerException {
 
-        ResourcesUtil resutil = ResourcesUtil.getInstance();
         try {
             if (_logger.isLoggable(Level.FINEST)) {
                _logger.log(Level.FINEST,
@@ -744,11 +755,9 @@ public class ApplicationManager extends AbstractManager
 
             String appName = event.getApplicationName();
             boolean cascade = true;
-            ServerContext sc = ApplicationServer.getServerContext();
             ConnectorResourcesLoader connecorResourcesLoader = 
-                      new ConnectorResourcesLoader(sc); 
-            resutil.setConfigContext(event.getConfigContext());
-            resutil.setReinitConfigContext(false);
+                      new ConnectorResourcesLoader(); 
+            ResourcesUtil.setEventConfigContext(event.getConfigContext());
  
             connecorResourcesLoader.loadEmbeddedRarRAConfigs(appName);
             boolean enabled = applicationDeployed(false, appName, event.getConfigContext());
@@ -777,7 +786,7 @@ public class ApplicationManager extends AbstractManager
             aele.initCause(th);
             throw aele;
         }finally {
-            resutil.setReinitConfigContext(true);
+            ResourcesUtil.resetEventConfigContext();
         }
     }
 
