@@ -1,37 +1,56 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 package com.sun.enterprise.server;
 
 import java.io.File;
+import java.util.List;
 import java.util.ArrayList;
 import java.io.IOException;
 
+import java.lang.reflect.Constructor;
+import java.net.URL;
+import java.net.URLClassLoader;
+
 import com.sun.enterprise.Switch;
 import com.sun.enterprise.server.J2EEServer;
 import com.sun.enterprise.security.audit.AuditModuleEventListenerImpl;
 import com.sun.enterprise.security.audit.SecurityServiceEventListenerImpl;
 import com.sun.enterprise.security.auth.realm.AuthRealmEventListenerImpl;
 import com.sun.enterprise.security.auth.realm.UserMgmtEventListenerImpl;
+import com.sun.enterprise.security.jmac.config.MessageSecurityConfigEventListenerImpl;
 
 import com.sun.enterprise.config.serverbeans.ServerBeansFactory;
 import com.sun.enterprise.config.serverbeans.TransactionService;
@@ -58,6 +77,7 @@ import com.sun.appserv.server.ServerLifecycleException;
 import com.sun.enterprise.server.pluggable.InternalServicesList;
 
 import com.sun.enterprise.admin.event.AdminEventListenerRegistry;
+import com.sun.enterprise.admin.event.MessageSecurityConfigEvent;
 import com.sun.enterprise.admin.event.jms.JmsServiceEvent;
 import com.sun.enterprise.admin.event.jms.JmsHostEvent;
 import com.sun.enterprise.admin.alert.AlertConfigurator;
@@ -72,38 +92,34 @@ import java.util.logging.Logger;
 import com.sun.logging.LogDomains;
 import com.sun.enterprise.server.logging.ServerLogManager;
 
+
+import com.sun.appserv.management.util.misc.TimingDelta;
+import com.sun.appserv.management.util.misc.RunnableBase;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+
 /**
  * ApplicationServer is the main entry point for iAS 7.0 J2EE server instance.
  * This code implements the server lifecycle state transitions and it relies on
  * various iAS 7.0 subsystems such as Web/EJB containers throught its lifecycle.
  * com.sun.enterprise.J2EEServer is the RI counerpart of this code; the evolving
  * iAS server implementation replaces functionality of RI J2EEServer.
+ *
+ * NOTES on services by name:
+ * Default services implementing lifecycle interface for orderly startup
+ * shutdown and hot deployment.
+ * Each service entry is of the form {name, className}.
+ * Note: Make sure LifecycleModuleService is the last subsystem in this list.
  */
 public class ApplicationServer {
-
     /** server logger */
-    static Logger _logger = LogDomains.getLogger(LogDomains.CORE_LOGGER);
-
-    public static java.io.PrintStream ostream = System.out;
-    public static java.io.PrintStream estream = System.err;
-
-    private static final String USER_TX = "java:comp/UserTransaction";
-    private static final String CODEBASE_PROP = "java.rmi.server.codebase";
-
-    /** 
-     * Default services implementing lifecycle interface for orderly startup/
-     * shutdown and hot deployment.
-     * Each service entry is of the form {name, className}.
-     * Note: Make sure LifecycleModuleService is the last subsystem in this list.
-     */
-    static String[][]servicesByName;
-
+    protected static final Logger _logger = LogDomains.getLogger(LogDomains.CORE_LOGGER);
 
     /** ServerContext -- server-wide runtime environment context **/
     private static ServerContext context = null;
 
     /** services -- standard runtime service objects **/
-    private ArrayList services = new ArrayList();
+    private  final List<ServerLifecycle> services = new ArrayList<ServerLifecycle>();
 
     /** common class loader for the server */
     private ClassLoader commonClassLoader;
@@ -129,12 +145,20 @@ public class ApplicationServer {
      * @exception ServerLifecycleException if this subsystem detects a fatal 
      *  error that prevents this subsystem from being used
      */
-    public void onInitialization(ServerContext context) 
+    public void onInitialization( final ServerContext context) 
                 throws ServerLifecycleException
     {
+        
         // sets the server context
         this.context = context;
 
+        // NOTE: side effects of reInitializeServerLoggers invoke various MBean related code
+        // that really ought not to run yet, and that code hits synchronized code being relied
+        // upon (indirectly via a Logger).  The thread initializing the MBeanServer (see PEMain)
+        // will try to acquire that same synchronized lock and then hang.
+        // See CR6555027.
+        com.sun.enterprise.util.FeatureAvailability.getInstance().waitForMBeanServer();
+        
         // initialize all server loggers that were not initialized
         // Note: reInitializeServerLoggers should be called after 
         //       setting the server context and before performing any
@@ -153,12 +177,11 @@ public class ApplicationServer {
         if(Boolean.getBoolean(com.sun.enterprise.server.PELaunch.USE_NEW_CLASSLOADER_PROPERTY))
             parentOfCommonCL = PELaunch.getSharedChain();
 
-        InstanceEnvironment env = this.context.getInstanceEnvironment();
+        final InstanceEnvironment env = this.context.getInstanceEnvironment();
 
         try {
-
-            String dir     =  env.getLibClassesPath();
-            String jarDir  = env.getLibPath();
+            final String dir     =  env.getLibClassesPath();
+            final String jarDir  = env.getLibPath();
 
             // constructs the common class loader. System class loader 
             // is the parent of common class loader. Common class loader 
@@ -169,6 +192,8 @@ public class ApplicationServer {
                                                 new File[] {new File(jarDir)},
                                                 parentOfCommonCL
                                                 );
+            logCommonClassLoaderDetails();
+            
 
             // ignore if null
             if (commonClassLoader == null) {
@@ -190,7 +215,7 @@ public class ApplicationServer {
         
         // sets the class loaders in the server context
         if (this.context instanceof ServerContextImpl) {
-            ServerContextImpl contextImpl = (ServerContextImpl) this.context;
+            final ServerContextImpl contextImpl = (ServerContextImpl) this.context;
 
             // set the common classloader in the server context
             contextImpl.setCommonClassLoader(commonClassLoader);
@@ -216,10 +241,10 @@ public class ApplicationServer {
             );
 
 	//added the pluggable interface way of getting the list of services
-	InternalServicesList servicesList = 
+	final InternalServicesList servicesList = 
 		context.getPluggableFeatureFactory().getInternalServicesList();
 
-	servicesByName = servicesList.getServicesByName();
+	final String[][] servicesByName = servicesList.getServicesByName();
 	if (servicesByName == null) {
 	      _logger.log(Level.SEVERE, "services.null");
 	      throw new ServerLifecycleException();
@@ -228,19 +253,17 @@ public class ApplicationServer {
         // security manager is set inside SecurityLifecycle
 
 	// Instantiate the built-in runtime services
-	services = instantiateRuntimeServices();
+        final List<ServerLifecycle> temp = instantiateRuntimeServices( context, servicesByName );
+        services.addAll( temp );
 	        
         // Initialize the built-in runtime services
-        for (int i = 0; i < services.size(); i++) {
-            Object service = services.get(i);
-            if (service instanceof ServerLifecycle) {
+        for ( final ServerLifecycle service : services) {
 	        try {
-                    ((ServerLifecycle) service).onInitialization(context);
-		} catch (ServerLifecycleException e) {		   
-		    _logger.log(Level.SEVERE, "service.notinit", 
-				new Object[] {service, e.toString()});
-		    throw e;
-		}
+                service.onInitialization(context);
+            } catch (ServerLifecycleException e) {		   
+                _logger.log(Level.SEVERE, "service.notinit", 
+                    new Object[] {service, e.toString()});
+                throw e;
             }
         }
 
@@ -253,14 +276,17 @@ public class ApplicationServer {
                 new SecurityServiceEventListenerImpl());
         AdminEventListenerRegistry.addUserMgmtEventListener(
                 new UserMgmtEventListenerImpl());
+        AdminEventListenerRegistry.addEventListener(
+                MessageSecurityConfigEvent.eventType,
+                new MessageSecurityConfigEventListenerImpl());
 
-        // Call RI J2EEServer initialization code. Note that all resources
-        // specified in server.xml will be loaded by the common class loader.
-	try {
-	    J2EEServer.main(context);
-	} catch (Exception e) {
-	    throw new ServerLifecycleException(e);
-	}
+            // Call RI J2EEServer initialization code. Note that all resources
+            // specified in server.xml will be loaded by the common class loader.
+        try {
+            J2EEServer.main(context);
+        } catch (Exception e) {
+            throw new ServerLifecycleException(e);
+        }
 
         // final connector class loader for the anonymous inner class
         final ClassLoader connCL = connectorClassLoader;
@@ -275,30 +301,123 @@ public class ApplicationServer {
                     }
                 }
         );
-
     }
+    
+    //Log all URLs loaded by the common classloader
+    private void logCommonClassLoaderDetails() {
+    	Logger _log = LogDomains.getLogger(LogDomains.LOADER_LOGGER);
+    	if (_log.isLoggable( Level.FINE ) ) {
+            if (commonClassLoader instanceof URLClassLoader) {
+            	URL[] u = ((URLClassLoader)commonClassLoader).getURLs();
+                StringBuffer sbuf = new StringBuffer();
+                for (int i = 0; i < u.length; i++) {
+                	sbuf.append(u[i].toExternalForm() + " ,");
+				}
+                
+            	_log.log(Level.FINE, "Common classloader contents: [ " + sbuf.toString() + " ].") ;
+            }
+    	}
+	}
 
+        static protected final ServerLifecycle
+    instantiateOneServerLifecycle(
+        final ServerContext serverContext,
+        final String classname )
+        throws ClassNotFoundException, InstantiationException, IllegalAccessException {
+        //final TimingDelta delta = new TimingDelta();
+        
+        final Class c  = Class.forName( classname );
+       
+        ServerLifecycle item = null;
+
+        // call a constructor that takes a ServerContext if it exists, since
+        // some services might make efficient use of it during construction.
+        // otherwise use a no-argument constructor
+        try {
+            final Constructor constructor = c.getConstructor( ServerContext.class );
+            item   = (ServerLifecycle)constructor.newInstance( serverContext );
+        }
+        catch( Exception e ) {
+            item = (ServerLifecycle)c.newInstance();
+        }
+
+        //System.out.println( "Instantiate " + classname + ": " + delta.elapsedMillis() );
+        return item;
+    }
+    
+    /** Index into String[] of the classname for a ServerLifecycle.*/
+    private static final int CLASSNAME_IDX = 1;
+    
+        protected List<ServerLifecycle>
+    instantiateRuntimeServices(
+        final ServerContext serverContext,
+        final String[][]    servicesByName ) throws ServerLifecycleException {
+        final List<String>  classnames  = new ArrayList<String>();
+        
+        for (int i=0; i < servicesByName.length; i++) {
+                final String[] serviceInfo = servicesByName[i];
+                final String classname = serviceInfo[ CLASSNAME_IDX ];
+                classnames.add( classname );
+        }
+        
+        return instantiateRuntimeServices( serverContext, classnames );
+    }
+    
+
+    private static final class Instantiator extends RunnableBase {
+        private final String        mClassname;
+        private final ServerContext mServerContext;
+        private volatile ServerLifecycle    mService;
+        
+        public Instantiator(
+            final ServerContext serverContext,
+            final String        classname ) {
+            mServerContext = serverContext;
+            mClassname     = classname;
+            mService       = null;
+        }
+        
+        protected void doRun()
+            throws ClassNotFoundException, InstantiationException, IllegalAccessException
+        {
+            mService = instantiateOneServerLifecycle(  mServerContext, mClassname );
+        }
+        public ServerLifecycle getService() {
+            waitDoneThrow();
+            return mService;
+        }
+    }
+    
+    /** Changing to RUN_IN_SEPARATE_THREAD achieves some efficiences at
+      startup, but leaving it as 'RUN_IN_CURRENT_THREAD' until time allows
+      testing. */
+    private static final RunnableBase.HowToRun HOW_TO_RUN =
+        RunnableBase.HowToRun.RUN_IN_CURRENT_THREAD;
+    
     /** 
      * Go through the list of built-in runtime services and instantiate them.
      */
-    protected ArrayList instantiateRuntimeServices() throws ServerLifecycleException{
-
-        ArrayList serviceList = new ArrayList();
-
+        protected List<ServerLifecycle>
+    instantiateRuntimeServices(
+        final ServerContext serverContext,
+        final List<String>  classnames )
+        throws ServerLifecycleException {
+        final Instantiator[]    instantiators = new Instantiator[ classnames.size() ];
+        
         // Instantiate service objects 
-        for (int i=0; i < servicesByName.length; i++) {
-            try {
-                String[] def = servicesByName[i];
-                Class c  = Class.forName(def[1]);
-                Object o = c.newInstance();
-
-                serviceList.add(o);
-            } catch (Exception ex) {
-	        _logger.log(Level.SEVERE, "server.exception", ex);
-		throw new ServerLifecycleException(ex.getMessage());
-	    }
-
+        int idx = 0;
+        for ( final String classname : classnames ) {
+            instantiators[idx]    = new Instantiator( serverContext, classname );
+            instantiators[idx].submit( );
+            ++idx;
         }
+        
+        // collect the resulting ServerLifecycle 
+        final List<ServerLifecycle> serviceList = new ArrayList<ServerLifecycle>();
+        for ( int i = 0; i < instantiators.length; ++i ) {
+            serviceList.add( instantiators[i].getService() );
+        }
+
         return serviceList;
     }
 
@@ -318,17 +437,13 @@ public class ApplicationServer {
     public void onStartup() throws ServerLifecycleException
     {
         // startup the built-in runtime services
-        for (int i = 0; i < services.size(); i++) {
-            Object service = services.get(i);
-
-            if (service instanceof ServerLifecycle) {
+        for ( final ServerLifecycle service : services) {
 	        try {
-                    ((ServerLifecycle) service).onStartup(context);
-		} catch (ServerLifecycleException e) {		  
-		    _logger.log(Level.SEVERE, "service.notstarted", 
-				new Object[] {service, e.toString()});
-		    throw e;
-		}
+                service.onStartup(context);
+            } catch (ServerLifecycleException e) {		  
+                _logger.log(Level.SEVERE, "service.notstarted", 
+                    new Object[] {service, e.toString()});
+                throw e;
             }
         }
         // _REVISIT_: Mover AlertConfigurator to configure after onReady()
@@ -406,16 +521,12 @@ public class ApplicationServer {
 
 
         // notify the built-in runtime services that we are ready
-        for (int i = 0; i < services.size(); i++) {
-            Object service = services.get(i);
-
-            if (service instanceof ServerLifecycle) {
+        for ( final ServerLifecycle service : services) {
 	        try {
-                    ((ServerLifecycle) service).onReady(context);
-		} catch (ServerLifecycleException e) {
-		  _logger.log(Level.SEVERE, "service.notready", 
-			      new Object[] {service, e.toString()});
-		}
+                service.onReady(context);
+            } catch (ServerLifecycleException e) {
+              _logger.log(Level.SEVERE, "service.notready", 
+                      new Object[] {service, e.toString()});
             }
         }
     }
@@ -431,19 +542,16 @@ public class ApplicationServer {
 	
         // shutdown the built-in runtime services (in the reverse order).
         for (int i = services.size(); i > 0; i--) {
-            Object service = services.get(i-1);
+            final ServerLifecycle service = services.get(i-1);
 
-	    _logger.log(Level.FINE,"service.shutdown",
-					services.get(i-1));
-	    try {
-                if (service instanceof ServerLifecycle) {
-                    ((ServerLifecycle) service).onShutdown();
-		}
-	    } catch (Exception e) {
-		//do nothing since we want to continue shutting down other LFC
-		//modules
-                _logger.log(Level.WARNING, "server.exception", e);	     
-	    }            
+	    _logger.log(Level.FINE,"service.shutdown", services.get(i-1));
+            try {
+                service.onShutdown();
+            } catch (Exception e) {
+            //do nothing since we want to continue shutting down other LFC
+            //modules
+                    _logger.log(Level.WARNING, "server.exception", e);	     
+            }            
         }
     }
 
@@ -471,14 +579,12 @@ public class ApplicationServer {
 	}
  // terminate the built-in runtime services (in the reverse order)
         for (int i = services.size(); i > 0; i--) {
-            Object service = services.get(i-1);
+            final ServerLifecycle service = services.get(i-1);
 
 	    _logger.log(Level.FINE,"service.shutdown",
 					services.get(i-1));
 	    try {
-                if (service instanceof ServerLifecycle) {
-                    ((ServerLifecycle) service).onTermination();
-                }
+            service.onTermination();
 	    } catch (Exception e) {
 		//do nothing since we want to continue shutting down other LFC
 		//modules
@@ -486,7 +592,7 @@ public class ApplicationServer {
 	    }
         }
 
-        services = null;
+        services.clear();
     }
 
     /**
