@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package com.sun.enterprise.server;
@@ -31,6 +44,7 @@ import com.sun.enterprise.admin.event.ModuleDeployEvent;
 import com.sun.enterprise.admin.event.ModuleDeployEventListener;
 import com.sun.enterprise.config.ConfigContext;
 import com.sun.enterprise.config.ConfigException;
+import com.sun.enterprise.connectors.util.ResourcesUtil;
 import com.sun.enterprise.deployment.backend.DeployableObjectType;
 import com.sun.enterprise.instance.ConnectorModulesManager;
 import com.sun.enterprise.management.StateManageable;
@@ -258,8 +272,7 @@ public class StandAloneConnectorModulesManager extends AbstractManager
             //the connector module is deployed in the backend.
             try {
                 ConnectorResourcesLoader connectorResourcesLoader = null;
-                ServerContext sc = ApplicationServer.getServerContext();
-                connectorResourcesLoader= new ConnectorResourcesLoader(sc);
+                connectorResourcesLoader= new ConnectorResourcesLoader();
                 connectorResourcesLoader.loadRAConfigs(event.getModuleName());
             } catch (ConfigException e) {
                 _logger.log(Level.WARNING, "" + e.getMessage());
@@ -358,7 +371,9 @@ public class StandAloneConnectorModulesManager extends AbstractManager
             jsr77 = true;
         }
 
+        
         try {
+            
             if (event.getModuleType().equals(event.TYPE_CONNECTOR)) {
                 String modID = event.getModuleName();
 
@@ -368,11 +383,10 @@ public class StandAloneConnectorModulesManager extends AbstractManager
 
                 boolean undeployed = false;
                 try {
-                    ResourcesUtil.getInstance().setConfigContext(event.getConfigContext());
-                    ResourcesUtil.getInstance().setReinitConfigContext(false);
+                    ResourcesUtil.setEventConfigContext(event.getConfigContext());
                 undeployed = moduleUndeployed(jsr77, modID, event.getCascade());
                 } finally {
-                    ResourcesUtil.getInstance().setReinitConfigContext(true);
+                    ResourcesUtil.resetEventConfigContext();
                 }
                 
                 if (!undeployed) {
@@ -427,11 +441,10 @@ public class StandAloneConnectorModulesManager extends AbstractManager
                 _logger.log(Level.FINEST, "Module : " + modID + " enabled...");
             }
             String appName = event.getModuleName();
-            ServerContext sc = ApplicationServer.getServerContext();
             ConnectorResourcesLoader connectorResourcesLoader = null;
             try {
                 connectorResourcesLoader=
-                      new ConnectorResourcesLoader(sc); 
+                      new ConnectorResourcesLoader(); 
             }catch(ConfigException ce) {
                 String msg = localStrings.getString(
                       "connector.connector_enable_failed",appName);
@@ -439,11 +452,10 @@ public class StandAloneConnectorModulesManager extends AbstractManager
                       "core.failed_to_get_connectorresourcesloader");
                 throw new AdminEventListenerException(msg);
             }
-            ResourcesUtil resutil = ResourcesUtil.getInstance();
-            resutil.setConfigContext(event.getConfigContext());
-            resutil.setReinitConfigContext(false);
-           
+            
             try {
+                ResourcesUtil.setEventConfigContext(event.getConfigContext());
+
                 connectorResourcesLoader.loadRAConfigs(appName);
                 realDeployed(event);
                 connectorResourcesLoader.load(appName);
@@ -455,7 +467,7 @@ public class StandAloneConnectorModulesManager extends AbstractManager
                 aele.initCause(th);
                 throw aele;
             }finally {
-                resutil.setReinitConfigContext(true);
+                ResourcesUtil.resetEventConfigContext();
             }
         }
     }
