@@ -1,25 +1,38 @@
 
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 package com.sun.enterprise.server.logging;
 
@@ -56,6 +69,7 @@ import com.sun.logging.LogDomains;
  * first to set the log levels correctly. We may miss setting some of the 
  * loglevels, because the values in domain.xml is not available when some
  * loggers are created 
+ * <p><b>NOT THREAD SAFE: mutable instance variables: verboseMode, customFilterError</b>
  */
 public class ServerLogManager extends BaseLogManager {
 
@@ -103,7 +117,14 @@ public class ServerLogManager extends BaseLogManager {
 
     public ServerLogManager() {
         super();
-        thisInstance = this;
+    }
+    
+        public static synchronized ServerLogManager
+    getInstance() {
+        if ( thisInstance == null ) {
+            thisInstance    = new ServerLogManager();
+        }
+        return thisInstance;
     }
 
     /**
@@ -548,8 +569,9 @@ public class ServerLogManager extends BaseLogManager {
                 return;
             }
             Iterator listIterator = listOfUnInitializedLoggers.iterator( );
+            final ServerLogManager mgr = getInstance();
             while( listIterator.hasNext( ) ) {
-                thisInstance.initializeLogger( (Logger) listIterator.next() );
+                mgr.initializeLogger( (Logger) listIterator.next() );
             }
         } catch( Exception e ) {
             new ErrorManager().error( 
