@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -87,6 +100,7 @@ import javax.enterprise.deploy.shared.ModuleType;
  */
 public class WebServiceTesterServlet extends HttpServlet implements MessageListener {
 
+    private WebServiceEndpoint svcEP;
     private static Logger logger = WsUtil.getDefaultLogger();
     private static Hashtable<String, Class> gsiClasses = new Hashtable<String, Class>();
     private static Hashtable<String, Object> ports = new Hashtable<String, Object>();
@@ -98,8 +112,9 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
             HttpServletResponse response, WebServiceEndpoint endpoint) {
                     
         try {
-            WebServiceTesterServlet servlet = new WebServiceTesterServlet();            
+            WebServiceTesterServlet servlet = new WebServiceTesterServlet(endpoint);            
             
+            response.setCharacterEncoding("UTF-8");
             if (request.getMethod().equalsIgnoreCase("GET")) {
                 servlet.doGet(request, response);
             } else {
@@ -128,7 +143,8 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
     
     
     /** Creates a new instance of WebServiceTesterServlet */
-    public WebServiceTesterServlet() {
+    public WebServiceTesterServlet(WebServiceEndpoint ep) {
+        svcEP = ep;
     }
     
     public void doGet(HttpServletRequest req, HttpServletResponse res) 
@@ -139,8 +155,18 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
         PrintWriter out = res.getWriter();
         String requestURL = req.getRequestURL().toString();
 
-        Endpoint myEndpoint = getEndpoint(requestURL);
+        Endpoint myEndpoint;
+        if(svcEP.implementedByWebComponent()) {
+            myEndpoint = WebServiceEngineImpl.getInstance().getEndpoint(req.getServletPath());
+        } else {
+            myEndpoint = WebServiceEngineImpl.getInstance().getEndpoint(req.getRequestURI());
+        }
         String seiClassName = myEndpoint.getDescriptor().getServiceEndpointInterface();
+        ClassLoader testerCL = svcEP.getWebService().getBundleDescriptor().getClassLoader();
+        if (testerCL != null ){
+            Thread.currentThread().setContextClassLoader(testerCL);
+        }
+
         
         // For now support Tester servlet for JAXWS based services only
         try {
@@ -231,7 +257,13 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
         String operationName = req.getParameter("action");
 
         try {
-            Endpoint myEndpoint = getEndpoint(requestURL);
+            Endpoint myEndpoint;
+            if(svcEP.implementedByWebComponent()) {
+                myEndpoint = WebServiceEngineImpl.getInstance().getEndpoint(req.getServletPath());
+            } else {
+                myEndpoint = WebServiceEngineImpl.getInstance().getEndpoint(req.getRequestURI());
+            }
+
 
             Class clientSEI = gsiClasses.get(requestURL);
             if (clientSEI==null) {
@@ -305,9 +337,11 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
                 myEndpoint.removeListener(this);
             } 
         
-        } catch(javax.xml.rpc.ServiceException e) {
-            throw new ServletException(e);
-        } catch(Exception e) {
+        } catch(Throwable e) {
+            out.print(localStrings.getLocalString(
+                       "enterprise.webservice.serviceExceptionError",
+                       "<H2>Service invocation threw an exception with message : {0}; Refer to the server log for more details</H2><BR><HR>", 
+                       new Object[] {e.getMessage()}));
             throw new ServletException(e);
         } 
                
@@ -422,22 +456,6 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
         return convertedValue;
     }
     
-    private Endpoint getEndpoint(String url) {
-        
-        WebServiceEngineImpl wsEngine = WebServiceEngineImpl.getInstance();
-        String requestUriRaw;
-        if (url.indexOf("/Tester")!=-1) {
-            requestUriRaw = url.substring(0, url.indexOf("/Tester"));
-        } else {
-            requestUriRaw = url;
-        }
-        String requestUri = (requestUriRaw.charAt(0) == '/') ?
-               requestUriRaw.substring(1) : requestUriRaw;
-        
-        return wsEngine.getEndpoint(requestUri);
-        
-    }
-    
     private void classNotAccessibleError(ClassLoader cl, String className, PrintWriter out) {
         
         out.print(localStrings.getLocalString(
@@ -481,7 +499,12 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
         throws ServletException, IOException {
 
         String requestURL = req.getRequestURL().toString();
-        Endpoint myEndpoint = getEndpoint(requestURL);
+        Endpoint myEndpoint;
+        if(svcEP.implementedByWebComponent()) {
+            myEndpoint = WebServiceEngineImpl.getInstance().getEndpoint(req.getServletPath());
+        } else {
+            myEndpoint = WebServiceEngineImpl.getInstance().getEndpoint(req.getRequestURI());
+        }
         
         // get our service qname
         QName serviceName =
@@ -592,14 +615,31 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
             return null;            
         }
         File[] classes = getListOfFiles(classesDir).toArray(new File[0]);
+        String resolvedServiceClass = null;
+        String svcClass = null;
         for (File f : classes) {            
-            if (!f.getName().endsWith(serviceClassName+".class"))
-                continue;
-            String svcClass = f.getAbsolutePath().substring(classesDirPath.length()+1);
+            if (f.getName().endsWith(serviceClassName+"_Service.class")){
+                resolvedServiceClass = f.getAbsolutePath().substring(classesDirPath.length()+1);
+                
+            } else {
+                if (f.getName().endsWith(serviceClassName+".class")){
+                    svcClass = f.getAbsolutePath().substring(classesDirPath.length()+1);
+                  
+                }
+            }         
+        }
+        //Incase there is a clash JAXWS resolves the serviceClass
+        //to serviceName_Service.class Use the first if it is present
+        //Fix for issue 3403
+        if ( resolvedServiceClass != null){
+            svcClass = resolvedServiceClass;
+        }    
+        if (svcClass != null) {
             svcClass = svcClass.substring(0, svcClass.indexOf(".class"));
             return svcClass.replaceAll("\\"+File.separator,".");
+        } else {
+            return null;        
         }
-        return null;        
     }
 
     private String getPortClass(Endpoint ep, Class serviceClass)
@@ -615,7 +655,7 @@ public class WebServiceTesterServlet extends HttpServlet implements MessageListe
             String getPortMethodName = "get" + 
                     JAXBRIContext.mangleNameToClassName(webEP.name());
             Method getPortMethod = 
-                    serviceClass.getMethod(getPortMethodName, null);
+                    serviceClass.getMethod(getPortMethodName, (Class[])null);
             return getPortMethod.getReturnType().getName();
         }
         return null;
