@@ -1,28 +1,41 @@
- 
+
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -56,10 +69,14 @@ import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.HttpRequest;
 import org.apache.catalina.HttpResponse;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
+*/
 import org.apache.catalina.LifecycleException;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.LifecycleListener;
+*/
 import org.apache.catalina.Logger;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Pipeline;
@@ -68,12 +85,11 @@ import org.apache.catalina.Request;
 import org.apache.catalina.Response;
 import org.apache.catalina.Session;
 import org.apache.catalina.Valve;
-// START GlassFish 247
-import org.apache.catalina.Wrapper;
-// END GlassFish 247
 import org.apache.catalina.deploy.LoginConfig;
 import org.apache.catalina.deploy.SecurityConstraint;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.util.LifecycleSupport;
+*/
 import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.StringManager;
 //START SJSAS 6202703
@@ -104,13 +120,18 @@ import org.apache.catalina.Auditor; // IASRI 4823322
  * requests.  Requests of any other type will simply be passed through.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 1.7 $ $Date: 2006/03/12 01:27:00 $
+ * @version $Revision: 1.17 $ $Date: 2007/05/05 05:31:52 $
  */
 
 
 public abstract class AuthenticatorBase
-        extends ValveBase
-        implements Authenticator, Lifecycle {
+    extends ValveBase
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
+    implements Authenticator, Lifecycle {
+    */
+    // START CR 6411114
+    implements Authenticator {
+    // END CR 6411114
     private static Log log = LogFactory.getLog(AuthenticatorBase.class);
     
     
@@ -152,12 +173,6 @@ public abstract class AuthenticatorBase
     protected Context context = null;
     
     
-    /**
-     * The debugging detail level for this component.
-     */
-    protected int debug = 0;
-    
-    
     /**
      * Return the MessageDigest implementation to be used when
      * creating session identifiers.
@@ -187,9 +202,11 @@ public abstract class AuthenticatorBase
     /**
      * The lifecycle event support for this component.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     protected LifecycleSupport lifecycle = new LifecycleSupport(this);
+    */
     
-    
+
     /**
      * A random number generator to use when generating session identifiers.
      */
@@ -220,8 +237,10 @@ public abstract class AuthenticatorBase
     /**
      * Has this component been started?
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     protected boolean started = false;
-    
+    */
+
     
     /**
      * "Expires" header always set to Date(1), so generate once only
@@ -312,7 +331,7 @@ public abstract class AuthenticatorBase
         
         super.setContainer(container);
         this.context = (Context) container;
-        
+        this.securePagesWithPragma = context.isSecurePagesWithPragma(); 
     }
     
     
@@ -419,7 +438,7 @@ public abstract class AuthenticatorBase
      * Return the flag that states, if proxy caching is disabled, what headers
      * we add to disable the caching.
      */
-    public boolean getSecurePagesWithPragma() {
+    public boolean isSecurePagesWithPragma() {
         return securePagesWithPragma;
     }
 
@@ -460,12 +479,10 @@ public abstract class AuthenticatorBase
         // END OF IASRI 4665318
                 
         // START GlassFish 247
-        // Select the Wrapper to be used for this Request
-        Wrapper wrapper = request.getWrapper();
-        if (wrapper == null) {
-            try {
+        if (!context.getAvailable()) {
+            try {    
                 ((HttpServletResponse) response.getResponse())
-                    .sendError(HttpServletResponse.SC_NOT_FOUND);
+                    .sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE);
             } catch (IllegalStateException e) {
                 ;
             } catch (IOException e) {
@@ -524,23 +541,6 @@ public abstract class AuthenticatorBase
             }
         }
         
-        // Special handling for form-based logins to deal with the case
-        // where the login form (and therefore the "j_security_check" URI
-        // to which it submits) might be outside the secured area
-        String contextPath = this.context.getPath();
-        String requestURI = hrequest.getDecodedRequestURI();
-        if (requestURI.startsWith(contextPath) &&
-                requestURI.endsWith(Constants.FORM_ACTION)) {
-            if (!authenticate(hrequest, hresponse, config)) {
-                if (log.isDebugEnabled())
-                    log.debug(" Failed authenticate() test ??" + requestURI );
-                // START OF IASRI 4665318
-                // return;
-                return END_PIPELINE;
-                // END OF IASRI 4665318
-            }
-        }
-        
         Realm realm = this.context.getRealm();
         // Is this request URI subject to a security constraint?
         SecurityConstraint [] constraints = realm.
@@ -553,7 +553,7 @@ public abstract class AuthenticatorBase
             // START OF IASRI 4665318
             // context.invokeNext(request, response);
             // return;
-            return INVOKE_NEXT;
+            return processSecurityCheck(hrequest,hresponse,config);
             // END OF IASRI 4665318
         }
         
@@ -616,10 +616,10 @@ public abstract class AuthenticatorBase
         //START SJSAS 6202703
         int preAuthenticateCheckResult = realm.preAuthenticateCheck(
                 hrequest, hresponse, constraints, disableProxyCaching,
-                securePagesWithPragma);
+                securePagesWithPragma, (sso != null));
         
         if(preAuthenticateCheckResult == Realm.AUTHENTICATE_NOT_NEEDED) {
-            return INVOKE_NEXT;
+            return processSecurityCheck(hrequest,hresponse,config);
         } else if(preAuthenticateCheckResult == Realm.AUTHENTICATE_NEEDED) {
             if (log.isDebugEnabled()) {
                 log.debug(" Calling authenticate()");
@@ -695,7 +695,21 @@ public abstract class AuthenticatorBase
         // END OF IASRI 4665318
         
     }
-    
+
+    /**
+     * A post-request processing implementation that does nothing.
+     *
+     * Very few Valves override this behaviour as most Valve logic
+     * is used for request processing.
+     */
+    public void postInvoke(Request request, Response response) 
+	throws IOException, ServletException {
+	    Realm realm = this.context.getRealm();
+	    HttpRequest hrequest = (HttpRequest) request;
+	    HttpResponse hresponse = (HttpResponse) response;
+	    realm.invokePostAuthenticateDelegate
+		(hrequest, hresponse, context);
+    }
     
     // ------------------------------------------------------ Protected Methods
     
@@ -962,6 +976,7 @@ public abstract class AuthenticatorBase
         Cookie cookie = new Cookie(Constants.SINGLE_SIGN_ON_COOKIE, value);
         cookie.setMaxAge(-1);
         cookie.setPath("/");
+        cookie.setSecure(hreq.isSecure());
         hres.addCookie(cookie);
         
         // Register this principal with our SSO valve
@@ -979,7 +994,32 @@ public abstract class AuthenticatorBase
         
     }
     
-    
+    // ------------------------------------------------------ Private Methods
+
+    private int processSecurityCheck(HttpRequest hrequest,
+				     HttpResponse hresponse,
+				     LoginConfig config) 
+	throws IOException {
+
+        // Special handling for form-based logins to deal with the case
+        // where the login form (and therefore the "j_security_check" URI
+        // to which it submits) might be outside the secured area
+        String contextPath = this.context.getPath();
+        String requestURI = hrequest.getDecodedRequestURI();
+        if (requestURI.startsWith(contextPath) &&
+                requestURI.endsWith(Constants.FORM_ACTION)) {
+            if (!authenticate(hrequest, hresponse, config)) {
+                if (log.isDebugEnabled())
+                    log.debug(" Failed authenticate() test ??" + requestURI );
+                // START OF IASRI 4665318
+                // return;
+                return END_PIPELINE;
+                // END OF IASRI 4665318
+            }
+        }
+	return INVOKE_NEXT;
+    }
+        
     // ------------------------------------------------------ Lifecycle Methods
     
     
@@ -988,35 +1028,41 @@ public abstract class AuthenticatorBase
      *
      * @param listener The listener to add
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public void addLifecycleListener(LifecycleListener listener) {
         
         lifecycle.addLifecycleListener(listener);
         
     }
-    
+    */
+
     
     /**
      * Get the lifecycle listeners associated with this lifecycle. If this
      * Lifecycle has no listeners registered, a zero-length array is returned.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public LifecycleListener[] findLifecycleListeners() {
         
         return lifecycle.findLifecycleListeners();
         
     }
-    
+    */
+
     
     /**
      * Remove a lifecycle event listener from this component.
      *
      * @param listener The listener to remove
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public void removeLifecycleListener(LifecycleListener listener) {
         
         lifecycle.removeLifecycleListener(listener);
         
     }
-    
+    */
+
     
     /**
      * Prepare for the beginning of active use of the public methods of this
@@ -1029,10 +1075,17 @@ public abstract class AuthenticatorBase
     public void start() throws LifecycleException {
         
         // Validate and update our current component state
+        /** CR 6411114 (Lifecycle implementation moved to ValveBase)
         if (started)
             throw new LifecycleException
                     (sm.getString("authenticator.alreadyStarted"));
         lifecycle.fireLifecycleEvent(START_EVENT, null);
+        */
+        // START CR 6411114
+        if (started)            // Ignore multiple starts
+            return;
+        super.start();
+        // END CR 6411114
         if ("org.apache.catalina.core.StandardContext".equals
                 (context.getClass().getName())) {
             try {
@@ -1047,8 +1100,10 @@ public abstract class AuthenticatorBase
                 log.error("Exception getting debug value", e);
             }
         }
+        /** CR 6411114 (Lifecycle implementation moved to ValveBase)
         started = true;
-        
+        */
+
         // Look up the SingleSignOn implementation in our request processing
         // path, if there is one
         Container parent = context.getParent();
@@ -1088,13 +1143,22 @@ public abstract class AuthenticatorBase
     public void stop() throws LifecycleException {
         
         // Validate and update our current component state
+        /** CR 6411114 (Lifecycle implementation moved to ValveBase)
         if (!started)
             throw new LifecycleException
-                    (sm.getString("authenticator.notStarted"));
+                (sm.getString("authenticator.notStarted"));
         lifecycle.fireLifecycleEvent(STOP_EVENT, null);
         started = false;
-        
+        */
+        // START CR 6411114
+        if (!started)       // Ignore stop if not started
+            return;
+        // END CR 6411114
+
         sso = null;
+        // START CR 6411114
+        super.stop();
+        // END CR 6411114
         
     }
     // BEGIN S1AS8 PE 4856062,4918627
