@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -39,10 +52,14 @@ import javax.servlet.http.HttpServletResponse;
 import org.apache.catalina.Container;
 import org.apache.catalina.HttpRequest;
 import org.apache.catalina.HttpResponse;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
+*/
 import org.apache.catalina.LifecycleException;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.LifecycleListener;
+*/
 import org.apache.catalina.Logger;
 import org.apache.catalina.Request;
 import org.apache.catalina.Response;
@@ -50,7 +67,9 @@ import org.apache.catalina.Session;
 import org.apache.catalina.SessionEvent;
 import org.apache.catalina.SessionListener;
 import org.apache.catalina.valves.ValveBase;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.util.LifecycleSupport;
+*/
 import org.apache.catalina.util.StringManager;
 
 
@@ -72,12 +91,17 @@ import org.apache.catalina.util.StringManager;
  * </ul>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 1.3 $ $Date: 2006/03/07 22:30:06 $
+ * @version $Revision: 1.8 $ $Date: 2007/06/28 21:47:38 $
  */
 
 public class SingleSignOn
     extends ValveBase
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     implements Lifecycle, SessionListener {
+    */
+    // START CR 6411114
+    implements SessionListener {
+    // END CR 6411114
 
 
     // ----------------------------------------------------- Instance Variables
@@ -90,23 +114,19 @@ public class SingleSignOn
     protected HashMap cache = new HashMap();
 
 
-    /**
-     * The debugging detail level for this component.
-     */
-    protected int debug = 0;
-
-
     /**
      * Descriptive information about this Valve implementation.
      */
-    protected static String info =
+    protected static final String info =
         "org.apache.catalina.authenticator.SingleSignOn";
 
 
     /**
      * The lifecycle event support for this component.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     protected LifecycleSupport lifecycle = new LifecycleSupport(this);
+    */
 
 
     /**
@@ -126,7 +146,9 @@ public class SingleSignOn
     /**
      * Component started flag.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     protected boolean started = false;
+    */
 
 
     // ------------------------------------------------------------- Properties
@@ -162,22 +184,26 @@ public class SingleSignOn
      *
      * @param listener The listener to add
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public void addLifecycleListener(LifecycleListener listener) {
 
         lifecycle.addLifecycleListener(listener);
 
     }
+    */
 
 
     /**
      * Get the lifecycle listeners associated with this lifecycle. If this 
      * Lifecycle has no listeners registered, a zero-length array is returned.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public LifecycleListener[] findLifecycleListeners() {
 
         return lifecycle.findLifecycleListeners();
 
     }
+    */
 
 
     /**
@@ -185,11 +211,13 @@ public class SingleSignOn
      *
      * @param listener The listener to remove
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public void removeLifecycleListener(LifecycleListener listener) {
 
         lifecycle.removeLifecycleListener(listener);
 
     }
+    */
 
 
     /**
@@ -203,11 +231,18 @@ public class SingleSignOn
     public void start() throws LifecycleException {
 
         // Validate and update our current component state
+        /** CR 6411114 (Lifecycle implementation moved to ValveBase)
         if (started)
             throw new LifecycleException
                 (sm.getString("authenticator.alreadyStarted"));
         lifecycle.fireLifecycleEvent(START_EVENT, null);
         started = true;
+        */
+        // START CR 6411114
+        if (started)            // Ignore multiple starts
+            return;
+        super.start();
+        // END CR 6411114
 
         if (debug >= 1)
             log("Started");
@@ -226,14 +261,23 @@ public class SingleSignOn
     public void stop() throws LifecycleException {
 
         // Validate and update our current component state
+        /** CR 6411114 (Lifecycle implementation moved to ValveBase)
         if (!started)
             throw new LifecycleException
                 (sm.getString("authenticator.notStarted"));
         lifecycle.fireLifecycleEvent(STOP_EVENT, null);
         started = false;
+        */
+        // START CR 6411114
+        if (!started)       // Ignore stop if not started
+            return;
+        // END CR 6411114
 
         if (debug >= 1)
             log("Stopped");
+        // START CR 6411114
+        super.stop();
+        // END CR 6411114
 
     }
 
@@ -413,7 +457,11 @@ public class SingleSignOn
      * @param ssoId Single sign on identifier
      * @param session Session to be associated
      */
-    protected void associate(String ssoId, Session session) {
+    public void associate(String ssoId, Session session) {
+
+        if (!started) {
+            return;
+        }
 
         if (debug >= 1)
             log("Associate sso id " + ssoId + " with session " + session);
