@@ -1,26 +1,39 @@
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -50,6 +63,7 @@ import javax.servlet.ServletContextAttributeEvent;
 import javax.servlet.ServletContextAttributeListener;
 
 import org.apache.catalina.Globals;
+import org.apache.catalina.ContainerEvent;
 import org.apache.catalina.Context;
 import org.apache.catalina.Host;
 import org.apache.catalina.Logger;
@@ -73,7 +87,7 @@ import org.apache.tomcat.util.http.mapper.MappingData;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.8 $ $Date: 2006/03/25 02:43:23 $
+ * @version $Revision: 1.15 $ $Date: 2007/07/03 18:05:00 $
  */
 
 public class ApplicationContext
@@ -89,15 +103,25 @@ public class ApplicationContext
      * @param context The associated Context instance
      */
     public ApplicationContext(String basePath, StandardContext context) {
+        this(basePath, null, context);
+    }
+
+    public ApplicationContext(
+                String basePath,
+                ArrayList<AlternateDocBase> alternateDocBases,
+                StandardContext context) {
         super();
         this.context = context;
         this.basePath = basePath;
+        this.alternateDocBases = alternateDocBases;
 
         //START PWC 6403328
         this.logPrefix = sm.getString("applicationContext.logPrefix",
                                       context.logName());
         //END PWC 6403328
 
+        setAttribute("com.sun.faces.useMyFaces",
+                     Boolean.valueOf(context.isUseMyFaces()));
     }
 
 
@@ -161,6 +185,13 @@ public class ApplicationContext
      */
     private String basePath = null;
 
+
+    /**
+     * Alternate doc bases
+     */
+    private ArrayList<AlternateDocBase> alternateDocBases = null;
+
+
     // START PWC 6403328
     /**
      * Log prefix string
@@ -169,15 +200,10 @@ public class ApplicationContext
     // END PWC 6403328
 
     /**
-     * Thread local mapping data.
+     * Thread local data used during request dispatch.
      */
-    private ThreadLocal localMappingData = new ThreadLocal();
-
-
-    /**
-     * Thread local URI message bytes.
-     */
-    private ThreadLocal localUriMB = new ThreadLocal();
+    private ThreadLocal<DispatchData> dispatchData =
+        new ThreadLocal<DispatchData>();
 
 
     // --------------------------------------------------------- Public Methods
@@ -263,25 +289,13 @@ public class ApplicationContext
     public ServletContext getContext(String uri) {
 
         // Validate the format of the specified argument
-        if ((uri == null) || (!uri.startsWith("/")))
+        if ((uri == null) || (!uri.startsWith("/"))) {
             return (null);
-
-        // Return the current context if requested
-        String contextPath = context.getPath();
-        if (!contextPath.endsWith("/"))
-            contextPath = contextPath + "/";
-
-        if (((contextPath.length() > 1) && (uri.startsWith(contextPath))) ||
-            ((contextPath.equals("/")) && (uri.equals("/")))) {
-            return (this);
         }
 
-        // Return other contexts only if allowed
-        if (!context.getCrossContext())
-            return (null);
+        Context child = null;
         try {
             Host host = (Host) context.getParent();
-            Context child = null;
             String mapuri = uri;
             while (true) {
                 child = (Context) host.findChild(mapuri);
@@ -292,17 +306,24 @@ public class ApplicationContext
                     break;
                 mapuri = mapuri.substring(0, slash);
             }
-            if (child == null) {
-                child = (Context) host.findChild("");
-            }
-            if (child != null)
-                return (child.getServletContext());
-            else
-                return (null);
         } catch (Throwable t) {
             return (null);
         }
 
+        if (child == null) {
+            return (null);
+        }
+
+        if (context.getCrossContext()) {
+            // If crossContext is enabled, can always return the context
+            return child.getServletContext();
+        } else if (child == context) {
+            // Can still return the current context
+            return context.getServletContext();
+        } else {
+            // Nothing to return
+            return (null);
+        }
     }
 
 
@@ -412,7 +433,21 @@ public class ApplicationContext
             return null;
         }
 
-        File file = new File(basePath, path);
+        File file = null;
+        if (alternateDocBases == null
+                || alternateDocBases.size() == 0) {
+            file = new File(basePath, path);
+        } else {
+            AlternateDocBase match = AlternateDocBase.findMatch(
+                                                path, alternateDocBases);
+            if (match != null) {
+                file = new File(match.getBasePath(), path);
+            } else {
+                // None of the url patterns for alternate doc bases matched
+                file = new File(basePath, path);
+            }
+        }
+
         return (file.getAbsolutePath());
 
     }
@@ -438,17 +473,16 @@ public class ApplicationContext
         if (path == null)
             return (null);
 
-        // Retrieve the thread local URI
-        MessageBytes uriMB = (MessageBytes) localUriMB.get();
-        if (uriMB == null) {
-            uriMB = new MessageBytes();
-            CharChunk uriCC = uriMB.getCharChunk();
-            uriCC.setLimit(-1);
-            localUriMB.set(uriMB);
-        } else {
-            uriMB.recycle();
+        // Use the thread local URI and mapping data
+        DispatchData dd = dispatchData.get();
+        if (dd == null) {
+            dd = new DispatchData();
+            dispatchData.set(dd);
         }
 
+        MessageBytes uriMB = dd.uriMB;
+        uriMB.recycle();
+
         // Get query string
         String queryString = null;
         int pos = path.indexOf('?');
@@ -459,11 +493,7 @@ public class ApplicationContext
         }
  
         // Retrieve the thread local mapping data
-        MappingData mappingData = (MappingData) localMappingData.get();
-        if (mappingData == null) {
-            mappingData = new MappingData();
-            localMappingData.set(mappingData);
-        }
+        MappingData mappingData = dd.mappingData;
 
         // Map the URI
         CharChunk uriCC = uriMB.getCharChunk();
@@ -545,9 +575,25 @@ public class ApplicationContext
             } else {
                 return null;
             }
+
         } else {
 
-            DirContext resources = context.getResources();
+            DirContext resources = null;
+
+            if (alternateDocBases == null
+                    || alternateDocBases.size() == 0) {
+                resources = context.getResources();
+            } else {
+                AlternateDocBase match = AlternateDocBase.findMatch(
+                                                path, alternateDocBases);
+                if (match != null) {
+                    resources = match.getResources();
+                } else {
+                    // None of the url patterns for alternate doc bases matched
+                    resources = context.getResources();
+                }
+            }
+
             if (resources != null) {
                 String fullPath = context.getName() + path;
                 String hostName = context.getParent().getName();
@@ -586,7 +632,22 @@ public class ApplicationContext
         if (path == null)
             return (null);
 
-        DirContext resources = context.getResources();
+        DirContext resources = null;
+
+        if (alternateDocBases == null
+                || alternateDocBases.size() == 0) {
+            resources = context.getResources();
+        } else {
+            AlternateDocBase match = AlternateDocBase.findMatch(
+                                path, alternateDocBases);
+            if (match != null) {
+                resources = match.getResources();
+            } else {
+                // None of the url patterns for alternate doc bases matched
+                resources = context.getResources();
+            }
+        }
+
         if (resources != null) {
             try {
                 Object resource = resources.lookup(path);
@@ -622,10 +683,26 @@ public class ApplicationContext
         if (path == null)
             return (null);
 
-        DirContext resources = context.getResources();
+        DirContext resources = null;
+
+        if (alternateDocBases == null
+                || alternateDocBases.size() == 0) {
+            resources = context.getResources();
+        } else {
+            AlternateDocBase match = AlternateDocBase.findMatch(
+                                path, alternateDocBases);
+            if (match != null) {
+                resources = match.getResources();
+            } else {
+                // None of the url patterns for alternate doc bases matched
+                resources = context.getResources();
+            }
+        }
+
         if (resources != null) {
             return (getResourcePathsInternal(resources, path));
         }
+
         return (null);
 
     }
@@ -786,14 +863,17 @@ public class ApplicationContext
             ServletContextAttributeListener listener =
                 (ServletContextAttributeListener) listeners[i];
             try {
-                context.fireContainerEvent("beforeContextAttributeRemoved",
-                                           listener);
+                context.fireContainerEvent(
+                    ContainerEvent.BEFORE_CONTEXT_ATTRIBUTE_REMOVED,
+                    listener);
                 listener.attributeRemoved(event);
-                context.fireContainerEvent("afterContextAttributeRemoved",
-                                           listener);
+                context.fireContainerEvent(
+                    ContainerEvent.AFTER_CONTEXT_ATTRIBUTE_REMOVED,
+                    listener);
             } catch (Throwable t) {
-                context.fireContainerEvent("afterContextAttributeRemoved",
-                                           listener);
+                context.fireContainerEvent(
+                    ContainerEvent.AFTER_CONTEXT_ATTRIBUTE_REMOVED,
+                    listener);
                 // FIXME - should we do anything besides log these?
                 log(sm.getString("applicationContext.attributeEvent"), t);
             }
@@ -861,25 +941,32 @@ public class ApplicationContext
                 (ServletContextAttributeListener) listeners[i];
             try {
                 if (replaced) {
-                    context.fireContainerEvent
-                        ("beforeContextAttributeReplaced", listener);
+                    context.fireContainerEvent(
+                        ContainerEvent.BEFORE_CONTEXT_ATTRIBUTE_REPLACED,
+                        listener);
                     listener.attributeReplaced(event);
-                    context.fireContainerEvent("afterContextAttributeReplaced",
-                                               listener);
+                    context.fireContainerEvent(
+                        ContainerEvent.AFTER_CONTEXT_ATTRIBUTE_REPLACED,
+                        listener);
                 } else {
-                    context.fireContainerEvent("beforeContextAttributeAdded",
-                                               listener);
+                    context.fireContainerEvent(
+                        ContainerEvent.BEFORE_CONTEXT_ATTRIBUTE_ADDED,
+                        listener);
                     listener.attributeAdded(event);
-                    context.fireContainerEvent("afterContextAttributeAdded",
-                                               listener);
+                    context.fireContainerEvent(
+                        ContainerEvent.AFTER_CONTEXT_ATTRIBUTE_ADDED,
+                        listener);
                 }
             } catch (Throwable t) {
-                if (replaced)
-                    context.fireContainerEvent("afterContextAttributeReplaced",
-                                               listener);
-                else
-                    context.fireContainerEvent("afterContextAttributeAdded",
-                                               listener);
+                if (replaced) {
+                    context.fireContainerEvent(
+                        ContainerEvent.AFTER_CONTEXT_ATTRIBUTE_REPLACED,
+                        listener);
+                } else {
+                    context.fireContainerEvent(
+                        ContainerEvent.AFTER_CONTEXT_ATTRIBUTE_ADDED,
+                        listener);
+                }
                 // FIXME - should we do anything besides log these?
                 log(sm.getString("applicationContext.attributeEvent"), t);
             }
@@ -1088,4 +1175,20 @@ public class ApplicationContext
     }
 
 
+    /**
+     * Internal class used as thread-local storage when doing path
+     * mapping during dispatch.
+     */
+    private final class DispatchData {
+
+        public MessageBytes uriMB;
+        public MappingData mappingData;
+
+        public DispatchData() {
+            uriMB = MessageBytes.newInstance();
+            CharChunk uriCC = uriMB.getCharChunk();
+            uriCC.setLimit(-1);
+            mappingData = new MappingData();
+        }
+    }
 }
