@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -46,10 +59,13 @@ import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
 import org.apache.catalina.Session;
 import org.apache.catalina.Manager;
+import org.apache.catalina.session.StandardSession;
 import org.apache.catalina.util.Enumerator;
 import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.StringManager;
-
+// START GlassFish 896
+import org.apache.coyote.tomcat5.SessionTracker;
+// END GlassFish 896
 
 /**
  * Wrapper around a <code>javax.servlet.http.HttpServletRequest</code>
@@ -65,7 +81,7 @@ import org.apache.catalina.util.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.5 $ $Date: 2006/01/27 19:35:20 $
+ * @version $Revision: 1.16 $ $Date: 2007/05/05 05:31:54 $
  */
 
 public class ApplicationHttpRequest extends HttpServletRequestWrapper {
@@ -74,17 +90,6 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
     // ------------------------------------------------------- Static Variables
 
 
-    /**
-     * The set of attribute names that are special for request dispatchers.
-     */
-    protected static final String specials[] =
-    { Globals.INCLUDE_REQUEST_URI_ATTR, Globals.INCLUDE_CONTEXT_PATH_ATTR,
-      Globals.INCLUDE_SERVLET_PATH_ATTR, Globals.INCLUDE_PATH_INFO_ATTR,
-      Globals.INCLUDE_QUERY_STRING_ATTR, Globals.FORWARD_REQUEST_URI_ATTR, 
-      Globals.FORWARD_CONTEXT_PATH_ATTR, Globals.FORWARD_SERVLET_PATH_ATTR, 
-      Globals.FORWARD_PATH_INFO_ATTR, Globals.FORWARD_QUERY_STRING_ATTR };
-
-
     /**
      * The string manager for this package.
      */
@@ -108,12 +113,29 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
         this.crossContext = crossContext;
         setRequest(request);
 
+        if (context.getManager() != null) {
+            isSessionVersioningSupported =
+                context.getManager().isSessionVersioningSupported();
+            if (isSessionVersioningSupported) {
+                HashMap<String, String> sessionVersions =
+                    (HashMap<String, String>) getAttribute(
+                        Globals.SESSION_VERSIONS_REQUEST_ATTRIBUTE);
+                if (sessionVersions != null) {
+                    requestedSessionVersion = sessionVersions.get(
+                        context.getPath());
+                }
+            }
+        }
     }
 
 
     // ----------------------------------------------------- Instance Variables
 
 
+    private String requestedSessionVersion = null;
+    private boolean isSessionVersioningSupported = false;
+
+
     /**
      * The context for this request.
      */
@@ -204,7 +226,14 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
     /**
      * Special attributes.
      */
-    protected Object[] specialAttributes = new Object[specials.length];
+    private HashMap specialAttributes = null;
+
+
+    /**
+     * true if this wrapper has been created for an include dispatch, false
+     * otherwise
+     */
+    private boolean isIncludeDispatch = false;
 
 
     // ------------------------------------------------- ServletRequest Methods
@@ -227,18 +256,21 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
             }
         }
 
-        int pos = getSpecial(name);
-        if (pos == -1) {
+        if (!ApplicationRequest.isSpecial(name)) {
             return getRequest().getAttribute(name);
         } else {
-            if ((specialAttributes[pos] == null) 
-                && (specialAttributes[5] == null) && (pos >= 5)) {
+            Object value = null;
+            if (specialAttributes != null) {
+                value = specialAttributes.get(name);
+            }
+            if (value == null && specialAttributes != null
+                    && name.startsWith("javax.servlet.forward")) {
                 // If it's a forward special attribute, and null, it means this
                 // is an include, so we check the wrapped request since 
                 // the request could have been forwarded before the include
                 return getRequest().getAttribute(name);
             } else {
-                return specialAttributes[pos];
+                return value;
             }
         }
 
@@ -253,7 +285,6 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
         return (new AttributeNamesEnumerator());
     }
 
-
     /**
      * Override the <code>removeAttribute()</code> method of the
      * wrapped request.
@@ -262,9 +293,13 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     public void removeAttribute(String name) {
 
-        if (!removeSpecial(name))
+        if (ApplicationRequest.isSpecial(name)) {
+            if (specialAttributes != null) {
+                specialAttributes.remove(name);
+            }
+        } else {
             getRequest().removeAttribute(name);
-
+        }
     }
 
 
@@ -285,10 +320,13 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
             return;
         }
 
-        if (!setSpecial(name, value)) {
+        if (ApplicationRequest.isSpecial(name)) {
+            if (specialAttributes != null) {
+                specialAttributes.put(name, value);
+            }
+        } else {
             getRequest().setAttribute(name, value);
         }
-
     }
 
 
@@ -538,16 +576,32 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
             if (other != null) {
                 Session localSession = null;
                 try {
-                    localSession =
-                        context.getManager().findSession(other.getId());
+                    if (isSessionVersioningSupported) {
+                        localSession =
+                            context.getManager().findSession(
+                                other.getId(),
+                                requestedSessionVersion);
+                        incrementSessionVersion((StandardSession) localSession,
+                                                context);
+                    } else {
+                        localSession =
+                            context.getManager().findSession(other.getId());
+                    }
                 } catch (IOException e) {
                     // Ignore
                 }
-                if (localSession == null && create) {
+
+                if ((localSession != null) && !localSession.isValid()) {
+                    localSession = null;
+                } else if (localSession == null && create) {
                     //START OF 6364900
                     localSession = 
                         context.getManager().createSession(other.getId());
-                    //END OF 6364900                    
+                    if (isSessionVersioningSupported) {
+                        incrementSessionVersion((StandardSession) localSession,
+                                                context);
+                    }
+                    //END OF 6364900
                     /* CR 6364900
                     localSession = context.getManager().createEmptySession();
                     localSession.setNew(true);
@@ -556,7 +610,12 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
                     localSession.setMaxInactiveInterval
                         (context.getManager().getMaxInactiveIntervalSeconds());
                     localSession.setId(other.getId());
-                     */
+                    */
+                    // START GlassFish 896
+                    SessionTracker tracker = (SessionTracker) getAttribute(
+                        Globals.SESSION_TRACKER);
+                    tracker.track(localSession);
+                    // END GlassFish 896
                 }
                 if (localSession != null) {
                     localSession.access();
@@ -589,16 +648,27 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
                 return (false);
             if (context == null)
                 return (false);
+
+            if (session != null
+                    && requestedSessionId.equals(session.getIdInternal())) {
+                return session.isValid();
+            }
+
             Manager manager = context.getManager();
             if (manager == null)
                 return (false);
-            Session session = null;
+            Session localSession = null;
             try {
-                session = manager.findSession(requestedSessionId);
+                if (isSessionVersioningSupported) {
+                    localSession = manager.findSession(requestedSessionId,
+                                                       requestedSessionVersion);
+                } else {
+                    localSession = manager.findSession(requestedSessionId);
+                }
             } catch (IOException e) {
-                session = null;
+                localSession = null;
             }
-            if ((session != null) && session.isValid()) {
+            if ((localSession != null) && localSession.isValid()) {
                 return (true);
             } else {
                 return (false);
@@ -775,67 +845,50 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
 
 
     /**
-     * Is this attribute name one of the special ones that is added only for
-     * included servlets?
+     * Initializes the special attributes of this request wrapper.
      *
-     * @param name Attribute name to be tested
+     * @param isIncludeDispatch true if the given attributes are for an
+     * include dispatch, false if they are for a forward dispatch
+     * @param requestUri The request URI
+     * @param contextPath The context path
+     * @param servletPath The servlet path
+     * @param pathInfo The path info
+     * @param queryString The query string
      */
-    protected boolean isSpecial(String name) {
+    void initSpecialAttributes(boolean isIncludeDispatch,
+                               String requestUri,
+                               String contextPath,
+                               String servletPath,
+                               String pathInfo,
+                               String queryString) {
 
-        for (int i = 0; i < specials.length; i++) {
-            if (specials[i].equals(name))
-                return (true);
+        this.isIncludeDispatch = isIncludeDispatch;
+
+        specialAttributes = new HashMap(5);
+
+        if (isIncludeDispatch) {        
+            specialAttributes.put(Globals.INCLUDE_REQUEST_URI_ATTR,
+                                  requestUri);
+            specialAttributes.put(Globals.INCLUDE_CONTEXT_PATH_ATTR,
+                                  contextPath);
+            specialAttributes.put(Globals.INCLUDE_SERVLET_PATH_ATTR,
+                                  servletPath);
+            specialAttributes.put(Globals.INCLUDE_PATH_INFO_ATTR,
+                                  pathInfo);
+            specialAttributes.put(Globals.INCLUDE_QUERY_STRING_ATTR,
+                                  queryString);
+        } else {
+            specialAttributes.put(Globals.FORWARD_REQUEST_URI_ATTR,
+                                  requestUri);
+            specialAttributes.put(Globals.FORWARD_CONTEXT_PATH_ATTR,
+                                  contextPath);
+            specialAttributes.put(Globals.FORWARD_SERVLET_PATH_ATTR,
+                                  servletPath);
+            specialAttributes.put(Globals.FORWARD_PATH_INFO_ATTR,
+                                  pathInfo);
+            specialAttributes.put(Globals.FORWARD_QUERY_STRING_ATTR,
+                                  queryString);
         }
-        return (false);
-
-    }
-
-
-    /**
-     * Get a special attribute.
-     *
-     * @return the special attribute pos, or -1 if it is not a special 
-     *         attribute
-     */
-    protected int getSpecial(String name) {
-        for (int i = 0; i < specials.length; i++) {
-            if (specials[i].equals(name)) {
-                return (i);
-            }
-        }
-        return (-1);
-    }
-
-
-    /**
-     * Set a special attribute.
-     * 
-     * @return true if the attribute was a special attribute, false otherwise
-     */
-    protected boolean setSpecial(String name, Object value) {
-        for (int i = 0; i < specials.length; i++) {
-            if (specials[i].equals(name)) {
-                specialAttributes[i] = value;
-                return (true);
-            }
-        }
-        return (false);
-    }
-
-
-    /**
-     * Remove a special attribute.
-     * 
-     * @return true if the attribute was a special attribute, false otherwise
-     */
-    protected boolean removeSpecial(String name) {
-        for (int i = 0; i < specials.length; i++) {
-            if (specials[i].equals(name)) {
-                specialAttributes[i] = null;
-                return (true);
-            }
-        }
-        return (false);
     }
 
 
@@ -928,34 +981,29 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
      */
     protected class AttributeNamesEnumerator implements Enumeration {
 
-        protected int pos = -1;
-        protected int last = -1;
         protected Enumeration parentEnumeration = null;
         protected String next = null;
+        private Iterator<String> specialNames = null;
 
         public AttributeNamesEnumerator() {
             parentEnumeration = getRequest().getAttributeNames();
-            for (int i = 0; i < specialAttributes.length; i++) {
-                if (getAttribute(specials[i]) != null) {
-                    last = i;
-                }
+            if (specialAttributes != null) {
+                specialNames = specialAttributes.keySet().iterator();
             }
         }
 
         public boolean hasMoreElements() {
-            return ((pos != last) || (next != null) 
-                    || ((next = findNext()) != null));
+            return (specialNames != null && specialNames.hasNext())
+                    || (next != null) 
+                    || ((next = findNext()) != null);
         }
 
         public Object nextElement() {
-            if (pos != last) {
-                for (int i = pos + 1; i <= last; i++) {
-                    if (getAttribute(specials[i]) != null) {
-                        pos = i;
-                        return (specials[i]);
-                    }
-                }
+
+            if (specialNames != null && specialNames.hasNext()) {
+                return specialNames.next();
             }
+
             String result = next;
             if (next != null) {
                 next = findNext();
@@ -969,14 +1017,43 @@ public class ApplicationHttpRequest extends HttpServletRequestWrapper {
             String result = null;
             while ((result == null) && (parentEnumeration.hasMoreElements())) {
                 String current = (String) parentEnumeration.nextElement();
-                if (!isSpecial(current)) {
+                if (!ApplicationRequest.isSpecial(current)
+                        || (isIncludeDispatch
+                            && current.startsWith("javax.servlet.forward")
+                            && getAttribute(current) != null)) {
                     result = current;
                 }
             }
             return result;
-       }
+        }
+    }
 
-   }
+    /**
+     * Increments the version of the given session, and stores it as a
+     * request attribute, so it can later be included in a response cookie.
+     */
+    private void incrementSessionVersion(StandardSession ss,
+                                         Context context) {
 
+        if (ss == null || context == null) {
+            return;
+        }
+
+        ss.incrementVersion();
+        String versionString = Long.toString(ss.getVersion());
+
+        HashMap<String, String> sessionVersions = (HashMap<String, String>)
+            getAttribute(Globals.SESSION_VERSIONS_REQUEST_ATTRIBUTE);
+        if (sessionVersions == null) {
+            sessionVersions = new HashMap<String, String>();
+            setAttribute(Globals.SESSION_VERSIONS_REQUEST_ATTRIBUTE,
+                         sessionVersions);
+        }
+        String path = context.getPath();
+        if ("".equals(path)) {
+            path = "/";
+        }
+        sessionVersions.put(path, versionString);
+    }
 
 }
