@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -73,7 +86,7 @@ import com.sun.org.apache.commons.logging.LogFactory;
  * with each context and server.
  *
  * @author Remy Maucherat
- * @version $Revision: 1.5 $ $Date: 2006/03/12 01:27:00 $
+ * @version $Revision: 1.8 $ $Date: 2007/05/05 05:31:54 $
  */
 
 public class NamingContextListener
@@ -764,25 +777,25 @@ public class NamingContextListener
                 value = env.getValue();
             } else if (type.equals("java.lang.Byte")) {
                 if (env.getValue() == null) {
-                    value = new Byte((byte) 0);
+                    value = Byte.valueOf((byte) 0);
                 } else {
                     value = Byte.decode(env.getValue());
                 }
             } else if (type.equals("java.lang.Short")) {
                 if (env.getValue() == null) {
-                    value = new Short((short) 0);
+                    value = Short.valueOf((short) 0);
                 } else {
                     value = Short.decode(env.getValue());
                 }
             } else if (type.equals("java.lang.Integer")) {
                 if (env.getValue() == null) {
-                    value = new Integer(0);
+                    value = Integer.valueOf(0);
                 } else {
                     value = Integer.decode(env.getValue());
                 }
             } else if (type.equals("java.lang.Long")) {
                 if (env.getValue() == null) {
-                    value = new Long(0);
+                    value = Long.valueOf(0);
                 } else {
                     value = Long.decode(env.getValue());
                 }
@@ -790,22 +803,22 @@ public class NamingContextListener
                 value = Boolean.valueOf(env.getValue());
             } else if (type.equals("java.lang.Double")) {
                 if (env.getValue() == null) {
-                    value = new Double(0);
+                    value = Double.valueOf(0);
                 } else {
                     value = Double.valueOf(env.getValue());
                 }
             } else if (type.equals("java.lang.Float")) {
                 if (env.getValue() == null) {
-                    value = new Float(0);
+                    value = Float.valueOf(0);
                 } else {
                     value = Float.valueOf(env.getValue());
                 }
             } else if (type.equals("java.lang.Character")) {
                 if (env.getValue() == null) {
-                    value = new Character((char) 0);
+                    value = Character.valueOf((char) 0);
                 } else {
                     if (env.getValue().length() == 1) {
-                        value = new Character(env.getValue().charAt(0));
+                        value = Character.valueOf(env.getValue().charAt(0));
                     } else {
                         throw new IllegalArgumentException();
                     }
