@@ -1,31 +1,42 @@
 
-
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
-
 package org.apache.catalina.servlets;
 
 
@@ -44,10 +55,9 @@ import java.io.RandomAccessFile;
 import java.io.Reader;
 import java.io.StringReader;
 import java.io.StringWriter;
-import java.nio.ByteBuffer;
-import java.util.Enumeration;
+import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.StringTokenizer;
-import java.util.Vector;
 
 import javax.naming.InitialContext;
 import javax.naming.NameClassPair;
@@ -67,6 +77,7 @@ import javax.xml.transform.stream.StreamResult;
 import javax.xml.transform.stream.StreamSource;
 
 import org.apache.catalina.Globals;
+import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.catalina.util.StringManager;
 import org.apache.catalina.util.URLEncoder;
@@ -79,13 +90,15 @@ import org.apache.naming.resources.ResourceAttributes;
 import org.apache.catalina.core.ApplicationHttpResponse;
 // END SJSAS 6231069
 
+import org.apache.catalina.core.AlternateDocBase;
+
 /**
  * The default resource-serving servlet for most web applications,
  * used to serve static resources such as HTML pages and images.
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.7 $ $Date: 2005/12/12 19:11:33 $
+ * @version $Revision: 1.17.4.1 $ $Date: 2007/09/11 17:41:37 $
  */
 
 public class DefaultServlet
@@ -155,12 +168,25 @@ public class DefaultServlet
     protected ProxyDirContext resources = null;
 
 
+    /**
+     * Alternate doc bases
+     */
+    protected ArrayList<AlternateDocBase> alternateDocBases = null;
+
+
     /**
      * File encoding to be used when reading static files. If none is specified
      * the platform default is used.
      */
     protected String fileEncoding = null;
     
+    
+    /**
+     * Full range marker.
+     */
+    protected static final ArrayList FULL = new ArrayList();
+    
+    
     // ----------------------------------------------------- Static Initializer
 
 
@@ -199,7 +225,7 @@ public class DefaultServlet
     /**
      * Size of file transfer buffer in bytes.
      */
-    private static final int BUFFER_SIZE = 4096;
+    protected static final int BUFFER_SIZE = 4096;
 
 
     // --------------------------------------------------------- Public Methods
@@ -233,14 +259,14 @@ public class DefaultServlet
         }
         try {
             value = getServletConfig().getInitParameter("listings");
-            listings = (new Boolean(value)).booleanValue();
+            listings = (Boolean.valueOf(value)).booleanValue();
         } catch (Throwable t) {
             ;
         }
         try {
             value = getServletConfig().getInitParameter("readonly");
             if (value != null)
-                readOnly = (new Boolean(value)).booleanValue();
+                readOnly = (Boolean.valueOf(value)).booleanValue();
         } catch (Throwable t) {
             ;
         }
@@ -295,6 +321,14 @@ public class DefaultServlet
             throw new UnavailableException("No resources");
         }
 
+        try {
+            alternateDocBases = (ArrayList<AlternateDocBase>)
+                getServletContext().getAttribute(
+                    Globals.ALTERNATE_RESOURCES_ATTR);
+        } catch(ClassCastException e) {
+            // Failed : Not the right type
+        }
+
     }
 
 
@@ -347,16 +381,7 @@ public class DefaultServlet
         throws IOException, ServletException {
 
         // Serve the requested resource, including the data content
-        try {
-            serveResource(request, response, true);
-        } catch( IOException ex ) {
-            // we probably have this check somewhere else too.
-            if( ex.getMessage() != null
-                && ex.getMessage().indexOf("Broken pipe") >= 0 ) {
-                // ignore it.
-            }
-            throw ex;
-        }
+        serveResource(request, response, true);
 
     }
 
@@ -673,7 +698,20 @@ public class DefaultServlet
                     path + "' headers only");
         }
 
-        CacheEntry cacheEntry = resources.lookupCache(path);
+        CacheEntry cacheEntry = null;
+        AlternateDocBase match = null;
+        if (alternateDocBases == null
+                || alternateDocBases.size() == 0) {
+            cacheEntry = resources.lookupCache(path);
+        } else {
+            match = AlternateDocBase.findMatch(path, alternateDocBases);
+            if (match != null) {
+                cacheEntry = match.getResources().lookupCache(path);
+            } else {
+                // None of the url patterns for alternate docbases matched
+                cacheEntry = resources.lookupCache(path);
+            }
+        }
 
         if (!cacheEntry.exists) {
             // Check if we're included so we can return the appropriate 
@@ -690,7 +728,7 @@ public class DefaultServlet
                  * Therefore, the only way we can let the including resource
                  * know about the missing resource is by throwing an 
                  * exception
-                 */
+                */
                 throw new FileNotFoundException(requestUri);
             }
 
@@ -716,8 +754,8 @@ public class DefaultServlet
                     requestUri = request.getRequestURI();
                 }
                 /* IASRI 4878272
-                response.sendError(HttpServletResponse.SC_NOT_FOUND,
-                                   requestUri);
+                 response.sendError(HttpServletResponse.SC_NOT_FOUND,
+                                    requestUri);
                 */
                 // BEGIN IASRI 4878272
                 response.sendError(HttpServletResponse.SC_NOT_FOUND);
@@ -747,7 +785,7 @@ public class DefaultServlet
             cacheEntry.attributes.setMimeType(contentType);
         }
 
-        Vector ranges = null;
+        ArrayList ranges = null;
         long contentLength = -1L;
 
         if (cacheEntry.context != null) {
@@ -756,8 +794,8 @@ public class DefaultServlet
             // suppress them
             if (!listings) {
                 /* IASRI 4878272
-                response.sendError(HttpServletResponse.SC_NOT_FOUND,
-                                   request.getRequestURI());
+                 response.sendError(HttpServletResponse.SC_NOT_FOUND,
+                                    request.getRequestURI());
                 */
                 // BEGIN IASRI 4878272
                 response.sendError(HttpServletResponse.SC_NOT_FOUND);
@@ -811,9 +849,10 @@ public class DefaultServlet
 
         }
 
-        if ( (cacheEntry.context != null) ||
-             ( ((ranges == null) || (ranges.isEmpty()))
-               && (request.getHeader("Range") == null) ) ) {
+        if ( (cacheEntry.context != null) 
+                || ( ((ranges == null) || (ranges.isEmpty()))
+                        && (request.getHeader("Range") == null) )
+                || (ranges == FULL) ) {
 
             // Set the appropriate output headers
             if (contentType != null) {
@@ -826,7 +865,12 @@ public class DefaultServlet
                 if (debug > 0)
                     log("DefaultServlet.serveFile:  contentLength=" +
                         contentLength);
-                response.setContentLength((int) contentLength);
+                if (contentLength < Integer.MAX_VALUE) {
+                    response.setContentLength((int) contentLength);
+                } else {
+                    // Set the content-length as String to be able to use a long
+                    response.setHeader("content-length", "" + contentLength);
+                }
             }
 
             InputStream renderResult = null;
@@ -835,7 +879,7 @@ public class DefaultServlet
                 if (content) {
                     // Serve the directory browser
                     renderResult =
-                        render(request.getContextPath(), cacheEntry);
+                        render(request.getContextPath(), cacheEntry, match);
                 }
 
             }
@@ -865,14 +909,18 @@ public class DefaultServlet
 
             if (ranges.size() == 1) {
 
-                Range range = (Range) ranges.elementAt(0);
+                Range range = (Range) ranges.get(0);
                 response.addHeader("Content-Range", "bytes "
                                    + range.start
                                    + "-" + range.end + "/"
                                    + range.length);
-                               System.out.println("2- " + contentLength);
- 
-                response.setContentLength((int) (range.end - range.start + 1));
+                long length = range.end - range.start + 1;
+                if (length < Integer.MAX_VALUE) {
+                    response.setContentLength((int) length);
+                } else {
+                    // Set the content-length as String to be able to use a long
+                    response.setHeader("content-length", "" + length);
+                }
 
                 if (contentType != null) {
                     if (debug > 0)
@@ -906,10 +954,10 @@ public class DefaultServlet
                         // Silent catch
                     }
                     if (ostream != null) {
-                        copy(cacheEntry, ostream, ranges.elements(),
+                        copy(cacheEntry, ostream, ranges.iterator(),
                              contentType);
                     } else {
-                        copy(cacheEntry, writer, ranges.elements(),
+                        copy(cacheEntry, writer, ranges.iterator(),
                              contentType);
                     }
                 }
@@ -989,7 +1037,7 @@ public class DefaultServlet
      * @param response The servlet response we are creating
      * @return Vector of ranges
      */
-    protected Vector parseRange(HttpServletRequest request,
+    protected ArrayList parseRange(HttpServletRequest request,
                                 HttpServletResponse response,
                                 ResourceAttributes resourceAttributes)
         throws IOException {
@@ -1014,7 +1062,7 @@ public class DefaultServlet
                 // If the ETag the client gave does not match the entity
                 // etag, then the entire entity is returned.
                 if (!eTag.equals(headerValue.trim()))
-                    return null;
+                    return FULL;
 
             } else {
 
@@ -1022,7 +1070,7 @@ public class DefaultServlet
                 // the last modification date of the entity, the entire entity
                 // is returned.
                 if (lastModified > (headerValueTime + 1000))
-                    return null;
+                    return FULL;
 
             }
 
@@ -1051,7 +1099,7 @@ public class DefaultServlet
 
         // Vector which will contain all the ranges which are successfully
         // parsed.
-        Vector result = new Vector();
+        ArrayList result = new ArrayList();
         StringTokenizer commaTokenizer = new StringTokenizer(rangeHeader, ",");
 
         // Parsing the range list
@@ -1114,7 +1162,7 @@ public class DefaultServlet
                 return null;
             }
 
-            result.addElement(currentRange);
+            result.add(currentRange);
         }
 
         return result;
@@ -1125,15 +1173,16 @@ public class DefaultServlet
     /**
      *  Decide which way to render. HTML or XML.
      */
-    protected InputStream render
-        (String contextPath, CacheEntry cacheEntry) {
+    protected InputStream render(String contextPath, CacheEntry cacheEntry,
+                                 AlternateDocBase match) {
+
         InputStream xsltInputStream =
             findXsltInputStream(cacheEntry.context);
 
         if (xsltInputStream==null) {
-            return renderHtml(contextPath, cacheEntry);
+            return renderHtml(contextPath, cacheEntry, match);
         } else {
-            return renderXml(contextPath, cacheEntry, xsltInputStream);
+            return renderXml(contextPath, cacheEntry, match, xsltInputStream);
         }
 
     }
@@ -1147,8 +1196,16 @@ public class DefaultServlet
      */
     protected InputStream renderXml(String contextPath,
                                     CacheEntry cacheEntry,
+                                    AlternateDocBase match,
                                     InputStream xsltInputStream) {
 
+        ProxyDirContext renderResources = null;
+        if (match != null) {
+            renderResources = match.getResources();
+        } else {
+            renderResources = resources;
+        }
+
         StringBuffer sb = new StringBuffer();
 
         sb.append("<?xml version=\"1.0\"?>");
@@ -1167,9 +1224,9 @@ public class DefaultServlet
         try {
 
             // Render the directory entries within this directory
-            NamingEnumeration enumeration = resources.list(cacheEntry.name);
-        
-            //rewriteUrl(contextPath) is expensive. cache result for later reuse
+            NamingEnumeration enumeration = renderResources.list(cacheEntry.name);
+            
+            // rewriteUrl(contextPath) is expensive. cache result for later reuse
             String rewrittenContextPath =  rewriteUrl(contextPath);
 
             while (enumeration.hasMoreElements()) {
@@ -1183,7 +1240,8 @@ public class DefaultServlet
                     continue;
 
                 CacheEntry childCacheEntry =
-                    resources.lookupCache(cacheEntry.name + resourceName);
+                    renderResources.lookupCache(cacheEntry.name
+                                                + resourceName);
                 if (!childCacheEntry.exists) {
                     continue;
                 }
@@ -1207,7 +1265,7 @@ public class DefaultServlet
                   .append("'");
 
                 sb.append(">");
-                sb.append(trimmed);
+                sb.append(RequestUtil.filter(trimmed));
                 if (childCacheEntry.context != null)
                     sb.append("/");
                 sb.append("</entry>");
@@ -1247,7 +1305,7 @@ public class DefaultServlet
             return (new ByteArrayInputStream(stream.toByteArray()));
         } catch (Exception e) {
             log("directory transform failure: " + e.getMessage());
-            return renderHtml(contextPath, cacheEntry);
+            return renderHtml(contextPath, cacheEntry, match);
         }
     }
 
@@ -1259,7 +1317,14 @@ public class DefaultServlet
      *  relative
      */
     protected InputStream renderHtml
-        (String contextPath, CacheEntry cacheEntry) {
+        (String contextPath, CacheEntry cacheEntry, AlternateDocBase match) {
+
+        ProxyDirContext renderResources = null;
+        if (match != null) {
+            renderResources = match.getResources();
+        } else {
+            renderResources = resources;
+        }
 
         String name = cacheEntry.name;
 
@@ -1282,7 +1347,7 @@ public class DefaultServlet
         PrintWriter writer = new PrintWriter(osWriter);
 
         StringBuffer sb = new StringBuffer();
-             
+        
         // rewriteUrl(contextPath) is expensive. cache result for later reuse
         String rewrittenContextPath =  rewriteUrl(contextPath);
 
@@ -1345,9 +1410,8 @@ public class DefaultServlet
         try {
 
             // Render the directory entries within this directory
-            NamingEnumeration enumeration = resources.list(cacheEntry.name);
+            NamingEnumeration enumeration = renderResources.list(cacheEntry.name);
             boolean shade = false;
-            
             while (enumeration.hasMoreElements()) {
 
                 NameClassPair ncPair = (NameClassPair) enumeration.nextElement();
@@ -1358,7 +1422,8 @@ public class DefaultServlet
                     continue;
 
                 CacheEntry childCacheEntry =
-                    resources.lookupCache(cacheEntry.name + resourceName);
+                    renderResources.lookupCache(cacheEntry.name
+                                                + resourceName);
                 if (!childCacheEntry.exists) {
                     continue;
                 }
@@ -1377,7 +1442,7 @@ public class DefaultServlet
                 if (childCacheEntry.context != null)
                     sb.append("/");
                 sb.append("\"><tt>");
-                sb.append(trimmed);
+                sb.append(RequestUtil.filter(trimmed));
                 if (childCacheEntry.context != null)
                     sb.append("/");
                 sb.append("</tt></a></td>\r\n");
@@ -1520,7 +1585,7 @@ public class DefaultServlet
     }
 
 
-    // -------------------------------------------------------- Private Methods
+    // -------------------------------------------------------- protected Methods
 
 
     /**
@@ -1533,7 +1598,7 @@ public class DefaultServlet
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
      */
-    private boolean checkIfMatch(HttpServletRequest request,
+    protected boolean checkIfMatch(HttpServletRequest request,
                                  HttpServletResponse response,
                                  ResourceAttributes resourceAttributes)
         throws IOException {
@@ -1578,7 +1643,7 @@ public class DefaultServlet
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
      */
-    private boolean checkIfModifiedSince(HttpServletRequest request,
+    protected boolean checkIfModifiedSince(HttpServletRequest request,
                                          HttpServletResponse response,
                                          ResourceAttributes resourceAttributes)
         throws IOException {
@@ -1594,6 +1659,7 @@ public class DefaultServlet
                     // The entity has not been modified since the date
                     // specified by the client. This is not an error case.
                     response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
+                    response.setHeader("ETag", getETag(resourceAttributes));
                     return false;
                 }
             }
@@ -1615,7 +1681,7 @@ public class DefaultServlet
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
      */
-    private boolean checkIfNoneMatch(HttpServletRequest request,
+    protected boolean checkIfNoneMatch(HttpServletRequest request,
                                      HttpServletResponse response,
                                      ResourceAttributes resourceAttributes)
         throws IOException {
@@ -1650,6 +1716,7 @@ public class DefaultServlet
                 if ( ("GET".equals(request.getMethod()))
                      || ("HEAD".equals(request.getMethod())) ) {
                     response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
+                    response.setHeader("ETag", getETag(resourceAttributes));
                     return false;
                 } else {
                     response.sendError
@@ -1673,7 +1740,7 @@ public class DefaultServlet
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
      */
-    private boolean checkIfUnmodifiedSince(HttpServletRequest request,
+    protected boolean checkIfUnmodifiedSince(HttpServletRequest request,
                                            HttpServletResponse response,
                                            ResourceAttributes resourceAttributes)
         throws IOException {
@@ -1706,7 +1773,7 @@ public class DefaultServlet
      *
      * @exception IOException if an input/output error occurs
      */
-    private void copy(CacheEntry cacheEntry, InputStream is,
+    protected void copy(CacheEntry cacheEntry, InputStream is,
                       ServletOutputStream ostream)
         throws IOException {
 
@@ -1756,7 +1823,7 @@ public class DefaultServlet
      *
      * @exception IOException if an input/output error occurs
      */
-    private void copy(CacheEntry cacheEntry, InputStream is, PrintWriter writer)
+    protected void copy(CacheEntry cacheEntry, InputStream is, PrintWriter writer)
         throws IOException {
 
         IOException exception = null;
@@ -1803,7 +1870,7 @@ public class DefaultServlet
      * @param range Range the client wanted to retrieve
      * @exception IOException if an input/output error occurs
      */
-    private void copy(CacheEntry cacheEntry, ServletOutputStream ostream,
+    protected void copy(CacheEntry cacheEntry, ServletOutputStream ostream,
                       Range range)
         throws IOException {
 
@@ -1838,7 +1905,7 @@ public class DefaultServlet
      * @param range Range the client wanted to retrieve
      * @exception IOException if an input/output error occurs
      */
-    private void copy(CacheEntry cacheEntry, PrintWriter writer,
+    protected void copy(CacheEntry cacheEntry, PrintWriter writer,
                       Range range)
         throws IOException {
 
@@ -1881,19 +1948,19 @@ public class DefaultServlet
      * @param contentType Content type of the resource
      * @exception IOException if an input/output error occurs
      */
-    private void copy(CacheEntry cacheEntry, ServletOutputStream ostream,
-                      Enumeration ranges, String contentType)
+    protected void copy(CacheEntry cacheEntry, ServletOutputStream ostream,
+                      Iterator ranges, String contentType)
         throws IOException {
 
         IOException exception = null;
 
-        while ( (exception == null) && (ranges.hasMoreElements()) ) {
+        while ( (exception == null) && (ranges.hasNext()) ) {
 
             InputStream resourceInputStream = cacheEntry.resource.streamContent();
             InputStream istream =
                 new BufferedInputStream(resourceInputStream, input);
 
-            Range currentRange = (Range) ranges.nextElement();
+            Range currentRange = (Range) ranges.next();
 
             // Writing MIME header.
             ostream.println();
@@ -1938,13 +2005,13 @@ public class DefaultServlet
      * @param contentType Content type of the resource
      * @exception IOException if an input/output error occurs
      */
-    private void copy(CacheEntry cacheEntry, PrintWriter writer,
-                      Enumeration ranges, String contentType)
+    protected void copy(CacheEntry cacheEntry, PrintWriter writer,
+                      Iterator ranges, String contentType)
         throws IOException {
 
         IOException exception = null;
 
-        while ( (exception == null) && (ranges.hasMoreElements()) ) {
+        while ( (exception == null) && (ranges.hasNext()) ) {
 
             InputStream resourceInputStream = cacheEntry.resource.streamContent();
             
@@ -1956,7 +2023,7 @@ public class DefaultServlet
                                                fileEncoding);
             }
 
-            Range currentRange = (Range) ranges.nextElement();
+            Range currentRange = (Range) ranges.next();
 
             // Writing MIME header.
             writer.println();
@@ -1999,7 +2066,7 @@ public class DefaultServlet
      * @param ostream The output stream to write to
      * @return Exception which occurred during processing
      */
-    private IOException copyRange(InputStream istream,
+    protected IOException copyRange(InputStream istream,
                                   ServletOutputStream ostream) {
 
         // Copy the input stream to the output stream
@@ -2032,7 +2099,7 @@ public class DefaultServlet
      * @param writer The writer to write to
      * @return Exception which occurred during processing
      */
-    private IOException copyRange(Reader reader, PrintWriter writer) {
+    protected IOException copyRange(Reader reader, PrintWriter writer) {
 
         // Copy the input stream to the output stream
         IOException exception = null;
@@ -2066,7 +2133,7 @@ public class DefaultServlet
      * @param end End of the range which will be copied
      * @return Exception which occurred during processing
      */
-    private IOException copyRange(InputStream istream,
+    protected IOException copyRange(InputStream istream,
                                   ServletOutputStream ostream,
                                   long start, long end) {
 
@@ -2118,7 +2185,7 @@ public class DefaultServlet
      * @param end End of the range which will be copied
      * @return Exception which occurred during processing
      */
-    private IOException copyRange(Reader reader, PrintWriter writer,
+    protected IOException copyRange(Reader reader, PrintWriter writer,
                                   long start, long end) {
 
         try {
@@ -2159,7 +2226,7 @@ public class DefaultServlet
     // ------------------------------------------------------ Range Inner Class
 
 
-    private class Range {
+    protected class Range {
 
         public long start;
         public long end;
@@ -2182,4 +2249,6 @@ public class DefaultServlet
         }
 
     }
+
+
 }
