@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -62,6 +75,7 @@ import javax.servlet.ServletResponse;
 import org.apache.catalina.Container;
 import org.apache.catalina.DefaultContext;
 import org.apache.catalina.Engine;
+import org.apache.catalina.Globals;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Session;
 //START OF 6364900
@@ -86,7 +100,7 @@ import com.sun.enterprise.util.uuid.UuidGenerator;
  * be subclassed to create more sophisticated Manager implementations.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 1.7 $ $Date: 2006/03/12 01:27:05 $
+ * @version $Revision: 1.23.4.1 $ $Date: 2007/10/30 00:17:08 $
  */
 
 public abstract class ManagerBase implements Manager, MBeanRegistration {
@@ -228,7 +242,7 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
      * session identifier.
      */
     protected HashMap sessions = new HashMap();
-
+    
     // Number of sessions created by this manager
     protected int sessionCounter=0;
 
@@ -258,6 +272,24 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     //END OF RIMOD# 5056989
 
 
+    /**
+     * Utility class used to call into services from
+     * com.sun.ejb.base.io.IOUtils, which provides object input and output
+     * streams for the serialization and deserialization of EJB references
+     * stored in HTTP sessions
+     */
+    private static final IOUtilsCaller webUtilsCaller;
+
+    /**
+     * Creates the utility class used to call into services from
+     * com.sun.ejb.base.io.IOUtils
+     */
+    static {
+        WebIOUtilsFactory factory = new WebIOUtilsFactory();
+        webUtilsCaller = factory.createWebIOUtil();            
+    }
+
+
     // ------------------------------------------------------------- Security classes
     private class PrivilegedSetRandomFile implements PrivilegedAction{
         
@@ -467,8 +499,8 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         boolean oldDistributable = this.distributable;
         this.distributable = distributable;
         support.firePropertyChange("distributable",
-                                   new Boolean(oldDistributable),
-                                   new Boolean(this.distributable));
+                                   Boolean.valueOf(oldDistributable),
+                                   Boolean.valueOf(this.distributable));
 
     }
 
@@ -552,8 +584,8 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         int oldMaxInactiveInterval = this.maxInactiveInterval;
         this.maxInactiveInterval = interval;
         support.firePropertyChange("maxInactiveInterval",
-                                   new Integer(oldMaxInactiveInterval),
-                                   new Integer(this.maxInactiveInterval));
+                                   Integer.valueOf(oldMaxInactiveInterval),
+                                   Integer.valueOf(this.maxInactiveInterval));
 
     }
 
@@ -582,8 +614,8 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         int oldSessionIdLength = this.sessionIdLength;
         this.sessionIdLength = idLength;
         support.firePropertyChange("sessionIdLength",
-                                   new Integer(oldSessionIdLength),
-                                   new Integer(this.sessionIdLength));
+                                   Integer.valueOf(oldSessionIdLength),
+                                   Integer.valueOf(this.sessionIdLength));
 
     }
 
@@ -614,7 +646,7 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     public void setRandomFile( String s ) {
     // as a hack, you can use a static file - and genarate the same
     // session ids ( good for strange debugging )
-        if (System.getSecurityManager() != null){
+        if (Globals.IS_SECURITY_ENABLED){
                 randomIS = (DataInputStream)AccessController.doPrivileged(new PrivilegedSetRandomFile());          
             } else {
                 FileInputStream fileInputStream = null;
@@ -772,7 +804,7 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
                 domain=ctx.getEngineName();
                 distributable = ctx.getDistributable();
                 StandardHost hst=(StandardHost)ctx.getParent();
-                String path = ctx.getPath();
+                String path = ctx.getEncodedPath();
                 if (path.equals("")) {
                     path = "/";
                 }   
@@ -801,7 +833,7 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
             }
         }
     }
-
+    
 
     /**
      * Add a property change listener to this component.
@@ -840,30 +872,7 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
         session.setValid(true);
         session.setCreationTime(System.currentTimeMillis());
         session.setMaxInactiveInterval(this.maxInactiveInterval);
-        //HERCULES:mod
         String sessionId = generateSessionId(session);
-
-        /*
-        String jvmRoute = getJvmRoute();
-        // @todo Move appending of jvmRoute generateSessionId()???
-        if (jvmRoute != null) {
-            sessionId += '.' + jvmRoute;
-        }
-         */
-        /*
-        synchronized (sessions) {
-            while (sessions.get(sessionId) != null){ // Guarantee uniqueness
-                duplicates++;
-                sessionId = generateSessionId();
-                // @todo Move appending of jvmRoute generateSessionId()???
-                if (jvmRoute != null) {
-                    sessionId += '.' + jvmRoute;
-                }
-            }
-        }
-         */
-        //end HERCULES:mod
-
         session.setId(sessionId);
         sessionCounter++;
 
@@ -955,6 +964,42 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     }
     
     
+    /**
+     * Finds and returns the session with the given id that also satisfies
+     * the given version requirement.
+     *
+     * This overloaded version of findSession() will be invoked only if
+     * isSessionVersioningSupported() returns true. By default, this method
+     * delegates to the version of findSession() that does not take any
+     * session version number.
+     *
+     * @param id The session id to match
+     * @param version The session version requirement to satisfy
+     *
+     * @return The session that matches the given id and also satisfies the
+     * given version requirement, or null if no such session could be found
+     * by this session manager
+     *
+     * @exception IOException if an IO error occurred
+     */
+    public Session findSession(String id, String version) throws IOException {
+
+        return findSession(id);
+    }
+
+
+    /**
+     * Returns true if this session manager supports session versioning, false
+     * otherwise.
+     *
+     * @return true if this session manager supports session versioning, false
+     * otherwise.
+     */
+    public boolean isSessionVersioningSupported() {
+        return false;
+    }
+
+
     /**
      * clear out the sessions cache
      * HERCULES:added
@@ -1009,6 +1054,15 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     // ------------------------------------------------------ Protected Methods
 
 
+    /**
+     * Gets the utility class used to call into services from
+     * com.sun.ejb.base.io.IOUtils.
+     */
+    protected static IOUtilsCaller getWebUtilsCaller() {
+        return webUtilsCaller;
+    }
+
+
     /**
      * Get new session class to be used in the doLoad() method.
      */
@@ -1062,43 +1116,6 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     }    
 
 
-    /**
-     * Generate and return a new session identifier.
-     */
-    protected synchronized String generateSessionIdPrevious() {
-
-        byte random[] = new byte[16];
-
-        // Render the result as a String of hexadecimal digits
-        StringBuffer result = new StringBuffer();
-        int resultLenBytes = 0;
-        while (resultLenBytes < this.sessionIdLength) {
-            getRandomBytes(random);
-            random = getDigest().digest(random);
-            for (int j = 0;
-                    j < random.length && resultLenBytes < this.sessionIdLength;
-                    j++) {
-                byte b1 = (byte) ((random[j] & 0xf0) >> 4);
-                byte b2 = (byte) (random[j] & 0x0f);
-                if (b1 < 10)
-                    result.append((char) ('0' + b1));
-                else
-                    result.append((char) ('A' + (b1 - 10)));
-                if (b2 < 10)
-                    result.append((char) ('0' + b2));
-                else
-                    result.append((char) ('A' + (b2 - 10)));
-                resultLenBytes++;
-            }
-        }
-        //START OF RIMOD# 5056989 -- Support for iWS6.0 style session ids
-        if (prefixSessionID_)
-            result.insert(0, prefix_);
-        //END OF RIMOD# 5056989
-        return (result.toString());
-    }
-
-
     // ------------------------------------------------------ Protected Methods
 
 
@@ -1461,4 +1478,11 @@ public abstract class ManagerBase implements Manager, MBeanRegistration {
     }
     //END OF 6364900    
 
+
+    /*
+     * Releases any resources held by this session manager.
+     */
+    public void release() {
+        clearSessions();
+    }
 }
