@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -34,6 +47,8 @@ import java.io.IOException;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
+import java.util.Iterator;
+import java.util.concurrent.ConcurrentHashMap;
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Lifecycle;
@@ -41,6 +56,7 @@ import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.Session;
 import org.apache.catalina.Store;
+import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.util.LifecycleSupport;
 import org.apache.catalina.security.SecurityUtil;
 import com.sun.org.apache.commons.logging.Log;
@@ -55,18 +71,17 @@ import com.sun.org.apache.commons.logging.LogFactory;
  * <b>IMPLEMENTATION NOTE</b>:  Correct behavior of session storing and
  * reloading depends upon external calls to the <code>start()</code> and
  * <code>stop()</code> methods of this class at the correct times.
- * HERCULES:mod implements SessionPurgeCapable
  *
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
- * @version $Revision: 1.8 $ $Date: 2006/03/12 01:27:05 $
+ * @version $Revision: 1.16.8.1 $ $Date: 2007/09/05 21:48:36 $
  */
 
 public abstract class PersistentManagerBase
     extends ManagerBase
     implements Lifecycle, PropertyChangeListener {
 
-    private static Log log = LogFactory.getLog(PersistentManagerBase.class);
+    private static final Log log = LogFactory.getLog(PersistentManagerBase.class);
 
     // ---------------------------------------------------- Security Classes
      private class PrivilegedStoreClear
@@ -159,7 +174,7 @@ public abstract class PersistentManagerBase
     /**
      * The descriptive name of this Manager implementation (for logging).
      */
-    protected static String name = "PersistentManagerBase";
+    protected static final String name = "PersistentManagerBase";
 
 
     /**
@@ -210,6 +225,19 @@ public abstract class PersistentManagerBase
     private int rejectedSessions = 0;
 
 
+    // START SJSAS 6406580
+    /**
+     * The set of invalidated Sessions for this Manager, keyed by
+     * session identifier.
+     */
+    protected ConcurrentHashMap<String, Long> invalidatedSessions
+        = new ConcurrentHashMap<String, Long>();
+
+    // Specifies for how long we're going to remember invalidated session ids
+    private long rememberInvalidatedSessionIdMilliSecs = 60000L;
+    // END SJSAS 6406580
+
+
     // ------------------------------------------------------------- Properties
 
     /**
@@ -218,6 +246,9 @@ public abstract class PersistentManagerBase
     public void backgroundProcess() {
         this.processExpires();
         this.processPersistenceChecks();
+        // START SJSAS 6406580
+        this.processInvalidatedSessions();
+        // END SJSAS 6406580
         if ((this.getStore() != null)
             && (this.getStore() instanceof StoreBase)) {
             ((StoreBase) this.getStore()).processExpires();
@@ -263,8 +294,8 @@ public abstract class PersistentManagerBase
         int oldBackup = this.maxIdleBackup;
         this.maxIdleBackup = backup;
         support.firePropertyChange("maxIdleBackup",
-                                   new Integer(oldBackup),
-                                   new Integer(this.maxIdleBackup));
+                                   Integer.valueOf(oldBackup),
+                                   Integer.valueOf(this.maxIdleBackup));
 
     }
 
@@ -291,8 +322,8 @@ public abstract class PersistentManagerBase
         int oldMaxIdleSwap = this.maxIdleSwap;
         this.maxIdleSwap = max;
         support.firePropertyChange("maxIdleSwap",
-                                   new Integer(oldMaxIdleSwap),
-                                   new Integer(this.maxIdleSwap));
+                                   Integer.valueOf(oldMaxIdleSwap),
+                                   Integer.valueOf(this.maxIdleSwap));
 
     }
 
@@ -321,8 +352,8 @@ public abstract class PersistentManagerBase
         int oldMinIdleSwap = this.minIdleSwap;
         this.minIdleSwap = min;
         support.firePropertyChange("minIdleSwap",
-                                   new Integer(oldMinIdleSwap),
-                                   new Integer(this.minIdleSwap));
+                                   Integer.valueOf(oldMinIdleSwap),
+                                   Integer.valueOf(this.minIdleSwap));
 
     }
 
@@ -350,6 +381,20 @@ public abstract class PersistentManagerBase
             ((Context) this.container).addPropertyChangeListener(this);
         }
 
+        // START SJSAS 6406580
+        if (container instanceof StandardContext) {
+            // Determine for how long we're going to remember invalidated
+            // session ids
+            StandardContext ctx = (StandardContext) container;
+            int frequency = ctx.getManagerChecksFrequency();
+            int reapIntervalSeconds = ctx.getBackgroundProcessorDelay();
+            rememberInvalidatedSessionIdMilliSecs
+                = frequency * reapIntervalSeconds * 1000L * 2;
+            if (rememberInvalidatedSessionIdMilliSecs <= 0) {
+                rememberInvalidatedSessionIdMilliSecs = 60000L;
+            }
+        }
+        // END SJSAS 6406580
     }
 
 
@@ -407,8 +452,8 @@ public abstract class PersistentManagerBase
         int oldMaxActiveSessions = this.maxActiveSessions;
         this.maxActiveSessions = max;
         support.firePropertyChange("maxActiveSessions",
-                                   new Integer(oldMaxActiveSessions),
-                                   new Integer(this.maxActiveSessions));
+                                   Integer.valueOf(oldMaxActiveSessions),
+                                   Integer.valueOf(this.maxActiveSessions));
 
     }
 
@@ -518,8 +563,8 @@ public abstract class PersistentManagerBase
         boolean oldSaveOnRestart = this.saveOnRestart;
         this.saveOnRestart = saveOnRestart;
         support.firePropertyChange("saveOnRestart",
-                                   new Boolean(oldSaveOnRestart),
-                                   new Boolean(this.saveOnRestart));
+                                   Boolean.valueOf(oldSaveOnRestart),
+                                   Boolean.valueOf(this.saveOnRestart));
 
     }
 
@@ -527,6 +572,15 @@ public abstract class PersistentManagerBase
     // --------------------------------------------------------- Public Methods
 
 
+    /*
+     * Releases any resources held by this session manager.
+     */
+    public void release() {
+        super.release();
+        clearStore();
+    }
+
+
     /**
      * Clear all sessions from the Store.
      */
@@ -601,6 +655,30 @@ public abstract class PersistentManagerBase
     }
 
 
+    /**
+     * Purges those session ids from the map of invalidated session ids whose
+     * time has come up
+     */
+    protected void processInvalidatedSessions() {
+
+        if (!started) {
+            return;
+        }
+
+        long timeNow = System.currentTimeMillis();
+        Iterator<String> iter = invalidatedSessions.keySet().iterator();
+        while (iter.hasNext()) {
+            String id = iter.next();
+            Long timeAdded = invalidatedSessions.get(id);
+            if ((timeAdded == null)
+                    || (timeNow - timeAdded.longValue() >
+                                    rememberInvalidatedSessionIdMilliSecs)) {
+                removeFromInvalidatedSessions(id);
+            }
+        }
+    }        
+
+
     /**
      * Return a new session object as long as the number of active
      * sessions does not exceed <b>maxActiveSessions</b>. If there
@@ -636,6 +714,12 @@ public abstract class PersistentManagerBase
      *  processing this request
      */
     public Session findSession(String id) throws IOException {
+        
+        //6406580 START
+        if(!this.isSessionIdValid(id)) {
+            return null;
+        }
+        //6406580 END        
 
         Session session = super.findSession(id);
         if (session != null)
@@ -806,6 +890,40 @@ public abstract class PersistentManagerBase
         }        
     }
 
+
+    // START SJSAS 6406580
+    /**
+     * Add this Session id to the set of invalidated Session ids for this
+     * Manager.
+     *
+     * @param sessionId session id to be added
+     */
+    public void addToInvalidatedSessions(String sessionId) {
+        invalidatedSessions.put(sessionId,
+                                Long.valueOf(System.currentTimeMillis()));
+    }
+
+    
+    /**
+     * Removes the given session id from the map of invalidated session ids.
+     *
+     * @param sessionId The session id to remove
+     */
+    public void removeFromInvalidatedSessions(String sessionId) {       
+        invalidatedSessions.remove(sessionId);
+    }    
+    
+
+    /**
+     * @return true if the given session id is not contained in the map of
+     * invalidated session ids, false otherwise
+     */
+    public boolean isSessionIdValid(String sessionId) {       
+        return (!invalidatedSessions.containsKey(sessionId));
+    }
+    // END SJSAS 6406580    
+
+
     /**
      * Save all currently active sessions in the appropriate persistence
      * mechanism, if any.  If persistence is not supported, this method
@@ -841,7 +959,6 @@ public abstract class PersistentManagerBase
 
     // ------------------------------------------------------ Protected Methods
 
-
     /**
      * Look for a session in the Store and, if found, restore
      * it in the Manager's list of active sessions if appropriate.
@@ -850,6 +967,54 @@ public abstract class PersistentManagerBase
      * is invalid or past its expiration.
      */
     protected Session swapIn(String id) throws IOException {
+        return swapIn(id, null);
+    }
+
+    /**
+     * Look for a session in the Store and, if found, restore
+     * it in the Manager's list of active sessions if appropriate.
+     * The session will be removed from the Store after swapping
+     * in, but will not be added to the active session list if it
+     * is invalid or past its expiration.
+     *
+     * @param id The session id
+     * @param version The requested session version
+     */
+    protected Session swapIn(String id, String version) throws IOException {
+
+        ClassLoader webappCl = null;
+        ClassLoader curCl = null;
+
+        if (getContainer() != null
+                    && getContainer().getLoader() != null) {
+            webappCl = getContainer().getLoader().getClassLoader();
+            curCl = Thread.currentThread().getContextClassLoader();
+        }
+
+        Session sess = null;
+
+        if (webappCl != null && curCl != webappCl) {
+            try {
+                Thread.currentThread().setContextClassLoader(webappCl);
+                sess = doSwapIn(id, version);
+            } finally {
+                Thread.currentThread().setContextClassLoader(curCl);
+            }
+        } else {
+            sess = doSwapIn(id, version);
+        }
+
+        return sess;
+    }
+
+    /**
+     * Look for a session in the Store and, if found, restore
+     * it in the Manager's list of active sessions if appropriate.
+     * The session will be removed from the Store after swapping
+     * in, but will not be added to the active session list if it
+     * is invalid or past its expiration.
+     */
+    private Session doSwapIn(String id, String version) throws IOException {
 
         if (store == null)
             return null;
@@ -869,7 +1034,11 @@ public abstract class PersistentManagerBase
                     }
                 }
             } else {
-                 session = store.load(id);
+		if (version != null) {
+                     session = ((StoreBase) store).load(id, version);
+                } else {
+                     session = store.load(id);
+                }
             }   
         } catch (ClassNotFoundException e) {
             log.error(sm.getString("persistentManager.deserializeError", id, e));
@@ -882,8 +1051,12 @@ public abstract class PersistentManagerBase
 
         if (!session.isValid()) {
             log.error("session swapped in is invalid or expired");
+            //6406580 START
+            /* - these lines are calling remove on store redundantly
             session.expire();
             removeSession(id);
+             */
+            //6406580 END            
             return (null);
         }
 
@@ -1133,7 +1306,8 @@ public abstract class PersistentManagerBase
                     if (log.isDebugEnabled())
                         log.debug(sm.getString
                             ("persistentManager.swapMaxIdle",
-                             session.getIdInternal(), new Integer(timeIdle)));
+                             session.getIdInternal(),
+                             Integer.valueOf(timeIdle)));
                     try {
                         swapOut(session);
                     } catch (IOException e) {
@@ -1164,7 +1338,7 @@ public abstract class PersistentManagerBase
         if(log.isDebugEnabled())
             log.debug(sm.getString
                 ("persistentManager.tooManyActive",
-                 new Integer(sessions.length)));        
+                 Integer.valueOf(sessions.length)));        
 
         int toswap = sessions.length - getMaxActiveSessions();
         long timeNow = System.currentTimeMillis();
@@ -1180,7 +1354,7 @@ public abstract class PersistentManagerBase
                         log.debug(sm.getString
                             ("persistentManager.swapTooManyActive",
                             sessions[i].getIdInternal(),
-                            new Integer(timeIdle)));                    
+                            Integer.valueOf(timeIdle)));                    
                     try {
                         swapOut(sessions[i]);
                     } catch (java.util.ConcurrentModificationException e1) {
@@ -1227,7 +1401,7 @@ public abstract class PersistentManagerBase
                             log.debug(sm.getString
                                 ("persistentManager.backupMaxIdle",
                                 session.getIdInternal(),
-                                new Integer(timeIdle))); 
+                                Integer.valueOf(timeIdle))); 
                         try {
                             writeSession(session);
                         } catch (java.util.ConcurrentModificationException e1) {
