@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -32,6 +45,7 @@ package org.apache.catalina.startup;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
+import java.io.InputStream;
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
@@ -74,10 +88,7 @@ import org.xml.sax.InputSource;
 public final class TldConfig  {
 
     // Names of JARs that are known not to contain any TLDs with listeners
-    private static HashSet noTldJars;
-
-    // Names of system jar files that are ignored if placed under WEB-INF
-    private static HashSet systemJars;
+    private static HashSet<String> noTldListeners;
 
     private static com.sun.org.apache.commons.logging.Log log=
         com.sun.org.apache.commons.logging.LogFactory.getLog( TldConfig.class );
@@ -85,76 +96,22 @@ public final class TldConfig  {
     private static final String FILE_URL_PREFIX = "file:";
     private static final int FILE_URL_PREFIX_LEN = FILE_URL_PREFIX.length();
 
-    // Names of system TLD URIs
-    private static HashSet<String> systemTldUris;
-
-    static {
-        systemTldUris = new HashSet();
-        systemTldUris.add("http://java.sun.com/jsf/core");
-        systemTldUris.add("http://java.sun.com/jsf/html");
-        systemTldUris.add("http://java.sun.com/jsp/jstl/core");
-    }
-
-    /*
-     * Initializes the set of JARs that are known not to contain any TLDs
+    // START CR 6402120   
+    /**
+     * The variable that indicates whether or not to create/use a serialized
+     * cache of TLD listeners.
      */
-    static {
-        noTldJars = new HashSet();
-        noTldJars.add("ant.jar");
-        noTldJars.add("catalina.jar");
-        noTldJars.add("catalina-ant.jar");
-        noTldJars.add("catalina-cluster.jar");
-        noTldJars.add("catalina-optional.jar");
-        noTldJars.add("catalina-i18n-fr.jar");
-        noTldJars.add("catalina-i18n-ja.jar");
-        noTldJars.add("catalina-i18n-es.jar");
-        noTldJars.add("commons-dbcp.jar");
-        noTldJars.add("commons-modeler.jar");
-        noTldJars.add("commons-logging-api.jar");
-        noTldJars.add("commons-beanutils.jar");
-        noTldJars.add("commons-fileupload-1.0.jar");
-        noTldJars.add("commons-pool.jar");
-        noTldJars.add("commons-digester.jar");
-        noTldJars.add("commons-logging.jar");
-        noTldJars.add("commons-collections.jar");
-        noTldJars.add("commons-el.jar");
-        noTldJars.add("jakarta-regexp-1.2.jar");
-        noTldJars.add("jasper-compiler.jar");
-        noTldJars.add("jasper-runtime.jar");
-        noTldJars.add("jmx.jar");
-        noTldJars.add("jmx-tools.jar");
-        noTldJars.add("jsp-api.jar");
-        noTldJars.add("jkshm.jar");
-        noTldJars.add("jkconfig.jar");
-        noTldJars.add("naming-common.jar");
-        noTldJars.add("naming-resources.jar");
-        noTldJars.add("naming-factory.jar");
-        noTldJars.add("naming-java.jar");
-        noTldJars.add("servlet-api.jar");
-        noTldJars.add("servlets-default.jar");
-        noTldJars.add("servlets-invoker.jar");
-        noTldJars.add("servlets-common.jar");
-        noTldJars.add("servlets-webdav.jar");
-        noTldJars.add("tomcat-util.jar");
-        noTldJars.add("tomcat-http11.jar");
-        noTldJars.add("tomcat-jni.jar");
-        noTldJars.add("tomcat-jk.jar");
-        noTldJars.add("tomcat-jk2.jar");
-        noTldJars.add("tomcat-coyote.jar");
-        noTldJars.add("xercesImpl.jar");
-        noTldJars.add("xmlParserAPIs.jar");
-        // JARs from J2SE runtime
-        noTldJars.add("sunjce_provider.jar");
-        noTldJars.add("ldapsec.jar");
-        noTldJars.add("localedata.jar");
-        noTldJars.add("dnsns.jar");
-    }
+    private static boolean cacheListeners = true;
+    // END CR 6402120   
+
+    // Names of system TLD URIs
+    private static HashSet<String> systemTldUris = new HashSet<String>();
+    private static HashSet<String> systemTldUrisJsf = new HashSet<String>();
 
     static {
-        systemJars = new HashSet();
-        systemJars.add("standard.jar");
-        systemJars.add("appserv-jstl.jar");
-        systemJars.add("jsf-impl.jar");
+        systemTldUrisJsf.add("http://java.sun.com/jsf/core");
+        systemTldUrisJsf.add("http://java.sun.com/jsf/html");
+        systemTldUris.add("http://java.sun.com/jsp/jstl/core");
     }
 
     // ----------------------------------------------------- Instance Variables
@@ -210,25 +167,62 @@ public final class TldConfig  {
 
     private ArrayList listeners=new ArrayList();
 
+    // START GlassFish 747
+    private HashMap<String, String[]> tldUriToLocationMap =
+            new HashMap<String, String[]>();
+    private String currentTldResourcePath;
+    private String currentTldJarFile;
+    private String currentTldJarEntryName;
+    // END GlassFish 747
+
+    private boolean useMyFaces;
+
+
     // --------------------------------------------------------- Public Methods
 
+
+    public void setUseMyFaces(boolean useMyFaces) {
+        this.useMyFaces = useMyFaces;
+    }
+
+
     /**
-     * Sets the list of JARs that are known not to contain any TLDs with
-     * listeners.
+     * Sets the list of JAR files that are known not to contain any
+     * TLDs that declare servlet listeners.
+     *
+     * Only shared JAR files (that is, those loaded by a delegation parent
+     * of the webapp's classloader) will be checked against this list.
      *
      * @param jarNames List of comma-separated names of JAR files that are 
-     * known not to contain any TLDs with listeners
+     * known not to contain any TLDs that declare servlet listeners
      */
     public static void setNoTldListeners(String jarNames) {
         if (jarNames != null) {
-            noTldJars.clear();
+            if (noTldListeners == null) {
+                noTldListeners = new HashSet<String>();
+            } else {
+                noTldListeners.clear();
+            }
             StringTokenizer tokenizer = new StringTokenizer(jarNames, ",");
             while (tokenizer.hasMoreElements()) {
-                noTldJars.add(tokenizer.nextToken());
+                noTldListeners.add(tokenizer.nextToken());
             }
         }
     }
 
+    /**
+     * Sets the list of JAR files that are known not to contain any
+     * TLDs that declare servlet listeners.
+     *
+     * Only shared JAR files (that is, those loaded by a delegation parent
+     * of the webapp's classloader) will be checked against this list.
+     *
+     * @param set HashSet containing the names of JAR file known not to
+     * contain any TLDs that declare servlet listeners
+     */
+    public static void setNoTldListeners(HashSet set) {
+        noTldListeners = set;
+    }
     
     // START SJSAS 8.1 5049111   
     /**
@@ -243,7 +237,34 @@ public final class TldConfig  {
     }
     // END SJSAS 8.1 5049111     
     
-    
+    // START CR 6402120
+    /**
+     * Sets the flag that indicates whether to create/use a serialized cache of
+     * listeners
+     *
+     * @param cache true to create/use a listener cache, false otherwise
+     */
+    public static void setCacheListeners(boolean cache) {
+        cacheListeners = cache;
+    } 
+ 
+    /**
+     * Indicates if a serialized cache of listeners is to be created and used
+     *
+     * @return true if the listener cache file is to be used, false otherwise.
+     */
+    public static boolean isCacheListeners() {
+        return cacheListeners;
+    }
+
+    /**
+     * @deprecated Provided for backwards compatibility only. Use
+     * setCacheListeners instead.
+     */
+    public static void setSingleProcess(boolean isSingleProcess) {
+        cacheListeners = isSingleProcess;
+    }
+    // END CR 6402120    
  
     /**
      * Set the validation feature of the XML parser used when
@@ -297,18 +318,50 @@ public final class TldConfig  {
 
     public void setContext(Context context) {
         this.context = context;
+        this.useMyFaces = ((StandardContext) context).isUseMyFaces();
     }
 
     public void addApplicationListener( String s ) {
-        //if(log.isDebugEnabled())
+        if (log.isDebugEnabled()) {
             log.debug( "Add tld listener " + s);
-        if (!isCurrentTldLocal || !systemTldUris.contains(currentTldUri)) {
+        }
+        if ((isCurrentTldLocal
+                    && !systemTldUris.contains(currentTldUri)
+                    && (!systemTldUrisJsf.contains(currentTldUri)
+                        || useMyFaces))
+                || (!isCurrentTldLocal
+                    && (!systemTldUrisJsf.contains(currentTldUri)
+                        || !useMyFaces))) {
             listeners.add(s);
         }
     }
 
     public void setTldUri(String uri) {
         this.currentTldUri = uri;
+        // START GlassFish 747
+        /*
+         * Add the mapping for the given URI only if
+         * - the corresponding TLD is local, and
+         * - the URI is not one of the standard (i.e., JSTL or JSF) ones, and
+         * - the URI is not already mapped (this check is necessary because 
+         *   taglibs specified in web.xml are supposed to take precedence)
+         */
+        if (isCurrentTldLocal
+                && !systemTldUris.contains(currentTldUri)
+                && (!systemTldUrisJsf.contains(currentTldUri)
+                    || useMyFaces)
+                && tldUriToLocationMap.get(currentTldUri) == null) {
+            String[] currentTldLocation = new String[2];
+            if (currentTldResourcePath != null) {
+                currentTldLocation[0] = currentTldResourcePath;
+            } else if (currentTldJarFile != null
+                    && currentTldJarEntryName != null) {
+                currentTldLocation[0] = "file:" + currentTldJarFile;
+                currentTldLocation[1] = currentTldJarEntryName;
+            }
+            tldUriToLocationMap.put(currentTldUri, currentTldLocation);
+        }
+        // END GlassFish 747
     }
 
     public String[] getTldListeners() {
@@ -331,7 +384,20 @@ public final class TldConfig  {
 
         File tldCache=null;
 
+        /* CR 6402120   
         if (context instanceof StandardContext) {
+        */
+        // START CR 6402120
+        if (log.isDebugEnabled()) { 
+            log.debug("Create/use TLD listener cache? "
+                      + (isCacheListeners()));
+        }
+
+        // If multiple JVMs are running, then do not create tldCache.ser 
+        // file as it will cause exceptions on the server side 
+        // because of unsynchronized access of tldCache.ser file.
+        if ((context instanceof StandardContext) && isCacheListeners()) {
+        // END CR 6402120   
             File workDir= (File)
                 ((StandardContext)context).getServletContext().getAttribute(Globals.WORK_DIR_ATTR);
             tldCache=new File( workDir, "tldCache.ser");
@@ -341,9 +407,12 @@ public final class TldConfig  {
         if( ! rescan ) {
             // find the cache
             if( tldCache!= null && tldCache.exists()) {
-                // just read it...
-                processCache(tldCache);
-                return;
+                try {
+                    processCache(tldCache);
+                    return;
+                } catch (Throwable t) {
+                    log.warn("Error scanning " + tldCache, t);
+                }
             }
         }
 
@@ -358,20 +427,19 @@ public final class TldConfig  {
         if (tldCache != null && tldCache.exists()) {
             long lastModified = getLastModified(resourcePaths, jarPaths);
             if (lastModified < tldCache.lastModified()) {
-                processCache(tldCache);
-                return;
+                try {
+                    processCache(tldCache);
+                    return;
+                } catch (Throwable t) {
+                    log.warn("Error scanning " + tldCache, t);
+                }
             }
         }
 
         // Scan each accumulated resource path for TLDs to be processed
-        Iterator paths = resourcePaths.iterator();
+        Iterator<String> paths = resourcePaths.iterator();
         while (paths.hasNext()) {
-            String path = (String) paths.next();
-            if (path.endsWith(".jar")) {
-                tldScanJar(path);
-            } else {
-                tldScanTld(path);
-            }
+            tldScanTld(paths.next());
         }
         if (jarPaths != null) {
             Iterator<JarPathElement> elems = jarPaths.values().iterator();
@@ -406,6 +474,11 @@ public final class TldConfig  {
             ((StandardContext)context).setTldScanTime(t2-t1);
         }
 
+        // START GlassFish 747
+        context.getServletContext().setAttribute(
+            Globals.JSP_TLD_URI_TO_LOCATION_MAP,
+            tldUriToLocationMap);
+        // END GlassFish 747
     }
 
     // -------------------------------------------------------- Private Methods
@@ -455,21 +528,24 @@ public final class TldConfig  {
         return lastModified;
     }
 
-    private void processCache(File tldCache ) throws IOException {
-        // read the cache and return;
-        try {
-            FileInputStream in=new FileInputStream(tldCache);
-            ObjectInputStream ois=new ObjectInputStream( in );
-            String list[]=(String [])ois.readObject();
-            if( log.isDebugEnabled() )
-                log.debug("Reusing tldCache " + tldCache + " " + list.length);
-            for( int i=0; list!=null && i<list.length; i++ ) {
-                context.addApplicationListener(list[i]);
-            }
-            ois.close();
-        } catch( ClassNotFoundException ex ) {
-            ex.printStackTrace();
+    /**
+     * Reads the cache of listeners specified in TLD files.
+     */
+    private void processCache(File tldCache ) throws Exception {
+        FileInputStream in=new FileInputStream(tldCache);
+        ObjectInputStream ois=new ObjectInputStream( in );
+        String list[]=(String [])ois.readObject();
+        if( log.isDebugEnabled() ) {
+            log.debug("Reusing tldCache " + tldCache + " " + list.length);
         }
+        for( int i=0; list!=null && i<list.length; i++ ) {
+            // Load the listener class. Failure to do so is an indication
+            // that the cache has become stale, in which case it must be
+            // ignored. See GlassFish Issue 2653.
+            context.getLoader().getClassLoader().loadClass(list[i]);
+            context.addApplicationListener(list[i]);
+        }
+        ois.close();
     }
 
     /**
@@ -491,34 +567,6 @@ public final class TldConfig  {
         // END SJSAS 6384538
     }
 
-    /**
-     * Scan the JAR file at the specified resource path for TLDs in the
-     * <code>META-INF</code> subdirectory, and scan each TLD for application
-     * event listeners that need to be registered.
-     *
-     * @param resourcePath Resource path of the JAR file to scan
-     *
-     * @exception Exception if an exception occurs while scanning this JAR
-     */
-    private void tldScanJar(String resourcePath) throws Exception {
-
-        if (log.isDebugEnabled()) {
-            log.debug(" Scanning JAR at resource path '" + resourcePath + "'");
-        }
-
-        URL url = context.getServletContext().getResource(resourcePath);
-        if (url == null) {
-            throw new IllegalArgumentException
-                                (sm.getString("contextConfig.tldResourcePath",
-                                              resourcePath));
-        }
-
-        File file = new File(url.getFile());
-        file = file.getCanonicalFile();
-        tldScanJar(file);
-
-    }
-
     /**
      * Scans all TLD entries in the given JAR for application listeners.
      *
@@ -551,6 +599,10 @@ public final class TldConfig  {
                 if (log.isTraceEnabled()) {
                     log.trace("  Processing TLD at '" + name + "'");
                 }
+                // START GlassFish 747
+                currentTldJarFile = jarPath;
+                currentTldJarEntryName = name;
+                // END GlassFish 747
                 try {
                     tldScanStream(new InputSource(jarFile.getInputStream(entry)),
                                   isLocal);
@@ -605,6 +657,11 @@ public final class TldConfig  {
             } finally {
                 isCurrentTldLocal = false;
                 currentTldUri = null;
+                // START GlassFish 747
+                currentTldJarFile = null;
+                currentTldJarEntryName = null;
+                currentTldResourcePath = null;
+                // END GlassFish 747
                 tldDigester.push(null);
                 tldDigester.clear();
             }
@@ -626,17 +683,22 @@ public final class TldConfig  {
             log.debug(" Scanning TLD at resource path '" + resourcePath + "'");
         }
 
-        InputSource inputSource = null;
+        InputStream tldStream =
+            context.getServletContext().getResourceAsStream(resourcePath);
+        if (tldStream == null) {
+            throw new ServletException
+                (sm.getString("contextConfig.tldResourcePath",
+                              resourcePath));
+        }
+
+        InputSource inputSource = new InputSource(tldStream);
+
+        // START GlassFish 747
+        currentTldResourcePath = resourcePath;
+        // END GlassFish 747
+
         try {
-            inputSource =
-                new InputSource(
-                    context.getServletContext().getResourceAsStream(resourcePath));
-            if (inputSource == null) {
-                throw new IllegalArgumentException
-                    (sm.getString("contextConfig.tldResourcePath",
-                                  resourcePath));
-            }
-            tldScanStream(inputSource);
+            tldScanStream(inputSource, true);
         } catch (Exception e) {
              throw new ServletException
                  (sm.getString("contextConfig.tldFileException", resourcePath,
@@ -679,6 +741,10 @@ public final class TldConfig  {
                 log.trace("   Adding path '" + resourcePath +
                     "' for URI '" + taglibs[i] + "'");
             }
+            // START GlassFish 747
+            tldUriToLocationMap.put(taglibs[i],
+                                    new String[] { resourcePath, null});
+            // END GlassFish 747
             resourcePaths.add(resourcePath);
         }
 
@@ -696,9 +762,8 @@ public final class TldConfig  {
      * Scans the web application's subdirectory identified by rootPath,
      * along with its subdirectories, for TLDs.
      *
-     * Initially, rootPath equals /WEB-INF. The /WEB-INF/classes and
-     * /WEB-INF/lib subdirectories are excluded from the search, as per the
-     * JSP 2.0 spec.
+     * Initially, rootPath equals /WEB-INF. /WEB-INF/tags and any of its
+     * subdirectories are excluded from the search, as per the JSP spec.
      *
      * @param resources The web application's resources
      * @param rootPath The path whose subdirectories are to be searched for
@@ -719,9 +784,7 @@ public final class TldConfig  {
             while (items.hasMoreElements()) {
                 NameClassPair item = (NameClassPair) items.nextElement();
                 String resourcePath = rootPath + "/" + item.getName();
-                if (!resourcePath.endsWith(".tld")
-                        && (resourcePath.startsWith("/WEB-INF/classes")
-                            || resourcePath.startsWith("/WEB-INF/lib"))) {
+                if (resourcePath.startsWith("/WEB-INF/tags")) {
                     continue;
                 }
                 if (resourcePath.endsWith(".tld")) {
@@ -754,8 +817,9 @@ public final class TldConfig  {
      * <CATALINA_HOME>/common/lib).
      *
      * The set of shared JARs to be scanned for TLDs is narrowed down by
-     * the <tt>noTldJars</tt> class variable, which contains the names of JARs
-     * that are known not to contain any TLDs with listeners.
+     * the <tt>noTldListeners</tt> class variable, which contains the names
+     * of JARs that are known not to contain any TLDs that declare servlet
+     * listeners.
      *
      * @return Map of JAR file paths
      */
@@ -792,11 +856,9 @@ public final class TldConfig  {
                      * that are not known not to contain any TLDs with
                      * listeners
                      */
-                    if ((loader != webappLoader
-                                || !systemJars.contains(file.getName()))
-                            && (loader == webappLoader
-                                || noTldJars == null
-                                || !noTldJars.contains(file.getName()))) {
+                    if (loader == webappLoader
+                            || noTldListeners == null
+                            || !noTldListeners.contains(file.getName())) {
                         JarPathElement elem = new JarPathElement(
                                 file, loader == webappLoader);
                         if (jarPathMap == null) {
