@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -46,14 +59,20 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 import org.apache.catalina.HttpResponse;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
+*/
 import org.apache.catalina.LifecycleException;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.LifecycleListener;
+*/
 import org.apache.catalina.Request;
 import org.apache.catalina.Response;
 import org.apache.catalina.Session;
+/** CR 6411114 (Lifecycle implementation moved to ValveBase)
 import org.apache.catalina.util.LifecycleSupport;
+*/
 import org.apache.catalina.util.StringManager;
 import org.apache.coyote.tomcat5.CoyoteRequest;
 
@@ -119,12 +138,17 @@ import org.apache.coyote.tomcat5.CoyoteRequest;
  *
  * @author Craig R. McClanahan
  * @author Jason Brittain
- * @version $Revision: 1.2 $ $Date: 2005/12/08 01:28:22 $
+ * @version $Revision: 1.5 $ $Date: 2007/05/05 05:32:41 $
  */
 
 public final class AccessLogValve
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     extends ValveBase
     implements Lifecycle {
+    */
+    // START CR 6411114
+    extends ValveBase {
+    // END CR 6411114
 
 
     // ----------------------------------------------------------- Constructors
@@ -168,7 +192,9 @@ public final class AccessLogValve
     /**
      * The lifecycle event support for this component.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     protected LifecycleSupport lifecycle = new LifecycleSupport(this);
+    */
 
 
     /**
@@ -222,7 +248,9 @@ public final class AccessLogValve
     /**
      * Has this component been started yet?
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     private boolean started = false;
+    */
 
 
     /**
@@ -864,8 +892,8 @@ public final class AccessLogValve
             else
                 value = "";
         } else if (pattern == 'r') {
-            StringBuffer sb = new StringBuffer();
             if (hreq != null) {
+                StringBuffer sb = new StringBuffer();
                 sb.append(hreq.getMethod());
                 sb.append(space);
                 sb.append(hreq.getRequestURI());
@@ -875,11 +903,10 @@ public final class AccessLogValve
                 }
                 sb.append(space);
                 sb.append(hreq.getProtocol());
+                value = sb.toString();
             } else {
-                sb.append("- - ");
-                sb.append(req.getProtocol());
+                value = "- - -";
             }
-            value = sb.toString();
         } else if (pattern == 'S') {
             if (request instanceof CoyoteRequest) {
                 Session sess = ((CoyoteRequest) request).getSessionInternal(false);
@@ -1061,22 +1088,26 @@ public final class AccessLogValve
      *
      * @param listener The listener to add
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public void addLifecycleListener(LifecycleListener listener) {
 
         lifecycle.addLifecycleListener(listener);
 
     }
+    */
 
 
     /**
      * Get the lifecycle listeners associated with this lifecycle. If this
      * Lifecycle has no listeners registered, a zero-length array is returned.
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public LifecycleListener[] findLifecycleListeners() {
 
         return lifecycle.findLifecycleListeners();
 
     }
+    */
 
 
     /**
@@ -1084,11 +1115,13 @@ public final class AccessLogValve
      *
      * @param listener The listener to add
      */
+    /** CR 6411114 (Lifecycle implementation moved to ValveBase)
     public void removeLifecycleListener(LifecycleListener listener) {
 
         lifecycle.removeLifecycleListener(listener);
 
     }
+    */
 
 
     /**
@@ -1102,11 +1135,19 @@ public final class AccessLogValve
     public void start() throws LifecycleException {
 
         // Validate and update our current component state
+        /** CR 6411114 (Lifecycle implementation moved to ValveBase)
         if (started)
             throw new LifecycleException
                 (sm.getString("accessLogValve.alreadyStarted"));
         lifecycle.fireLifecycleEvent(START_EVENT, null);
         started = true;
+        */
+        // START CR 6411114
+        if (started)            // Ignore multiple starts
+            return;
+
+        super.start();
+        // END CR 6411114
 
         // Initialize the timeZone, Date formatters, and currentDate
         TimeZone tz = TimeZone.getDefault();
@@ -1144,13 +1185,22 @@ public final class AccessLogValve
     public void stop() throws LifecycleException {
 
         // Validate and update our current component state
+        /** CR 6411114
         if (!started)
             throw new LifecycleException
                 (sm.getString("accessLogValve.notStarted"));
         lifecycle.fireLifecycleEvent(STOP_EVENT, null);
         started = false;
+        */
+        // START CR 6411114
+        if (!started)       // Ignore stop if not started
+            return;
+        // END CR 6411114
 
         close();
+        // START CR 6411114
+        super.stop();
+        // END CR 6411114
 
     }
 
