@@ -1,29 +1,42 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
- */ 
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
+ */
 package org.apache.coyote.http11;
 
 import java.io.InputStream;
@@ -54,10 +67,32 @@ import javax.management.MBeanRegistration;
  */
 public class Http11Protocol implements ProtocolHandler, MBeanRegistration
 {
+    // START SJSAS 6439313     
+    protected boolean blocking = false;
+    // END SJSAS 6439313     
+
+    /**
+     * The <code>SelectorThread</code> implementation class. Not used when 
+     * Coyote is used.
+     */
+    protected String selectorThreadImpl = null; 
+    
+    
     public Http11Protocol() {
-        create();
+        // START SJSAS 6439313 
+        this(false,false,null);
     }
 
+    
+    public Http11Protocol(boolean secure, boolean blocking, 
+                          String selectorThreadImpl) {
+        this.secure = secure;
+        this.blocking = blocking; 
+        this.selectorThreadImpl = selectorThreadImpl;
+        // END SJSAS 6439313
+        create();
+    }
+   
     protected void create() {
         cHandler = new Http11ConnectionHandler( this );
         setSoLinger(Constants.DEFAULT_CONNECTION_LINGER);
@@ -81,7 +116,7 @@ public class Http11Protocol implements ProtocolHandler, MBeanRegistration
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
     /** Pass config info
@@ -156,7 +191,8 @@ public class Http11Protocol implements ProtocolHandler, MBeanRegistration
             log.error(sm.getString("http11protocol.endpoint.initerror"), ex);
             throw ex;
         }
-        log.info(sm.getString("http11protocol.init", "" + ep.getPort()));
+        log.info(sm.getString("http11protocol.init", "" + ep.getPort(),
+                System.getProperty("product.name")));
 
     }
     
@@ -188,7 +224,8 @@ public class Http11Protocol implements ProtocolHandler, MBeanRegistration
             log.error(sm.getString("http11protocol.endpoint.starterror"), ex);
             throw ex;
         }
-        log.info(sm.getString("http11protocol.start", "" + ep.getPort()));
+        log.info(sm.getString("http11protocol.start", "" + ep.getPort(),
+                System.getProperty("product.name")));
     }
 
     public void destroy() throws Exception {
@@ -411,6 +448,16 @@ public class Http11Protocol implements ProtocolHandler, MBeanRegistration
         setAttribute("keystoreType", k);
     }
 
+    // START GlassFish Issue 657
+    public void setTruststore(String truststore) {
+        setAttribute("truststore", truststore);
+    }
+
+    public void setTruststoreType(String truststoreType) {
+        setAttribute("truststoreType", truststoreType);
+    }    
+    // END GlassFish Issue 657
+
     public String getClientauth() {
         return getProperty("clientauth");
     }
@@ -451,7 +498,18 @@ public class Http11Protocol implements ProtocolHandler, MBeanRegistration
     	secure=b;
         setAttribute("secure", "" + b);
     }
+    
+    // START SJSAS 6439313     
+    public boolean getBlocking() {
+        return blocking;
+    }
 
+    public void setBlocking( boolean b ) {
+    	blocking=b;
+        setAttribute("blocking", "" + b);
+    }
+    // END SJSAS 6439313     
+    
     public String getCiphers() {
         return getProperty("ciphers");
     }
@@ -671,7 +729,7 @@ public class Http11Protocol implements ProtocolHandler, MBeanRegistration
         }
     }
 
-    protected static com.sun.org.apache.commons.logging.Log log 
+    protected static final com.sun.org.apache.commons.logging.Log log 
         = com.sun.org.apache.commons.logging.LogFactory.getLog(Http11Protocol.class);
 
     // -------------------- Various implementation classes --------------------
