@@ -1,26 +1,39 @@
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -74,7 +87,7 @@ import com.sun.appserv.ProxyHandler;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 1.12.2.1 $ $Date: 2006/09/05 19:52:26 $
+ * @version $Revision: 1.32.2.1.2.2 $ $Date: 2007/11/27 01:53:48 $
  */
 
 public class CoyoteAdapter
@@ -87,7 +100,24 @@ public class CoyoteAdapter
 
     public static final int ADAPTER_NOTES = 1;
 
+    static final String JVM_ROUTE = System.getProperty("jvmRoute");
 
+    protected static final boolean ALLOW_BACKSLASH = 
+        Boolean.valueOf(System.getProperty("org.apache.coyote.tomcat5.CoyoteAdapter.ALLOW_BACKSLASH", "false")).booleanValue();
+
+    private static final boolean COLLAPSE_ADJACENT_SLASHES =
+        Boolean.valueOf(System.getProperty(
+            "com.sun.enterprise.web.collapseAdjacentSlashes", "true")).booleanValue();
+
+    /**
+     * When mod_jk is used, the adapter must be invoked the same way 
+     * Tomcat does by invoking service(...) and the afterService(...). This
+     * is a hack to make it compatible with Tomcat 5|6.
+     */
+    private boolean compatWithTomcat = false;
+    
+    private String serverName = System.getProperty("product.name");
+    
     // ----------------------------------------------------------- Constructors
 
 
@@ -232,30 +262,70 @@ public class CoyoteAdapter
                         request.setAttribute(Globals.CERTIFICATES_ATTR,
                                              certs);
                     }
+                    
                 }
                 // END S1AS 6188932
-
+                
+                response.addHeader("Server",serverName);
+                
                 // Calling the container
                 connector.getContainer().invoke(request, response);
-              
             }
-
+         /* GlassFish Issue 79    
             response.finishResponse();
             req.action( ActionCode.ACTION_POST_REQUEST , null);
 
-        } catch (IOException e) {
+         } catch (IOException e) {
             ;
+         } catch (Throwable t) {
+             log.error(sm.getString("coyoteAdapter.service"), t);
+         } finally {
+             // Recycle the wrapper request and response
+             request.recycle();
+             response.recycle();
+         }*/
+        // START GlassFish Issue 798
+        } catch (IOException e) {
+            // Recycle the wrapper request and response
+            request.recycle();
+            response.recycle();
         } catch (Throwable t) {
+            log.error(sm.getString("coyoteAdapter.service"), t);
+                        // Recycle the wrapper request and response
+            request.recycle();
+            response.recycle();
+        } 
+        // END GlassFish Issue 798
+        
+        if ( compatWithTomcat ) {
+            afterService(req,res);
+        }
+        
+    }
+
+    // START GlassFish Issue 798
+    /**
+     * Finish the response and close the connection based on the connection
+     * header.
+     */
+    public void afterService(Request req,Response res) throws Exception{
+        CoyoteRequest request = (CoyoteRequest) req.getNote(ADAPTER_NOTES);
+        CoyoteResponse response = (CoyoteResponse) res.getNote(ADAPTER_NOTES);
+        
+        if ( request == null || response == null) return;
+        
+        try{
+            response.finishResponse();
+            req.action( ActionCode.ACTION_POST_REQUEST , null);
+        }catch (Throwable t) {
             log.error(sm.getString("coyoteAdapter.service"), t);
         } finally {
             // Recycle the wrapper request and response
             request.recycle();
             response.recycle();
         }
-
     }
-
-
+    // END GlassFish Issue 798
     // ------------------------------------------------------ Protected Methods
 
 
@@ -301,16 +371,18 @@ public class CoyoteAdapter
           req.getURLDecoder().convert(decodedURI, false);
         } catch (IOException ioe) {
           res.setStatus(400);
-          res.setMessage("Invalid URI");
-          throw ioe;
+          res.setMessage("Invalid URI: " + ioe.getMessage());
+          return false;
         }
 
+        /* GlassFish Issue 2339
         // Normalize decoded URI
         if (!normalize(req.decodedURI())) {
             res.setStatus(400);
             res.setMessage("Invalid URI");
             return false;
         }
+        */
 
         // Set the remote principal
         String principal = req.getRemoteUser().toString();
@@ -334,7 +406,16 @@ public class CoyoteAdapter
         // START CR 6309511
         // URI character decoding
         request.convertURI(decodedURI);
-        
+
+        // START GlassFish Issue 2339
+        // Normalize decoded URI
+        if (!normalize(decodedURI)) {
+            res.setStatus(400);
+            res.setMessage("Invalid URI");
+            return false;
+        }
+        // END GlassFish Issue 2339
+
         // Parse session Id
         request.parseSessionId();
         // END CR 6309511
@@ -344,7 +425,9 @@ public class CoyoteAdapter
         // won't be considered by the mapping algorithm.
         CharChunk uriCC = decodedURI.getCharChunk();
         int semicolon = uriCC.indexOf(';');
+        String sessionVersionString = null;
         if (semicolon > 0) {
+            sessionVersionString = request.parseSessionVersion();
             decodedURI.setChars
                 (uriCC.getBuffer(), uriCC.getStart(), semicolon);
         }
@@ -352,7 +435,10 @@ public class CoyoteAdapter
         // Request mapping.
         connector.getMapper().map(req.serverName(), decodedURI, 
                                   request.getMappingData());
-        
+        // START GlassFish 1024
+        request.setDefaultContext(request.getMappingData().isDefaultContext);
+        // END GlassFish 1024
+
         // START SJSAS 6253524
         // request.setContext((Context) request.getMappingData().context);
         // END SJSAS 6253524
@@ -360,6 +446,7 @@ public class CoyoteAdapter
         Context ctx = (Context) request.getMappingData().context;
         request.setContext(ctx);
         // END SJSAS 6253524
+
         request.setWrapper((Wrapper) request.getMappingData().wrapper);
 
         // Filter trace method
@@ -408,14 +495,28 @@ public class CoyoteAdapter
                 redirectPath = redirectPath + ";jsessionid=" 
                     + request.getRequestedSessionId();
             }            
+            // START GlassFish 936
+            redirectPath = urlEncoder.encodeURL(redirectPath);
+            // END GlassFish 936
             if (query != null) {
                 // This is not optimal, but as this is not very common, it
                 // shouldn't matter
                 redirectPath = redirectPath + "?" + query;
             }
-            // START GlassFish 936
-            redirectPath = urlEncoder.encodeURL(redirectPath);
-            // END GlassFish 936
+
+            // START CR 6590921
+            boolean authPassthroughEnabled = 
+                connector.getAuthPassthroughEnabled();
+            ProxyHandler proxyHandler = connector.getProxyHandler();
+            if (authPassthroughEnabled && proxyHandler != null) {
+
+                if (proxyHandler.getSSLKeysize(
+                        (HttpServletRequest)request.getRequest()) > 0) {
+                    request.setSecure(true);
+                }
+            }
+            // END CR 6590921
+
             response.sendRedirect(redirectPath);
             return false;
         }
@@ -432,6 +533,10 @@ public class CoyoteAdapter
         request.parseJrouteCookie();
         // END SJSAS 6346226
 
+        if (sessionVersionString != null) {
+            request.parseSessionVersionString(sessionVersionString);
+        }
+
         return true;
     }
 
@@ -600,6 +705,17 @@ public class CoyoteAdapter
      */
     public static boolean normalize(MessageBytes uriMB) {
 
+        int type = uriMB.getType();
+        if (type == MessageBytes.T_CHARS) {
+            return normalizeChars(uriMB);
+        } else {
+            return normalizeBytes(uriMB);
+        }
+    }
+
+
+    private static boolean normalizeBytes(MessageBytes uriMB) {
+
         ByteChunk uriBC = uriMB.getByteChunk();
         byte[] b = uriBC.getBytes();
         int start = uriBC.getStart();
@@ -615,10 +731,16 @@ public class CoyoteAdapter
         // Replace '\' with '/'
         // Check for null byte
         for (pos = start; pos < end; pos++) {
-            if (b[pos] == (byte) '\\')
-                b[pos] = (byte) '/';
-            if (b[pos] == (byte) 0)
+            if (b[pos] == (byte) '\\') {
+                if (ALLOW_BACKSLASH) {
+                    b[pos] = (byte) '/';
+                } else {
+                    return false;
+                }
+            }
+            if (b[pos] == (byte) 0) {
                 return false;
+            }
         }
 
         // The URL must start with '/'
@@ -627,11 +749,13 @@ public class CoyoteAdapter
         }
 
         // Replace "//" with "/"
-        for (pos = start; pos < (end - 1); pos++) {
-            if (b[pos] == (byte) '/') {
-                while ((pos + 1 < end) && (b[pos + 1] == (byte) '/')) {
-                    copyBytes(b, pos, pos + 1, end - pos - 1);
-                    end--;
+        if (COLLAPSE_ADJACENT_SLASHES) {
+            for (pos = start; pos < (end - 1); pos++) {
+                if (b[pos] == (byte) '/') {
+                    while ((pos + 1 < end) && (b[pos + 1] == (byte) '/')) {
+                        copyBytes(b, pos, pos + 1, end - pos - 1);
+                        end--;
+                    }
                 }
             }
         }
@@ -693,6 +817,109 @@ public class CoyoteAdapter
     }
 
 
+    private static boolean normalizeChars(MessageBytes uriMB) {
+
+        CharChunk uriCC = uriMB.getCharChunk();
+        char[] c = uriCC.getChars();
+        int start = uriCC.getStart();
+        int end = uriCC.getEnd();
+
+        // URL * is acceptable
+        if ((end - start == 1) && c[start] == (char) '*')
+          return true;
+
+        int pos = 0;
+        int index = 0;
+
+        // Replace '\' with '/'
+        // Check for null char
+        for (pos = start; pos < end; pos++) {
+            if (c[pos] == (char) '\\') {
+                if (ALLOW_BACKSLASH) {
+                    c[pos] = (char) '/';
+                } else {
+                    return false;
+                }
+            }
+            if (c[pos] == (char) 0) {
+                return false;
+            }
+        }
+
+        // The URL must start with '/'
+        if (c[start] != (char) '/') {
+            return false;
+        }
+
+        // Replace "//" with "/"
+        if (COLLAPSE_ADJACENT_SLASHES) {
+            for (pos = start; pos < (end - 1); pos++) {
+                if (c[pos] == (char) '/') {
+                    while ((pos + 1 < end) && (c[pos + 1] == (char) '/')) {
+                        copyChars(c, pos, pos + 1, end - pos - 1);
+                        end--;
+                    }
+                }
+            }
+        }	
+
+        // If the URI ends with "/." or "/..", then we append an extra "/"
+        // Note: It is possible to extend the URI by 1 without any side effect
+        // as the next character is a non-significant WS.
+        if (((end - start) > 2) && (c[end - 1] == (char) '.')) {
+            if ((c[end - 2] == (char) '/') 
+                || ((c[end - 2] == (char) '.') 
+                    && (c[end - 3] == (char) '/'))) {
+                c[end] = (char) '/';
+                end++;
+            }
+        }
+
+        uriCC.setEnd(end);
+
+        index = 0;
+
+        // Resolve occurrences of "/./" in the normalized path
+        while (true) {
+            index = uriCC.indexOf("/./", 0, 3, index);
+            if (index < 0)
+                break;
+            copyChars(c, start + index, start + index + 2, 
+                      end - start - index - 2);
+            end = end - 2;
+            uriCC.setEnd(end);
+        }
+
+        index = 0;
+
+        // Resolve occurrences of "/../" in the normalized path
+        while (true) {
+            index = uriCC.indexOf("/../", 0, 4, index);
+            if (index < 0)
+                break;
+            // Prevent from going outside our context
+            if (index == 0)
+                return false;
+            int index2 = -1;
+            for (pos = start + index - 1; (pos >= 0) && (index2 < 0); pos --) {
+                if (c[pos] == (char) '/') {
+                    index2 = pos;
+                }
+            }
+            copyChars(c, start + index2, start + index + 3,
+                      end - start - index - 3);
+            end = end + index2 - index - 3;
+            uriCC.setEnd(end);
+            index = index2;
+        }
+
+        uriCC.setChars(c, start, end);
+
+        return true;
+
+    }
+
+
     // ------------------------------------------------------ Protected Methods
 
 
@@ -707,6 +934,17 @@ public class CoyoteAdapter
     }
 
 
+    /**
+     * Copy an array of chars to a different position. Used during 
+     * normalization.
+     */
+    private static void copyChars(char[] c, int dest, int src, int len) {
+        for (int pos = 0; pos < len; pos++) {
+            c[pos + dest] = c[pos + src];
+        }
+    }
+
+
     /**
      * Log a message on the Logger associated with our Container (if any)
      *
@@ -775,5 +1013,32 @@ public class CoyoteAdapter
         }
     }
     // END SJSAS 6349248
+
+    
+    /**
+     * Return true when an instance is executed the same way it does in Tomcat.
+     */
+    public boolean isCompatWithTomcat() {
+        return compatWithTomcat;
+    }
+
+    
+    /**
+     * <tt>true</tt> if this class needs to be compatible with Tomcat
+     * Adapter class. Since Tomcat Adapter implementation doesn't support 
+     * the afterService method, the afterService method must be invoked 
+     * inside the service method.
+     */
+    public void setCompatWithTomcat(boolean compatWithTomcat) {
+        this.compatWithTomcat = compatWithTomcat;
+
+        // Add server header
+        if (compatWithTomcat){
+            serverName = "Apache/" + serverName;
+        } else {
+            // Recalculate.
+            serverName = System.getProperty("product.name");
+        }
+    }
        
 }
