@@ -1,40 +1,50 @@
-
-
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
- */ 
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
+ */
 package org.apache.jasper.compiler;
 
 import java.io.File;
 import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
-import java.io.OutputStreamWriter;
-import java.io.PrintStream;
 import java.io.PrintWriter;
-import java.io.UnsupportedEncodingException;
+import java.io.Writer;
 import java.net.URL;
 import java.net.URLConnection;
+import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.StringTokenizer;
@@ -43,16 +53,9 @@ import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
 import org.apache.jasper.Options;
 import org.apache.jasper.servlet.JspServletWrapper;
-import org.apache.jasper.util.SystemLogHandler;
-import org.apache.tools.ant.BuildException;
-import org.apache.tools.ant.DefaultLogger;
-import org.apache.tools.ant.Project;
-import org.apache.tools.ant.taskdefs.Javac;
-import org.apache.tools.ant.types.Path;
-import org.apache.tools.ant.types.PatternSet;
 
 /**
- * Main JSP compiler class. This class uses Ant for compiling.
+ * Main JSP compiler class.
  *
  * @author Anil K. Vijendran
  * @author Mandar Raje
@@ -61,107 +64,68 @@ import org.apache.tools.ant.types.PatternSet;
  * @author Remy Maucherat
  * @author Mark Roth
  */
+
 public class Compiler {
-    private static com.sun.org.apache.commons.logging.Log log=
-        com.sun.org.apache.commons.logging.LogFactory.getLog( Compiler.class );
+    private static com.sun.org.apache.commons.logging.Log commonsLog =
+        com.sun.org.apache.commons.logging.LogFactory.getLog(Compiler.class);
+    private static com.sun.org.apache.commons.logging.Log noOpLog =
+        new com.sun.org.apache.commons.logging.impl.NoOpLog();
 
     // ----------------------------------------------------------------- Static
 
-
-    // Some javac are not thread safe; use a lock to serialize compilation, 
-    static Object javacLock = new Object();
-
-
     // ----------------------------------------------------- Instance Variables
 
-
     protected JspCompilationContext ctxt;
 
     private ErrorDispatcher errDispatcher;
     private PageInfo pageInfo;
     private JspServletWrapper jsw;
-    private JasperAntLogger logger;
     private TagFileProcessor tfp;
+    private JavaCompiler javaCompiler;
+    private boolean jspcMode;
+    private com.sun.org.apache.commons.logging.Log log;
+    private SmapUtil smapUtil;
+    private Options options;
+    private Node.Nodes pageNodes;
+    private long jspModTime;
 
-    protected Project project=null;
-
-    protected Options options;
-
-    protected Node.Nodes pageNodes;
     // ------------------------------------------------------------ Constructor
 
-
-    public Compiler(JspCompilationContext ctxt) {
-        this(ctxt, null);
-    }
-
-
+    // Compiler for parsing only
     public Compiler(JspCompilationContext ctxt, JspServletWrapper jsw) {
         this.jsw = jsw;
         this.ctxt = ctxt;
+        this.jspcMode = false;
         this.options = ctxt.getOptions();
+        this.log = commonsLog;
+        this.smapUtil = new SmapUtil(ctxt);
+        this.errDispatcher = new ErrorDispatcher(jspcMode);
+        this.javaCompiler = new AntJavaCompiler();
+        javaCompiler.init(ctxt, errDispatcher, jspcMode);
     }
 
-    // Lazy eval - if we don't need to compile we probably don't need the project
-    private Project getProject() {
-
-        if( project!=null ) return project;
-
-        // Initializing project
-        project = new Project();
-        logger = new JasperAntLogger();
-        logger.setOutputPrintStream(System.out);
-        logger.setErrorPrintStream(System.err);
-	logger.setMessageOutputLevel(Project.MSG_INFO);
-        project.addBuildListener( logger);
-	if (System.getProperty("catalina.home") != null) {
-            project.setBasedir( System.getProperty("catalina.home"));
-        }
-        
-        if( options.getCompiler() != null ) {
-            if( log.isDebugEnabled() )
-                log.debug("Compiler " + options.getCompiler() );
-            project.setProperty("build.compiler", options.getCompiler() );
-        }
-        project.init();
-        return project;
+    public Compiler(JspCompilationContext ctxt, JspServletWrapper jsw,
+                    boolean jspcMode) {
+        this.jsw = jsw;
+        this.ctxt = ctxt;
+        this.jspcMode = jspcMode;
+        this.options = ctxt.getOptions();
+        this.log = jspcMode? noOpLog: commonsLog;
+        this.smapUtil = new SmapUtil(ctxt);
+        this.errDispatcher = new ErrorDispatcher(jspcMode);
+        initJavaCompiler();
     }
 
-    class JasperAntLogger extends DefaultLogger {
-
-        private StringBuffer reportBuf = new StringBuffer();
-
-        protected void printMessage(final String message,
-                                    final PrintStream stream,
-                                    final int priority) {
-        }
-
-        protected void log(String message) {
-            reportBuf.append(message);
-            reportBuf.append(System.getProperty("line.separator"));
-        }
-
-        protected String getReport() {
-            String report = reportBuf.toString();
-            reportBuf.setLength(0);
-            return report;
-        }
-    }
 
     // --------------------------------------------------------- Public Methods
 
 
     /** 
-     * Compile the jsp file into equivalent servlet in .java file
-     * @return a smap for the current JSP page, if one is generated,
-     *         null otherwise
+     * Compile the jsp file into equivalent servlet in java source
      */
-    private String[] generateJava() throws Exception {
+    private void generateJava() throws Exception {
         
-        String[] smapStr = null;
-
         long t1, t2, t3, t4;
-
         t1 = t2 = t3 = t4 = 0;
 
         if (log.isDebugEnabled()) {
@@ -174,7 +138,7 @@ public class Compiler {
                                 ctxt.getJspFile());
 
         JspConfig jspConfig = options.getJspConfig();
-        JspConfig.JspProperty jspProperty =
+        JspProperty jspProperty =
             jspConfig.findJspProperty(ctxt.getJspFile());
 
         /*
@@ -196,24 +160,20 @@ public class Compiler {
         if (jspProperty.getIncludeCoda() != null) {
 	    pageInfo.setIncludeCoda(jspProperty.getIncludeCoda());
         }
+        if (options.isDefaultBufferNone() && pageInfo.getBufferValue() == null){
+            // Set to unbuffered if not specified explicitly
+            pageInfo.setBuffer(0);
+        }
 
         String javaFileName = ctxt.getServletJavaFileName();
         ServletWriter writer = null;
 
         try {
             // Setup the ServletWriter
-            String javaEncoding = ctxt.getOptions().getJavaEncoding();
-            OutputStreamWriter osw = null; 
-
-            try {
-                osw = new OutputStreamWriter(
-                            new FileOutputStream(javaFileName), javaEncoding);
-            } catch (UnsupportedEncodingException ex) {
-                errDispatcher.jspError("jsp.error.needAlternateJavaEncoding",
-                                       javaEncoding);
-            }
-
-            writer = new ServletWriter(new PrintWriter(osw));
+            Writer javaWriter = javaCompiler.getJavaWriter(
+                                    javaFileName,
+                                    ctxt.getOptions().getJavaEncoding());
+            writer = new ServletWriter(new PrintWriter(javaWriter));
             ctxt.setWriter(writer);
 
             // Reset the temporary variable counter for the generator.
@@ -228,7 +188,7 @@ public class Compiler {
                 Generator.generate(writer, this, pageNodes);
                 writer.close();
                 writer = null;
-                return null;
+                return;
             }
 
             // Validate and process attributes
@@ -290,7 +250,7 @@ public class Compiler {
                 }
             }
             // Remove the generated .java file
-            new File(javaFileName).delete();            
+            javaCompiler.doJavaFile(false);
             throw e;
         } finally {
             if (writer != null) {
@@ -304,7 +264,7 @@ public class Compiler {
         
         // JSR45 Support
         if (! options.isSmapSuppressed()) {
-            smapStr = SmapUtil.generateSmap(ctxt, pageNodes);
+            smapUtil.generateSmap(pageNodes);
         }
 
         // If any proto type .java and .class files was generated,
@@ -313,14 +273,13 @@ public class Compiler {
         // .class file need to be removed, to make sure that javac would
         // generate .class again from the new .java file just generated.
         tfp.removeProtoTypeFiles(ctxt.getClassFileName());
-
-        return smapStr;
     }
 
+
     /** 
      * Compile the servlet from .java file to .class file
      */
-    private void generateClass(String[] smap)
+    private void generateClass()
         throws FileNotFoundException, JasperException, Exception {
 
         long t1 = 0;
@@ -328,140 +287,67 @@ public class Compiler {
             t1 = System.currentTimeMillis();
         }
 
-        String javaEncoding = ctxt.getOptions().getJavaEncoding();
         String javaFileName = ctxt.getServletJavaFileName();
         String classpath = ctxt.getClassPath(); 
-
         String sep = System.getProperty("path.separator");
 
-        StringBuffer errorReport = new StringBuffer();
-
-        StringBuffer info=new StringBuffer();
-        info.append("Compile: javaFileName=" + javaFileName + "\n" );
-        info.append("    classpath=" + classpath + "\n" );
-
-        // Start capturing the System.err output for this thread
-        SystemLogHandler.setThread();
-
-        // Initializing javac task
-        getProject();
-        Javac javac = (Javac) project.createTask("javac");
-
         // Initializing classpath
-        Path path = new Path(project);
-        /* PWC 1.2 6311155
-        path.setPath(System.getProperty("java.class.path"));
-        info.append("    cp=" + System.getProperty("java.class.path") + "\n");
-        */
-        // START PWC 1.2 6311155
+        ArrayList<File> cpath = new ArrayList<File>();
+        HashSet<String> paths = new HashSet<String>();
+
+        // Process classpath, which includes system classpath from compiler
+        // options, plus the context classpath from the classloader
         String sysClassPath = options.getSystemClassPath();
-        path.setPath(sysClassPath);
-        info.append("    cp=" + sysClassPath + "\n");
-        // END PWC 1.2 6311155
+        if (sysClassPath != null) {
+            StringTokenizer tokenizer = new StringTokenizer(sysClassPath, sep);
+            while (tokenizer.hasMoreElements()) {
+                String path = tokenizer.nextToken();
+                if (! paths.contains(path)) {
+                    paths.add(path);
+                    cpath.add(new File(path));
+                }
+            }
+        }
         StringTokenizer tokenizer = new StringTokenizer(classpath, sep);
         while (tokenizer.hasMoreElements()) {
-            String pathElement = tokenizer.nextToken();
-            File repository = new File(pathElement);
-            path.setLocation(repository);
-            info.append("    cp=" + repository + "\n");
+            String path = tokenizer.nextToken();
+            if (! paths.contains(path)) {
+                paths.add(path);
+                cpath.add(new File(path));
+            }
         }
-
-        if( log.isDebugEnabled() )
-            /* PWC 1.2 6311155
-            log.debug( "Using classpath: " + System.getProperty("java.class.path") + sep
-                       + classpath);
-            */
-            // START PWC 1.2 6311155
+        if(log.isDebugEnabled()) {
             log.debug("Using classpath: " + sysClassPath + sep + classpath);
-            //  END PWC 1.2 6311155
+        }
+        javaCompiler.setClassPath(cpath);
         
-        // Initializing sourcepath
-        Path srcPath = new Path(project);
-        srcPath.setLocation(options.getScratchDir());
-
-        info.append("    work dir=" + options.getScratchDir() + "\n");
+        // Set debug info
+        javaCompiler.setDebug(options.getClassDebugInfo());
 
         // Initialize and set java extensions
         String exts = System.getProperty("java.ext.dirs");
         if (exts != null) {
-            Path extdirs = new Path(project);
-            extdirs.setPath(exts);
-            javac.setExtdirs(extdirs);
-            info.append("    extension dir=" + exts + "\n");
-        }
-
-        // Configure the compiler object
-        javac.setEncoding(javaEncoding);
-        javac.setClasspath(path);
-        javac.setDebug(ctxt.getOptions().getClassDebugInfo());
-        javac.setSrcdir(srcPath);
-        javac.setOptimize(! ctxt.getOptions().getClassDebugInfo() );
-        javac.setFork(ctxt.getOptions().getFork());
-        info.append("    srcDir=" + srcPath + "\n" );
-
-        // Set the Java compiler to use
-        if (options.getCompiler() != null) {
-            javac.setCompiler(options.getCompiler());
-            info.append("    compiler=" + options.getCompiler() + "\n");
+            javaCompiler.setExtdirs(exts);
         }
 
         if (options.getCompilerTargetVM() != null) {
-            javac.setTarget(options.getCompilerTargetVM());
-            info.append("   compilerTargetVM=" + options.getCompilerTargetVM() + "\n");
+            javaCompiler.setTargetVM(options.getCompilerTargetVM());
         }
 
         if (options.getCompilerSourceVM() != null) {
-            javac.setSource(options.getCompilerSourceVM());
-            info.append("   compilerSourceVM=" + options.getCompilerSourceVM() + "\n");
+            javaCompiler.setSourceVM(options.getCompilerSourceVM());
         }
 
-        // Build includes path
-        PatternSet.NameEntry includes = javac.createInclude();
+        // Start java compilation
+        JavacErrorDetail[] javacErrors =
+            javaCompiler.compile(ctxt.getFullClassName(), pageNodes);
 
-        includes.setName(ctxt.getJavaPath());
-        info.append("    include="+ ctxt.getJavaPath() + "\n" );
+        if (javacErrors != null) {
+            // If there are errors, always generate java files to disk.
+            javaCompiler.doJavaFile(true);
 
-        BuildException be = null;
-
-        try {
-            if (ctxt.getOptions().getFork()) {
-                javac.execute();
-            } else {
-                synchronized(javacLock) {
-                    javac.execute();
-                }
-            }
-        } catch (BuildException e) {
-            be = e;
-            log.error( "Javac exception ", e);
-            log.error( "Env: " + info.toString());
-        }
-
-        errorReport.append(logger.getReport());
-
-        // Stop capturing the System.err output for this thread
-        String errorCapture = SystemLogHandler.unsetThread();
-        if (errorCapture != null) {
-            errorReport.append(System.getProperty("line.separator"));
-            errorReport.append(errorCapture);
-        }
-
-        if (!ctxt.keepGenerated()) {
-            File javaFile = new File(javaFileName);
-            javaFile.delete();
-        }
-
-        if (be != null) {
-            String errorReportString = errorReport.toString();
-            log.error("Error compiling file: " + javaFileName + " "
-                      + errorReportString);
-            JavacErrorDetail[] javacErrors = ErrorDispatcher.parseJavacErrors(
-                        errorReportString, javaFileName, pageNodes);
-            if (javacErrors != null) {
-                errDispatcher.javacError(javacErrors);
-            } else {
-                errDispatcher.javacError(errorReportString, be);
-            }
+            log.error("Error compiling file: " + javaFileName);
+            errDispatcher.javacError(javacErrors);
         }
 
         if (log.isDebugEnabled()) {
@@ -469,42 +355,33 @@ public class Compiler {
             log.debug("Compiled " + javaFileName + " " + (t2-t1) + "ms");
         }
 
-	if (ctxt.isPrototypeMode()) {
-	    return;
-	}
+        // Save or delete the generated Java files, depending on the
+        // value of "keepgenerated" attribute
+        javaCompiler.doJavaFile(ctxt.keepGenerated());
 
         // JSR45 Support
-        if (! options.isSmapSuppressed()) {
-            SmapUtil.installSmap(smap);
+        if (!ctxt.isPrototypeMode() && !options.isSmapSuppressed()) {
+            smapUtil.installSmap();
         }
 
         // START CR 6373479
         if (jsw != null && jsw.getServletClassLastModifiedTime() <= 0) {
-            File targetFile = new File(ctxt.getClassFileName());
-            jsw.setServletClassLastModifiedTime(targetFile.lastModified());
+            jsw.setServletClassLastModifiedTime(
+                javaCompiler.getClassLastModified());
         }
         // END CR 6373479
-    }
 
-    /** 
-     * Compile the jsp file from the current engine context
-     */
-    public void compile()
-        throws FileNotFoundException, JasperException, Exception
-    {
-        compile(true);
-    }
+        if (options.getSaveBytecode()) {
+            javaCompiler.saveClassFile(ctxt.getFullClassName(),
+                                       ctxt.getClassFileName());
+        }
 
-    /**
-     * Compile the jsp file from the current engine context.  As an side-
-     * effect, tag files that are referenced by this page are also compiled.
-     * @param compileClass If true, generate both .java and .class file
-     *                     If false, generate only .java file
-     */
-    public void compile(boolean compileClass)
-        throws FileNotFoundException, JasperException, Exception
-    {
-        compile(compileClass, false);
+        // On some systems, due to file caching, the time stamp for the updated
+        // JSP file may actually be greater than that of the newly created byte
+        // codes in the cache.  In such cases, adjust the cache time stamp to
+        // JSP page time, to avoid unnecessary recompilations.
+        ctxt.getRuntimeContext().adjustBytecodeTime(ctxt.getFullClassName(),
+                                                    jspModTime);
     }
 
     /**
@@ -513,19 +390,30 @@ public class Compiler {
      *
      * @param compileClass If true, generate both .java and .class file
      *                     If false, generate only .java file
-     * @param jspcMode true if invoked from JspC, false otherwise
      */
-    public void compile(boolean compileClass, boolean jspcMode)
+    public void compile(boolean compileClass)
         throws FileNotFoundException, JasperException, Exception
     {
-        if (errDispatcher == null) {
-            this.errDispatcher = new ErrorDispatcher(jspcMode);
-        }
-
         try {
-            String[] smap = generateJava();
+            // Create the output directory for the generated files
+            // Always try and create the directory tree, in case the generated
+            // directories were deleted after the server was started.
+            ctxt.getOutputDir();
+            ctxt.makeOutputDir();
+
+            // If errDispatcher is nulled from a previous compilation of the
+            // same page, instantiate one here.
+            if (errDispatcher == null) {
+                errDispatcher = new ErrorDispatcher(jspcMode);
+            }
+            generateJava();
             if (compileClass) {
-                generateClass(smap);
+                generateClass();
+            }
+            else {
+                // If called from jspc to only compile to .java files,
+                // make sure that .java files are written to disk.
+                javaCompiler.doJavaFile(ctxt.keepGenerated());
             }
         } finally {
             if (tfp != null) {
@@ -537,9 +425,9 @@ public class Compiler {
             // memory footprint.
             tfp = null;
             errDispatcher = null;
-            logger = null;
-            project = null;
-            pageInfo = null;
+            if (!jspcMode) {
+                pageInfo = null;
+            }
             pageNodes = null;
             if (ctxt.getWriter() != null) {
                 ctxt.getWriter().close();
@@ -583,6 +471,52 @@ public class Compiler {
         }
 
         long jspRealLastModified = 0;
+        // START PWC 6468930
+        File targetFile;
+        
+        if (checkClass) {
+            targetFile = new File(ctxt.getClassFileName());
+        } else {
+            targetFile = new File(ctxt.getServletJavaFileName());
+        }
+        
+        // Get the target file's last modified time. File.lastModified()
+        // returns 0 if the file does not exist.
+        long targetLastModified = targetFile.lastModified();
+
+        // Check cached class file
+        if (checkClass) {
+            JspRuntimeContext rtctxt = ctxt.getRuntimeContext();
+            String className = ctxt.getFullClassName();
+            long cachedTime = rtctxt.getBytecodeBirthTime(className);
+            if (cachedTime > targetLastModified) {
+                targetLastModified = cachedTime;
+            } else {
+                // Remove from cache, since the bytecodes from the file is more
+                // current, so that JasperLoader won't load the cached version
+                rtctxt.setBytecode(className, null);
+            }
+        }
+
+        if (targetLastModified == 0L)
+            return true;
+
+        // Check if the jsp exists in the filesystem (instead of a jar
+        // or a remote location). If yes, then do a File.lastModified()
+        // to determine its last modified time. This is more performant 
+        // (fewer stat calls) than the ctxt.getResource() followed by 
+        // openConnection(). However, it only works for file system jsps.
+        // If the file has indeed changed, then need to call URL.OpenConnection() 
+        // so that the cache loads the latest jsp file
+        if (jsw != null) {
+            File jspFile = jsw.getJspFile();
+            if (jspFile != null) {
+                jspRealLastModified = jspFile.lastModified();
+            }
+        }
+        if (jspRealLastModified == 0 ||
+            targetLastModified < jspRealLastModified) {
+        // END PWC 6468930
         try {
             URL jspUrl = ctxt.getResource(jsp);
             if (jspUrl == null) {
@@ -596,7 +530,10 @@ public class Compiler {
             e.printStackTrace();
             return true;
         }
-
+        // START PWC 6468930
+        }
+        // END PWC 6468930
+        /* PWC 6468930
         long targetLastModified = 0;
         File targetFile;
         
@@ -611,10 +548,14 @@ public class Compiler {
         }
 
         targetLastModified = targetFile.lastModified();
+        */
         if (checkClass && jsw != null) {
             jsw.setServletClassLastModifiedTime(targetLastModified);
         }
+
         if (targetLastModified < jspRealLastModified) {
+            // Remember JSP mod time
+            jspModTime = jspRealLastModified;
             if( log.isDebugEnabled() ) {
                 log.debug("Compiler: outdated: " + targetFile + " " +
                     targetLastModified );
@@ -647,6 +588,12 @@ public class Compiler {
                 includeUconn.getInputStream().close();
 
                 if (includeLastModified > targetLastModified) {
+                    // START GlassFish 750
+                    if (include.endsWith(".tld")) {
+                        ctxt.clearTaglibs();
+                        ctxt.clearTagFileJarUrls();
+                    }
+                    // END GlassFish 750
                     return true;
                 }
             } catch (Exception e) {
@@ -676,6 +623,14 @@ public class Compiler {
     }
 
 
+    /**
+     * Sets the info about the page under compilation
+     */
+    public void setPageInfo(PageInfo pageInfo) {
+	this.pageInfo = pageInfo;
+    }
+
+
     public JspCompilationContext getCompilationContext() {
 	return ctxt;
     }
@@ -722,4 +677,54 @@ public class Compiler {
             // Remove as much as possible, ignore possible exceptions
         }
     }
+
+    /**
+     * Get an instance of JavaCompiler.
+     * If Running with Mustang (JDK1.6), use a Jsr199JavaCompiler that
+     * supports JSR199,
+     * else if eclipse's JDT compiler is avalable, use that.
+     * The default is to use javac from ant.
+     * NOTE: When the appserver can be built with and runs only with JDK1.6,
+     * this should be changed to instantiate Jsr199JavaCompiler with new
+     * operator directly.
+     */
+    private void initJavaCompiler() {
+        Class c = getClassFor("javax.tools.ToolProvider");
+        if (c != null) {
+            // JDK1.6
+            c = getClassFor("org.apache.jasper.compiler.Jsr199JavaCompiler");
+            if (c != null) {
+                try {
+                    javaCompiler = (JavaCompiler) c.newInstance();
+                } catch (Exception ex) {
+                }
+            }
+        }
+        if (javaCompiler == null) {
+            c = getClassFor("org.eclipse.jdt.internal.compiler.Compiler");
+            if (c != null) {
+                c = getClassFor("org.apache.jasper.compiler.JDTJavaCompiler");
+                if (c != null) {
+                    try {
+                        javaCompiler = (JavaCompiler) c.newInstance();
+                    } catch (Exception ex) {
+                    }
+                }
+            }
+        }
+        if (javaCompiler == null) {
+            javaCompiler = new AntJavaCompiler();
+        }
+
+        javaCompiler.init(ctxt, errDispatcher, jspcMode);
+    }
+
+    private Class getClassFor(String className) {
+        Class c = null;
+        try {
+            c = Class.forName(className, false, getClass().getClassLoader());
+        } catch (ClassNotFoundException ex) {
+        }
+        return c;
+    }
 }
