@@ -1,29 +1,40 @@
-
-
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
- */ 
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
+ */
 
 package org.apache.jasper.runtime;
 
@@ -39,6 +50,9 @@ import org.apache.jasper.Constants;
 import org.apache.jasper.compiler.Localizer;
 import org.apache.jasper.security.SecurityUtil;
 
+import com.sun.enterprise.web.io.ByteWriter;
+
+
 /**
  * Write text to a character-output stream, buffering characters so as
  * to provide for the efficient writing of single characters, arrays,
@@ -48,9 +62,14 @@ import org.apache.jasper.security.SecurityUtil;
  * buffered. 
  * 
  * This needs revisiting when the buffering problems in the JSP spec
- * are fixed -akv 
+ * are fixed -akv  What buffering problems? -kmc
+ *
+ * Add method for writing bytes.  This allows static texts to be pre-encoded,
+ * for performance.  Note that this can be done only if the page is unbuffered.
+ * -kmc
  *
  * @author Anil K. Vijendran
+ * @author Kin-man Chung
  */
 public class JspWriterImpl extends JspWriter {
 
@@ -60,6 +79,8 @@ public class JspWriterImpl extends JspWriter {
     private int nextChar;
     private boolean flushed = false;
     private boolean closed = false;
+    protected boolean implementsByteWriter = true;
+    protected ByteWriter byteOut;
     
     public JspWriterImpl() {
 	super( Constants.DEFAULT_BUFFER_SIZE, true );
@@ -123,6 +144,7 @@ public class JspWriterImpl extends JspWriter {
 	flushed = false;
         closed = false;
         out = null;
+        byteOut = null;
 	nextChar = 0;
         response = null;
     }
@@ -198,6 +220,12 @@ public class JspWriterImpl extends JspWriter {
         if (out != null) {
             out.flush();
         }
+        // START 6426898
+        else {
+            // Cause response headers to be sent
+            response.flushBuffer();
+        }
+        // END 6426898
     }
 
     /**
@@ -212,8 +240,8 @@ public class JspWriterImpl extends JspWriter {
         if (out != null)
             out.close();
         out = null;
+        byteOut = null;
         closed = true;
-        //            cb = null;
     }
 
     /**
@@ -230,6 +258,60 @@ public class JspWriterImpl extends JspWriter {
     }
 
 
+    /**
+     * Attempt to write a String pre-encoded with the page encoding.
+     *
+     * @param bytesOK If true, write out the byte array,
+     *                else, write out the String.
+     * @param buf     The text encoded with the page encoding
+     * @param str     The original text
+     */
+    public void write(boolean bytesOK, byte buf[], String str)
+            throws IOException {
+
+        ensureOpen();
+        if (bufferSize == 0 && bytesOK) {
+            initByteOut();
+            if (implementsByteWriter) {
+                write(buf, 0, buf.length, str.length());
+                return;
+            }
+        }
+        write(str);
+    }
+
+    /* Returns true if bytes should be outputted.
+     * Used by ServletResponseWrapperInclude.
+    */
+    boolean shouldOutputBytes() {
+        if (bufferSize > 0) {
+            return false;
+        }
+        try {
+            initByteOut();
+        } catch (IOException ex) {
+        }
+        return implementsByteWriter;
+    }
+
+    private void initByteOut() throws IOException {
+        initOut();
+        if (byteOut == null) {
+            try {
+                byteOut = (ByteWriter) out;
+                implementsByteWriter = true;
+            } catch (ClassCastException ex) {
+                implementsByteWriter = false;
+            }
+        }
+    }
+
+    public void write(byte buf[], int off, int len, int strlen)
+            throws IOException {
+        byteOut.write(buf, off, len, strlen);
+    }
+
+
     /**
      * Write a single character.
      */
@@ -361,11 +443,7 @@ public class JspWriterImpl extends JspWriter {
     public void write(String s) throws IOException {
         // Simple fix for Bugzilla 35410
         // Calling the other write function so as to init the buffer anyways
-        if(s == null) {
-            write(s, 0, 0);
-        } else {
-            write(s, 0, s.length());
-        }
+        write(s, 0, (s != null) ? s.length() : 0);
     }
 
 
@@ -612,4 +690,14 @@ public class JspWriterImpl extends JspWriter {
         println();
     }
 
+
+    // START PWC 6512276
+    public boolean hasData() {
+        if (bufferSize != 0 && nextChar != 0) {
+            return true;
+        }
+
+        return false;
+    }
+    // END PWC 6512276
 }
