@@ -1,28 +1,41 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package org.apache.jasper.runtime;
@@ -33,8 +46,9 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
+import java.util.Collections;
 import java.util.Enumeration;
-import java.util.Hashtable;
+import java.util.HashMap;
 import java.util.Iterator;
 
 import javax.servlet.Servlet;
@@ -94,8 +108,6 @@ public class PageContextImpl extends PageContext {
     // Logger
     private static Log log = LogFactory.getLog(PageContextImpl.class);
 
-    static ExpressionFactory expFactory;
-
     // per-servlet state
     private BodyContentImpl[] outs;
     private int depth;
@@ -105,25 +117,23 @@ public class PageContextImpl extends PageContext {
     private JspFactory factory;
     private boolean needsSession;
     private String errorPageURL;
-    private boolean autoFlush;
     private int	bufferSize;
     private JspApplicationContextImpl jspApplicationContext;
     private ELResolver elResolver;
     private ELContext elContext;
 
     // page-scope attributes
-    private transient Hashtable	attributes;
+    private HashMap attributes;
+    private boolean isNametableInitialized;
 
     // per-request state
-    private transient ServletRequest request;
-    private transient ServletResponse response;
-    private transient Object page;
-    private transient HttpSession session;
-    private boolean isIncluded;
+    private ServletRequest request;
+    private ServletResponse response;
+    private HttpSession session;
 
     // initial output stream
-    private transient JspWriter out;
-    private transient JspWriterImpl baseOut;
+    private JspWriter out;
+    private JspWriterImpl baseOut;
 
     /*
      * Constructor.
@@ -131,7 +141,7 @@ public class PageContextImpl extends PageContext {
     PageContextImpl(JspFactory factory) {
         this.factory = factory;
 	this.outs = new BodyContentImpl[0];
-	this.attributes = new Hashtable(16);
+	this.attributes = new HashMap(16);
 	this.depth = -1;
     }
 
@@ -162,10 +172,8 @@ public class PageContextImpl extends PageContext {
 	this.needsSession = needsSession;
 	this.errorPageURL = errorPageURL;
 	this.bufferSize = bufferSize;
-	this.autoFlush = autoFlush;
 	this.request = request;
  	this.response = response;
-        expFactory = JspApplicationContextImpl.expressionFactory;
 
 	// Setup session (if required)
 	if (request instanceof HttpServletRequest && needsSession)
@@ -183,38 +191,34 @@ public class PageContextImpl extends PageContext {
         }
         this.out = baseOut;
 
-	// register names/values as per spec
-	setAttribute(OUT, this.out);
-	setAttribute(REQUEST, request);
-	setAttribute(RESPONSE, response);
-
-	if (session != null)
-	    setAttribute(SESSION, session);
-
-	setAttribute(PAGE, servlet);
-	setAttribute(CONFIG, config);
-	setAttribute(PAGECONTEXT, this);
-	setAttribute(APPLICATION, context);
+        this.isNametableInitialized = false;
         setAttribute(Constants.FIRST_REQUEST_SEEN, "true", APPLICATION_SCOPE);
 
-	isIncluded = request.getAttribute(
-	    "javax.servlet.include.servlet_path") != null;
+    }
+
+    private void initializePageScopeNameTable() {
+        isNametableInitialized = true;
+        // register names/values as per spec
+        setAttribute(OUT, this.out);
+        setAttribute(REQUEST, request);
+        setAttribute(RESPONSE, response);
+
+        if (session != null)
+            setAttribute(SESSION, session);
+
+        setAttribute(PAGE, servlet);
+        setAttribute(CONFIG, config);
+        setAttribute(PAGECONTEXT, this);
+        setAttribute(APPLICATION, context);
     }
 
     public void release() {
         out = baseOut;
 	try {
-	    if (isIncluded) {
-		((JspWriterImpl)out).flushBuffer();
-                // push it into the including jspWriter
-	    } else {
-                // Old code:
-                //out.flush();
-                // Do not flush the buffer even if we're not included (i.e.
-                // we are the main page. The servlet will flush it and close
-                // the stream.
-                ((JspWriterImpl)out).flushBuffer();
-            }
+            // Do not flush the buffer even if we're not included (i.e.
+            // we are the main page. The servlet will flush it and close
+            // the stream.
+            ((JspWriterImpl)out).flushBuffer();
 	} catch (IOException ex) {
 	    log.warn("Internal error flushing the buffer in release()");
 	}
@@ -228,7 +232,6 @@ public class PageContextImpl extends PageContext {
 	needsSession = false;
 	errorPageURL = null;
 	bufferSize = JspWriter.DEFAULT_BUFFER;
-	autoFlush = true;
 	request = null;
 	response = null;
         depth = -1;
@@ -258,6 +261,9 @@ public class PageContextImpl extends PageContext {
     }
 
     private Object doGetAttribute(String name){
+        if (!isNametableInitialized) {
+            initializePageScopeNameTable();
+        }
         return attributes.get(name);
     }
 
@@ -283,6 +289,9 @@ public class PageContextImpl extends PageContext {
     private Object doGetAttribute(String name, int scope){
         switch (scope) {
             case PAGE_SCOPE:
+                if (!isNametableInitialized) {
+                    initializePageScopeNameTable();
+                }
                 return attributes.get(name);
 
             case REQUEST_SCOPE:
@@ -324,6 +333,9 @@ public class PageContextImpl extends PageContext {
 
     private void doSetAttribute(String name, Object attribute){
         if (attribute != null) {
+            if (!isNametableInitialized) {
+                initializePageScopeNameTable();
+            }
             attributes.put(name, attribute);
         } else {
             removeAttribute(name, PAGE_SCOPE);
@@ -354,6 +366,9 @@ public class PageContextImpl extends PageContext {
         if (o != null) {
             switch (scope) {
             case PAGE_SCOPE:
+                if (!isNametableInitialized) {
+                    initializePageScopeNameTable();
+                }
                 attributes.put(name, o);
                 break;
 
@@ -402,6 +417,9 @@ public class PageContextImpl extends PageContext {
     private void doRemoveAttribute(String name, int scope){
         switch (scope) {
         case PAGE_SCOPE:
+            if (!isNametableInitialized) {
+                initializePageScopeNameTable();
+            }
             attributes.remove(name);
             break;
 
@@ -436,7 +454,7 @@ public class PageContextImpl extends PageContext {
         if (SecurityUtil.isPackageProtectionEnabled()){
             return ((Integer)AccessController.doPrivileged(new PrivilegedAction(){
                 public Object run(){
-                    return new Integer(doGetAttributeScope(name));
+                    return Integer.valueOf(doGetAttributeScope(name));
                 }
             })).intValue();
         } else {
@@ -445,6 +463,11 @@ public class PageContextImpl extends PageContext {
     }
 
     private int doGetAttributeScope(String name){
+
+        if (!isNametableInitialized) {
+            initializePageScopeNameTable();
+        }
+
         if (attributes.get(name) != null)
             return PAGE_SCOPE;
 
@@ -491,6 +514,10 @@ public class PageContextImpl extends PageContext {
 
     private Object doFindAttribute(String name){
 
+        if (!isNametableInitialized) {
+            initializePageScopeNameTable();
+        }
+
         Object o = attributes.get(name);
         if (o != null)
             return o;
@@ -530,7 +557,10 @@ public class PageContextImpl extends PageContext {
     private Enumeration<String> doGetAttributeNamesInScope(int scope){
         switch (scope) {
         case PAGE_SCOPE:
-            return attributes.keys();
+            if (!isNametableInitialized) {
+                initializePageScopeNameTable();
+            }
+            return Collections.enumeration(attributes.keySet());
             
         case REQUEST_SCOPE:
             return request.getAttributeNames();
@@ -642,30 +672,12 @@ public class PageContextImpl extends PageContext {
 				  true);
     }
 
-    public void include(final String relativeUrlPath, final boolean flush)
-	        throws ServletException, IOException {
-        if (SecurityUtil.isPackageProtectionEnabled()){
-            try{
-                AccessController.doPrivileged(new PrivilegedExceptionAction(){
-                    public Object run() throws Exception{
-                        doInclude(relativeUrlPath, flush);
-                        return null;
-                    }
-                });
-            } catch (PrivilegedActionException e){
-                Exception ex =  e.getException();
-                if (ex instanceof IOException){
-                    throw (IOException)ex;
-                } else {
-                    throw (ServletException)ex;
-                }
-            }
-        } else {
-            doInclude(relativeUrlPath, flush);
-        }
-    }
-
-    private void doInclude(String relativeUrlPath, boolean flush)
+    /*
+     * No need to execute include inside a privileged block, since it calls
+     * ApplicationDispatcher include, which in turn is executed inside a
+     * privileged block
+     */
+    public void include(String relativeUrlPath, boolean flush)
 	        throws ServletException, IOException {
         JspRuntimeLibrary.include(request, response, relativeUrlPath, out,
 				  flush);
@@ -683,8 +695,7 @@ public class PageContextImpl extends PageContext {
                                                                                 
             celResolver.add(new ImplicitObjectELResolver());
             // Add ELResolvers registered in JspApplicationContext
-            JspApplicationContextImpl jaContext =
-                (JspApplicationContextImpl) getJspApplicationContext();
+            JspApplicationContextImpl jaContext = getJspApplicationContext();
             Iterator<ELResolver> it = jaContext.getELResolvers();
             while (it.hasNext()) {
                 celResolver.add(it.next());
@@ -719,31 +730,12 @@ public class PageContextImpl extends PageContext {
         return jspApplicationContext;
     }
                          
-    public void forward(final String relativeUrlPath)
-            throws ServletException, IOException {
-
-        if (SecurityUtil.isPackageProtectionEnabled()){
-            try{
-                AccessController.doPrivileged(new PrivilegedExceptionAction(){
-                    public Object run() throws Exception{
-                        doForward(relativeUrlPath);
-                        return null;
-                    }
-                });
-            } catch (PrivilegedActionException e){
-                Exception ex =  e.getException();
-                if (ex instanceof IOException){
-                    throw (IOException)ex;
-                } else {
-                    throw (ServletException)ex;
-                }
-            }
-        } else {
-            doForward(relativeUrlPath);
-        }
-    }
-
-    private void doForward(String relativeUrlPath)
+    /*
+     * No need to execute forward inside a privileged block, since it calls
+     * ApplicationDispatcher forward, which in turn is executed inside a
+     * privileged block
+     */
+    public void forward(String relativeUrlPath)
         throws ServletException, IOException{
 
         // JSP.4.5 If the buffer was flushed, throw IllegalStateException
@@ -881,7 +873,7 @@ public class PageContextImpl extends PageContext {
              */
             request.setAttribute("javax.servlet.jsp.jspException", t);
             request.setAttribute("javax.servlet.error.status_code",
-                new Integer(HttpServletResponse.SC_INTERNAL_SERVER_ERROR));
+                Integer.valueOf(HttpServletResponse.SC_INTERNAL_SERVER_ERROR));
             request.setAttribute("javax.servlet.error.request_uri",
             ((HttpServletRequest) request).getRequestURI());
             request.setAttribute("javax.servlet.error.servlet_name",
@@ -914,13 +906,13 @@ public class PageContextImpl extends PageContext {
             // to get a stack trace for the real problem
             if (t instanceof IOException) throw (IOException)t;
             if (t instanceof ServletException) throw (ServletException)t;
-                if (t instanceof RuntimeException) throw (RuntimeException)t;
+            if (t instanceof RuntimeException) throw (RuntimeException)t;
 
             Throwable rootCause = null;
-                if (t instanceof JspException) {
-                    rootCause = ((JspException) t).getRootCause();
-                } else if (t instanceof ELException) {
-                    rootCause = t.getCause();
+            if (t instanceof JspException) {
+                rootCause = ((JspException) t).getRootCause();
+            } else if (t instanceof ELException) {
+                rootCause = t.getCause();
             }
 
             if (rootCause != null) {
@@ -931,6 +923,14 @@ public class PageContextImpl extends PageContext {
         }
     }
 
+    private static ExpressionFactory getExpressionFactory(
+            PageContext pageContext) {
+
+        PageContextImpl pc =
+            (PageContextImpl) JspContextWrapper.getRootPageContext(pageContext);
+        return pc.getJspApplicationContext().getExpressionFactory();
+    }
+
     /**
      * Evaluates an EL expression
      *
@@ -956,6 +956,7 @@ public class PageContextImpl extends PageContext {
                         ELContextImpl elContext =
                             (ELContextImpl) pageContext.getELContext();
                         elContext.setFunctionMapper(functionMap);
+                        ExpressionFactory expFactory = getExpressionFactory(pageContext);
                         ValueExpression expr =
                              expFactory.createValueExpression(
                                                    elContext,
@@ -975,6 +976,7 @@ public class PageContextImpl extends PageContext {
         } else {
             ELContextImpl elContext = (ELContextImpl)pageContext.getELContext();
             elContext.setFunctionMapper(functionMap);
+            ExpressionFactory expFactory = getExpressionFactory(pageContext);
             ValueExpression expr = expFactory.createValueExpression(
                                            elContext,
                                            expression,
@@ -992,6 +994,7 @@ public class PageContextImpl extends PageContext {
         // ELResolvers are not used in createValueExpression
         ELContextImpl elctxt = (ELContextImpl)pageContext.getELContext();
         elctxt.setFunctionMapper(functionMap);
+        ExpressionFactory expFactory = getExpressionFactory(pageContext);
         return expFactory.createValueExpression(elctxt,
                                                 expression,
                                                 expectedType);
@@ -1006,6 +1009,7 @@ public class PageContextImpl extends PageContext {
                                                                                 
         ELContextImpl elctxt = (ELContextImpl)pageContext.getELContext();
         elctxt.setFunctionMapper(functionMap);
+        ExpressionFactory expFactory = getExpressionFactory(pageContext);
         return expFactory.createMethodExpression(
                                     elctxt,
                                     expression,
@@ -1023,6 +1027,7 @@ public class PageContextImpl extends PageContext {
     public static void setMethodVariable(PageContext pageContext,
                                          String variable,
                                          MethodExpression expression) {
+        ExpressionFactory expFactory = getExpressionFactory(pageContext);
         ValueExpression exp =
             expFactory.createValueExpression(expression, Object.class);
         setValueVariable(pageContext, variable, exp);
