@@ -1,36 +1,50 @@
 
 
 /*
- * The contents of this file are subject to the terms
- * of the Common Development and Distribution License
- * (the "License").  You may not use this file except
- * in compliance with the License.
- *
- * You can obtain a copy of the license at
- * glassfish/bootstrap/legal/CDDLv1.0.txt or
- * https://glassfish.dev.java.net/public/CDDLv1.0.html.
- * See the License for the specific language governing
- * permissions and limitations under the License.
- *
- * When distributing Covered Code, include this CDDL
- * HEADER in each file and include the License file at
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
- * add the following below this CDDL HEADER, with the
- * fields enclosed by brackets "[]" replaced with your
- * own identifying information: Portions Copyright [yyyy]
- * [name of copyright owner]
- *
- * Copyright 2005 Sun Microsystems, Inc. All rights reserved.
- *
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
+ * 
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
+ * 
  * Portions Copyright Apache Software Foundation.
+ * 
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 package org.apache.tomcat.util.buf;
 
-import java.text.*;
-import java.util.*;
+
 import java.io.Serializable;
 import java.io.IOException;
+import java.text.DateFormat;
+import java.util.Date;
 
 /**
  * This class is used to represent a subarray of bytes in an HTTP message.
@@ -47,25 +61,25 @@ import java.io.IOException;
 public final class MessageBytes implements Cloneable, Serializable {
     // primary type ( whatever is set as original value )
     private int type = T_NULL;
-
+    
     public static final int T_NULL = 0;
     /** getType() is T_STR if the the object used to create the MessageBytes
-        was a String */
+     * was a String */
     public static final int T_STR  = 1;
     /** getType() is T_STR if the the object used to create the MessageBytes
-        was a byte[] */ 
+     * was a byte[] */
     public static final int T_BYTES = 2;
     /** getType() is T_STR if the the object used to create the MessageBytes
-        was a char[] */ 
+     * was a char[] */
     public static final int T_CHARS = 3;
-
+    
     private int hashCode=0;
-    // did we computed the hashcode ? 
+    // did we computed the hashcode ?
     private boolean hasHashCode=false;
-
+    
     // Is the represented object case sensitive ?
     private boolean caseSensitive=true;
-
+    
     // Internal objects to represent array + offset, and specific methods
     private ByteChunk byteC=new ByteChunk();
     private CharChunk charC=new CharChunk();
@@ -75,7 +89,7 @@ public final class MessageBytes implements Cloneable, Serializable {
     // true if a String value was computed. Probably not needed,
     // strValue!=null is the same
     private boolean hasStrValue=false;
-
+    
     /**
      * Creates a new, uninitialized MessageBytes object.
      * @deprecated Use static newInstance() in order to allow
@@ -83,52 +97,52 @@ public final class MessageBytes implements Cloneable, Serializable {
      */
     public MessageBytes() {
     }
-
+    
     /** Construct a new MessageBytes instance
      */
     public static MessageBytes newInstance() {
-	return factory.newInstance();
+        return factory.newInstance();
     }
-
+    
     /** Configure the case sensitivity
      */
     public void setCaseSenitive( boolean b ) {
-	caseSensitive=b;
+        caseSensitive=b;
     }
-
+    
     public MessageBytes getClone() {
-	try {
-	    return (MessageBytes)this.clone();
-	} catch( Exception ex) {
-	    return null;
-	}
+        try {
+            return (MessageBytes)this.clone();
+        } catch( Exception ex) {
+            return null;
+        }
     }
-
+    
     public boolean isNull() {
 //		should we check also hasStrValue ???
-		return byteC.isNull() && charC.isNull() && ! hasStrValue;
-	// bytes==null && strValue==null;
+        return byteC.isNull() && charC.isNull() && ! hasStrValue;
+        // bytes==null && strValue==null;
     }
     
     /**
      * Resets the message bytes to an uninitialized (NULL) state.
      */
     public void recycle() {
-	type=T_NULL;
-	byteC.recycle();
-	charC.recycle();
-
-	strValue=null;
-	caseSensitive=true;
-
-	hasStrValue=false;
-	hasHashCode=false;
-	hasIntValue=false;
-    hasLongValue=false;
-	hasDateValue=false;	
+        type=T_NULL;
+        byteC.recycle();
+        charC.recycle();
+        
+        strValue=null;
+        caseSensitive=true;
+        
+        hasStrValue=false;
+        hasHashCode=false;
+        hasIntValue=false;
+        hasLongValue=false;
+        hasDateValue=false;
     }
-
-
+    
+    
     /**
      * Sets the content to the specified subarray of bytes.
      *
@@ -143,23 +157,23 @@ public final class MessageBytes implements Cloneable, Serializable {
         hasHashCode=false;
         hasIntValue=false;
         hasLongValue=false;
-        hasDateValue=false; 
+        hasDateValue=false;
     }
-
+    
     /** Set the encoding. If the object was constructed from bytes[]. any
      *  previous conversion is reset.
      *  If no encoding is set, we'll use 8859-1.
      */
     public void setEncoding( String enc ) {
-	if( !byteC.isNull() ) {
-	    // if the encoding changes we need to reset the converion results
-	    charC.recycle();
-	    hasStrValue=false;
-	}
-	byteC.setEncoding(enc);
+        if( !byteC.isNull() ) {
+            // if the encoding changes we need to reset the converion results
+            charC.recycle();
+            hasStrValue=false;
+        }
+        byteC.setEncoding(enc);
     }
-
-    /** 
+    
+    /**
      * Sets the content to be a char[]
      *
      * @param c the bytes
@@ -173,62 +187,65 @@ public final class MessageBytes implements Cloneable, Serializable {
         hasHashCode=false;
         hasIntValue=false;
         hasLongValue=false;
-        hasDateValue=false; 
+        hasDateValue=false;
     }
-
+    
     /** Remove the cached string value. Use it after a conversion on the
      *	byte[] or after the encoding is changed
      *  XXX Is this needed ?
      */
     public void resetStringValue() {
-	if( type != T_STR ) {
-	    // If this was cread as a byte[] or char[], we remove
-	    // the old string value
-	    hasStrValue=false;
-	    strValue=null;
-	}
+        if( type != T_STR ) {
+            // If this was cread as a byte[] or char[], we remove
+            // the old string value
+            hasStrValue=false;
+            strValue=null;
+        }
     }
-
-    /** 
+    
+    /**
      * Set the content to be a string
      */
     public void setString( String s ) {
-        if (s == null)
-            return;
         strValue=s;
-        hasStrValue=true;
         hasHashCode=false;
         hasIntValue=false;
         hasLongValue=false;
-        hasDateValue=false; 
-        type=T_STR;
+        hasDateValue=false;
+        if (s == null) {
+            hasStrValue=false;
+            type=T_NULL;
+        } else {
+            hasStrValue=true;
+            type=T_STR;
+        }
     }
-
+    
     // -------------------- Conversion and getters --------------------
-
+    
     /** Compute the string value
      */
     public String toString() {
-	if( hasStrValue ) return strValue;
-	hasStrValue=true;
-	
-	switch (type) {
-	case T_CHARS:
-	    strValue=charC.toString();
-	    return strValue;
-	case T_BYTES:
-	    strValue=byteC.toString();
-	    return strValue;
-	}
-	return null;
+        if( hasStrValue ) return strValue;
+        hasStrValue=true;
+        
+        switch (type) {
+            case T_CHARS:
+                strValue=charC.toString();
+                return strValue;
+            case T_BYTES:
+                strValue=byteC.toString();
+                return strValue;
+        }
+        return null;
     }
-
+    
     //----------------------------------------
     /** Return the type of the original content. Can be
      *  T_STR, T_BYTES, T_CHARS or T_NULL
      */
     public int getType() {
-	return type;
+        return type;
     }
     
     /**
@@ -236,25 +253,25 @@ public final class MessageBytes implements Cloneable, Serializable {
      * Valid only if T_BYTES or after a conversion was made.
      */
     public ByteChunk getByteChunk() {
-	return byteC;
+        return byteC;
     }
-
+    
     /**
      * Returns the char chunk, representing the char[] and offset/length.
      * Valid only if T_CHARS or after a conversion was made.
      */
     public CharChunk getCharChunk() {
-	return charC;
+        return charC;
     }
-
+    
     /**
      * Returns the string value.
      * Valid only if T_STR or after a conversion was made.
      */
     public String getString() {
-	return strValue;
+        return strValue;
     }
-
+    
     /** Unimplemented yet. Do a char->byte conversion.
      */
     public void toBytes() {
@@ -267,235 +284,245 @@ public final class MessageBytes implements Cloneable, Serializable {
         byte bb[] = strValue.getBytes();
         byteC.setBytes(bb, 0, bb.length);
     }
-
+    
     /** Convert to char[] and fill the CharChunk.
      *  XXX Not optimized - it converts to String first.
      */
     public void toChars() {
-	if( ! charC.isNull() ) {
+        if( ! charC.isNull() ) {
             type=T_CHARS;
-	    return;
-	}
-	// inefficient
-	toString();
+            return;
+        }
+        // inefficient
+        toString();
         type=T_CHARS;
-	char cc[]=strValue.toCharArray();
-	charC.setChars(cc, 0, cc.length);
+        char cc[]=strValue.toCharArray();
+        charC.setChars(cc, 0, cc.length);
     }
     
-
+    
     /**
      * Returns the length of the original buffer.
      * Note that the length in bytes may be different from the length
      * in chars.
      */
     public int getLength() {
-	if(type==T_BYTES)
-	    return byteC.getLength();
-	if(type==T_CHARS) {
-	    return charC.getLength();
-	}
-	if(type==T_STR)
-	    return strValue.length();
-	toString();
-	if( strValue==null ) return 0;
-	return strValue.length();
+        if(type==T_BYTES)
+            return byteC.getLength();
+        if(type==T_CHARS) {
+            return charC.getLength();
+        }
+        if(type==T_STR)
+            return strValue.length();
+        toString();
+        if( strValue==null ) return 0;
+        return strValue.length();
     }
-
+    
     // -------------------- equals --------------------
-
+    
     /**
      * Compares the message bytes to the specified String object.
      * @param s the String to compare
      * @return true if the comparison succeeded, false otherwise
      */
     public boolean equals(String s) {
-	if( ! caseSensitive )
-	    return equalsIgnoreCase( s );
-	switch (type) {
-	case T_STR:
-	    if( strValue==null && s!=null) return false;
-	    return strValue.equals( s );
-	case T_CHARS:
-	    return charC.equals( s );
-	case T_BYTES:
-	    return byteC.equals( s );
-	default:
-	    return false;
-	}
-    }
+        if( s == null )
+            return false;
 
+        if( ! caseSensitive )
+            return equalsIgnoreCase( s );
+        switch (type) {
+            case T_STR:
+                if( strValue==null && s!=null) return false;
+                return strValue.equals( s );
+            case T_CHARS:
+                return charC.equals( s );
+            case T_BYTES:
+                return byteC.equals( s );
+            default:
+                return false;
+        }
+    }
+    
     /**
      * Compares the message bytes to the specified String object.
      * @param s the String to compare
      * @return true if the comparison succeeded, false otherwise
      */
     public boolean equalsIgnoreCase(String s) {
-	switch (type) {
-	case T_STR:
-	    if( strValue==null && s!=null) return false;
-	    return strValue.equalsIgnoreCase( s );
-	case T_CHARS:
-	    return charC.equalsIgnoreCase( s );
-	case T_BYTES:
-	    return byteC.equalsIgnoreCase( s );
-	default:
-	    return false;
-	}
+        switch (type) {
+            case T_STR:
+                if( strValue==null && s!=null) return false;
+                return strValue.equalsIgnoreCase( s );
+            case T_CHARS:
+                return charC.equalsIgnoreCase( s );
+            case T_BYTES:
+                return byteC.equalsIgnoreCase( s );
+            default:
+                return false;
+        }
     }
-
-    public boolean equals(MessageBytes mb) {
-	switch (type) {
-	case T_STR:
-	    return mb.equals( strValue );
-	}
-
-	if( mb.type != T_CHARS &&
-	    mb.type!= T_BYTES ) {
-	    // it's a string or int/date string value
-	    return equals( mb.toString() );
-	}
-
-	// mb is either CHARS or BYTES.
-	// this is either CHARS or BYTES
-	// Deal with the 4 cases ( in fact 3, one is simetric)
-	
-	if( mb.type == T_CHARS && type==T_CHARS ) {
-	    return charC.equals( mb.charC );
-	} 
-	if( mb.type==T_BYTES && type== T_BYTES ) {
-	    return byteC.equals( mb.byteC );
-	}
-	if( mb.type== T_CHARS && type== T_BYTES ) {
-	    return byteC.equals( mb.charC );
-	}
-	if( mb.type== T_BYTES && type== T_CHARS ) {
-	    return mb.byteC.equals( charC );
-	}
-	// can't happen
-	return true;
+    
+    public boolean equals(Object obj) {
+        
+        if (obj == null || (!(obj instanceof MessageBytes))) {
+            return false;
+        }
+        
+        MessageBytes mb = (MessageBytes) obj;
+        
+        switch (type) {
+            case T_STR:
+                return mb.equals( strValue );
+        }
+        
+        if( mb.type != T_CHARS &&
+                mb.type!= T_BYTES ) {
+            // it's a string or int/date string value
+            return equals( mb.toString() );
+        }
+        
+        // mb is either CHARS or BYTES.
+        // this is either CHARS or BYTES
+        // Deal with the 4 cases ( in fact 3, one is simetric)
+        
+        if( mb.type == T_CHARS && type==T_CHARS ) {
+            return charC.equals( mb.charC );
+        }
+        if( mb.type==T_BYTES && type== T_BYTES ) {
+            return byteC.equals( mb.byteC );
+        }
+        if( mb.type== T_CHARS && type== T_BYTES ) {
+            return byteC.equals( mb.charC );
+        }
+        if( mb.type== T_BYTES && type== T_CHARS ) {
+            return mb.byteC.equals( charC );
+        }
+        // can't happen
+        return true;
     }
-
+    
     
     /**
      * Returns true if the message bytes starts with the specified string.
      * @param s the string
      */
     public boolean startsWith(String s) {
-	switch (type) {
-	case T_STR:
-	    return strValue.startsWith( s );
-	case T_CHARS:
-	    return charC.startsWith( s );
-	case T_BYTES:
-	    return byteC.startsWith( s );
-	default:
-	    return false;
-	}
+        switch (type) {
+            case T_STR:
+                return strValue.startsWith( s );
+            case T_CHARS:
+                return charC.startsWith( s );
+            case T_BYTES:
+                return byteC.startsWith( s );
+            default:
+                return false;
+        }
     }
-
+    
     /**
      * Returns true if the message bytes starts with the specified string.
      * @param s the string
      * @param pos The start position
      */
     public boolean startsWithIgnoreCase(String s, int pos) {
-	switch (type) {
-	case T_STR:
-	    if( strValue==null ) return false;
-	    if( strValue.length() < pos + s.length() ) return false;
-	    
-	    for( int i=0; i<s.length(); i++ ) {
-		if( Ascii.toLower( s.charAt( i ) ) !=
-		    Ascii.toLower( strValue.charAt( pos + i ))) {
-		    return false;
-		}
-	    }
-	    return true;
-	case T_CHARS:
-	    return charC.startsWithIgnoreCase( s, pos );
-	case T_BYTES:
-	    return byteC.startsWithIgnoreCase( s, pos );
-	default:
-	    return false;
-	}
+        switch (type) {
+            case T_STR:
+                if( strValue==null ) return false;
+                if( strValue.length() < pos + s.length() ) return false;
+                
+                for( int i=0; i<s.length(); i++ ) {
+                    if( Ascii.toLower( s.charAt( i ) ) !=
+                            Ascii.toLower( strValue.charAt( pos + i ))) {
+                        return false;
+                    }
+                }
+                return true;
+            case T_CHARS:
+                return charC.startsWithIgnoreCase( s, pos );
+            case T_BYTES:
+                return byteC.startsWithIgnoreCase( s, pos );
+            default:
+                return false;
+        }
     }
-
     
-
+    
+    
     // -------------------- Hash code  --------------------
     public  int hashCode() {
-	if( hasHashCode ) return hashCode;
-	int code = 0;
-
-	if( caseSensitive ) 
-	    code=hash(); 
-	else
-	    code=hashIgnoreCase();
-	hashCode=code;
-	hasHashCode=true;
-	return code;
+        if( hasHashCode ) return hashCode;
+        int code = 0;
+        
+        if( caseSensitive )
+            code=hash();
+        else
+            code=hashIgnoreCase();
+        hashCode=code;
+        hasHashCode=true;
+        return code;
     }
-
-    // normal hash. 
+    
+    // normal hash.
     private int hash() {
-	int code=0;
-	switch (type) {
-	case T_STR:
-	    // We need to use the same hash function
-	    for (int i = 0; i < strValue.length(); i++) {
-		code = code * 37 + strValue.charAt( i );
-	    }
-	    return code;
-	case T_CHARS:
-	    return charC.hash();
-	case T_BYTES:
-	    return byteC.hash();
-	default:
-	    return 0;
-	}
+        int code=0;
+        switch (type) {
+            case T_STR:
+                // We need to use the same hash function
+                for (int i = 0; i < strValue.length(); i++) {
+                    code = code * 37 + strValue.charAt( i );
+                }
+                return code;
+            case T_CHARS:
+                return charC.hash();
+            case T_BYTES:
+                return byteC.hash();
+            default:
+                return 0;
+        }
     }
-
+    
     // hash ignoring case
     private int hashIgnoreCase() {
-	int code=0;
-	switch (type) {
-	case T_STR:
-	    for (int i = 0; i < strValue.length(); i++) {
-		code = code * 37 + Ascii.toLower(strValue.charAt( i ));
-	    }
-	    return code;
-	case T_CHARS:
-	    return charC.hashIgnoreCase();
-	case T_BYTES:
-	    return byteC.hashIgnoreCase();
-	default:
-	    return 0;
-	}
+        int code=0;
+        switch (type) {
+            case T_STR:
+                for (int i = 0; i < strValue.length(); i++) {
+                    code = code * 37 + Ascii.toLower(strValue.charAt( i ));
+                }
+                return code;
+            case T_CHARS:
+                return charC.hashIgnoreCase();
+            case T_BYTES:
+                return byteC.hashIgnoreCase();
+            default:
+                return 0;
+        }
     }
-
+    
     public int indexOf(char c) {
-	return indexOf( c, 0);
+        return indexOf( c, 0);
     }
-
+    
     // Inefficient initial implementation. Will be replaced on the next
     // round of tune-up
     public int indexOf(String s, int starting) {
-	toString();
-	return strValue.indexOf( s, starting );
+        toString();
+        return strValue.indexOf( s, starting );
     }
     
     // Inefficient initial implementation. Will be replaced on the next
     // round of tune-up
     public int indexOf(String s) {
-	return indexOf( s, 0 );
+        return indexOf( s, 0 );
     }
     
     public int indexOfIgnoreCase(String s, int starting) {
-	toString();
-	String upper=strValue.toUpperCase();
-	String sU=s.toUpperCase();
-	return upper.indexOf( sU, starting );
+        toString();
+        String upper=strValue.toUpperCase();
+        String sU=s.toUpperCase();
+        return upper.indexOf( sU, starting );
     }
     
     /**
@@ -504,49 +531,48 @@ public final class MessageBytes implements Cloneable, Serializable {
      * @param starting The start position
      */
     public int indexOf(char c, int starting) {
-	switch (type) {
-	case T_STR:
-	    return strValue.indexOf( c, starting );
-	case T_CHARS:
-	    return charC.indexOf( c, starting);
-	case T_BYTES:
-	    return byteC.indexOf( c, starting );
-	default:
-	    return -1;
-	}
+        switch (type) {
+            case T_STR:
+                return strValue.indexOf( c, starting );
+            case T_CHARS:
+                return charC.indexOf( c, starting);
+            case T_BYTES:
+                return byteC.indexOf( c, starting );
+            default:
+                return -1;
+        }
     }
-
+    
     /** Copy the src into this MessageBytes, allocating more space if
      *  needed
      */
-    public void duplicate( MessageBytes src ) throws IOException
-    {
-	// START CR 6309514
-	// Discard previous state before duplicating
-	recycle();
-	// END CR 6309514
-
-	switch( src.getType() ) {
-	case MessageBytes.T_BYTES:
-	    type=T_BYTES;
-	    ByteChunk bc=src.getByteChunk();
-	    byteC.allocate( 2 * bc.getLength(), -1 );
-	    byteC.append( bc );
-	    break;
-	case MessageBytes.T_CHARS:
-	    type=T_CHARS;
-	    CharChunk cc=src.getCharChunk();
-	    charC.allocate( 2 * cc.getLength(), -1 );
-	    charC.append( cc );
-	    break;
-	case MessageBytes.T_STR:
-	    type=T_STR;
-	    String sc=src.getString();
-	    this.setString( sc );
-	    break;
-	}
+    public void duplicate( MessageBytes src ) throws IOException {
+        // START CR 6309514
+        // Discard previous state before duplicating
+        recycle();
+        // END CR 6309514
+        
+        switch( src.getType() ) {
+            case MessageBytes.T_BYTES:
+                type=T_BYTES;
+                ByteChunk bc=src.getByteChunk();
+                byteC.allocate( 2 * bc.getLength(), -1 );
+                byteC.append( bc );
+                break;
+            case MessageBytes.T_CHARS:
+                type=T_CHARS;
+                CharChunk cc=src.getCharChunk();
+                charC.allocate( 2 * cc.getLength(), -1 );
+                charC.append( cc );
+                break;
+            case MessageBytes.T_STR:
+                type=T_STR;
+                String sc=src.getString();
+                this.setString( sc );
+                break;
+        }
     }
-
+    
     // -------------------- Deprecated code --------------------
     // efficient int, long and date
     // XXX used only for headers - shouldn't be
@@ -564,25 +590,25 @@ public final class MessageBytes implements Cloneable, Serializable {
      * instance to avoid synchronization problems.
      */
     public void setTime(long t, DateFormat df) {
-	// XXX replace it with a byte[] tool
-	recycle();
-	if( dateValue==null)
-	    dateValue=new Date(t);
-	else
-	    dateValue.setTime(t);
-	if( df==null )
-	    strValue=DateTool.format1123(dateValue);
-	else
-	    strValue=DateTool.format1123(dateValue,df);
-	hasStrValue=true;
-	hasDateValue=true;
-	type=T_STR;   
+        // XXX replace it with a byte[] tool
+        recycle();
+        if( dateValue==null)
+            dateValue=new Date(t);
+        else
+            dateValue.setTime(t);
+        if( df==null )
+            strValue=DateTool.format1123(dateValue);
+        else
+            strValue=DateTool.format1123(dateValue,df);
+        hasStrValue=true;
+        hasDateValue=true;
+        type=T_STR;
     }
-
+    
     public void setTime(long t) {
-	setTime( t, null );
+        setTime( t, null );
     }
-
+    
     /** Set the buffer to the representation of an int
      */
     public void setInt(int i) {
@@ -622,10 +648,10 @@ public final class MessageBytes implements Cloneable, Serializable {
         hasHashCode=false;
         hasIntValue=true;
         hasLongValue=false;
-        hasDateValue=false; 
+        hasDateValue=false;
         type=T_BYTES;
     }
-
+    
     /** Set the buffer to the representation of an long
      */
     public void setLong(long l) {
@@ -665,83 +691,81 @@ public final class MessageBytes implements Cloneable, Serializable {
         hasHashCode=false;
         hasIntValue=false;
         hasLongValue=true;
-        hasDateValue=false; 
+        hasDateValue=false;
         type=T_BYTES;
     }
-
+    
     /**
      *  @deprecated The buffer are general purpose, caching for headers should
      *  be done in headers
      */
-    public  long getTime()
-    {
-     	if( hasDateValue ) {
-	    if( dateValue==null) return -1;
-	    return dateValue.getTime();
-     	}
-	
-     	long l=DateTool.parseDate( this );
-     	if( dateValue==null)
-     	    dateValue=new Date(l);
-     	else
-     	    dateValue.setTime(l);
-     	hasDateValue=true;
-     	return l;
+    public  long getTime() {
+        if( hasDateValue ) {
+            if( dateValue==null) return -1;
+            return dateValue.getTime();
+        }
+        
+        long l=DateTool.parseDate( this );
+        if( dateValue==null)
+            dateValue=new Date(l);
+        else
+            dateValue.setTime(l);
+        hasDateValue=true;
+        return l;
     }
     
-
+    
     // Used for headers conversion
     /** Convert the buffer to an int, cache the value
-     */ 
-    public int getInt() 
-    {
-	if( hasIntValue )
-	    return intValue;
-	
-	switch (type) {
-	case T_BYTES:
-	    intValue=byteC.getInt();
-	    break;
-	default:
-	    intValue=Integer.parseInt(toString());
-	}
-	hasIntValue=true;
-	return intValue;
+     */
+    public int getInt() {
+        if( hasIntValue )
+            return intValue;
+        
+        switch (type) {
+            case T_BYTES:
+                intValue=byteC.getInt();
+                break;
+            default:
+                intValue=Integer.parseInt(toString());
+        }
+        hasIntValue=true;
+        return intValue;
     }
-
+    
     // Used for headers conversion
     /** Convert the buffer to an long, cache the value
-     */ 
+     */
     public long getLong() {
         if( hasLongValue )
             return longValue;
         
         switch (type) {
-        case T_BYTES:
-            longValue=byteC.getLong();
-            break;
-        default:
-            longValue=Long.parseLong(toString());
+            case T_BYTES:
+                longValue=byteC.getLong();
+                break;
+            default:
+                longValue=Long.parseLong(toString());
         }
-
+        
         hasLongValue=true;
         return longValue;
-
-     }
-
+        
+    }
+    
     // -------------------- Future may be different --------------------
     
     private static MessageBytesFactory factory=new MessageBytesFactory();
-
+    
     public static void setFactory( MessageBytesFactory mbf ) {
-	factory=mbf;
+        factory=mbf;
     }
     
     public static class MessageBytesFactory {
-	protected MessageBytesFactory() {
-	}
-	public MessageBytes newInstance() {
-	    return new MessageBytes();
-	}
+        protected MessageBytesFactory() {
+        }
+        public MessageBytes newInstance() {
+            return new MessageBytes();
+        }
     }
 }
