@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 
@@ -129,7 +142,7 @@ abstract class JDOConcreteBeanGenerator {
     /**
      * Signature with CVS keyword substitution for identifying the generated code
      */
-    static final String SIGNATURE = "$RCSfile: JDOConcreteBeanGenerator.java,v $ $Revision: 1.2 $"; //NOI18N
+    static final String SIGNATURE = "$RCSfile: JDOConcreteBeanGenerator.java,v $ $Revision: 1.4 $"; //NOI18N
     
     JDOConcreteBeanGenerator(ClassLoader loader,
                              Model model,
@@ -441,8 +454,8 @@ abstract class JDOConcreteBeanGenerator {
 
             // private static fields:
             CMPTemplateFormatter.addPrivateField(
-                CMPROTemplateFormatter.privateStaticVariablesTemplate,
-                Modifier.STATIC, 
+                CMPROTemplateFormatter.privateStaticFinalVariablesTemplate,
+                Modifier.STATIC + Modifier.FINAL, 
                 concreteImplWriter);
         }
 
@@ -769,15 +782,6 @@ abstract class JDOConcreteBeanGenerator {
         // For the following methods the method body exists only for
         // updateable beans
 
-        // jdoClosePersistenceManager 
-        if (isUpdateable) {
-            body = CMPTemplateFormatter.jdoClosePersistenceManagerBody;
-        }
-
-        CMPTemplateFormatter.addGenericMethod(
-                CMPTemplateFormatter.jdoClosePersistenceManager_, 
-                body, concreteImplWriter);
-
         // assertPersistenceManagerIsNull 
         if (isUpdateable) {
             body = CMPTemplateFormatter.assertPersistenceManagerIsNullBody;
@@ -799,8 +803,18 @@ abstract class JDOConcreteBeanGenerator {
         // For the following methods the method body exists for
         // all bean types
 
-        // Add jdoGetPersistenceManager as method returning 
-        // PersistenceManager:
+        // jdoClosePersistenceManager 
+        if (isUpdateable) {
+            body = CMPTemplateFormatter.jdoClosePersistenceManagerBody;
+        } else {
+            body = CMPROTemplateFormatter.jdoClosePersistenceManagerBody;
+        }
+
+        CMPTemplateFormatter.addGenericMethod(
+                CMPTemplateFormatter.jdoClosePersistenceManager_, 
+                body, concreteImplWriter);
+
+        // Add jdoGetPersistenceManager as method returning PersistenceManager:
         if (isUpdateable) {
             body = CMPTemplateFormatter.jdoGetPersistenceManagerBody;
         } else {
@@ -811,6 +825,34 @@ abstract class JDOConcreteBeanGenerator {
                 CMPTemplateFormatter.jdoPersistenceManagerClass_,
                 body, concreteImplWriter);
 
+        // Add methods that use PK as the argument 
+        oneParam[0] = CMPTemplateFormatter.key_;
+        if (isUpdateable) {
+            body = CMPTemplateFormatter.jdoGetPersistenceManager0Body;
+        } else {
+            body = CMPROTemplateFormatter.jdoGetPersistenceManager0Body;
+
+            // Also add jdoGetPersistenceManagerByPK for RO beans only as method returning 
+            // PersistenceManager for this PK:
+            concreteImplWriter.addMethod(CMPROTemplateFormatter.jdoGetPersistenceManagerByPK_, // name
+                    Modifier.PUBLIC , // modifiers
+                    CMPTemplateFormatter.jdoPersistenceManagerClass_, // returnType
+                    oneParam, // parameterNames
+                    pkClassParam, //parameterTypes
+                    null,// exceptions
+                    CMPROTemplateFormatter.jdoGetPersistenceManagerByPKBody, //body
+                    null);// comments
+        }
+
+        concreteImplWriter.addMethod(CMPTemplateFormatter.jdoGetPersistenceManager0_, // name
+                Modifier.PUBLIC , // modifiers
+                CMPTemplateFormatter.jdoPersistenceManagerClass_, // returnType
+                oneParam, // parameterNames
+                pkClassParam, //parameterTypes
+                null,// exceptions
+                body, // body
+                null);// comments
+
         // Add jdoLookupPersistenceManagerFactory as static synchronized method:
         oneParam[0] = concreteImplName;
         MessageFormat mformat = null;
@@ -864,13 +906,24 @@ abstract class JDOConcreteBeanGenerator {
         // Add jdoReleasePersistenceManager as method 
         // with PersistenceManager as a param:
         oneParam[0] = CMPTemplateFormatter.jdoPersistenceManagerClass_;
-        
+        concreteImplWriter.addMethod(CMPTemplateFormatter.jdoReleasePersistenceManager_, // name
+            Modifier.PRIVATE, // modifiers
+            CMPTemplateFormatter.void_, // returnType
+            param0, // parameterNames
+            oneParam,// parameterTypes
+            null,// exceptions
+            CMPTemplateFormatter.jdoReleasePersistenceManagerBody, // body
+            null);// comments
+
+        // Add jdoReleasePersistenceManager0 as method 
+        // with PersistenceManager as a param that is different 
+        // between updateable and read-only beans:
         if (isUpdateable) {
             body = CMPTemplateFormatter.jdoReleasePersistenceManagerBody;
         } else {
-            body = CMPROTemplateFormatter.jdoReleasePersistenceManagerBody;
+            body = CMPROTemplateFormatter.jdoReleasePersistenceManager0Body;
         }
-        concreteImplWriter.addMethod(CMPTemplateFormatter.jdoReleasePersistenceManager_, // name
+        concreteImplWriter.addMethod(CMPTemplateFormatter.jdoReleasePersistenceManager0_, // name
             Modifier.PRIVATE, // modifiers
             CMPTemplateFormatter.void_, // returnType
             param0, // parameterNames
