@@ -10,11 +10,11 @@ DOMAIN=$7
 
 # Only create domain if domain doesn't exist
 if [ ! -d $DOMAIN_ROOT/$DOMAIN ]; then
-   $BASEDIR/bin/asadmin create-domain --domaindir $DOMAIN_ROOT --adminport $ADMIN_PORT --adminuser $ADMIN_USER --passwordfile $PASSWORD_FILE --instanceport $INSTANCE_PORT $DOMAIN   
+   $BASEDIR/bin/asadmin create-domain --domaindir $DOMAIN_ROOT --adminport $ADMIN_PORT --user $ADMIN_USER --passwordfile $PASSWORD_FILE --instanceport $INSTANCE_PORT $DOMAIN   
    result=$?
    if [ $result -ne 0 ]; then
       echo "Could not create default domain $DOMAIN at $DOMAIN_ROOT.\n"
-      echo "Create required domains manually using asadmin \n"
+      echo "Create required domains manually using asadmin2 \n"
       exit 1
    fi
 fi
