@@ -13,16 +13,18 @@ require "ctime.pl";
 use Env qw(HOST);
 $rm = "rm -rf";
 $pkgaddLog = "/tmp/pkgaddLog.$$";
-$asadmin = "/usr/share/sunappserver/bin/asadmin";
-$confRoot = "/usr/share/sunappserver/config";
-$sampleDir = "/usr/share/sunappserver/samples";
-$installDir = "/usr/share/sunappserver";
+$asadmin = "/usr/share/glassfishv2/bin/asadmin";
+$confRoot = "/usr/share/glassfishv2/config";
+$sampleDir = "/usr/share/glassfishv2/samples";
+$installDir = "/usr/share/glassfishv2";
 $javaHome = "/usr/lib/jvm/java-1.5.0-sun";
-$basedir = "/usr/share/sunappserver";
-$bindir = "/usr/share/sunappserver/bin";
-$install = "/usr/share/sunappserver/config";
+$sixHome = "/usr/lib/jvm/java-6-sun";
+$openHome = "/usr/lib/jvm/java-6-openjdk";
+$basedir = "/usr/share/glassfishv2";
+$bindir = "/usr/share/glassfishv2/bin";
+$install = "/usr/share/glassfishv2/config";
 $shareDir = "/usr/share";
-$docs = "/usr/share/sunappserver/docs";
+$docs = "/usr/share/glassfishv2/docs";
 $imqDir = "$installDir/imq";
 $arch=`dpkg-architecture -qDEB_HOST_ARCH`;
 chomp $arch;
@@ -49,9 +51,20 @@ $adminUser = $NVP{adminuser};
 $adminPort = $NVP{adminport};
 $adminPasswd = $NVP{adminpasswd};
 $instanceport = $NVP{serverport};
+$adminProfile = $NVP{adminprofile};
+$adminSecure = $NVP{adminsecure};
 
 
-# install existing packages
+# check if sun-java-5 directory exists or replace with sun-java-6 
+if (! -e $javaHome || ! -d $javaHome)
+  {
+    $javaHome = $sixHome;
+    if (! -e $javaHome || ! -d $javaHome)
+    {
+        $javaHome = $openHome;
+    }
+  }
+
 $retCode = system("mkdir -p $basedir/bin");
 
 print "/bin/sh $install/install/appservenv $install $basedir\n";
@@ -108,7 +121,8 @@ if ($retCode ne "0")
      if (m/^AS_WEBCONSOLE_LIB=/) {s/=.*$/="$installDir\/lib"/;}
      if (m/^AS_CONFIG=/) {s/=.*$/="$installDir\/config"/;}
      if (m/^AS_INSTALL=/) {s/=.*$/="$installDir"/;}
-     if (m/^AS_DERBY_INSTALL=/) {s/=.*$/="$installDir\/javadb"/;}
+     if (m/^AS_DERBY_INSTALL=/) {s/=.*$/="\/usr\/share\/javadb"/;}
+     if (m/^AS_JDMK_HOME=/) {s/=.*$/="$installDir\/lib\/SUNWjdmk\/5.1"/;}
      if (m/^AS_WEBSERVICES_LIB=/) {s/=.*$/="$installDir\/lib"/;}
      if (m/^AS_JAVA=/) {s/=.*$/="$javaHome"/}
      if (m/^AS_NATIVE_LAUNCHER=/) {s/=.*$/="false"/}
@@ -118,6 +132,20 @@ if ($retCode ne "0")
   close(F3);
   close(F4);
 
+  system("cp $confRoot/../lib/install/templates/asadminenv.conf $confRoot");
+  rename("$confRoot/asadminenv.conf", "$confRoot/asadminenv.conf.orig");
+  open(F31, "<$confRoot/asadminenv.conf.orig");
+  open(F32, ">$confRoot/asadminenv.conf") || die ("asadminenv.conf cannot be opened\n");
+  while (<F31>)
+  {
+     if (m/^AS_ADMIN_PORT=/) {s/=.*$/=$adminPort/;}
+     if (m/^AS_ADMIN_PROFILE=/) {s/=.*$/=$adminProfile/;}
+     if (m/^AS_ADMIN_SECURE=/) {s/=.*$/=$adminSecure/;}
+     print F32 ("$_");
+  }
+  close(F31);
+  close(F32);
+
   if (-f "$imqDir/etc/imqenv.conf")
   {
     rename("$imqDir/etc/imqenv.conf", "$imqDir/etc/imqenv.conf.orig");
@@ -143,12 +171,19 @@ if ($retCode ne "0")
   $retCode = system("sh $install/install/create.sh $basedir $domainRoot $adminPort $adminUser $installDir/config/install/default/passwordfile $instanceport domain1");
   if ($retCode eq "0")
   {
-	# start the domain up
+        # start the domain up
         $retCode = system("sh $install/install/start.sh $bindir $adminUser $installDir/config/install/default/passwordfile");
   }
+# start the domain up
+#  $retCode = system("sh $install/install/start.sh $bindir $adminUser $installDir/config/install/default/passwordfile");
+#  if ($retCode ne "0")
+#  {
+#	print "Failed to start the domain1.\n";
+#	exit 1;
+#  }
 
-#  $docRoot = "$domainRoot/domain1/docroot";
-#  $retCode = system("cp -r $basedir/lib/install/docs/ $basedir");
+  $docRoot = "$domainRoot/domain1/docroot";
+# $retCode = system("cp -r $basedir/lib/install/docs/ $basedir");
   
 #  $retCode = system("cd $sampleDir; find . -name docs > docslist;");
 #  $retCode = system("./xcp $docRoot/samples $sampleDir/docslist $sampleDir");
