@@ -6,7 +6,7 @@ BIN_DIR=$1
 numRunning=`$BIN_DIR/asadmin list-domains | grep -v "not running" | grep -c "running"`
 
 if [ $numRunning -ne 0 ]; then
-   $BIN_DIR/asadmin stop-domain
+   $BIN_DIR/asadmin stop-appserv
    result=$?
    if [ $result -ne 0 ]; then
       exit 1
