@@ -1,10 +1,9 @@
-<?xml version='1.0' encoding='iso-8859-1'?>
+<?xml version='1.0' encoding='UTF-8'?>
 <!-- ident   "%Z%%M% %I%   %E% SMI" -->
 <!DOCTYPE helpset PUBLIC "-//Sun Microsystems Inc.//DTD JavaHelp HelpSet Version 2.0//EN" "helpset_1_0.dtd">
 
 <helpset version="2.0" xml:lang="en">
-<title>Sun Java System Application Server Platform Edition 9 Admin Console
-Online Help</title>
+<title>Sun Java System Application Server 9.1 Update 1 Administration Console Online Help</title>
 <maps>
 	<homeID></homeID>
 	<mapref location="map.jhm"/>
