@@ -1,24 +1,39 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the "License").  You may not use this file except 
- * in compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * glassfish/bootstrap/legal/CDDLv1.0.txt or 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html. 
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * // Copyright (c) 1998, 2007, Oracle. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * HEADER in each file and include the License file at 
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable, 
- * add the following below this CDDL HEADER, with the 
- * fields enclosed by brackets "[]" replaced with your 
- * own identifying information: Portions Copyright [yyyy] 
- * [name of copyright owner]
+ *
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
-// Copyright (c) 1998, 2006, Oracle. All rights reserved.  
 package oracle.toplink.essentials.internal.ejb.cmp3;
 
 import java.util.*;
@@ -32,11 +47,13 @@ import oracle.toplink.essentials.ejb.cmp3.persistence.SEPersistenceUnitInfo;
 import oracle.toplink.essentials.logging.AbstractSessionLog;
 import oracle.toplink.essentials.internal.ejb.cmp3.EntityManagerSetupImpl;
 import oracle.toplink.essentials.ejb.cmp3.persistence.PersistenceUnitProcessor;
+import oracle.toplink.essentials.ejb.cmp3.persistence.Archive;
 import oracle.toplink.essentials.exceptions.*;
 import oracle.toplink.essentials.logging.SessionLog;
 import oracle.toplink.essentials.ejb.cmp3.EntityManagerFactoryProvider;
 import oracle.toplink.essentials.PersistenceProvider;
 
+import javax.persistence.PersistenceException;
 import javax.persistence.spi.ClassTransformer;
 import oracle.toplink.essentials.config.TopLinkProperties;
 
@@ -50,7 +67,7 @@ import oracle.toplink.essentials.config.TopLinkProperties;
  *
  * @see oracle.toplink.essentials.ejb.cmp3.EntityManagerFactoryProvider
  */
-public class JavaSECMPInitializer {
+public class JavaSECMPInitializer implements PersistenceInitializationActivator {
 
     // Used when byte code enhancing
     public static Instrumentation globalInstrumentation;
@@ -62,12 +79,6 @@ public class JavaSECMPInitializer {
     // The JavaSECMPInitializer - a singleton
     protected static JavaSECMPInitializer javaSECMPInitializer;
 
-    // We create an EntityManagerSetupImpl for each persistence unit the following
-    // two variables maintain a dictionary of both the EntityManagerSetupImpls and
-    // their associated PersistenceUnitInfo objects
-    protected HashMap<String, EntityManagerSetupImpl> emSetupImpls = null;
-    protected HashMap<String, SEPersistenceUnitInfo> emSetupPersistenceUnitInfos = null;
-
     protected ClassLoader sessionClassLoader = null;
 
     /**
@@ -75,13 +86,14 @@ public class JavaSECMPInitializer {
      * Get the singleton entityContainer.
      * @return EnityContainer
      */
-    public static JavaSECMPInitializer getJavaSECMPInitializer(Map properties) {
+    public static synchronized JavaSECMPInitializer getJavaSECMPInitializer() {
         if (javaSECMPInitializer == null) {
-           initializeFromMain(properties);
-        }   
+           javaSECMPInitializer = new JavaSECMPInitializer();
+        }
+        AbstractSessionLog.getLog().setLevel(JavaSECMPInitializer.getTopLinkLoggingLevel());
         return javaSECMPInitializer;
     }
-
+    
   /**
    * Return whether initialization has occured without actually triggering
    * initialization
@@ -106,8 +118,6 @@ public class JavaSECMPInitializer {
      */
     protected JavaSECMPInitializer() {
         super();
-        emSetupImpls = new HashMap<String, EntityManagerSetupImpl>();
-        emSetupPersistenceUnitInfos = new HashMap<String, SEPersistenceUnitInfo>();
     }
 
     /**
@@ -116,33 +126,49 @@ public class JavaSECMPInitializer {
      * This method will prepare to call predeploy, call it and finally register the
      * transformer returned to be used for weaving.
      */
-    protected boolean callPredeploy(SEPersistenceUnitInfo persistenceUnitInfo, Map m) {
+    protected boolean callPredeploy(SEPersistenceUnitInfo persistenceUnitInfo, Map m, PersistenceInitializationActivator persistenceActivator) {
+        ClassLoader tempLoader = null;
         // we will only attempt to deploy when TopLink is specified as the provider or the provider is unspecified
         String providerClassName = persistenceUnitInfo.getPersistenceProviderClassName();
-        if (providerClassName == null || providerClassName.equals("") || providerClassName.equals(EntityManagerFactoryProvider.class.getName()) || providerClassName.equals(PersistenceProvider.class.getName())){
-            Set tempLoaderSet = PersistenceUnitProcessor.buildClassSet(persistenceUnitInfo);
-            // Create the temp loader that will not cache classes for entities in our persistence unit
-            ClassLoader tempLoader = createTempLoader(tempLoaderSet);
-            persistenceUnitInfo.setNewTempClassLoader(tempLoader);
+        if (persistenceActivator.isPersistenceProviderSupported(providerClassName)){
+            EntityManagerSetupImpl emSetupImpl = EntityManagerFactoryProvider.getEntityManagerSetupImpl(persistenceUnitInfo.getPersistenceUnitRootUrl()+persistenceUnitInfo.getPersistenceUnitName());
+            
+            // if we already have an EntityManagerSetupImpl this PU has already been processed.  Use the existing one
+            if (emSetupImpl != null && !emSetupImpl.isUndeployed()){
+                return false;
+            }
+            Set tempLoaderSet = PersistenceUnitProcessor.buildClassSet(persistenceUnitInfo, Thread.currentThread().getContextClassLoader());
 
-            EntityManagerSetupImpl emSetupImpl = new EntityManagerSetupImpl();
-            emSetupImpls.put(persistenceUnitInfo.getPersistenceUnitName(), emSetupImpl);
-            emSetupPersistenceUnitInfos.put(persistenceUnitInfo.getPersistenceUnitName(), persistenceUnitInfo);   
-           
+            Map mergedProperties = EntityManagerFactoryProvider.mergeMaps(m, persistenceUnitInfo.getProperties());
+
+            String weaving = EntityManagerFactoryProvider.getConfigPropertyAsString(TopLinkProperties.WEAVING, mergedProperties, null);
+            // Bug#4452468  When globalInstrumentation is null, there is no weaving
+            if (globalInstrumentation == null) {
+                if (weaving == null) {
+                   mergedProperties.put(TopLinkProperties.WEAVING, "false");
+                   weaving = "false";
+                }else if (weaving.equalsIgnoreCase("true")){
+                    throw new PersistenceException(EntityManagerSetupException.wrongWeavingPropertyValue());
+                }
+            }
+            
+            // Bug#2741: If weaving disabled then use regular loader, not a temp one
+            if (weaving != null && (weaving.equalsIgnoreCase("false") || weaving.equalsIgnoreCase("static"))) {
+                shouldCreateInternalLoader=false;
+            }
+            // Create the temp loader that will not cache classes for entities in our persistence unit
+            tempLoader = createTempLoader(tempLoaderSet);
+            persistenceUnitInfo.setNewTempClassLoader(tempLoader);
+            persistenceUnitInfo.setClassLoader(getMainLoader());
+            if (emSetupImpl == null){
+                emSetupImpl = new EntityManagerSetupImpl();
+                EntityManagerFactoryProvider.addEntityManagerSetupImpl(persistenceUnitInfo.getPersistenceUnitRootUrl()+persistenceUnitInfo.getPersistenceUnitName(), emSetupImpl);
+            }
             // Make the callback
             AbstractSessionLog.getLog().log(SessionLog.FINER, "cmp_init_invoke_predeploy", persistenceUnitInfo.getPersistenceUnitName());
 
-            Map mergedProperties = EntityManagerFactoryProvider.mergeMaps(m, persistenceUnitInfo.getProperties());
-            //Bug#4452468  When globalInstrumentation is null, there is no weaving
-            if (globalInstrumentation == null && EntityManagerFactoryProvider.getConfigPropertyAsString(TopLinkProperties.WEAVING, mergedProperties, null) == null) {
-                if (m == null) {
-                    m = new HashMap();
-                }
-                m.put(TopLinkProperties.WEAVING, "false");
-            }
-    
             // A call to predeploy will partially build the session we will use
-            final ClassTransformer transformer = emSetupImpl.predeploy(persistenceUnitInfo, m);
+            final ClassTransformer transformer = emSetupImpl.predeploy(persistenceUnitInfo, mergedProperties);
     
             // If we got a transformer then register it 
             if ((transformer != null) && (globalInstrumentation != null)) {
@@ -162,7 +188,6 @@ public class JavaSECMPInitializer {
             } else if (globalInstrumentation == null) {
                 AbstractSessionLog.getLog().log(SessionLog.FINER, "cmp_init_globalInstrumentation_is_null");
             }
-            persistenceUnitInfo.setClassLoader(getMainLoader());
             return true;
         }
         return false;
@@ -196,17 +221,6 @@ public class JavaSECMPInitializer {
 
         return tempLoader;
     }
-    
-  /**
-   * Return the setup class for a given entity manager name 
-   * @param emName 
-   */
-    public EntityManagerSetupImpl getEntityManagerSetupImpl(String emName){
-        if (emName == null){
-            return (EntityManagerSetupImpl)emSetupImpls.get("");
-        }
-        return (EntityManagerSetupImpl)emSetupImpls.get(emName);
-    }
 
     public static ClassLoader getMainLoader() {
         return Thread.currentThread().getContextClassLoader();
@@ -218,11 +232,11 @@ public class JavaSECMPInitializer {
    * and creates any required transformers.
    * Second the deploy process creates a TopLink session based on that metadata.
    */
-    protected void initPersistenceUnits(URL url, Map m) {
-        Iterator<SEPersistenceUnitInfo> persistenceUnits = PersistenceUnitProcessor.getPersistenceUnits(url, sessionClassLoader).iterator();
+    protected void initPersistenceUnits(Archive archive, Map m, PersistenceInitializationActivator persistenceActivator) {
+        Iterator<SEPersistenceUnitInfo> persistenceUnits = PersistenceUnitProcessor.getPersistenceUnits(archive, sessionClassLoader).iterator();
         while (persistenceUnits.hasNext()){
             SEPersistenceUnitInfo persistenceUnitInfo = persistenceUnits.next();
-            callPredeploy(persistenceUnitInfo, m);
+            callPredeploy(persistenceUnitInfo, m, persistenceActivator);
         }
     }
 
@@ -233,11 +247,12 @@ public class JavaSECMPInitializer {
      * contains that list.  It will then initialize the container with that list.
      * If succeeded return true, false otherwise.
      */
-    public void initialize(Map m) {
+    public void initialize(Map m, PersistenceInitializationActivator persistenceActivator) {
         sessionClassLoader = getMainLoader();
-        for (URL url: PersistenceUnitProcessor.findPersistenceArchives()){
-            AbstractSessionLog.getLog().log(SessionLog.FINER, "cmp_init_initialize", url);
-            initPersistenceUnits(url, m);
+        final Set<Archive> pars = PersistenceUnitProcessor.findPersistenceArchives();
+        for (Archive archive: pars){
+            AbstractSessionLog.getLog().log(SessionLog.FINER, "cmp_init_initialize", archive);
+            initPersistenceUnits(archive, m, persistenceActivator);
         }
     }
 
@@ -254,29 +269,18 @@ public class JavaSECMPInitializer {
         // Create JavaSECMPInitializer singleton
         javaSECMPInitializer = new JavaSECMPInitializer();
         // Initialize it
-        javaSECMPInitializer.initialize(new HashMap());
+        javaSECMPInitializer.initialize(new HashMap(), javaSECMPInitializer);
     }
 
     /**
-     *  Initialize the static entityContainer from a main method.  The entityContainer
-     *  can also be initialized from a premain method. with slightly different behavior
-     * @param cls the class of the JavaSECMPInitializer.  We will instantiate it reflectively
-     * @param m a map containing the set of properties to intantiate with.
+     * Returns whether the given persistence provider class is supported by this implementation
+     * @param providerClassName
+     * @return
      */
-    public static void initializeFromMain(Map m) {
-        if (javaSECMPInitializer != null) {
-            return;
-        }
-
-        javaSECMPInitializer = new JavaSECMPInitializer();
-        AbstractSessionLog.getLog().setLevel(JavaSECMPInitializer.getTopLinkLoggingLevel());
-
-        AbstractSessionLog.getLog().log(SessionLog.FINER, "cmp_init_initialize_from_main");
-
-        // Initialize it
-        javaSECMPInitializer.initialize(m);
+    public boolean isPersistenceProviderSupported(String providerClassName){
+        return (providerClassName == null) || providerClassName.equals("") || providerClassName.equals(EntityManagerFactoryProvider.class.getName()) || providerClassName.equals(PersistenceProvider.class.getName());
     }
-
+    
     /**
      *  INTERNAL:
      *  Create a list of java.lang.Class that contains the classes of all the entities
