@@ -1,24 +1,39 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the "License").  You may not use this file except 
- * in compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * glassfish/bootstrap/legal/CDDLv1.0.txt or 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html. 
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * // Copyright (c) 1998, 2007, Oracle. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * HEADER in each file and include the License file at 
- * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable, 
- * add the following below this CDDL HEADER, with the 
- * fields enclosed by brackets "[]" replaced with your 
- * own identifying information: Portions Copyright [yyyy] 
- * [name of copyright owner]
+ *
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
+ * 
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
-// Copyright (c) 1998, 2006, Oracle. All rights reserved.  
 package oracle.toplink.essentials.mappings;
 
 import java.util.*;
@@ -55,16 +70,16 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
     protected transient DatabaseTable relationTable;
 
     /** The field in the source table that corresponds to the key in the relation table */
-    protected transient Vector sourceKeyFields;
+    protected transient Vector<DatabaseField> sourceKeyFields;
 
     /**  The field in the target table that corresponds to the key in the relation table */
-    protected transient Vector targetKeyFields;
+    protected transient Vector<DatabaseField> targetKeyFields;
 
     /** The field in the intermediate table that corresponds to the key in the source table */
-    protected transient Vector sourceRelationKeyFields;
+    protected transient Vector<DatabaseField> sourceRelationKeyFields;
 
     /** The field in the intermediate table that corresponds to the key in the target table */
-    protected transient Vector targetRelationKeyFields;
+    protected transient Vector<DatabaseField> targetRelationKeyFields;
 
     /** Query used for single row deletion. */
     protected transient DataModifyQuery deleteQuery;
@@ -276,7 +291,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Return all the source key fields associated with the mapping.
      */
-    public Vector getSourceKeyFields() {
+    public Vector<DatabaseField> getSourceKeyFields() {
         return sourceKeyFields;
     }
 
@@ -299,7 +314,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Return all the source realtion key fields associated with the mapping.
      */
-    public Vector getSourceRelationKeyFields() {
+    public Vector<DatabaseField> getSourceRelationKeyFields() {
         return sourceRelationKeyFields;
     }
 
@@ -322,7 +337,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Return all the target keys associated with the mapping.
      */
-    public Vector getTargetKeyFields() {
+    public Vector<DatabaseField> getTargetKeyFields() {
         return targetKeyFields;
     }
 
@@ -345,10 +360,20 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Return all the target relation key fields associated with the mapping.
      */
-    public Vector getTargetRelationKeyFields() {
+    public Vector<DatabaseField> getTargetRelationKeyFields() {
         return targetRelationKeyFields;
     }
 
+    /**
+     * INTERNAL:
+     * Returns whether the mapping has any constraint dependencies, such as
+     * join table entries. This dependency is important for deletes from
+     * unidirectional relationships.
+     */
+    public boolean hasConstraintDependency() {
+        // Set a dependency on the owning side
+        return !isReadOnly();
+    }
     protected boolean hasCustomDeleteQuery() {
         return hasCustomDeleteQuery;
     }
@@ -599,7 +624,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * If a user does not specify the source key then the primary keys of the source table are used.
      */
     protected void initializeSourceKeysWithDefaults(AbstractSession session) {
-        List primaryKeyFields = getDescriptor().getPrimaryKeyFields();
+        List<DatabaseField> primaryKeyFields = getDescriptor().getPrimaryKeyFields();
         for (int index = 0; index < primaryKeyFields.size(); index++) {
             getSourceKeyFields().addElement(primaryKeyFields.get(index));
         }
@@ -639,7 +664,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * If a user does not specify the target key then the primary keys of the target table are used.
      */
     protected void initializeTargetKeysWithDefaults(AbstractSession session) {
-        List primaryKeyFields = getReferenceDescriptor().getPrimaryKeyFields();
+        List<DatabaseField> primaryKeyFields = getReferenceDescriptor().getPrimaryKeyFields();
         for (int index = 0; index < primaryKeyFields.size(); index++) {
             getTargetKeyFields().addElement(primaryKeyFields.get(index));
         }
@@ -947,33 +972,37 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Delete entries related to this mapping from the relation table.
      */
-    public void preDelete(DeleteObjectQuery query) throws DatabaseException {
+    public void preDelete(WriteObjectQuery query) throws DatabaseException {
         Object objectsIterator = null;
         ContainerPolicy containerPolicy = getContainerPolicy();
 
-        if (isReadOnly()) {
-            return;
-        }
-
         Object objects = getRealCollectionAttributeValueFromObject(query.getObject(), query.getSession());
 
-        if (shouldObjectModifyCascadeToParts(query)) {
+        if (shouldObjectModifyCascadeToPartsForPreDelete(query)) {
             //this must be done up here because the select must be done before the entry in the relation table is deleted.
             objectsIterator = containerPolicy.iteratorFor(objects);
         }
-        prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
-        query.getSession().executeQuery(getDeleteAllQuery(), query.getTranslationRow());
+        
+        if (!isReadOnly()) {
+            prepareTranslationRow(query.getTranslationRow(), query.getObject(), query.getSession());
+            query.getSession().executeQuery(getDeleteAllQuery(), query.getTranslationRow());
+        }
 
         // If privately owned delete the objects, this does not handle removed objects (i.e. verify delete, not req in uow).
         // Does not try to optimize delete all like 1-m, (rarely used and hard to do).
-        if (shouldObjectModifyCascadeToParts(query)) {
+        if (shouldObjectModifyCascadeToPartsForPreDelete(query)) {
             if (objects != null) {
                 //objectsIterator will not be null because cascade check will still return true.
                 while (containerPolicy.hasNext(objectsIterator)) {
-                    DeleteObjectQuery deleteQuery = new DeleteObjectQuery();
-                    deleteQuery.setObject(containerPolicy.next(objectsIterator, query.getSession()));
-                    deleteQuery.setCascadePolicy(query.getCascadePolicy());
-                    query.getSession().executeQuery(deleteQuery);
+                    Object object = containerPolicy.next(objectsIterator, query.getSession());
+                    // Do not cascade to objects that are currently being processed
+                    if (!query.getSession().getCommitManager().isCommitInPreModify(object) &&
+                            shouldObjectDeleteCascadeToPart(query, object)) {
+                        DeleteObjectQuery deleteQuery = new DeleteObjectQuery();
+                        deleteQuery.setObject(object);
+                        deleteQuery.setCascadePolicy(query.getCascadePolicy());
+                        query.getSession().executeQuery(deleteQuery);
+                    }
                 }
             }
         }
@@ -1137,7 +1166,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Set the source fields.
      */
-    public void setSourceKeyFields(Vector sourceKeyFields) {
+    public void setSourceKeyFields(Vector<DatabaseField> sourceKeyFields) {
         this.sourceKeyFields = sourceKeyFields;
     }
 
@@ -1169,7 +1198,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Set the source fields.
      */
-    public void setSourceRelationKeyFields(Vector sourceRelationKeyFields) {
+    public void setSourceRelationKeyFields(Vector<DatabaseField> sourceRelationKeyFields) {
         this.sourceRelationKeyFields = sourceRelationKeyFields;
     }
 
@@ -1191,7 +1220,7 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Set the target fields.
      */
-    public void setTargetKeyFields(Vector targetKeyFields) {
+    public void setTargetKeyFields(Vector<DatabaseField> targetKeyFields) {
         this.targetKeyFields = targetKeyFields;
     }
 
@@ -1223,7 +1252,27 @@ public class ManyToManyMapping extends CollectionMapping implements RelationalMa
      * INTERNAL:
      * Set the target fields.
      */
-    public void setTargetRelationKeyFields(Vector targetRelationKeyFields) {
+    public void setTargetRelationKeyFields(Vector<DatabaseField> targetRelationKeyFields) {
         this.targetRelationKeyFields = targetRelationKeyFields;
     }
+
+    /**
+     * Return whether any process leading to object deletion should also affect its parts. 
+     * Used in preDelete. Note that dependencies are reversed for deletes: 
+     * Join table entries must be removed before the related objects can be deleted.
+     * So the owning side must be deleted before the non-owning side is processed. 
+     */
+    protected boolean shouldObjectModifyCascadeToPartsForPreDelete(ObjectLevelModifyQuery query) {
+        // Always cascade for privately-owned parts
+        if (isPrivateOwned()) {
+            return true;
+        }
+
+        // The non-owning side is read-only for join table mappings.
+        if (query.shouldCascadeOnlyDependentParts()) {
+            return isReadOnly();
+        } 
+        
+        return query.shouldCascadeAllParts();
+    }
 }
