@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -33,6 +46,7 @@ import java.lang.Object;
 // import java.util.Stack;
 import com.sun.enterprise.util.collection.ArrayListStack; 
 // IASRI 4698847 END
+import java.lang.reflect.Method;
 
 import org.omg.IOP.Codec;
 import org.omg.IOP.ServiceContext;
@@ -55,6 +69,13 @@ import org.omg.CosTSInteroperation.TAG_OTS_POLICY;
 import com.sun.jts.CosTransactions.CurrentTransaction;
 import com.sun.corba.ee.impl.corba.EnvironmentImpl;
 import com.sun.corba.ee.impl.txpoa.TSIdentificationImpl;
+import com.sun.corba.ee.spi.ior.IOR;
+import com.sun.corba.ee.spi.ior.ObjectKeyTemplate;
+import com.sun.corba.ee.spi.ior.ObjectAdapterId;
+import com.sun.corba.ee.spi.ior.iiop.IIOPProfile;
+import com.sun.corba.ee.spi.orb.ORBVersionFactory;
+import com.sun.corba.ee.spi.oa.rfm.ReferenceFactoryManager;
+import com.sun.corba.ee.impl.orbutil.ORBConstants;
  
 import java.util.logging.Logger;
 import java.util.logging.Level;
@@ -64,8 +85,8 @@ import com.sun.logging.LogDomains;
  * This is the implementation of the JTS PI-based client/server interceptor.
  * This will be called during request/reply invocation path.
  *
- * @author Ram Jeyaraman 11/11/2000, $Author: sheetalv $
- * @version 1.0, $Revision: 1.2.4.2 $ on $Date: 2006/03/31 21:20:36 $
+ * @author Ram Jeyaraman 11/11/2000, $Author: tcfujii $
+ * @version 1.0, $Revision: 1.8 $ on $Date: 2007/05/05 05:36:42 $
  */
 public class InterceptorImpl extends org.omg.CORBA.LocalObject
         implements ClientRequestInterceptor, ServerRequestInterceptor {
@@ -89,13 +110,15 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
     public static final String CLIENT_POLICY_CHECKING =
         "com.sun.jts.pi.CLIENT_POLICY_CHECKING";
     public static final String INTEROP_MODE = "com.sun.jts.pi.INTEROP_MODE";
+    // The ReferenceFactoryManager from the orb.
+    private static ReferenceFactoryManager rfm = null;
 	
 	/*
 		Logger to log transaction messages
 	*/  
 	 static Logger _logger = LogDomains.getLogger(LogDomains.TRANSACTION_LOGGER);
 
-    public static ThreadLocal otsThreadLocal =
+    public static final ThreadLocal otsThreadLocal =
         new ThreadLocal() {
             protected java.lang.Object initialValue() {
                 Object[] threadLocalState = new Object[2];
@@ -114,6 +137,7 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
     // static initializer block
 
     static {
+	    /**
         Any any = ORB.init().create_any();
         any.insert_boolean(false);
         nullContext = new PropagationContext(
@@ -128,6 +152,7 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
             new TransIdentity(null, null, new otid_t(-1, 0, new byte[0])),
             new TransIdentity[0],
             any);
+	    ***/
     }
 
     public static ORB txOrb = null;
@@ -221,7 +246,8 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
             // policy component must be treated as FORBIDS.
             otsPolicyValue = OTSPolicyImpl._ADAPTS.value();
         } else {
-            TypeCode typeCode = ORB.init().get_primitive_tc(TCKind.tk_short);
+            // TypeCode typeCode = ORB.init().get_primitive_tc(TCKind.tk_short);
+            TypeCode typeCode = txOrb.get_primitive_tc(TCKind.tk_short);
             Any any = null;
             try {
                 any = this.codec.decode_value(otsComp.component_data, typeCode);
@@ -289,7 +315,7 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
         // Currently, checked behaviour is disabled and shall be reinstated
         // once OTS RTF redrafts the OTS spec based on PI. An issue needs to be
         // filed.
-        Object target = ri.effective_target();
+        org.omg.CORBA.Object target = ri.effective_target();
         if ( StubAdapter.isStub(target) && StubAdapter.isLocal(target) ) { 
             // target is local
             // load a dummy context and discard the current tx context.
@@ -298,8 +324,25 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
             // load a null context and discard the current tx context.
             hctx.value = nullContext;
         } else {
-            // Call JTS proprietary interceptor to get current tx.
-            sender.sending_request(ri.request_id(), hctx);
+	    IOR ior = ((com.sun.corba.ee.spi.orb.ORB)txOrb).getIOR( target, false ) ;
+	    IIOPProfile prof = ior.getProfile() ;
+	    ObjectKeyTemplate oktemp = prof.getObjectKeyTemplate() ;
+	    if (oktemp.getORBVersion().equals(ORBVersionFactory.getFOREIGN()))
+	    {
+                hctx.value = nullContext;
+	    } else {
+		    /*
+	        ObjectAdapterId oaid = oktemp.getObjectAdapterId() ;
+	        String[] adapterName = oaid.getAdapterName() ;
+	        boolean isEjbCall = isEjbAdapterName(adapterName);
+	        if (!isEjbCall) {
+                    hctx.value = nullContext;
+	        } else {
+                    // Call JTS proprietary interceptor to get current tx.
+                    sender.sending_request(ri.request_id(), hctx);
+	        } */
+                sender.sending_request(ri.request_id(), hctx);
+	    }
         }
 
         // add service context.
@@ -336,6 +379,9 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
             // transaction to fail.
         }
 
+	if (svc == null) {
+	    return;
+	}
         // a tx svc context is available.
 
         // check if TransactionService is available.
@@ -394,6 +440,10 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
 		return;
 	}
 
+	if (svc == null) {
+	    return;
+	}
+
         // a tx svc context is available.
 
         // Set up the Environment instance with exception information.
@@ -458,6 +508,9 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
             // transaction to fail.
         }
 
+	if (svc == null) {
+	    return;
+	}
         // a tx svc context is available.
 
         // check if TransactionService is available.
@@ -772,5 +825,33 @@ public class InterceptorImpl extends org.omg.CORBA.LocalObject
 
     public static void setOrb(ORB orb) {
         txOrb = orb;
+        Any any = txOrb.create_any();
+        any.insert_boolean(false);
+        nullContext = new PropagationContext(
+            0,
+            new TransIdentity(null, null, new otid_t(0, 0, new byte[0])),
+            new TransIdentity[0],
+            any);
+
+        any.insert_boolean(true);
+        dummyContext = new PropagationContext(
+            -1,
+            new TransIdentity(null, null, new otid_t(-1, 0, new byte[0])),
+            new TransIdentity[0],
+            any);
+	try {
+	    rfm = (ReferenceFactoryManager)orb.resolve_initial_references(
+	          ORBConstants.REFERENCE_FACTORY_MANAGER ) ;
+	} catch (Exception ex) {
+	    _logger.log(Level.WARNING,ex.getMessage(), ex);
+	}
     }
+
+     public static boolean isEjbAdapterName( String[] adapterName ) {
+         boolean result = false ;
+	 if (rfm != null)
+	     result = rfm.isRfmName( adapterName ) ;
+         return result ;
+     }
+
 }
