@@ -1,30 +1,44 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 package com.sun.enterprise.tools.common;
 
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.Serializable;
+import java.util.StringTokenizer;
 import com.sun.enterprise.tools.common.dd.webservice.Webservices;
 import com.sun.enterprise.tools.common.dd.webservice.PortComponentType;
 import com.sun.enterprise.tools.common.dd.webservice.WebserviceDescriptionType;
@@ -56,6 +70,15 @@ import com.sun.enterprise.config.serverbeans.WebModule;
 import com.sun.enterprise.config.ConfigBean;
 import com.sun.enterprise.admin.server.core.AdminService;
 
+import org.w3c.dom.Document;
+import javax.xml.parsers.DocumentBuilderFactory;
+import org.xml.sax.InputSource;
+import javax.xml.xpath.XPathFactory;
+import javax.xml.xpath.XPath;
+import javax.xml.namespace.NamespaceContext;
+import java.io.BufferedInputStream;
+import com.sun.enterprise.webservice.monitoring.NamespaceContextImpl;
+
 /**
  * This is the mechanism to provide web service information for a given module. 
  * A WebServiceInfoProvider implementation is a class that extends the 
@@ -243,12 +266,25 @@ public class AppServWebServiceInfoProvider implements WebServiceInfoProvider {
                     final String wsFile = (String) propMap.get(
                         WebServiceInfoProvider.WS_XML_PROP_NAME);
 
+                    // port contains name-space:port-name
+                    // get the second part of this string (after the ":")
+                    String port = null;
+                    StringTokenizer st = new StringTokenizer(
+                    pt.getWsdlPort().getLocalPart(),
+                    ":");
+                    while(st.hasMoreTokens()) {
+                        port = st.nextToken();
+                    }
+
+                    final String serviceUrl =
+                    getServiceURL(wsdl,port) ;
+
                     // populate web service endpoint info object
                     WebServiceEndpointInfoImpl wsInfo = new 
                         WebServiceEndpointInfoImpl(pt.getPortComponentName(),
                             uri, appId, bundleName, isAppStandAloneModule,
                             wsdlFile, mapping, wsFile,implType,implName,
-                            implClass, wsDescrInfo.isSecure());
+                            implClass, serviceUrl, wsDescrInfo.isSecure());
 
                     wsInfo.putField(
                     WebServiceEndpointInfo.SUN_WEB_XML_KEY, (Serializable)propMap.get(
@@ -285,6 +321,28 @@ public class AppServWebServiceInfoProvider implements WebServiceInfoProvider {
         return aList;
     }
 
+    private String getServiceURL(String wsdlFile, String localPart) {
+        String endpointURL = null;
+        try {
+            DocumentBuilderFactory dFactory = 
+                DocumentBuilderFactory.newInstance();
+            InputSource inputSource = new InputSource(new
+                BufferedInputStream(new FileInputStream(wsdlFile)));
+            Document wsdlDoc = dFactory.newDocumentBuilder().parse(new
+            BufferedInputStream(new FileInputStream(wsdlFile)));
+            XPathFactory xPathFactory = XPathFactory.newInstance();
+            XPath xPath = xPathFactory.newXPath();
+            NamespaceContext context = new NamespaceContextImpl(wsdlDoc);
+            xPath.setNamespaceContext(context);
+            String xpathExpression = "/:definitions/:service/:port[@name='"+ 
+                localPart+"']/soap:address/@location";
+            endpointURL = xPath.evaluate(xpathExpression, inputSource);
+            return endpointURL;
+        } catch (Exception e) {
+            return null;
+        }
+    }
+
     /**
      * Returns the List of WebServiceEndpointInfos for the provided WEB module.
      *
