@@ -1,24 +1,37 @@
 /*
- * The contents of this file are subject to the terms 
- * of the Common Development and Distribution License 
- * (the License).  You may not use this file except in
- * compliance with the License.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  * 
- * You can obtain a copy of the license at 
- * https://glassfish.dev.java.net/public/CDDLv1.0.html or
- * glassfish/bootstrap/legal/CDDLv1.0.txt.
- * See the License for the specific language governing 
- * permissions and limitations under the License.
+ * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
  * 
- * When distributing Covered Code, include this CDDL 
- * Header Notice in each file and include the License file 
- * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
- * If applicable, add the following below the CDDL Header, 
- * with the fields enclosed by brackets [] replaced by
- * you own identifying information: 
- * "Portions Copyrighted [year] [name of copyright owner]"
+ * The contents of this file are subject to the terms of either the GNU
+ * General Public License Version 2 only ("GPL") or the Common Development
+ * and Distribution License("CDDL") (collectively, the "License").  You
+ * may not use this file except in compliance with the License. You can obtain
+ * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
+ * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
+ * language governing permissions and limitations under the License.
  * 
- * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
+ * When distributing the software, include this License Header Notice in each
+ * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
+ * Sun designates this particular file as subject to the "Classpath" exception
+ * as provided by Sun in the GPL Version 2 section of the License file that
+ * accompanied this code.  If applicable, add the following below the License
+ * Header, with the fields enclosed by brackets [] replaced by your own
+ * identifying information: "Portions Copyrighted [year]
+ * [name of copyright owner]"
+ * 
+ * Contributor(s):
+ * 
+ * If you wish your version of this file to be governed by only the CDDL or
+ * only the GPL Version 2, indicate your decision by adding "[Contributor]
+ * elects to include this software in this distribution under the [CDDL or GPL
+ * Version 2] license."  If you don't indicate a single choice of license, a
+ * recipient has the option to distribute your version of this file under
+ * either the CDDL, the GPL Version 2 or to extend the choice of license to
+ * its licensees as provided above.  However, if you add GPL Version 2 code
+ * and therefore, elected the GPL Version 2 license, then the option applies
+ * only if the new code is made subject to such option by the copyright
+ * holder.
  */
 
 /*
@@ -56,6 +69,7 @@ public class ElementToObjectMapper {
     public BaseElement getElementObject(String element) throws ElementNotFoundException, ClassNotFoundException, 
                                                 InstantiationException, IllegalAccessException{
         String eleClassName = (String)(elementMap.get(element));
+	
         if (eleClassName == null)
             throw new ElementNotFoundException(element+" "+stringManager.getString("upgrade.transform.elementObjectMapper.elementNotFound"));
         return (BaseElement)((Class.forName(eleClassName)).newInstance());
@@ -87,7 +101,7 @@ public class ElementToObjectMapper {
         elementMap.put("applications", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
         elementMap.put("lifecycle-module", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
         elementMap.put("j2ee-application", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
-        elementMap.put("ejb-module", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
+		elementMap.put("ejb-module", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
         elementMap.put("web-module", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
         elementMap.put("connector-module", "com.sun.enterprise.tools.upgrade.transform.elements.GenericComponent");
         elementMap.put("http-listener", "com.sun.enterprise.tools.upgrade.transform.elements.HttpListener");
@@ -169,8 +183,8 @@ public class ElementToObjectMapper {
         elementMap.put("iiop-endpoint", "com.sun.enterprise.tools.upgrade.transform.elements.UnSupportedElement");
         
         // Additions for AS9 upgrade
-        elementMap.put("node-agents", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
-        elementMap.put("node-agent", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
+        elementMap.put("node-agents", "com.sun.enterprise.tools.upgrade.transform.elements.UnSupportedElement");
+        elementMap.put("node-agent", "com.sun.enterprise.tools.upgrade.transform.elements.UnSupportedElement");
         elementMap.put("lb-configs", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("lb-config", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("cluster-ref", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
@@ -182,10 +196,11 @@ public class ElementToObjectMapper {
         elementMap.put("listener-config", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("filter-config", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("access-log", "com.sun.enterprise.tools.upgrade.transform.elements.AccessLog");
+        // Added for AS8.2 EE
+        elementMap.put("access-log", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("http-access-log", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("request-processing", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("keep-alive", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
-        elementMap.put("connection-pool", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("http-protocol", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("http-file-cache", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("security-map", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
@@ -199,7 +214,9 @@ public class ElementToObjectMapper {
         elementMap.put("event", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("load-balancers", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
         elementMap.put("load-balancer", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
+        //Will be supported when domain is created afresh. CR - 6480041		
         elementMap.put("jms-availability", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
+        elementMap.put("connection-pool", "com.sun.enterprise.tools.upgrade.transform.elements.GenericElement");
     }
     /*
      * This method builds key map.  The keyMap holds key if any for an element for comparison puprose
@@ -213,7 +230,7 @@ public class ElementToObjectMapper {
         }
         // all resources
         keyMap.put("custom-resource", "jndi-name");
-        keyMap.put("jdbc-resource","pool-name");
+        keyMap.put("jdbc-resource","jndi-name");
         keyMap.put("external-jndi-resource", "jndi-name");
         keyMap.put("jdbc-connection-pool", "name");
         keyMap.put("mail-resource", "jndi-name");
@@ -250,6 +267,7 @@ public class ElementToObjectMapper {
         keyMap.put("security-map","name");
         keyMap.put("load-balancer","name");
         keyMap.put("management-rule","name");
+        keyMap.put("system-property","name");
     }
     private void buildElementStructureMap(){
         
@@ -318,7 +336,8 @@ public class ElementToObjectMapper {
         //web-container-availability? , ejb-container-availability? , property*)
         elementStructureMap.put("web-container-availability", getListWithElementNames(new String[]{}));
         elementStructureMap.put("ejb-container-availability", getListWithElementNames(new String[]{"jms-availability","property"}));
-        elementStructureMap.put("jms-availability", getListWithElementNames(new String[]{"property"}));
+        //Will be supported when domain is created afresh. CR - 6480041
+        //elementStructureMap.put("jms-availability", getListWithElementNames(new String[]{"property"}));
         elementStructureMap.put("security-map", getListWithElementNames(new String[]{"property"}));
         elementStructureMap.put("principal", getListWithElementNames(new String[]{}));
         //principal | user-group)+ , backend-principal)
@@ -344,7 +363,7 @@ public class ElementToObjectMapper {
         elementStructureMap.put("access-log", getListWithElementNames(new String[]{}));
         elementStructureMap.put("http-access-log", getListWithElementNames(new String[]{}));
         elementStructureMap.put("server-ref", getListWithElementNames(new String[]{}));
-        
+        elementStructureMap.put("clusters", getListWithElementNames(new String[]{"system-property","property"}));
     }
     private java.util.List getListWithElementNames(String[] succeedingElements){
         java.util.ArrayList eleList = new java.util.ArrayList();
